"""
Point2Play Source Code
Copyright 2003,2004,2005 TransGaming Technologies Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""

import ConfigParser, os, copy, config, os.path, stat, shutil, sys, signal, base64, stat, string, p2p_diag, gtk, time, config_gui, cedega_version_parse, urllib2, misc_utils
import gddb
import gui_utils, log

_TG_CONFIG_VERSION=12
REAPING=0

# Title of WineX as seen by users. This has been abstracted out in case it ever
# needs to be renamed, such as when distributed with an OEM package or something.
WINEX_DISPLAY_TITLE = "Cedega"

# CDROM Eject Monitoring constants
CDROMEJECTMONITOR_DEFAULT = 0
CDROMEJECTMONITOR_ON      = 1
CDROMEJECTMONITOR_OFF     = 2

SETUP_SCRIPT_NAME = "tg_game_setup.sh"  # for systemwide games - the set-up-for-local-user script

# how many user-defined debug options shall we save between sessions?
NUM_DEBUG_OPTIONS_TO_SAVE = 5

# Current version of Point2Play
# 
# If a test version, have a string between the main verson and the test string
# ie. "1.3 Test#3" is good, "1.3test3" is not.
VERSION_STRING = "6.0"

import re

def version_cmp(x, y):
    """ Sorts Cedega/WineX versions correctly """

    # get rid of <401f>-like stuff at beginning of line
    while True:
        match = re.search('^(<[^>]+>)(.*$)', x)
        if match:
            x = match.group(2)
        else:
            break
    while True:
        match = re.search('^(<[^>]+>)(.*$)', y)
        if match:
            y = match.group(2)
        else:
            break    

    # for beta package names consisting of at least one letter followed by Stuff and then a number:
    match1 = re.search('^([A-Za-z].*?)([0-9]*)$', x)
    match2 = re.search('^([A-Za-z].*?)([0-9]*)$', y)
    if match1 and match2:  # both are beta packages
        if match1.group(1) == match2.group(1):  # same beta run
            try:
                return int(match1.group(2)) - int(match2.group(2))  # compare the numbers
            except: # just in case something funny happened
                return cmp(x, y) 
        else: # different beta run - do a string sort
            return cmp(x, y)  # don't bother with numbers
    if match1 or match2:  # just one of them is a beta package
        return cmp(x, y)

    # If we got here, we are comparing two different versions.
    # Split each incoming version string into pieces to do this properly

    splitter = re.compile('([-.]|\d+|[^-.\d]+)')
    xlist = splitter.split(x)
    ylist = splitter.split(y)
    for current_x, current_y in map(None, xlist, ylist):
        if current_x == '' and current_y == '':
            continue
        if current_x == None:
            return -1
        elif current_y == None:
            return 1
        if current_x == '' and current_y == '':
            continue
        elif current_x == '-':
            return -1
        elif current_y == '-':
            return 1
        elif current_x == '.' and current_y == '.':
            continue
        elif current_x == '.':
            return -1
        elif current_y == '.':
            return 1;
        else:
            match1 = re.search('^\d+$', current_x)
            match2 = re.search('^\d+$', current_y)
            if match1 and match2:
                match1 = re.search('^0', current_x)
                match2 = re.search('^0', current_y)
                if match1 or match2:
                    if cmp(current_x, current_y):
                        return cmp(current_x, current_y)
                else:
                    diff = int(current_x) - int(current_y)
                    if diff:
                        return diff
            else:
                current_x = string.lower(current_x)
                current_y = string.lower(current_y)
                if cmp(current_x, current_y):
                    return cmp(current_x, current_y)
        
    # I shouldn't ever get here right? FIXME?
    return 0



class Point2PlayError(Exception):
    """Base class for Point2Play exceptions"""
    def __init__(self, message ):
        self.message = message

    def __str__( self ):
        return `self.message`

try:
    import tests
except:
    log.logger.critical(_("Unable to load test case utilities"))
    sys.exit(1)

class badConfigFile(Point2PlayError):
    """Raised when the given config file is missing information"""

class gameFolderDoesNotExist(Point2PlayError):
    """Raised when unable to find given game folder name. """

class shortcutDoesNotExist(Point2PlayError):
    """Raised when unable to find a shortcut in a certain game folder. """

class winexNotFound(Point2PlayError):
    """Raised when winex binary could not be found to run application"""

class badWinexVersion(Point2PlayError):
    """Raised when winex version can't be found"""

class renameGameFolderFailed(Point2PlayError):
    """Raised when unable to rename game folder."""

class pathError(Exception):
    """Raised when _{unix|windows}_to_{windows|unix}_path fails"""
    def __init__( self, message ):
        self.message = message

    def __str__( self ):
        return `self.message`

def _unix_to_windows_path( path, root, drive_letter = "C" ):
    drive_letter = drive_letter.upper( )
    if len( drive_letter ) != 1:
        raise pathError( _("Drive letter not 1 letter") )
    if drive_letter < "A" or drive_letter > "Z":
        raise pathError( _("Drive letter not A-Z") )
    temp = path.replace( root, drive_letter + ":" )
    new_path = temp.replace( "/", "\\" )
    return new_path

def _windows_to_unix_path( path, root, drive_letter = "C" ):
    drive_letter = drive_letter.upper()
    if len( drive_letter ) != 1:
        raise pathError( _("Drive letter not 1 letter") )
    if drive_letter < "A" or drive_letter > "Z":
        raise pathError( _("Drive letter not A-Z") )
    temp = path.replace( drive_letter + ":", root )
    new_path = temp.replace( "\\", "/" )
    return new_path

class Point2Play:
    # This is a helper function for the patching functionality of P2P
    # We will drop to each subdirectory of the given path, copying them
    # from the patch directory into the WineX directory in P2PFOLDER/.winex_ver/etc.etc.
    #
    # The way this patch mechanism works is really really dumb -- but its
    # only intended to update scripts, so it should be quite fast... 
    # anyway, this should not stick around for long, at least as it is now
    def patchhelper_cp_contents(self, dirpath, recursed_path, ver):
        # First we need to check that the source directory exists. If it doesn't.
        # don't do anything. Most likely there is no patches directory AT ALL or
        # no patches exist for that version of WineX.
        if not os.access(dirpath, os.F_OK):
           return
       
        # Ok, if the directory exists, then we have some new files!


        # Create the dest directory if its not there already, since 'cp' will barf
        # otherwise
        if not os.access(ver["path"] + "/" + recursed_path, os.F_OK):
           try:
              os.makedirs(ver["path"] + "/" + recursed_path)
           except:
              print "Error creating %s during patching process" % ver["path"] + "/" + recursed_path     
              return

        # Get the list of files in the directory and look at them
        # each individually
        files_in_dir = os.listdir (dirpath)
        for indv_file in files_in_dir:
            mode = os.stat(dirpath + "/" + indv_file)[stat.ST_MODE]

            # If the individual file is actually a subdirectory, we need to recurse
            # into it.
            if stat.S_ISDIR ( mode ):
               self.patchhelper_cp_contents (dirpath + "/" + indv_file, recursed_path + indv_file + "/", ver)
            else:
               # This is an actual file, blindly copy it over the original.
               # (This first iteration should only be for small stuff)

               # Copy the file
               os.system("cp -f %s %s 2> /dev/null" % (dirpath + '/' + indv_file, ver["path"] + "/" + recursed_path) )

            
    def __init__( self, filename ):
        """ Filename is the cedega general config file
        """

        try: 
            self.P2PPATH = os.getenv('P2PPATH') + "/"
        except:
            self.P2PPATH = "./"

        try:
            self.P2PFOLDER = os.getenv('P2PFOLDER')
            assert self.P2PFOLDER
        except:
            self.P2PFOLDER = os.getenv('HOME') + "/.cedega/"
        if self.P2PFOLDER[-1] != "/":
            self.P2PFOLDER = self.P2PFOLDER + "/"

        try:
            self.WINEXGLOBALPREFIX = os.getenv('WINEXGLOBALPREFIX')
            assert self.WINEXGLOBALPREFIX
        except:
            self.WINEXGLOBALPREFIX = os.getenv('HOME') + "/.transgaming_global/"
            os.environ['WINEXGLOBALPREFIX'] = self.WINEXGLOBALPREFIX
        if self.WINEXGLOBALPREFIX[-1] != "/":
            self.WINEXGLOBALPREFIX = self.WINEXGLOBALPREFIX + "/"

        try: # if someone changes WINEXGLOBALPREFIX, we need to make sure
             # the proper FONTINSTALL_DIR is in the environment for 
             # the tg-font-install script.
            self.FONTINSTALL_DIR = os.getenv('FONTINSTALL_DIR')
            assert self.FONTINSTALL_DIR
        except:
            self.FONTINSTALL_DIR = self.WINEXGLOBALPREFIX + "Fonts/"
            os.environ['FONTINSTALL_DIR'] = self.FONTINSTALL_DIR
        if self.FONTINSTALL_DIR[-1] != "/":
            self.FONTINSTALL_DIR = self.FONTINSTALL_DIR + "/"

        # same for the MOZCTLINSTALL_DIR, for the tg-mozctl-install script.

        try: # if someone changes WINEXGLOBALPREFIX, we need to make sure
             # the proper MOZCTLINSTALL_DIR is in the environment for 
             # the tg-mozctl-install script.
            self.MOZCTLINSTALL_DIR = os.getenv('MOZCTLINSTALL_DIR')
            assert self.MOZCTLINSTALL_DIR
        except:
            self.MOZCTLINSTALL_DIR = self.WINEXGLOBALPREFIX
            os.environ['MOZCTLINSTALL_DIR'] = self.MOZCTLINSTALL_DIR
        if self.MOZCTLINSTALL_DIR[-1] != "/":
            self.MOZCTLINSTALL_DIR = self.MOZCTLINSTALL_DIR + "/"

        self.gamedir = self.P2PFOLDER

        try: # grab the location of the system-wide game directory 
            self.SYSTEMWIDE_GAMEDIR = os.getenv('SYSTEMWIDE_GAMEDIR')
            assert self.SYSTEMWIDE_GAMEDIR            
        except: # use canonical location and place in the environment
            self.SYSTEMWIDE_GAMEDIR = "/usr/lib/transgaming_games"
            os.environ['SYSTEMWIDE_GAMEDIR'] = self.SYSTEMWIDE_GAMEDIR
        
        self.SYSTEMWIDE_GAMEDIR = self.SYSTEMWIDE_GAMEDIR + "/"

        try:
            self.USER_DEFINED_PROFILES_PATH = os.getenv('USER_DEFINED_PROFILES_PATH') + "/"
        except:
            self.USER_DEFINED_PROFILES_PATH = self.P2PFOLDER + "/configuration_profiles/"

        try:
            self.DEFAULT_PROFILES_PATH = os.getenv('DEFAULT_PROFILES_PATH') + "/"
        except:
            self.DEFAULT_PROFILES_PATH = self.P2PFOLDER + "/.default_configuration_profiles/"

        try:
            self.ORIGINAL_DEFAULT_PROFILES_PATH = os.getenv('ORIGINAL_DEFAULT_PROFILES_PATH') + "/"
        except:
            self.ORIGINAL_DEFAULT_PROFILES_PATH = self.P2PPATH + "default_configuration_profiles/"

        try:
            self.MOZCTLINSTALLER_PATH = os.getenv('MOZCTLINSTALLER_PATH') + "/"
        except:
            self.MOZCTLINSTALLER_PATH = self.P2PPATH + "transgaming_mozctl/"

        try:
            self.FONTINSTALLER_PATH = os.getenv('FONTINSTALLER_PATH') + "/"
        except:
            self.FONTINSTALLER_PATH = self.P2PPATH + "transgaming_fontinstaller/"

        try:
            self.GLOBAL_DELTA_FILENAME = os.getenv('GLOBAL_DELTA_FILENAME')
            assert self.GLOBAL_DELTA_FILENAME
        except:
            self.GLOBAL_DELTA_FILENAME = ".global.delta"

        self.VERSION_STRING = "6.0"

        # Place to hold instance of Point2Play_gui
        self.Point2Play_gui = None
        # Parent window needed for popping up error dialogs
        self.parent_window = None

        # Some useful stuff predefined..
        self.last_gddb = None
        self.versions = []           # WineX Versions that are known by Point2Play
        self.default_winex = None  # The version of WineX user set as default
        self.LocalesInstalled = None # Languages installed through P2P 
        self.current_default_cedega_profile = None  # This will be the user-proof profile which matches the current cedega version.
        self.current_user_cedega_profile = None # This will be the user-editable profile which matches the current cedega version.        
        self.skip = 0

        self.renice_wineserver = 0
        self.CMDLINE_RENICING = 0

        # First we will go through the master Point2PlayRC file and pick up any versions of
        # WineX that are installed system-wide (if it exists)
        #
        # NOTE: Eventually this file will serve as a general master rc file, but for now
        #       it is only for system-wide WineX Installs
        global_rc_file = ConfigParser.ConfigParser()
        global_default = None
        if (os.access(self.P2PPATH + "Point2PlayRC", os.F_OK) == 1 and os.access(self.P2PPATH + "Point2PlayRC", os.R_OK) == 1):
            try:
                global_rc_file.readfp( file( self.P2PPATH + "Point2PlayRC" ) )
                ver_from_file = global_rc_file.options("winex")
                for ver in ver_from_file:
                   if ver == "default":
                       self.default_winex = global_rc_file.get("winex","default")
                       # if the default used in the local rc doesn't exist let's use the global one.
                       global_default = global_rc_file.get("winex","default")
                   else:
                       InitialPath = global_rc_file.get("winex", ver)

                       # We take a different approach depending on whether this is a retail WineX or CVS
                       # entry
                       num_colons = InitialPath.count(":")
                       
                       if (num_colons == 1):
                          # 1 colon == Retail release
                          # <WineXPath>:<Script Path>
                          (WineXPath, ScriptPath) = InitialPath.split(":",1)
                          RetailInstallDir = None
                       elif (num_colons == 2):
                          # 2 colons == CVS release
                          # <WineXPath>:<Script Path>:<Retail WineX Path>
                          (WineXPath, ScriptPath, RetailInstallDir) = InitialPath.split(":",2)
                       else:
                          # Unknown format string, ignore it
                          continue

                       versionparse = cedega_version_parse.ParseVersion(ver)
                       ver = versionparse.get_parsed_version_string()

                       # Only append a version once
                       found_duplicate = 0
                       for i in range(0, len( self.versions ) ):
                          if self.versions[i]["version"] == ver:
                             found_duplicate = 1
                          
                       if (found_duplicate == 1):
                          print _("Warning: Multiple defined %s Version %s in RC file %s, only the first one used") % (WINEX_DISPLAY_TITLE, ver, self.P2PPATH + "Point2PlayRC")
                       else:
                          self.versions.append( { "version": ver, "path": WineXPath, "local":0, "script": ScriptPath, "retailinstalldir": RetailInstallDir, "version_credential":versionparse.get_version_credential() } )
            except:
               print _("There was a problem processing your global RC file %s. You may have problems accessing some versions of %s from Cedega you had configured") % (self.P2PPATH + "Point2PlayRC", WINEX_DISPLAY_TITLE)
               pass


        # Next we need to sort out the [transgaming] section of the (local) rc file
        self.config_file_name = filename
        log.logger.info("Filename is " + filename)
        self.debug = 0
        config_file = ConfigParser.ConfigParser()
        config_file.readfp( file( filename, "r" ) )

        try:
            # note: we used to have gamedir as the single required option in the
            # transgaming section but this got in the way of relocatable packages.
            # If we eventually make gamedir different from P2PFOLDER, or permit
            # game sharing, we will have to rework many things. So for now I've
            # somewhat arbitrarily chosen "wizard_version_last_run" to be the 
            # item to define the transgaming section and allow the check for 
            # existence.
            
            self.config = { "wizard_version_last_run": config_file.get( "transgaming", "wizard_version_last_run" ) }
        except ConfigParser.NoOptionError, e:
            raise badConfigFile( _("Missing options %s") % e.value )
        except ConfigParser.NoSectionError, e:
            raise badConfigFile( _("Missing [transgaming] section") )

        # Iterate through all of the Cedega engine versions installed 
        # Note: to make the packages relocatable, we now use  
        # a magic string in the .cedegarc called $ENGINE_FOLDER_ROOT
        # to be replaced with whaterver the user's P2PFOLDER variable 
        # contains plus the string ".winex_ver".

        local_global_set = 0
        try:
           file2 = config_file.options("winex")
           for ver in file2:
              if ver == "default":
                 # User's default trumps the global one
                 self.default_winex = config_file.get("winex","default")
                 local_global_set = 1
              else:
                 InitialPath = config_file.get("winex", ver)

#                 log.logger.info("Just read in " + InitialPath)
                 InitialPath = self.decode_cedegarc_enginepath(InitialPath)
#                 log.logger.info("Decoded it to: " + InitialPath)

                 # We take a different approach depending on whether this is a retail WineX or CVS
                 # entry
                 num_colons = InitialPath.count(":")
  
                 if (num_colons == 0):
                    # 0 colons == P2P installed WineX
                    WineXPath = InitialPath
                    RetailInstallDir = None
                    ScriptPath = None
                 elif (num_colons == 1):
                    # 1 colon == Retail release
                    # <WineXPath>:<Script Path>
                    (WineXPath, ScriptPath) = InitialPath.split(":",1)
                    RetailInstallDir = None
                 elif (num_colons == 2):
                    # 2 colons == CVS release
                    # <WineXPath>:<Script Path>:<Retail WineX Path>
                    (WineXPath, ScriptPath, RetailInstallDir) = InitialPath.split(":",2)
                 else:
                    # Unknown format string, ignore it
                    continue
                    
                 # Check if this version supports 4.0 features.
                 versionparse = cedega_version_parse.ParseVersion(ver)
                 ver = versionparse.get_parsed_version_string()
 
                 # Check to make sure there isn't a duplicate entry
                 found_duplicate = 0
                 for i in range(0, len( self.versions ) ):
                     if self.versions[i]["version"] == ver:
                     # If the duplicate is from the global rc file, delete it. local has precidence
                       if self.versions[i]["local"] == 0:
                          del self.versions[i]
                       else:
                          found_duplicate = 1
                          
                 if (found_duplicate == 1):
                     print _("Warning: Multiple defined %s Version %s in RC file %s, only the first one used") % (WINEX_DISPLAY_TITLE, ver, self.config_file_name)
                 else:
                     # Versions here trump any match version in the global rc file
                     self.versions.append( { "version": ver, "path": WineXPath, "local":1, "script": ScriptPath, "retailinstalldir": RetailInstallDir, "version_credential":versionparse.get_version_credential() } )
        except:
           print _("There was a problem processing your local RC file %s. You may have problems accessing some versions of %s from Cedega you had configured") % (self.config_file_name, WINEX_DISPLAY_TITLE)
           pass

 
        if (local_global_set == 1):
           # Check to make sure the default exists, if not, set it to the global one if there was one
           default_found = 0
           for i in range(0, len(self.versions) ):
              if self.versions[i]["version"] == self.default_winex:
                 default_found = 1

           if default_found == 0:
              self.default_winex = global_default

        # Now that we have a list of all of the installed versions of WineX, we need to check if
        # we have any impending updates to those particular versions of WineX
        for ver in self.versions:
           if ver["local"] == 1:
              winex_ver_dir = self.P2PPATH + 'patches/' + ver["version"]
              self.patchhelper_cp_contents(winex_ver_dir, "", ver)

        try:
            self.config["accept_license"] = int(config_file.get( "transgaming", "accept_license" ))
        except:
            self.config["accept_license"] = 0

        # Find out if the user wants default, on or off for CDROM eject monitoring
        try:
            self.config["cdrom_eject_monitoring"] = int(config_file.get( "transgaming", "cdrom_eject_monitoring"))
        except:
            self.config["cdrom_eject_monitoring"] = 0 # default
        
        # Find out if the user wants automatic icon syncing with the Window Manager menu system or not
        try:
            self.config["autowmiconsync"] = int(config_file.get( "transgaming", "autowmiconsync"))
        except:
            self.config["autowmiconsync"] = 0 # AUTOWMICONSYNC_ON

        # The user will only be harassed about his stack-challenged distro only once
        try:
            self.config["warned_about_pthreadstacksize"] = int(config_file.get( "transgaming", "warned_about_pthreadstacksize" ) )
        except:
            self.config["warned_about_pthreadstacksize"] = 0   

        # the user can choose to hardcode pthreads on or off in the config file
        try:
            self.config["pthreads_forced"] = config_file.get( "transgaming", "pthreads_forced")
        except:
            self.config["pthreads_forced"] = "auto"

        try:
            self.config["username"] = config_file.get("transgaming","username")
            try:
                password = config_file.get("transgaming","password")
                self.config["password"] = base64.decodestring( password )
            except:
                self.config["password"] = ""
        except:
            self.config["username"] = ""
            self.config["password"] = ""                

        try:
            self.config["wizard_version_last_run"] = config_file.get("transgaming", "wizard_version_last_run")
        except:
            self.config["wizard_version_last_run"] = ""
        
        try:
            self.config["last_gui_version_run"] = config_file.get("transgaming", "last_gui_version_run")
        except:
            self.config["last_gui_version_run"] = ""

        try:
            self.config["delta_migration_required"] = config_file.get("transgaming", "delta_migration_required")
        except: # if it isn't there, we need to migrate
            self.config["delta_migration_required"] = "True"

        try:
            self.config["debugger_use_logfile"] = config_file.get("transgaming", "debugger_use_logfile")
        except: 
            self.config["debugger_use_logfile"] = "False"

        try:
            self.config["debugger_output_to_console"] = config_file.get("transgaming", "debugger_output_to_console")
        except: 
            self.config["debugger_output_to_console"] = "True"

        try:
            self.config["debugger_last_logfile"] = config_file.get("transgaming", "debugger_last_logfile")
        except: 
            self.config["debugger_last_logfile"] = ""

        try:
            self.config["debugger_last_channel"] = config_file.get("transgaming", "debugger_last_channel")
        except: 
            self.config["debugger_last_channel"] = ""


        for config_item, info_list in gui_utils.gui_config_items.iteritems():
            self.config[config_item] = gui_utils.load_value_from_config_file(config_item, config_file)
        try:
            proxy_password = config_file.get("transgaming", "proxy_password")
            self.config["proxy_password"] = base64.decodestring( proxy_password )
        except:
            self.config["proxy_password"] = ""


        try:
            game_dir_list = os.listdir( self.gamedir )
        except:
            raise badConfigFile( _("gamedir option points to bad directory: " + self.gamedir) )

        
        # Next we need to slurp in the joysticks configurations
        self.joysticks = []
        try:
           file3 = config_file.options( "joysticks" )
           for joystick in file3:
               joystick_info = config_file.get("joysticks", joystick)
               (jsname, jsdetails) = joystick_info.split("=", 1)
               self.joysticks.append ( {"name": jsname, "details": jsdetails} )
        except:
           # Its possible the user doesn't have a [joystick] entry, especially for
           # those users with legacy config files. Ignore.
           pass
           
        self.profiles = {}
        self.debug_options = [] # storage for commonly-used options in the debugger

        # If the user's P2PFOLDER/.defaults_configuration_profiles/ directory is empty ( or missing files ), 
        # copy over the default profiles from the transgaming_cedega/ directory.
        self.populate_default_profiles_directory()

        # Ok, let's check the packaged default Cedega profiles, and see if their user-editable equivalents are
        # in place.  If not, copy the defaults over.
        self.match_and_copy_default_cedega_profiles()

        for filename in os.listdir(self.USER_DEFINED_PROFILES_PATH):
            self.profiles[filename] = []            

        # slurp in the matchings of configuration profiles with games
        try:
            profiles = config_file.options("profiles")
            for profile in profiles:
                profile_info = config_file.get("profiles", profile)
                (profile_name,profile_games)=profile_info.split("=",1)

                # only keep track of the profiles that are attached to real filenames
                try:
                    x = self.profiles[profile_name]  # see if it's a real filename
                    # convert string that looks like '"a", "b", "c"' or '"a","b","c"' to list
                    profile_games_list = []
                    for g in profile_games.split(","):
                        match = re.search('^\s*\"(.*)\"\s*$', g)
                        if match:
                            profile_games_list.append(match.group(1))
                    self.profiles[profile_name] = profile_games_list
                except:
                    log.logger.info("The configuration profile " + profile + " referred to in the .cedegarc file does not appear to exist. Ignoring." )
        except:
            pass # no profiles section, but that's OK. We'll make one when we write the config file.

        # slurp in the debugging options
        try:
            debug_opts = config_file.options("debug_options")
            log.logger.info("Found " + str(len(debug_opts)) + " personal debug options...")
            for option_entry in debug_opts:
                debug_option = config_file.get("debug_options", option_entry)
                log.logger.info("Got a debug option from .cedegarc: " + option_entry + " ... " + debug_option)
                self.debug_options.append(debug_option)
        except:
            pass # no debug_options section, but that's OK.
                 # We'll make one when we write the config file.

        # Next we need to slurp in the font aliases
        self.font_aliases = []
        try:
           fontaliases = config_file.options( "font_aliases" )
           for font_alias in fontaliases:
               font_info = config_file.get("font_aliases", font_alias)
               (aliasfrom, aliasto) = font_info.split("=", 1)
               self.font_aliases.append ( {"aliasfrom": aliasfrom, "aliasto": aliasto} )
        except:
           # Its possible the user doesn't have a [font_aliases] entry, especially for
           # those users with legacy config files. Set up some defaults
           self.font_aliases.append( {"aliasfrom": "MS Serif", "aliasto": "Times New Roman"} )
           self.font_aliases.append( {"aliasfrom": "Helv", "aliasto": "Arial"})
           self.font_aliases.append( {"aliasfrom": "MS Sans Serif", "aliasto": "Arial"})
           self.font_aliases.append( {"aliasfrom": "System", "aliasto": "Arial" })
           self.font_aliases.append( {"aliasfrom": "Tahoma", "aliasto": "Arial"} ) 

           pass

        # Next, we need to load in the list of locales that we have installed
        # through Cedega GUI
        try:
           languages_installed = config_file.options("languages")
           for lang_locale in languages_installed:
              langvers = config_file.get("languages", lang_locale)
              (lang_p2pver, lang_revis, credits) = langvers.split(":", 2)
              if not self.LocalesInstalled:
                 self.LocalesInstalled = { lang_locale : { "p2pver":lang_p2pver, "revver":lang_revis, "credits":credits } }
              else:
                 self.LocalesInstalled[lang_locale] = { "p2pver":lang_p2pver, "revver":lang_revis, "credits":credits } 

        except:
           # It's ok to not have any language entries in the RC file.. just let it go
           pass

        # Next we get the extra font path (currently only one but should in the future be multiples)
        self.extra_font_path = []
        try:
           fontpaths = config_file.options( "font_pathes" )
           for font_path in fontpaths:
               font_info = config_file.get("font_pathes", font_path)
               self.extra_font_path.append ( font_info )
        except:
            # If there is none we just ignore - possible that the user hasn't set one yet or
            # this is an old config
            pass

        """Only use the directories"""
        game_list = None
        needs_writing = []
        for dir_entry in game_dir_list:
            # If we need to specify this particular game directory should be written to disk 
            # or not (due to creating a default config for a particular game)
            add_to_writing_list = 0

            if dir_entry[0] == ".":
                continue
            if dir_entry == "configuration_profiles":
                continue
            dir_name = self.gamedir + dir_entry
            if os.path.isdir( dir_name ):
                shortcuts = []
                try:
                    game_config_file = file( dir_name + "/games.ini" )
                    game_config = ConfigParser.ConfigParser()
                    game_config.readfp( game_config_file )
                    for shortcut_entry in game_config.sections():
                        try:
                           commandlineargs = game_config.get( shortcut_entry, "commandlineargs" )
                        except:
                           # new addition to game config file - if it's absent, make it blank
                           commandlineargs = "" 

                        try:
                           cedega_app = game_config.get( shortcut_entry, "cedega_app" )
                        except:
                           # new addition to game config file - if it's absent, make it true
                           cedega_app = "Y"

                        try:
                           path = game_config.get( shortcut_entry, "path" )
                           icon = game_config.get( shortcut_entry, "icon" )
                           workdir = game_config.get( shortcut_entry, "workdir" )
                           
                           try:
                               gddb = game_config.get( shortcut_entry, "gddb" )
                           except:
                               gddb = ""

                        except:
                            print _("Error parsing \"%s\" - Game \"%s\" is missing information") % (dir_name + "/games.ini", shortcut_entry) 
                            continue

                        # just a quick sanity check - if the path is a URL, CedegaApp should be N.
                        if misc_utils.is_URL(path):
                            cedega_app = "N"

                        # Add to our list of available

                        shortcuts.append( { "Name": shortcut_entry,
                                            "Path": self.decode_games_ini_path(path),
                                            "Icon": self.decode_games_ini_path(icon),
                                            "Workdir": self.decode_games_ini_path(workdir),
                                            "DirName" : self.decode_games_ini_path(os.path.basename(dir_name)),
                                            "CommandLineArguments" : commandlineargs,
                                            "gddb" : gddb,
                                            "CedegaApp" : cedega_app } )

                    if add_to_writing_list == 1:
                       needs_writing.append(dir_entry)

                except:
                    pass

                if game_list != None:
                    game_list[dir_entry] = shortcuts
                else:
                    game_list = { dir_entry: shortcuts }
        self._games = game_list

        # If we have some games.inis that need updating due to a game config file being newly created.
        if needs_writing:
           for nw_single in needs_writing:
              self.write_shortcut_entries(nw_single)
          

        try:
            self.tests = tests.tests()
        except tests.noTestsFound:
            pass

    def new_systemwide_games(self):
        """ If there are new systemwide games, this function returns a list of the folders.
            Otherwise it returns None. 
            Also return None if there are no engines installed. """

        if not len (self.versions):
            log.logger.info("No engines; skipping the search for system-wide games.")
            return None

        if not os.access( self.SYSTEMWIDE_GAMEDIR, os.F_OK ):
            log.logger.info("No system-wide games - " + self.SYSTEMWIDE_GAMEDIR + " not found.")
            return None
        
        systemwide_game_folders = os.listdir(self.SYSTEMWIDE_GAMEDIR)

        if not systemwide_game_folders:
            log.logger.info("No system-wide games - " + self.SYSTEMWIDE_GAMEDIR + " is empty.")
            return None

        # see if the games are new
        game_dir_list = []
        for curr_dirname in systemwide_game_folders:
            log.logger.info("Examining under the systemwide game folder: " + curr_dirname)
            if os.path.isdir( self.SYSTEMWIDE_GAMEDIR + curr_dirname ):
                if not self._games or curr_dirname not in self._games:
                    log.logger.info("Found a new system-wide game called " + curr_dirname)

                    setup_script = self.SYSTEMWIDE_GAMEDIR + curr_dirname + "/" + SETUP_SCRIPT_NAME
                    if not os.access(setup_script, os.F_OK):
                        log.logger.info("The system-wide game folder " + curr_dirname + " does not appear to have a setup script (" + setup_script + "). Skipping.") # FIXME - right? or should we just copy everything?
                        continue
                    if not os.access(setup_script, os.X_OK):
                        log.logger.info("The setup script " + setup_script + " exists but is not executable. Skipping.") # FIXME - popup message?
                        continue
                    game_dir_list.append(curr_dirname)

        if game_dir_list:
            return game_dir_list
        else:
            return None

    def setup_new_systemwide_games(self, new_game_folder_list):
        """ This function sets up each newly-installed system-wide game listed in 
            new_game_folder_list. ONLY get the list from new_systemwide_games(). It
            filters out the bad folders.

            For each game folder N in new_game_folder_list (which are directories under SYSTEMWIDE_GAMEDIR):
               Create a new game folder called N with the correct user-related framework files
               Run SYSTEMWIDE_GAMEDIR/N/tg_game_setup.sh
               add N to the internal list of game folders

            Note: the calling code is responsible for updating the GUI.

        """

        for curr_dirname in new_game_folder_list:
            new_game_folder = self.gamedir + curr_dirname

            # create a new game folder using the winex script to create all necessary 
            # framework files
            install_profile =  self.create_install_profile()
            try:
                retval = self.winex( 1, curr_dirname, "", None, new_game_folder + "/" + "games.ini.new", self.default_winex, OverrideGameProfile=install_profile)
            except Point2PlayError, e:
                log.logger.warning("Failed to create new game folder called " + new_game_folder)
                # FIXME - do a popup?
                self.delete_install_profile()
                continue

            # if we get here, we should just issue warnings but not skip anything more
            # since the directory will be there, just perhaps left empty. FIXME?

            self.delete_install_profile()
            if retval:
                # We should have a pid now to wait on.
                pid = retval
                (p, v) = os.waitpid(pid, os.WNOHANG)
                while p == 0 and v == 0:
                    (p, v) = os.waitpid(pid, os.WNOHANG)
            self.create_new_game_folder( curr_dirname )

            saved_dir = os.getcwd()
            os.chdir(self.SYSTEMWIDE_GAMEDIR + curr_dirname)
            os.environ['TG_SW_GAME'] = curr_dirname
            status = os.system("./" + SETUP_SCRIPT_NAME)
            log.logger.info("Running " + SETUP_SCRIPT_NAME)
            if status != 0:
                log.logger.warning("Error - " + SETUP_SCRIPT_NAME + " returned bad error status: " + str(status))
            os.chdir(saved_dir)

            # if we made it here, we can add the directory to the list of game folders
            log.logger.info("Adding " + curr_dirname + " to user's gamedir...")
            self.add_game_folder_and_shortcuts_to_gui(curr_dirname)


    def encode_cedegarc_enginepath(self, path):
        """ Given an engine path from a .cedegarc file, replace the part that matches
            P2PFOLDER/.winex_ver with "$ENGINE_FOLDER_ROOT". Return the new string. 
            (note P2PFOLDER already has the trailing /) """

        if path[0:len(self.P2PFOLDER + ".winex_ver/")] == self.P2PFOLDER + ".winex_ver/":
            return "$ENGINE_FOLDER_ROOT/" + path[len(self.P2PFOLDER + ".winex_ver/"):]
        return path

    def decode_cedegarc_enginepath(self, path):
        """ Given an engine path from a .cedegarc file, replace the part that matches 
            "$ENGINE_FOLDER_ROOT" with P2PFOLDER/.winex_ver. Return the new string.
            (note P2PFOLDER already has the trailing /) """

        if path[0:len("$ENGINE_FOLDER_ROOT/")] == "$ENGINE_FOLDER_ROOT/":
            return self.P2PFOLDER + ".winex_ver/" + path[len("$ENGINE_FOLDER_ROOT/"):]
        return path

    def encode_games_ini_path(self, path):
        """ Given a games.ini path, replace the part that matches
            P2PFOLDER with "$GAME_FOLDER_ROOT". Return the new string.
            (note P2PFOLDER already has the trailing /) """

        FILE_URL_PREFIX="file://"
        if path[0:len(self.P2PFOLDER)] == self.P2PFOLDER:
            return "$GAME_FOLDER_ROOT/" + path[len(self.P2PFOLDER):]

        # if it's a file URL:
        if path[0:len(FILE_URL_PREFIX)] == FILE_URL_PREFIX and path[len(FILE_URL_PREFIX):len(self.P2PFOLDER)+len(FILE_URL_PREFIX)] == self.P2PFOLDER:
            return FILE_URL_PREFIX + "$GAME_FOLDER_ROOT/" + path[len(FILE_URL_PREFIX)+len(self.P2PFOLDER):]
        return path

    def decode_games_ini_path(self, path):
        """ Given a games.ini path, replace the part that matches 
            "$GAME_FOLDER_ROOT" with P2PFOLDER. Return the new string.
            (note P2PFOLDER already has the trailing /) """

        FILE_URL_PREFIX="file://"
        if path[0:len("$GAME_FOLDER_ROOT/")] == "$GAME_FOLDER_ROOT/":
            return self.P2PFOLDER + path[len("$GAME_FOLDER_ROOT/"):]

        # if it's a file URL:
        if path[0:len(FILE_URL_PREFIX)] == FILE_URL_PREFIX and path[len(FILE_URL_PREFIX):len("$GAME_FOLDER_ROOT/") + len(FILE_URL_PREFIX)] == "$GAME_FOLDER_ROOT/":
            return FILE_URL_PREFIX + self.P2PFOLDER + path[len(FILE_URL_PREFIX) + len("$GAME_FOLDER_ROOT/"):]
        return path

    def getextra(self):
        extradll = None
        try:
            extra = os.getenv('TGEXTRA_MOD')
            extradll = extra + "/libd3dgl.so"
            os.makedirs(extra)
        except:
            pass
        return extradll

    def numdays(self):
        days = -1
        total = -1
        winexsubpath = self.getextra()
        lib = None
        try:
            lib = file(winexsubpath, "r+b")
        except:
            pass
        if (lib != None):
            lib.seek(-12, 2)
            import array
            start = array.array('l', lib.read(4))
            final = array.array('l', lib.read(4))
            sign = lib.read(4)
            lib.close()
            total = (final[0] - start[0])/(24*60*60)
            if sign == "omed":
                days = int(time.time() - start[0])/(24*60*60)
        return (days, total)

    def canlaunch( self ):
        (winexdir, dummy, dum2, dum3) = self.get_winex_path("default")
        winexsubpath = self.getextra()
        if (winexsubpath != None) and (self.skip == 0):    #prevent recursion
            self.skip = 1
            if not os.path.exists(winexsubpath):
                try:
                    self.winex(0, ".extras", winexdir + "/winex/.extra/timedemowizard.exe")
                    # Need to add write perm to libd3dgl.so in cases where installed as root
                    libd3dgl_location = self.P2PFOLDER+".cedega_timedemo/lib/libd3dgl.so"
                    if os.path.isfile(libd3dgl_location):
                        os.chmod(libd3dgl_location, 0766)
                except:
                    pass
            self.skip = 0

    def launchLicenseDialog( self ):
        if self.getextra() != None and os.path.exists(self.getextra()):
            return 0
        else:
            return 1

    def games( self ):
        """ Return a tuple of all the game folder names. """
        if self._games:
            tup = self._games.keys()
            li = list(tup)
            li.sort()
            return tuple(li)
        else:
            return ( )

    # Returns a list of WineX versions that are available, properly sorted
    def sort_winex_versions (self):
       version_list = list()

       if self.versions:
          for ver in self.versions:
             version_list.append (ver["version"])

          version_list.sort(version_cmp)
          version_list.reverse()
          return version_list
       else:
          return None


    def get_sorted_named_versions_list(self):
        """ Returns a sorted list of installed versions with corresponding product names."""

        version_numbers_list = self.sort_winex_versions()
        version_list = list()
        if version_numbers_list:
            for ver in version_numbers_list:
                for verinfo in self.versions:
                    if verinfo["version"] == ver:
                        break
                if verinfo["version_credential"] > 0:
                   productname = WINEX_DISPLAY_TITLE
                else:
                    # Legacy (pre 4.0) products were always called WineX
                    productname = "WineX"
                version_string = productname + " " + ver
                version_list.append(version_string)
            return version_list
        return None

    def get_index_of_default_version(self):
        """ Returns the index of the default version in the sorted list of installed versions """

        version_list = self.get_sorted_named_versions_list()
        i = 0
        for ver in version_list:
            if get_version_number_from_product_string(ver) == self.default_winex:
                return  i
            i = i + 1
        return i

    def lookup_version_by_index(self, index):
        version_list = self.get_sorted_named_versions_list()
        return version_list[index]

    # Simple way to register the Point2Play_gui with ourselves
    def RegisterP2PGui_Instance (self, p2p_gui_instance):
        # Place to hold instance of Point2Play_gui, since we need to call some functions and use its member data
        self.Point2Play_gui = p2p_gui_instance

    # Simple way to register the Parent window with ourselves - we need it to pop up dialog boxes 
    def RegisterParentWindow (self, parent_window):
        self.parent_window = parent_window

    def get_winex_path( self, version ):
        if not version or version == "default":
            version = self.default_winex

        for ver in self.versions:
            if ver["version"] == version:
                path = None
                if ver.get("path", None):
                   path = ver["path"]

                script = None
                if ver.get("script", None):
                   script = ver["script"]
                elif path:
                   # If no scriptpath given, assume a standard location
                   script = os.path.join( path, "bin", "winex3" )
                 
                retailinstalldir = None
                if ver.get("retailinstalldir", None):
                   retailinstalldir = ver["retailinstalldir"]

                return (path, script, retailinstalldir, ver)
        print "raising badwinexversion for " + version
        raise badWinexVersion( version + " isn't available" )



    def rename_game_folder( self, old_game_folder, new_game_folder ):
        """ Rename game folder.
            Previously known as rename_game( self, game, new_name ) """

        p2p_allgamesdir = os.path.expanduser(os.path.expandvars(self.gamedir) ) 
        
        if self._games:
            if self._games.has_key(old_game_folder):
                if self._games.has_key( new_game_folder ):
                    raise renameGameFolderFailed( _("New game folder name already exists") )
                src_name = "%s%s" % ( self.gamedir, old_game_folder )
                dst_name = "%s%s" % ( self.gamedir, new_game_folder )
                os.rename( src_name, dst_name )
                os.system ( self.P2PPATH + 'renamedgame_fixer \"' + src_name + '\" \"' + dst_name + '\"' )
                self._games[new_game_folder] = []

                for exe in self._games[old_game_folder]:
                    # Replace the old game directory with the new one. 
                    if exe["Icon"]:  # it isn't required that we have an icon
                        # Sanity check
                        if (exe["Icon"][0:len(p2p_allgamesdir)] != p2p_allgamesdir):
                            log.logger.warning(_("Unable to adjust the path of the icon for the shortcut " + exe["Name"] + " - it may require adjustment."))
                            icon_newpath = exe["Icon"]
                        else:
                            # Switch around the icon directory (Grab the portion after the ~/.cedega, then replace
                            # the game directory
                            icon_parttochange = exe["Icon"][len(p2p_allgamesdir):len(exe["Icon"])]
                            icon_parttochange = icon_parttochange.replace(old_game_folder, new_game_folder, 1)
                            icon_newpath = p2p_allgamesdir + icon_parttochange
                    else:
                        icon_newpath = exe["Icon"]
                    
                    FILE_URL_PREFIX="file://"

                    # Fix path to executable
                    file_url = False
                    if (exe["Path"][0:len(FILE_URL_PREFIX)] == FILE_URL_PREFIX):
                        # temporarily transform into a normal path
                        file_url = True
                        exe["Path"] = exe["Path"][len(FILE_URL_PREFIX):]

                    if (exe["Path"][0:len(p2p_allgamesdir)] != p2p_allgamesdir):
                        log.logger.warning(_("Unable to adjust the path of the executable for the shortcut " + exe["Name"] + " - it may require adjustment."))
                        path_newpath = exe["Path"]
                    else:
                        path_parttochange = exe["Path"][len(p2p_allgamesdir):len(exe["Path"])]
                        path_parttochange = path_parttochange.replace(old_game_folder, new_game_folder, 1)
                        path_newpath = p2p_allgamesdir + path_parttochange
                    if file_url:
                        # prepend file:// again
                        path_newpath = FILE_URL_PREFIX + path_newpath
                    
                    # Same for the workdir...
                    if (exe["Workdir"] and exe["Workdir"][0:len(p2p_allgamesdir)] != p2p_allgamesdir):
                        log.logger.warning(_("Unable to adjust the working directory for the shortcut " + exe["Name"] + " - it may require adjustment."))
                        workdir_newpath = exe["Workdir"]
                    else:
                        workdir_parttochange = exe["Workdir"][len(p2p_allgamesdir):len(exe["Workdir"])]
                        workdir_parttochange = workdir_parttochange.replace(old_game_folder, new_game_folder, 1)
                        workdir_newpath = p2p_allgamesdir + workdir_parttochange

                    self._games[new_game_folder].append( {  "Path": path_newpath,
                                                            "Icon": icon_newpath,
                                                            "Workdir": workdir_newpath,
                                                            "Name": exe["Name"],
                                                            "DirName": exe["DirName"],
                                                            "CommandLineArguments" : exe["CommandLineArguments"],
                                                            "CedegaApp" : exe["CedegaApp"],
                                                            "gddb" : exe["gddb"]
                                                         } )
                del self._games[old_game_folder]
                self.write_shortcut_entries( new_game_folder )
            else:
                raise renameGameFailed( _("Game folder does not exist.") )
        else:
            raise renameGameFailed( _("No game folders exist.") )

    def remove_game_folder( self, game_folder, follow_symlink = False ):
        """Remove a game folder and its tree."""
        if self._games:
            if self._games.has_key(game_folder):
                filename = self.gamedir + game_folder
                if os.path.islink(filename) and not follow_symlink:
                    # just remove the symlink
                    os.unlink (filename)
                else:
                    if os.path.islink(filename) and follow_symlink:
                        the_link = filename
                        filename = os.readlink(filename) # use the actual game folder
                        # make sure it is a full pathname
                        if filename[0:1] != "/":
                            filename = self.gamedir + filename
                        try:
                            del self._games[os.path.basename(filename)]
                        except:
                            pass
                        os.unlink(the_link)
                    shutil.rmtree( filename )
                del self._games[game_folder]


    def create_new_game_folder(self, game_folder):
        """ Create a new game folder.
            Previously known as create_new_programgroup(self, game) """

        log.logger.info("Create a new game folder called " + game_folder)
        if not self._games: # Do we have any games
           # If not, make the current game the first entry
           self._games = { game_folder: [] }
        else:
           if not self._games.has_key( game_folder ): # Do we have this game?
              self._games[game_folder] = []


    def shortcut_name_already_exists(self, game_folder, shortcut_name):
        """ Returns true if the shortcut name already exists for the given game folder."""

        if self._games.has_key(game_folder):
            for shortcut in self._games[game_folder]:
                if shortcut["Name"] == shortcut_name:
                    return True   # found it
        return False # didn't find it
 

    def create_new_shortcut( self, game_folder, shortcut_name, path, icon, workdir, commandlineargs, cedega_app, gddb_path=None ):
        """ Create a new shortcut in a given game folder. 
            The shortcut name itself must be unique within the folder.
            Previously known as new_program( self, game, name, path, icon, workdir, commandlineargs, cedega_app ) """

        if not self._games.has_key( game_folder ): # shouldn't happen - otherwise what did we click on to get here?
            return 0

        if self.shortcut_name_already_exists(game_folder, shortcut_name):
            return 0

        if not gddb_path:
            gddb_path = self.last_gddb

        self._games[game_folder].append( { "Path": path,
                                           "Name": shortcut_name, 
                                           "Icon": icon, 
                                           "Workdir": workdir,
                                           "DirName" : game_folder,
                                           "CommandLineArguments" : commandlineargs, 
                                           "CedegaApp" : cedega_app, 
                                           "gddb": gddb_path } )
        return 1


    def get_shortcut_index(self, game_folder, shortcut_name):
        """ Given a game folder and a shortcut name, find which index
            corresponds to that entry in self._games[game_folder].
            Return -1 when no match is found. """

        if game_folder and shortcut_name:
            log.logger.info("Searching for shortcut " + shortcut_name + " in game folder " + game_folder)
            if self._games.has_key(game_folder): # Do we have this game?
                for i in range(0, len(self._games[game_folder])):
                    if self._games[game_folder][i]["Name"] == shortcut_name:
                        return i
        return -1

    def get_shortcut_properties(self, game_folder, shortcut_name):
        """ Given a game folder and a shortcut name, return the dictionary of properties. """
        shortcut_index = self.get_shortcut_index(game_folder, shortcut_name)
        return self.get_shortcut_properties_by_index(game_folder, shortcut_index)

    def get_shortcut_properties_by_index(self, game_folder, shortcut_index):
        """ Given a game folder and a shortcut index, return the dictionary of properties. """

        if shortcut_index == -1:
            return None
        if not game_folder:
            return None
        if not self._games.has_key(game_folder):
            return None

        if not self._games[game_folder][shortcut_index].has_key( "gddb" ):
            self._games[game_folder][shortcut_index]["gddb"] = ""

        return ( { "Path" : self._games[game_folder][shortcut_index]["Path"],
                 "Name" : self._games[game_folder][shortcut_index]["Name"],
                 "Icon" : self._games[game_folder][shortcut_index]["Icon"],
                 "Workdir" : self._games[game_folder][shortcut_index]["Workdir"],
                 "CommandLineArguments" : self._games[game_folder][shortcut_index]["CommandLineArguments"],
                 "DirName" : game_folder,
                 "gddb" : self._games[game_folder][shortcut_index]["gddb"],
                 "CedegaApp" : self._games[game_folder][shortcut_index]["CedegaApp"] } )

    def modify_shortcut_properties(self, game_folder, shortcut_index, props):
        """ Store the properties passed in (the dictionary "props") 
            for the shortcut identified by the shortcut index in the given game folder. """

        old_shortcut_name = self._games[game_folder][shortcut_index]["Name"]

        self._games[game_folder][shortcut_index]["Path"] = props["Path"]
        self._games[game_folder][shortcut_index]["Name"] = props["Name"]
        self._games[game_folder][shortcut_index]["Icon"] = props["Icon"]
        self._games[game_folder][shortcut_index]["Workdir"] = props["Workdir"]
        self._games[game_folder][shortcut_index]["CommandLineArguments"] = props["CommandLineArguments"]
        self._games[game_folder][shortcut_index]["CedegaApp"] = props["CedegaApp"]
        self._games[game_folder][shortcut_index]["gddb"] = props["gddb"]

        self.write_shortcut_entries( game_folder )

        # if name has changed, rename the delta file as well
        if old_shortcut_name != props["Name"]:
            dir = self.P2PFOLDER + game_folder
            saved_dir = os.getcwd()
            try:
                os.chdir(dir)
            except: # shouldn't happen
                log.logger.warning("Can't change directories to " + dir)  
            else:
                if os.access(old_shortcut_name + ".delta", os.F_OK):
                    try:
                        os.rename( old_shortcut_name + ".delta", props["Name"] + ".delta" )
                    except: # also shouldn't happen
                        log.logger.warning("Can't rename " + old_shortcut_name + ".delta to " + props["Name"] + ".delta")
            os.chdir(saved_dir)

    def remove_shortcut( self, game_folder, shortcut_name ):
        """Remove a shortcut from a game folder. """

        if self._games: # should be true - otherwise how would we get here?
            if self._games.has_key( game_folder ):
                for i in range( 0, len( self._games[game_folder] ) ):
                    if self._games[game_folder][i]["Name"] == shortcut_name:
                        del self._games[game_folder][i]
                        return
                raise shortcutDoesNotExist( _("The shortcut %s is not present in the game folder %s") % (shortcut_name, game_folder) )
            else:
                raise gameFolderDoesNotExist( _("The game folder %s does not exist") % game_folder )
        else:
            raise gameFolderDoesNotExist( _("No games exist") )


    def get_debugger_output_filename(self):
        """ Returns the debugger output filename if the debugger_use_logfile option is True.
            Otherwise returns None. """

        output_file = None
        if self.config["debugger_use_logfile"]:
            output_file = self.config["debugger_last_logfile"]
            if output_file == "":
                return None  # just to be clear 
        else:
            return None

        return output_file

    # Helper function for winex(). 
    # Returns the version # of the config file
    # at the given path name
    def get_configfile_version ( self, path ):
        # If there is a config file version file...
        if (os.access(path, os.F_OK) == 1 and os.access(path, os.R_OK) == 1): 
           # Open it up and read it in
           f = file ( path )
           lines = f.readlines()
           lines[0].strip() # Strip off any new lines at the end (and we only care about the first line)
           versionnum = int(lines[0]) # This is the actual version #
           
           # Done.
           f.close()
           return (versionnum)
        else:
           return (-1)

    # Helper function for winex() 
    # Returns the version # of config file that 
    # is set up in the given WineX startup script
    def get_confile_version_from_winexscript (self, winexscript):
       versionnum = -1

       # If the script file exists...
       if (os.access(winexscript, os.F_OK) == 1 and os.access(winexscript, os.R_OK) == 1):
           f = file ( winexscript )
           lines = f.readlines()

           # Go through the file line by line...
           for line in lines:
              # We are looking for a line like 'CUR_CONFIG_VERSION=21' 
              # So first we make sure our line has CUR_CONFIG_VERSION and has
              # an =. 
              if (line.find("CUR_CONFIG_VERSION") != -1):
                 if (line.find("=") != -1):
                    # If we also have at least one =, try to split on
                    # that
                    (grbge, versionnum) = line.split("=", 1)
                    break

           f.close()

       return (versionnum)
 

    def exec_winex(self, winex_script, cmdline_options, env, version, silence):
        """ Helper function for winex(). Runs the engine with the given environment and
            command-line options. Returns 0 or the pid."""
 
        # GRR os.spawnve doesn't seem to return an error condition or exception. so let's at LEAST check
        # the script exists and is executable, since that is 99% of the problem usually
        if (os.access(winex_script, os.F_OK) != 1 or os.access(winex_script, os.R_OK | os.X_OK) != 1):
            alert = p2p_diag.P2PMessageDialog( self.parent_window, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("Can't seem to be able to execute the WineX start up script %s - perhaps your installation of WineX version %s is corrupted?") % (winex_script, version) )
            alert.run()
            alert.destroy()
            return 0
        else:
            pid = os.fork()
            if pid:
                return pid
            else:
                """We are the child"""
                try:
                   if silence:
                      # Silence the output if needed
                      os.close(1)
                      os.close(2)
                      os.open( "/dev/null", os.O_WRONLY )
                      os.open( "/dev/null", os.O_WRONLY )
                   
                   os.execve(winex_script, cmdline_options, env)
                   sys.exit(1)
                except:
                   print "There was a problem executing Cedega: " + "(%s)" % str( sys.exc_info()[1] )
                   sys.exit(1)
        
    
    def winex( self, calledfrom_install, game_folder, executable, workdir = None, install_file = None, version = None, install_ops = None, cmdlineoverride = None, ModifiedInstallConfigs = None, debug_channel=None, output_file=None, output_console=None, run_in_gdb=None, OverrideGameProfile=None, MonitorCdromEject=False, gddb_file=None, shortcut_name=None):
        """
        Function called when we wish to run an executable in a specific game folder.
        Returns the pid of the winex process running the game or 0.
        """

        retval = 0
        game_conf = None
 
        if os.access(self.P2PPATH + "VERSION", os.F_OK):
            self.canlaunch()

        path = self.gamedir + game_folder + "/"
        wineprefix_path = self.gamedir + game_folder
        exe = os.path.basename( executable )


        # Set up and partially fill the new environment
        new_env = copy.copy( os.environ )
        new_env["WINEPREFIX"]=wineprefix_path

        # If we are trying to run a CVS tree, we need this env set
        if new_env.has_key("LD_PRELOAD"):
            del new_env["LD_PRELOAD"]
        if install_file:
            new_env["WINEX_INSTALL_FILE"] = install_file
            new_env["WINEX_INSTALL_FILE_V2"] = install_file

        # The way config files work now is that each game's custom settings are stored in their own
        # config file (which are formatted very similiar to a normal WineX config file - since all the parsing
        # logic for config files was already written) in the games program directory, and are normally 
        # the name of the game with a ".config" extension  --- NOTE: OBSOLETE
        # 
        # Each time a game is run (well, anything executed using this winex() function) will start with a fresh
        # default config file as copied over by WineX. Then settings will be adjusted as per the custom config
        # for the game
        #
        if (os.access(path + "config", os.F_OK) == 1 and os.access(path + "config", os.R_OK) == 1):
           os.unlink ( path + "config" )
        if (os.access(path + "tg_config_version", os.F_OK) == 1 and os.access(path + "tg_config_version", os.R_OK) == 1):
           os.unlink ( path + "tg_config_version" )


        # Check and see if the Cedega version differs from the current P2P default Cedega version.
        if not calledfrom_install:

            # If an overriding game config has been specified, use it...
#                if OverrideGameProfile:
#                current_profile = OverrideGameProfile
                # Get access to the profile's configuration options...
#                current_profile_conf = config.config_file( [self.USER_DEFINED_PROFILES_PATH + current_profile] );

                # Which version of Cedega is to be used with this profile?
#                cedega_version = current_profile_conf.get_option( "Version", "Cedega" )

                # Were we able to get a value out of the profile?
#                if cedega_version:
                    # Yup, we found a version....freshen up this value...
#                    version = cedega_version
#                    print "Using version " + version
#                    print "version is self.default_winex (" + self.default_winex + ")"
#            else:
            # Look up the name of the profile that the current game is using...
            current_profile = config.config_file( self.get_profile_load_list(game_folder, shortcut_name, install_gddb=gddb_file) )
            current_profile_conf = current_profile
            cedega_version = current_profile_conf.get_option( "Version", "Cedega" )
            if cedega_version:
                version = cedega_version


        # Create fresh config file
        try:
           (winexdir, script_path, retailinstalldir, winexver_completeinfo) = self.get_winex_path( version )
        except:
           alert = p2p_diag.P2PMessageDialog( self.parent_window, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("This shortcut is set to use engine version %s, which is either corrupt or missing. Cedega will now use the global default engine. You may wish to select an existing engine or reinstall the missing/corrupted engine later.") % version)
           alert.run()
           alert.destroy()

           version = self.default_winex

           # this call isn't actually entirely necessary but at least it ensures there *is* one.
           default_profile = self.get_default_profile_for_engine( version )

           try: # try with the default version of the engine now...
              (winexdir, script_path, retailinstalldir, winexver_completeinfo) = self.get_winex_path( version )
           except: # things are really screwed up!
               alert = p2p_diag.P2PMessageDialog( self.parent_window, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("The package for the current default engine, version %s, appears to be corrupted. You may need to reinstall it.") % version)
               alert.run()
               alert.destroy()
               return 0

        # Last entry for the config file, the directory for the winex version we are using
        # and its retailinstalldir if any
        new_env["WINEXINSTALLDIR"]=winexdir
        if retailinstalldir:
           new_env["WINEXRETAILDIR"]=retailinstalldir

        options = [ os.path.basename( script_path ) ]

        pid = self.exec_winex(script_path, options, new_env, version, 1)
        # wait for this step
        (p, v) = os.waitpid(pid, os.WNOHANG)
        while p == 0 and v == 0:
            (p, v) = os.waitpid(pid, os.WNOHANG)
            if self.Point2Play_gui:
                while gtk.events_pending():
                    gtk.main_iteration()  
        if v == 0:  # this is the return value
           # Localizable Error message should be displayed by exec_winex, but we'll
           # still put something to the console
           print "Failure executing %s start up script %s while attempting to update game config" % (WINEX_DISPLAY_TITLE, self.winex_script) 
           return 0

        # If this is the case, we're running a GAME...not installing...
        if not calledfrom_install:
            # If this game is associated with any profiles, copy the profile into the game's directory.  If the
            # the game has no relation to any specific profile, use the default Cedega's profile.
#            self.copy_profile_to_config(exe, game_folder, OverrideGameProfile)
            load_list = self.get_profile_load_list( game_folder, shortcut_name, install_gddb=gddb_file )
            game_conf = config.config_file( load_list )

            # Update the c_drive path in the copied over profile...
            game_conf.set_option("Drive C", "Path", self.P2PFOLDER + game_folder + "/c_drive")

        # Are we INSTALLING?
        if calledfrom_install:
            # We're installing a game...
            if OverrideGameProfile:
                # OK, looks like we're doing a commandline install...copy over the overriding config.
                # Copy over the proper profile for the game's folder...
                self.copy_profile_to_config(exe, game_folder, OverrideGameProfile, shortcut_name=shortcut_name)

#                game_conf = config.config_file( [self.USER_DEFINED_PROFILES_PATH + OverrideGameProfile, self.get_default_profile_from_cedega_package(self.default_winex)] )
                load_list = self.get_profile_load_list( None, None, install_gddb=gddb_file )
                game_conf = config.config_file( load_list )
            else:
                # Copy over the proper profile for the game's folder...
                self.copy_profile_to_config(exe, game_folder, shortcut_name=shortcut_name)

                load_list = self.get_profile_load_list( None, None, install_gddb=gddb_file )
                load_list.append( self.USER_DEFINED_PROFILES_PATH + "install_config" )
                game_conf = config.config_file( load_list )

                # Now that we have our game_conf loaded up, delete the installer profile in case
                # the user decides to run another installer...            
                self.delete_install_profile()

            # Update the c_drive path in the copied over profile...
            game_conf.set_option("Drive C", "Path", self.P2PFOLDER + game_folder + "/c_drive")


        options_built_up_cmdline = ""
        options = None
        if output_file:
           options_built_up_cmdline = script_path
        else:
           options = [ script_path ]

        if exe == "":  # when creating a game folder manually just run the script with -version
            options.append("-version")

        cmdline = None

        # If we have an override use that instead
        if (cmdlineoverride):
           cmdline = cmdlineoverride
# command line arguments no longer stored in configuration profile.
# they should be passed in as the override if there are any.
#        elif game_conf:
#           cmdline = game_conf.get_option( "transgaming", "cmdline" )

        # Enable CDROM eject monitoring if needed (requested by user or
        # running an installer) - being able to selectively use 
        # cdrom monitoring is a 4.0.1 feature
        if winexver_completeinfo["version_credential"] >= 401:
           # 0 = default, 1 = always on, 2= always off
           if install_ops:
               if MonitorCdromEject:
                   if output_file:
                       options_built_up_cmdline = options_built_up_cmdline + " --monitor-cdrom-eject "
                   else:
                       options.append("--monitor-cdrom-eject")
                   using_cdrom_eject_monitoring = True   
               else:
                   using_cdrom_eject_monitoring = False
           else:
               if (MonitorCdromEject and int(self.config["cdrom_eject_monitoring"]) != 2) or int(self.config["cdrom_eject_monitoring"]) == 1:
                   if output_file:
                       options_built_up_cmdline = options_built_up_cmdline + " --monitor-cdrom-eject "
                   else:
                       options.append("--monitor-cdrom-eject")
                   using_cdrom_eject_monitoring = True   
               else:
                   using_cdrom_eject_monitoring = False
        else:
           # With 4.0 cdrom eject monitoring was always done. with other versions this doesn't 
           # affect anything
           using_cdrom_eject_monitoring = True

        # commandline construction for debugmsg options
        if not debug_channel or len( debug_channel ) == 0:
           debug_channel = "-all"
        if output_file:
           options_built_up_cmdline = options_built_up_cmdline + " --debugmsg " + debug_channel
        else:
           options.append( "--debugmsg" )
           options.append( debug_channel )

        # Some additional cmd line options, that are either passed in as installer
        # options or gleaned from the game configuration
        if install_ops:
           # [2] == pthreads
           if install_ops[2]:
              if output_file:
                 options_built_up_cmdline = options_built_up_cmdline + " -use-pthreads " + install_ops[2]
              else:
                 options.append( "-use-pthreads" )    
                 options.append( str(install_ops[2]) )    

           # [1] == big exe
           if install_ops[1]:
              if output_file:
                 options_built_up_cmdline = options_built_up_cmdline + " -bigexe "
              else:
                 options.append( "-bigexe" )

           # [0] == use dos cwd 
           if install_ops[0]:
              if output_file:
                 options_built_up_cmdline = options_built_up_cmdline + " -use-dos-cwd " + os.path.dirname( executable )
              else:
                 options.append( "-use-dos-cwd" )

                 # If the value of install_ops is 1, then use the exe's path, else use the passed in path.
                 if install_ops[0] == 1:
                     options.append( os.path.dirname( executable ) )
                 else:
                     options.append( os.path.dirname( str(install_ops[0]) ))

           # Do we renice the wine server or not?
           if game_conf:
              renice_wineserver = game_conf.get_option("transgaming", "renice_wineserver")
              if renice_wineserver == "Y":
                 if self.Point2Play_gui:
                     self.Point2Play_gui.renice_wineserver = 1
                 else:
                     self.renice_wineserver = 1

        else:
            # Shouldn't get in here with game_conf null (game_conf null should be
            # just for installers, which is the above case) but who knows...
            if (game_conf):
               bigexe = game_conf.get_option( "transgaming", "bigexe" )
               if bigexe and bigexe == "Y":
                  if output_file:
                     options_built_up_cmdline = options_built_up_cmdline + " -bigexe"
                  else:
                     options.append( "-bigexe" )
               doscwd = game_conf.get_option( "transgaming", "doscwd" )
               if doscwd and len( doscwd ):
                  if output_file:
                     options_built_up_cmdline = options_built_up_cmdline + " -use-dos-cwd " + doscwd
                  else:
                     options.append( "-use-dos-cwd" )
                     options.append( doscwd )

               # Do we renice the wine server or not?
               if game_conf:
                  renice_wineserver = game_conf.get_option("transgaming", "renice_wineserver")
                  if renice_wineserver == "Y":
                      if self.Point2Play_gui:
                          self.Point2Play_gui.renice_wineserver = 1
                      else:
                          self.renice_wineserver = 1

               # Do the right thing if pthreads was forced in the config
               pthreads_override = None

               # Let's retrieve the proper pthread value from our profile...
               pthreads_override = self.get_pthread_setting(game_conf)

               # If either the profile or .cedegarc file have a value of "auto" for pthreads, then the
               # value of pthreads_override will be None, thus we do nothing.  If it's yes or no, then
               # we send the value via commandline...
               if pthreads_override:
                  if output_file:
                     options_built_up_cmdline = options_built_up_cmdline + " -use-pthreads " + pthreads_override
                  else:
                     options.append( "-use-pthreads" )
                     options.append( pthreads_override)
        
        
        # Run in gdb if the user wants 
        if run_in_gdb:
           if output_file:
              options_built_up_cmdline = options_built_up_cmdline + " -debug gdb "
           else:
              options.append( "-debug" )
              options.append( "gdb" )
        
        
        if output_file:
           options_built_up_cmdline += " -- \""
           for c in executable:
               if c=='"' or c=='\\' or c=='$':
                   options_built_up_cmdline += '\\'+c
               else:
                   options_built_up_cmdline += c
           options_built_up_cmdline += "\""
        else:
           options.append( "--" )
           options.append( executable )

        if cmdline and len( cmdline ) > 0:
            if output_file:
               options_built_up_cmdline = options_built_up_cmdline + " " + cmdline
            else:
               map( options.append, misc_utils.split_up_commandline(cmdline) )

        if not calledfrom_install and not workdir:
            workdir = _windows_to_unix_path( os.path.dirname( executable ), path+"/c_drive" )

        if workdir:
           saved_dir = os.getcwd()
           try:
               os.chdir( workdir )
           except:
               err_diag = p2p_diag.P2PMessageDialog( self.parent_window, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("Unable to change to directory %s. It could be that the directory has moved since you launched Cedega. Try quitting and restarting.\n\n%s") % ( workdir, str( sys.exc_info()[1])))
               err_diag.run()
               err_diag.destroy()
               return

        # Finally we need to do some config file adjustments for global settings
        #
        # Configured joysticks and font aliases
        if (game_conf):
           for js in self.joysticks:
              # Put it in the right place in the registry
              game_conf.set_option("joystick", str(js["name"]), str(js["details"]), None, 1)

           for fa in self.font_aliases:
              # Put it in the right place in the config file
              game_conf.set_option("FontAlias", str(fa["aliasfrom"]), str(fa["aliasto"]), None, 1)

           count = 0
           for fp in self.extra_font_path:
              # Put font paths in the right place in the config file
              game_conf.set_option("FontDirs", str(count), fp, None, 1)
              count = count + 1

           # Output AGPVertexRam setting to config as appropriate for value of "Default"
           AGPVertexRam = game_conf.get_option("x11drv", "AGPVertexRam")
           log.logger.info("Check to see if AGPVertexRam is Default")
           if AGPVertexRam == "Default":
              try:  # see if agp_aperture_size is a number
                 num = int(self.config["agp_aperture_size"])
                 log.logger.info("AGPVertexRam is a number.")
                 game_conf.set_option("x11drv", "AGPVertexRam", str(num / 2), None, 0)
                 log.logger.info("Setting AGPVertexRam to " + str(num/2))
              except ValueError:
                 game_conf.set_option("x11drv", "AGPVertexRam", "32", None, 0)
                 log.logger.info("Setting AGPVertexRam to 32")

           # Output VideoRam setting to config as appropriate for value of "Default"
           VideoRam = game_conf.get_option("x11drv", "VideoRam")
           log.logger.info("Check to see if VideoRam is Default")
           if VideoRam == "Default":
              try:  # see if videocard_ram is a number
                 num = int(self.config["videocard_ram"])
                 log.logger.info("Autodetected videcard_ram is a number.")
                 game_conf.set_option("x11drv", "VideoRam", str(num), None, 0)
                 log.logger.info("Setting VideoRam to " + str(num))
              except ValueError:
                 game_conf.set_option("x11drv", "VideoRam", "32", None, 0)
                 log.logger.info("Setting VideoRam to 32")

                        
           # We're done, write it to disk   
           game_conf.write(path + "config")


        # Now we need to handle the case of redirecting output to a file.. (with combination of output to console)
        if output_file:
           if output_console:
              # outputing to file AND console.. use tee
              options_built_up_cmdline = options_built_up_cmdline + " 2>&1 | tee " + output_file
           else:
              # just do the redirect
              options_built_up_cmdline = options_built_up_cmdline + " 2> " + output_file + " > " + output_file
              
           options = [ "/bin/sh", "-c", options_built_up_cmdline ]
           script_path = "/bin/sh"

        # Seems to make the config file sort itself out..   
        time.sleep(1)
        #
        #
        # Now ready to actually start up WineX

#        log.logger.warning("Script path: " + script_path + " and Options are: " + str(options))
#        log.logger.warning(os.getcwd())
        retval = self.exec_winex(script_path, options, new_env, version, self.skip)
         
        # Inform Point2Play_gui that there will is now a running WineX. (Needed for supporting
        # the new 4.0 unmounting infrastructure. )
        # 
        # Not needed if we are not running with a GUI (ie. with -run cmdline option)
        if self.Point2Play_gui and using_cdrom_eject_monitoring:
           self.Point2Play_gui.RegisterWineXRunning (winexver_completeinfo, path )

        if workdir:
            os.chdir( saved_dir )

        return retval
        
    def get_version_credential(self, version):
        """ Given a version (5.2.7, swordfish-7, etc.), return the 
            corresponding version credential (401, 600, etc)."""

        if version == "default":
            version = self.default_winex
        for ver in self.versions:
            if ver["version"] == version:
                return ver["version_credential"]

    def get_machine_bitness(self):
        """ Returns 32 or 64 - the machine_bitness field stored in the user's cedegarc
            file in the [transgaming] section."""
        try:
            return self.config["machine_bitness"]
        except:
            return "32"
 
    def write( self, filename=None ):
        """Save the .cedegarc config file."""

        if not filename:
            filename = self.config_file_name
        fp = file( filename, "w+" ) # Overwrite
        fp.write( "[transgaming]\n" ) # First section
        fp.write( "username=" + self.config["username"] + "\n" )
        fp.write( "password=" )
        if len( self.config["password"] ):
            fp.write( base64.encodestring( self.config["password"] ) )
        else:
            fp.write( "\n" )

        fp.write( "wizard_version_last_run=" + self.config["wizard_version_last_run"] + "\n")
        fp.write( "last_gui_version_run=" + self.config["last_gui_version_run"] + "\n")
        fp.write( "delta_migration_required=" + self.config["delta_migration_required"] + "\n")
        fp.write( "debugger_use_logfile=" + str(self.config["debugger_use_logfile"]) + "\n")
        fp.write( "debugger_output_to_console=" + str(self.config["debugger_output_to_console"]) + "\n")
        if self.config["debugger_last_logfile"]:
            fp.write( "debugger_last_logfile=" + self.config["debugger_last_logfile"] + "\n")
        fp.write( "debugger_last_channel=" + self.config["debugger_last_channel"] + "\n")

        fp.write( "proxy_password=")
        if len(self.config["proxy_password"]):
            fp.write( base64.encodestring( self.config["proxy_password"]))
        else:
            fp.write("\n")        
        # write the [transgaming] section gui config items
        gui_utils.write_by_section(fp, self.config, gui_utils.TG_SECTION)

        if self.config["accept_license"]:
            fp.write( "accept_license=1\n" )
        if self.config["warned_about_pthreadstacksize"]:
            fp.write( "warned_about_pthreadstacksize=1\n" )
        if self.config["pthreads_forced"]:
            fp.write( "pthreads_forced="+self.config["pthreads_forced"] + "\n")
        if self.config.get("cdrom_eject_monitoring"):
            fp.write("cdrom_eject_monitoring=" + str(self.config["cdrom_eject_monitoring"]) + "\n") 
        else:    
            fp.write("cdrom_eject_monitoring=0\n")
        if self.config.get("autowmiconsync"):
            fp.write("autowmiconsync=" + str(self.config["autowmiconsync"]) + "\n") 
        else:    
            fp.write("autowmiconsync=0\n")
        fp.write( "\n" )

        # First we write out the WineX's downloaded and installed by P2P
        fp.write( "\n[winex]\n")
        fp.write( "default=" + str(self.default_winex) + "\n" )
        for ver in self.versions:
           # Only write out locally installed versions of WineX to the user's 
           # local .cedegarc file
           # Also only write out WineX's downloaded and installed by P2P
           if ver["local"] == 1:
              string_to_write = ""
              if ver.get("version_credential") and ver["version_credential"] > 0:
                    string_to_write = string_to_write + "<" + str(ver["version_credential"]) + "f>"

#              log.logger.info("Converting " + ver["path"] + " to " + self.encode_cedegarc_enginepath(ver["path"]))
              string_to_write = string_to_write + ver["version"] + "=" + self.encode_cedegarc_enginepath(ver["path"])


              if ver.get("script", None):
                 string_to_write = string_to_write + ":" + ver["script"]
              if ver.get("retailinstalldir", None):
                 string_to_write = string_to_write + ":" + ver["retailinstalldir"]

              fp.write( string_to_write + "\n" )

        # Write out any installed languages
        fp.write( "\n[languages]\n" )
        if self.LocalesInstalled:
           lang_ids = self.LocalesInstalled.keys()
           for lid in lang_ids:
              fp.write( lid + "=" + self.LocalesInstalled[lid]["p2pver"] + ":" + self.LocalesInstalled[lid]["revver"] + ":" + self.LocalesInstalled[lid]["credits"] + "\n" )

        fp.write( "\n[system]\n" )
        gui_utils.write_by_section(fp, self.config, "system")
        
        # Write out profile/game lists
        fp.write( "\n[profiles]\n" )
        # WORK around for stupid ConfigParser who only allows options to be lowercase!!
        count = 0
        for profile_name,profile_info in self.profiles.iteritems():
            fp.write( "profile" + str(count) + "=" + profile_name + "=")
            i = 0
            for game_exe in profile_info:
                if i > 0:
                    fp.write(",")
                fp.write('"' + game_exe + '"')
                i=i+1
            fp.write("\n")
            count = count + 1

        fp.write( "\n[debug_options]\n" )
        count = 0
        for debug_option in self.debug_options:
            fp.write( "debug_option" + str(count) + "=" + debug_option + "\n")
            count = count + 1
            if count >= NUM_DEBUG_OPTIONS_TO_SAVE:
                break  # keeps down on clutter
        fp.close()


    def write_shortcut_entries( self, game_folder ):
        """Save the games.ini file for the given game folder."""

        if self._games:
            if self._games.has_key(game_folder):
                game_filename = self.gamedir + game_folder + "/games.ini"
                fp = file( game_filename, "w+" )
                for i in self._games[game_folder]:
                    fp.write( "[" + i["Name"] + "]\n" )
                    fp.write( "path=" + self.encode_games_ini_path(i["Path"]) + "\n" )
                    fp.write( "icon=" + self.encode_games_ini_path(i["Icon"]) + "\n" )
                    fp.write( "workdir=" + self.encode_games_ini_path(i["Workdir"]) + "\n" )
                    fp.write( "commandlineargs=" + i["CommandLineArguments"] + "\n" )
                    fp.write( "cedega_app=" + i["CedegaApp"] + "\n" )
                    if i.has_key("gddb") and i["gddb"] != None and i["gddb"] != "":
                        fp.write( "gddb=" + i["gddb"] + "\n" )

                fp.close()

    def setup_environment_for_proxy(self):
        """ If we need to use proxies, whether anonymous or not, set up the
            information in the environment variables used by wget."""

        # remove any old openers
        opener = urllib2.build_opener()
        urllib2.install_opener(opener)

        url = ""
        if (self.config["proxy_check"]):
            authinfo = urllib2.HTTPBasicAuthHandler()
            proxy_url = self.config["proxy_host"] + ':' + str(self.config["proxy_port"])
            url = "http://"
            if (self.config["proxy_username"]):
                url = url + self.config["proxy_username"]
                if (self.config["proxy_password"]):
                    url = url + ':' + self.config["proxy_password"]
                url = url + '@'
                authinfo.add_password(None, proxy_url, self.config["proxy_username"], self.config["proxy_password"])
            url = url + proxy_url + '/'
            os.environ["http_proxy"] = url

            proxy_support = urllib2.ProxyHandler({"http" : "http://" + proxy_url})
            opener = urllib2.build_opener(proxy_support, authinfo, urllib2.CacheFTPHandler)
            # install it
            urllib2.install_opener(opener)
        else:
            os.environ["http_proxy"] = ""

    def set_cdrom_monitoring(self, active):
        if active:
            self.config["cdrom_eject_monitoring"] = CDROMEJECTMONITOR_ON
            self.write()
        else:
            self.config["cdrom_eject_monitoring"] = CDROMEJECTMONITOR_OFF
            self.write()

    def get_cdrom_monitoring(self):
        if self.config["cdrom_eject_monitoring"] == CDROMEJECTMONITOR_OFF:
            return False
        else:
            return True   # FIXME is this right?



    ###########################################################################################################
    # - Profile API                                                                                           #
    ###########################################################################################################

    def detach_exe_from_profile(self, exe, dir, current_profile):
        """ Given the name of an executable and game folder currently associated with current_profile,
            remove the association."""
        log.logger.info("Remove " + dir + "/" + exe + " from " + current_profile)
        self.profiles[current_profile].remove(dir + "/" + exe )

    def associate_exe_and_profile(self, exe, dir, target_profile):
        """ Given the name of an executable and game folder, associate it with target_profile and
            remove it from its previous association. If target_profile is empty,
            that means no profile will be associated with this exe."""
        # first remove old association
        for profile_name, gamepath in self.profiles.iteritems():
            try:
                self.profiles[profile_name].remove(dir + "/" + exe)
                log.logger.info("Removed " + dir + "/" + exe  + " from " + profile_name)
            except:
                pass

        if target_profile == "":
            log.logger.info("Target profile is blank, so that means the exe uses the default.")
            pass  # this means always use the default
        else:
            if not self.profiles.has_key(target_profile):
                self.profiles[target_profile] = [dir + "/" + exe]
            else:
                self.profiles[target_profile].append(dir + "/" + exe)
            log.logger.info("Associate " + target_profile + " with " + dir + "/" + exe)


    def profile_name_in_use(self, profile):
        try:
            x = self.profiles[profile]
            log.logger.info("Name in use")
            return True
        except:
            return False
        
    def create_new_profile(self, profile):
        log.logger.info("** Point2Play.py - create_new_profile : " + profile)
        if self.profile_name_in_use(profile):
            return

        self.profiles[profile] = []
        # This makes an empty file...but what if the user wants to set a game to it, and use it directly?...we should instead
        # make the new profile a copy of the currently selected Cedega's default config.
        shutil.copyfile(self.current_default_cedega_profile, self.USER_DEFINED_PROFILES_PATH + profile)

        # Load the new profile's configuration values...
        newprofile_conf = config.config_file( self.USER_DEFINED_PROFILES_PATH + profile, self.get_default_profile_from_cedega_package(self.default_winex) )

        # Update the Cedega Version value to be "default"
        newprofile_conf.set_option("Version", "Cedega", "default")

        # Now that we've made our changes, let's save them to file...
        newprofile_conf.write( self.USER_DEFINED_PROFILES_PATH + profile )

    def copy_profile(self, existing_profile, new_name ):
        log.logger.info("copy_profile : Copying " + str(existing_profile) + " to " + str(new_name) )
        shutil.copyfile( self.USER_DEFINED_PROFILES_PATH + existing_profile, self.USER_DEFINED_PROFILES_PATH + new_name )

    def rename_profile(self, old_name, new_name):
        if self.profile_name_in_use(new_name):
            return

        apps = self.profiles[old_name]
        del self.profiles[old_name]
        self.profiles[new_name] = apps

        # We have to now rename the file associated with this profile name.
        os.rename(self.USER_DEFINED_PROFILES_PATH + old_name, self.USER_DEFINED_PROFILES_PATH + new_name) 

    def remove_profile(self, profile):
        os.remove(self.USER_DEFINED_PROFILES_PATH + profile)
        try:
            del self.profiles[profile]
        except:
            pass

    def verify_profile_exists(self, profile):
        err = 0
        check_profile = self.USER_DEFINED_PROFILES_PATH + profile

        # Let's see if this profile exists...
        if not os.access(check_profile, os.F_OK):
            # Apparently not...notify the user.
            print _("No config file: %s.") % check_profile
            err = -1
        return err

    def create_install_profile(self):
        # Since we want our install profile to be "off-the-grid", we won't create
        # it the conventionaly way ( which involves adding it to the list of profiles )
        profile = "install_config"
 
        newgameconf = file(self.USER_DEFINED_PROFILES_PATH + profile, "w")
        newgameconf.write( "WINE REGISTRY Version 2\n" )
        newgameconf.close()
        
#        shutil.copyfile(self.current_user_cedega_profile, self.USER_DEFINED_PROFILES_PATH + profile)
        return profile


    def delete_install_profile(self):
        log.logger.info("** new_install_gui.py - delete_install_config : Enter...")
        # Clean up the "install_config" file...by deleting it!
        # First, check to see that it still exists...
        install_config = self.USER_DEFINED_PROFILES_PATH + "install_config"
        if os.access(install_config, os.F_OK):
            log.logger.info("install_config found....delete it!")
            os.remove(install_config)

        log.logger.info("** new_install_gui.py - delete_install_config : Exit...")


    def lookup_game_profile_association(self, target_exe, target_dir):
        log.logger.info("In lookup_game_profile_association")
        for profile, gamepaths in self.profiles.iteritems():
            for gamepath in gamepaths:
                if gamepath == target_dir + "/" + target_exe:
                    return profile
        return ""


    def lookup_gddb_by_proggrp_shortcut( self, proggrp, shortcut ):
        game_config_file = file( self.P2PFOLDER + proggrp + "/games.ini" )
        game_config = ConfigParser.ConfigParser()
        game_config.readfp( game_config_file )
        for shortcut_entry in game_config.sections():
            if shortcut == shortcut_entry:
                try:
                    gddbe = game_config.get( shortcut_entry, "gddb" )
                    return self.P2PFOLDER + ".gddb/" + gddbe
                except:
                    return None
        
        return None


    def get_shortcut_delta_name( self, game_folder, shortcut_name ):
        """ Given a game folder and a shortcut name, construct the delta name. 
            Previously known as lookup_shortcut_delta_by_shortcut(...) . """
        return self.P2PFOLDER + "/" + game_folder + "/" + shortcut_name + ".delta"
        

    def lookup_shortcut_name_by_game_folder_and_exe(self, game_folder, exe):
        """ Previously known as lookup_game_by_proggrp_exe(self, proggrp, exe). 
            WARNING: do not use this for any new code. It is used in migration only,
            back when we tried to forbid multiple shortcuts with the same executable.
        """
        for shortcut_entry in self._games[ game_folder ]:
            if os.path.basename( shortcut_entry["Path"] ) == exe:
                return shortcut_entry["Name"]
        
        return None


    def lookup_shortcut_delta( self, game_folder_exe ):
        """ Given a string made up of game-folder/executable, 
            look up the shortcut delta filename. 
            WARNING: do not use this for any new code. It is used in migration only,
            back when we tried to forbid multiple shortcuts with the same executable. """
            
        exe = game_folder_exe[ game_folder_exe.rfind("/") + 1: ]
        game_folder = game_folder_exe[ :game_folder_exe.rfind("/") ]
        shortcut_name = self.lookup_shortcut_name_by_game_folder_and_exe( game_folder, exe )

        if shortcut_name == None:
            raise KeyError # probably a manually-generated game folder
        else:
            return self.P2PFOLDER + "/" + game_folder + "/" + shortcut_name + ".delta"
    

    def lookup_game_exe(self, game_folder, shortcut_name):
        """ Given a game folder name and a shortcut name, return the path of the executable."""

        shortcut_properties = self.get_shortcut_properties(game_folder, shortcut_name)
        if shortcut_properties:
            log.logger.info("Looking up " + shortcut_name + " - " + game_folder)
            return os.path.basename(shortcut_properties["Path"])
        else:
            log.logger.info("Can't find a game folder called " + game_folder)
            return ""

    def get_profile_list(self):
         tup = self.profiles.keys()
         tup.sort(lambda x, y: cmp(string.lower(x), string.lower(y)))
         li = list(tup)
         li.sort()
         return tuple(li)

    def get_index_of_current_profile(self, current_profile):
        profile_list = self.get_profile_list()
        i = 0
        for profile in profile_list:
            if current_profile == profile:
                return i
            i=i+1

        # FIXME - if we got here, we should return the index of the current
        # default version of Cedega.

        return -1

    def get_nth_profile(self, n):
        profile_list = self.get_profile_list()
        return profile_list[n]

    def get_default_profile_product(self, file):
        period = file.find(".")
        underscore = file.find("_")
        return file[(period+1):underscore]

    def get_default_profile_version(self, file):
        underscore = file.find("_")
        return file[(underscore+1):]

    def get_default_profile_product_and_version(self, file):
        period = file.find(".")
        underscore = file.find("_")
        return file[(period+1):underscore], file[(underscore+1):]

    def populate_default_profiles_directory(self, copy_all=0):

        # Let's get a list of all the config files that are available in the package. 
        packaged_configs = os.listdir(self.ORIGINAL_DEFAULT_PROFILES_PATH)
        log.logger.info("Looking through " + self.ORIGINAL_DEFAULT_PROFILES_PATH)
        # Now, quickly run thorugh and check to see if any of these config files are missing in the user's
        # self.DEFAULT_PROFILES_PATH directory...if so, copy them over.  This will aid in fixing problems which
        # may occur in the event that a user renames/deletes a default...this way, they will always be there.
        for packaged_config in packaged_configs:
            if not packaged_config == "CVS":
                if not os.access( self.DEFAULT_PROFILES_PATH + packaged_config, os.F_OK ) and copy_all == 0:
                    shutil.copy( self.ORIGINAL_DEFAULT_PROFILES_PATH + packaged_config, self.DEFAULT_PROFILES_PATH + packaged_config )


    def get_default_profile_for_engine( self, version ):
        if version == None:
            return None

        default_profile = self.DEFAULT_PROFILES_PATH + "config.cedega_" + version
        if not os.access(default_profile, os.F_OK):
            if version == self.default_winex:  
                return None
            return self.get_default_profile_for_engine( self.default_winex )
        return default_profile

    def get_default_profile_from_cedega_package(self, version):
        # If there is not a default profile matching the passed in version in Point2Play, we will grab the
        # default profile from the Cedega package directly.

        if version == None:
            return None

        default_profile = self.P2PFOLDER + ".winex_ver/winex-" + version + "/.transgaming/config.cedega_" + version

        if not os.access(default_profile, os.F_OK):
            # Hrmmm...this version of Cedega doesn't have a packaged default profile.  As a last precaution to
            # a crash, copy the Cedega versions config file.
            default_profile = self.P2PFOLDER + ".winex_ver/winex-" + version + "/.transgaming/config"
            if not os.access(default_profile, os.F_OK):
                # Not good...if the Cedega package doesn't even have a config file...print an error.
                log.logger.warning( _("The Cedega version (" + str(version) + ") you are defaulting to is missing the following file : " + default_profile))
                return None

        # Ok, now return the name of the file to be copied.
        return default_profile


    def match_and_copy_default_cedega_profiles(self):
        # Ok, since we never edit the shipped default profiles, we'll
        # copy them to provide the user with editable versions of them.
        for i in range(0, len( self.versions ) ):

            # First things first...let's get make things easier to read...
            version = self.versions[i]["version"]
            default_profile = self.DEFAULT_PROFILES_PATH + "config.cedega_" + version
            user_profile = self.USER_DEFINED_PROFILES_PATH + "cedega_" + version

            # Before we try and copy a default profile into the user defined directory,
            # let's see if one already exists, so that we don't already overwrite a
            # possibly edited file with the default...
            if not os.access( user_profile, os.F_OK ):
                # Ok, the file doesn't exist...check to see if a default profile exists for this version...
                if not os.access(default_profile, os.F_OK):
                    # If the default profile is not in the default profile location, grab it from the Cedega package.
                    default_profile = self.get_default_profile_from_cedega_package(version)
                if default_profile:
                    shutil.copy(default_profile, user_profile)
                    # As we add the new profile to the directory, let's take it's name and add it to our list of profiles.
                    profile_to_add = "cedega_" + version
                    self.profiles[profile_to_add] = []

            if version == self.default_winex:
                self.current_default_cedega_profile = default_profile
                self.current_user_cedega_profile = user_profile
                log.logger.info("self.current_default_cedega_profile now == " + str(self.current_default_cedega_profile))
                log.logger.info("self.current_user_cedega_profile now == " + str(self.current_user_cedega_profile))


    # When the user clicks "Play", a profile will be copied over to the respective game directory.  This
    # profile will be a copy of the profile that is currently linked to the game.  If no profile is linked
    # to this game, use the user-editable profile that corresponds to the currently selected Cedega.
    def copy_profile_to_config(self, exe, game_folder, override_profile=None, shortcut_name=None):
        plist = self.get_profile_load_list( game_folder, shortcut_name )
        pconfig = config.config_file( plist )
        pconfig.write( self.P2PFOLDER + game_folder + "/config" )
        return
        
        if exe:
            # Now, let's see if there's a profile associated with the current exe...
            profile = self.lookup_game_profile_association( exe, game_folder )
            if profile:
                # This game is linked to a profile...
                profile_to_copy = self.USER_DEFINED_PROFILES_PATH + profile
            else:
                if self.current_user_cedega_profile:
                    # If there is not profile associated with this file, copy over the profile that corresponds to the currently selected
                    # Cedega version...
                    profile_to_copy = self.current_user_cedega_profile
                else:
                    print "There was an error with the default profile..."
                    return -1

            # If the user has provided an override profile from the command line, override
            # the associated profile...
            if override_profile:
                profile_to_copy = self.USER_DEFINED_PROFILES_PATH + override_profile

            # We'll copy the desired profile over to the game's directory and
            # rename it to "config"
            dest_config_file = self.P2PFOLDER + game_folder + "/config"

            log.logger.info("Point2Play.py - copy_profile_to_config : src = " + profile_to_copy + " dest = " + dest_config_file)

            # "Copy that, copying file right now..."
            shutil.copyfile(profile_to_copy, dest_config_file)


    def retrieve_current_default_profile(self):
        # Ok, we need to get the untouched default config file of the currently selected version of Cedega...
        default_game_config = config.config_file(self.current_default_cedega_profile)

    def set_default_winex(self, version_number):
        self.default_winex = version_number
        self.update_current_default_cedega_profiles(version_number)
        if version_number:
            c = config.config_file( [self.P2PFOLDER + self.GLOBAL_DELTA_FILENAME] )
            c.set_option( "Version", "Cedega", version_number )
            c.write( self.P2PFOLDER + self.GLOBAL_DELTA_FILENAME )

    def get_browser_path(self):
        """ Returns the path of the global preferences browser field, or empty string if there is none."""
        c = config.config_file( [self.P2PFOLDER + self.GLOBAL_DELTA_FILENAME] )
        browser = c.get_option("wine", "Browser")
        if browser:
            return browser
        else: # might be None
            return ""

    # When the user changes the current Cedega version, point to new current default profiles.
    def update_current_default_cedega_profiles(self, version):
        if version:
            # If the user launches Point2Play, then erases a default uneditable profile ( in .default_configuration_profiles ), then we might try
            # and access a file that's not there...let's make sure it's there.
            default_profile = self.DEFAULT_PROFILES_PATH + "config.cedega_" + version
            packaged_default_profile = self.ORIGINAL_DEFAULT_PROFILES_PATH + "config.cedega_" + version

            # Before we do the setting of anything, do we have a default profile for this version of Cedega?
            if not os.access(default_profile, os.F_OK):
                # Hrmm....apparently not.  Ok, in this case, let's check out the default_configuration_profiles folder in the 
                # transgaming_cedega package...do we have a matching version of the file there?
                if os.access(packaged_default_profile, os.F_OK):
                    # Yup...copy it over.
                    shutil.copyfile(packaged_default_profile, default_profile)
                else:
                    # So, this must be a Cedega engine package that's newer than our Cedega GUI...in that case, let's get the default
                    # profile from the Cedega package.
                    cedega_packaged_default_profile = self.get_default_profile_from_cedega_package(version)
                    if cedega_packaged_default_profile != None:
                        shutil.copyfile(cedega_packaged_default_profile, default_profile)
                    else:
                        no_config_dialog = p2p_diag.P2PMessageDialog( self.parent_window, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("A default profile for your copy of Cedega %s could not be found.  Perhaps the package was corrupted. You may want to try downloading it again and reinstalling.") % str(version) )
                        no_config_dialog.run()
                        no_config_dialog.destroy()              
                        return

            # Ok, it seems we have the default file to match the current default version...let's see if we have the user-editable version.
            if not os.access(self.USER_DEFINED_PROFILES_PATH + "cedega_" + version, os.F_OK):
                self.match_and_copy_default_cedega_profiles()

            self.current_default_cedega_profile = self.DEFAULT_PROFILES_PATH + "config.cedega_" + version
            self.current_user_cedega_profile = self.USER_DEFINED_PROFILES_PATH + "cedega_" + version
            log.logger.info("self.current_default_cedega_profile now == " + self.current_default_cedega_profile)
            log.logger.info("self.current_user_cedega_profile now == " + self.current_user_cedega_profile)
        else:
            log.logger.info("self.current_default_cedega_profile now == None")
            log.logger.info("self.current_user_cedega_profile now == None")
            self.current_default_cedega_profile = None 
            self.current_user_cedega_profile = None

    # Get the profile name from a full path string containing the profile name at the end.
    def get_profile_from_full_path(self, full_path_profile):
        last_slash = full_path_profile.rfind("/")
        return full_path_profile[(last_slash+1):]

    # Is this profile a copy of one of the defaults?
    def is_profile_cedega_profile(self, filename):
        for i in range(0, len( self.versions ) ):
            version = self.versions[i]["version"]
            cedega_file = "cedega_" + version
            if ( cedega_file == filename ):
                return 1
        return 0


    ###########################################################################################################
    # - Migration API                                                                                         #
    ###########################################################################################################

    def check_if_migration_needed(self):
        point2playrc = os.environ["HOME"] + "/.point2playrc"
        point2play_dir = os.environ["HOME"] + "/.point2play"
        transgaming_dir = os.environ["HOME"] + "/.transgaming"

        # Phase 1
        if os.access(point2play_dir, os.F_OK) and os.access(point2playrc, os.F_OK):
            # Check and see if .point2play is a symlink...if so, throw up a warning, and move on...
            if os.path.islink(point2play_dir):
                symlink_detected_dialog = p2p_diag.P2PMessageDialog( self.parent_window, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("Cedega has detected that %s/.point2play is not a valid directory, it may be a symlink. Cedega cannot migrate the games and configuration profiles from this location.  Please read the How-To for instructions on manually migrating your games and configuration profiles.") % os.environ["HOME"] )
                symlink_detected_dialog.run()
                symlink_detected_dialog.destroy()
            else:
                do_migration_dialog = p2p_diag.P2PMessageDialog( self.parent_window, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_YES_NO, _("Games and settings from a Point2Play setup have been detected.  The functionality of Point2Play is now included in Cedega.  To use your previously installed games and settings Cedega will migrate them to the correct location for use.\n\nWARNING! This operation cannot be reversed.\n\nIf NO is selected you may launch the migration tool in the future by selecting Run Setup Wizard from the Tools menu.") )
                migrate = do_migration_dialog.run()
                do_migration_dialog.destroy()

                if migrate == gtk.RESPONSE_YES:
                    # Do migration....
                    self.migrate_point2play_and_point2playrc()
                else:
                    migration_later_dialog = p2p_diag.P2PMessageDialog( self.parent_window, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("You may launch the migration tool in the future by selecting Run Setup Wizard from the Tools menu.") )
                    migration_later_dialog.run()
                    migration_later_dialog.destroy()

        # Phase 2
        if os.access(transgaming_dir, os.F_OK):
            # Check and see if .transgaming directory is a symlink...if so, throw up a warning, and move on...
            if os.path.islink(transgaming_dir):
                symlink_detected_dialog = p2p_diag.P2PMessageDialog( self.parent_window, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("Cedega has detected that %s/.transgaming is not a valid directory, it may be a symlink. Cedega cannot migrate the games and configuration profiles from this location.  Please read the How-To for instructions on manually migrating your games and configuration profiles.") % os.environ["HOME"] )
                symlink_detected_dialog.run()
                symlink_detected_dialog.destroy()
            else:
                # Migrate the .transgaming folder...
                do_migration_dialog = p2p_diag.P2PMessageDialog( self.parent_window, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_YES_NO, _("Games and settings from an older command line version of Cedega have been detected.  To use your previously installed games Cedega will migrate them to the correct location for use.\n\nWARNING! This operation cannot be reversed.\n\nIf NO is selected you may launch the migration tool in the future by selecting Run Setup Wizard from the Tools menu.") )
                migrate = do_migration_dialog.run()
                do_migration_dialog.destroy()
                if migrate == gtk.RESPONSE_YES:
                    # Do migration....
                    self.migrate_transgaming_folder()
                else:
                    migration_later_dialog = p2p_diag.P2PMessageDialog( self.parent_window, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("You may launch the migration tool in the future by selecting Run Setup Wizard from the Tools menu.") )
                    migration_later_dialog.run()
                    migration_later_dialog.destroy()
                

    def create_directory_for_migration(self, dirname):
        # If the requested directory doesn't already exist, create it.
        if not os.access( dirname, os.F_OK ):
            os.mkdir( dirname )


    def separate_version_and_credential(self, whole_version):
        last_bracket = whole_version.find('>')
        version = whole_version[(last_bracket+1):]
        credential = whole_version[1:(last_bracket-1)]
        return version, credential 


    def separate_profile_from_games(self, profile_string):
        equals = profile_string.find('=')
        profile = profile_string[:equals]
        games = profile_string[(equals+1):]
        return profile, games


    def migrate_games_ini(self, filename):
        newlines = []
        # If the games.ini file exists....
        if os.access( filename, os.F_OK ):
            # Open it...
            filePtr=open( filename, 'r')
            if filePtr:
                lines = filePtr.readlines()
                # Read it line by line...
                for line in lines:
                    # Replace every reference to "point2play" to "cedega"
                    line = line.replace("point2play", "cedega")
                    newlines.append(line)
            filePtr.close()

            # Now that we have the changed lines...write them...
            filePtr=open( filename, 'w' )
            if filePtr:
                filePtr.writelines(newlines)
            filePtr.close()
        
        

    def add_game_folder_and_shortcuts_to_gui(self, game_folder):
        """ Add game folder and its shortcuts to the GUI.
            Previously known as add_gamegroup_and_exes_to_gui(self, game). """

        # Check and see if we have this game folder.  If we don't, it will be created.
        self.create_new_game_folder(game_folder)

        dir_name = self.gamedir + game_folder
        if os.path.isdir( dir_name ):
            shortcuts = []
            try:
                game_config_file = file( dir_name + "/games.ini" )
                game_config = ConfigParser.ConfigParser()
                game_config.readfp( game_config_file )
                for shortcut_entry in game_config.sections():
                    try:
                       commandlineargs = game_config.get(shortcut_entry, "commandlineargs" )
                    except:
                       # new addition to game config file - if it's absent, make it blank
                       commandlineargs = ""
                    try:
                       cedega_app = game_config.get( shortcut_entry, "cedega_app" )
                    except:
                       # new addition to game config file - if it's absent, make it true
                       cedega_app = "Y"

                    try:
                       path = game_config.get( shortcut_entry, "path" )
                       icon = game_config.get( shortcut_entry, "icon" )
                       workdir = game_config.get( shortcut_entry, "workdir" )
                    except:
                        print _("Error parsing \"%s\" - Game \"%s\" is missing information") % (dir_name + "/games.ini", shortcut_entry)
                        continue

                    try:
                       gddb = game_config.get( shortcut_entry, "gddb" )
                    except:
                       gddb = ""

                    # just a quick sanity check - if the path is a URL, CedegaApp should be N.
                    if misc_utils.is_URL(path):
                        cedega_app = "N"

                    # Add to our list of available

                    shortcuts.append( { "Name": shortcut_entry,
                                        "Path": self.decode_games_ini_path(path),
                                        "Icon": self.decode_games_ini_path(icon),
                                        "Workdir": self.decode_games_ini_path(workdir),
                                        "DirName" : self.decode_games_ini_path(os.path.basename(dir_name)),
                                        "CommandLineArguments" : commandlineargs,
                                        "gddb" : gddb,
                                        "CedegaApp" : cedega_app } )

                if add_to_writing_list == 1:
                    needs_writing.append(game_folder)

            except:
                pass

            # Add the shortcuts to the list
            self._games[game_folder] = shortcuts
                             

    def migrate_point2play_and_point2playrc(self):
        dirs = []
        files_not_copied = 0
        old_default_cedega_version = ""
        point2play_dir = os.environ["HOME"] + "/.point2play"

        # Next we need to sort out the [transgaming] section of the (local) rc file
        point2playrc = ConfigParser.ConfigParser()
        point2playrc.readfp( file( os.environ["HOME"] + "/.point2playrc", "r" ) )

        cedegarc = ConfigParser.ConfigParser()
        try:  # figure out the path to the .cedegarc file or use a plausible location
            cedegarc_path = os.getenv('CEDEGA_RC_PATH')
            assert cedegarc_path
        except:
            log.logger.info("Setting cedegarc file to $HOME/.cedegarc - environment variable CEDEGA_RC_PATH was either unset or empty.")
            cedegarc_path = os.getenv('HOME') + "/.cedegarc"        

        cedegarc.readfp( file( cedegarc_path, "r" ) )
        for dir in os.listdir(point2play_dir):
            #for name in dirs:
            dirs.append(dir)

        # The new installation of Cedega will already have a complete set of .default_configuration_profiles.
        # As for the .winex_ver and configuration_profiles, we will copy over files and directories selectively.
        # We do NOT want to copy over the .language folder.
        try:
            dirs.remove(".default_configuration_profiles")
        except:
            pass
        try:
            dirs.remove(".winex_ver")
        except:
            pass
        try:
            dirs.remove("configuration_profiles")
        except:
            pass      
        try:
            dirs.remove(".languages")
        except:
            pass

        # ** GAMES **
        # Copy over the games...
        for dir in dirs:
            srcdir = os.path.join( point2play_dir,  dir )

            destdir = os.path.join( self.P2PFOLDER, dir )

            # If the destination directory doesn't already exists, move the game directory!
            if not os.access( destdir, os.F_OK ):
                shutil.move( srcdir, destdir )
            else:
                overwrite_dialog = p2p_diag.P2PMessageDialog( self.parent_window, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_YES_NO, _("The directory %s already exists.  Do you wish to overwrite it with your previously installed %s directory?") % (destdir, dir) )
                overwrite = overwrite_dialog.run()
                overwrite_dialog.destroy()
                if overwrite == gtk.RESPONSE_YES:
                    try:
                        # Remove the directory for overwriting...
                        shutil.rmtree( destdir )
                    except:
                        pass
                    try:
                        # Move it!
                        shutil.move( srcdir, destdir )
                    except:
                        pass
                else:
                    files_not_copied = 1
 
            # Update the games.ini file for each game directory
            self.migrate_games_ini(destdir + "/games.ini")

            # Update the GUI so that the games show up...
            self.add_game_folder_and_shortcuts_to_gui(dir)


        # ** username/psswd **
        # Has the user already entered their username/password in the new Cedega?
        new_username = cedegarc.get("transgaming", "username")
        # If not...
        if not new_username:
            # Get the old one...
            old_username = point2playrc.get("transgaming", "username")
            # if it's there...
            if old_username:
                # And migrate it over...
                self.config["username"] = old_username
                # Along with the old password...
                old_password = point2playrc.get("transgaming", "password")
                password = base64.decodestring(old_password)
                self.config["password"] = password

        # ** .winex_ver **
        # Check to see if .cedega has a .winex_ver folder already...if not, make one...
        self.create_directory_for_migration( self.P2PFOLDER + ".winex_ver" )
 
        try:
            # Loop through the .point2playrc file's [winex] entries...
            for version, version_dir in point2playrc.items( "winex" ):
                # If the version is default, skip it...
                if version == "default":
                    # Does the new config have a default value?
                    new_default_cedega_version = cedegarc.get("winex", "default")
                    if new_default_cedega_version == None or new_default_cedega_version == "None":
                        # Nope...Get the old default version..
                        old_default_cedega_version = point2playrc.get("winex", "default")
                    continue

                # Separate the version number from the credential...
                (version_num, credential) = self.separate_version_and_credential(version)
                value = cedegarc.has_option( "winex", version )
                dest_dir = os.path.join( self.P2PFOLDER, ".winex_ver/winex-" + version_num )
                
                if value == False:

                    # Does the src directory exist?  If so, copy it over...
                    if os.access( version_dir, os.F_OK ):
                        # First, check and see if there's any crunkiness going on...
                        if os.access( dest_dir, os.F_OK ):
                            try:
                                # Hmm...their config file says there's no .winex_ver here, but we found one.
                                shutil.rmtree( dest_dir )
                            except:
                                pass
                        try:
                            # Now copy it...
                            shutil.move( version_dir, dest_dir )
                        except:
                            pass

                    # Update the config file.
                    self.versions.append( { "version":version_num, "path":dest_dir, "local": 1, "script":None, "retailinstalldir":None, "version_credential":credential} )
                else:
                    overwrite_dialog = p2p_diag.P2PMessageDialog( self.parent_window, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_YES_NO, _("You have already downloaded Cedega %s using the new Cedega.  Do you wish to overwrite it with your previously installed Cedega %s directory?") % (version_num, version_num) )
                    overwrite = overwrite_dialog.run()
                    overwrite_dialog.destroy()
                    if overwrite == gtk.RESPONSE_YES:
                        # Does the src directory exist?  If so, copy it over...
                        if os.access( version_dir, os.F_OK ):
                            try:
                                # Trash the old...!
                                shutil.rmtree( dest_dir )
                            except:
                                pass
                            try:
                                # Make your move...
                                shutil.move( version_dir, dest_dir )
                            except:
                                pass
                    else:
                        files_not_copied = 1
 
        except ConfigParser.NoOptionError, e:
            raise badConfigFile( _("Missing options %s") % e.value )
        except ConfigParser.NoSectionError, e:
            raise badConfigFile( _("Missing [transgaming] section") )


        # ** PROFILES **
        try:
            # Loop through the profiles in our .point2playrc file and see if they need to be copied over and
            # reassociated...
            for profile_num, profile_name in point2playrc.items("profiles"):
                # Parse the string from the config file...
                (profile, games) = self.separate_profile_from_games(profile_name)

                # The profile in the new location...
                profile_to_copy = os.path.join( self.P2PFOLDER, "configuration_profiles", profile )

                # Check and see if profile alread exists in the new locale...if not, move it over..
                if not os.access(profile_to_copy, os.F_OK):
                    src_profile = os.path.join( point2play_dir, "configuration_profiles", profile )
                    if os.access( src_profile, os.F_OK ):
                        try:
                            shutil.move(src_profile, profile_to_copy)
                        except:
                            pass
                        # Add the new profiles to the set of profiles...
                        self.profiles[profile] = []
                    else:
                        # Hrmm...the profile was in the config, but the actual file was missing...forget it.
                        continue
                else:
                    overwrite_dialog = p2p_diag.P2PMessageDialog( self.parent_window, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_YES_NO, _("The profile %s already exists. Do you wish to overwrite the currently installed version with the version from your previous Point2Play install?") % (profile) )
                    overwrite = overwrite_dialog.run()
                    overwrite_dialog.destroy()
                    # Did the user want to overwrite the file?
                    if overwrite == gtk.RESPONSE_YES:
                        src_profile = os.path.join( point2play_dir, "configuration_profiles", profile )
                        # Check and see if we have a file to copy...
                        if os.access( src_profile, os.F_OK ):
                            try:
                                # Yup, remove the old profile
                                os.remove(profile_to_copy)
                            except:
                                pass
                            try:
                                # And copy the profile to the new location...
                                shutil.move(src_profile, profile_to_copy)
                            except:
                                pass
                    else:
                        files_not_copied = 1

                # Reassociation : Does this profile have associated games that need to be maintained?
                if games:
                    # How many games are associated with this one profile?
                    numgames_assoc = games.count(',') + 1

                    # Individually determine if a game/profile association needs to be kept, or forgotten
                    # due to a more recent installation and association of the same game in the new Cedega.
                    for i in range(numgames_assoc):
                        found = 0
                		# The profile values are stored in "dir/exe" format...
                        # first let's get rid of the quotes...
                        quote1 = games.find('"')
                        quote2 = games.find('"', (quote1+1))
                        game = games[(quote1+1):(quote2)] # game = dir/exe

                        # Let's get all the games for all the profiles, and see if this game is already
                        # associated with a profile...
                        values =  self.profiles.values()
                        for sublist in values:
                            if game in sublist:
                                # Aha!  It's already associated with a profile...in this case, we don't carry over
                                # the previous association.
                                found = 1
                        if found == 0:
                            # This game is not associated with any profiles in the new location...hook up the previous
                            # association.
                            self.associate_exe_and_profile(game[(game.find('/')+1):], game[:(game.find('/'))], profile)

                        # Chop off the game we already dealt with, so that next cycle, we handle a new one...
                        games = games[(quote2+1):]
        except:
            print _("There was a problem with migrating your old profiles.  These can be moved manually later.")


        # If we are going to migrate the old default cedega, now is the
        # time to do it...
        if old_default_cedega_version:
            self.set_default_winex(old_default_cedega_version)
        self.Point2Play_gui.setup_default_version_menu()


        # Finalize and write changes to the config file...
        self.write()

        if files_not_copied == 1:
            # Display message...
            remove_p2p_dir_dialog = p2p_diag.P2PMessageDialog( self.parent_window, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("Not all of your previous games or settings were migrated.  You may wish to manually delete the .point2play directory found in your home folder.")) 
            remove = remove_p2p_dir_dialog.run()
            remove_p2p_dir_dialog.destroy()
        else:
            # Erase the folder...
            try:
                shutil.rmtree(point2play_dir)
            except:
                pass

        # Delete the now forgotten .point2playrc file...before it falls into dilapidation...Farewell, Aquarius...she was
        # a good ship...
        try:
            os.remove(os.environ["HOME"] + "/.point2playrc")
        except:
            pass
        done_migration_dialog = p2p_diag.P2PMessageDialog( self.parent_window, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("Migration of your previously installed games and settings is now complete."))
        done = done_migration_dialog.run()
        done_migration_dialog.destroy()


    def migrate_transgaming_folder(self):
        dot_transgaming_dir = os.path.join( self.P2PFOLDER, "Dot TransGaming")
        transgaming_drive = os.path.join( os.environ["HOME"], "TransGaming_Drive")

        if not os.access( dot_transgaming_dir, os.F_OK ):
            src_dir = os.environ["HOME"] + "/.transgaming"
            shutil.move( src_dir, dot_transgaming_dir )

            # Now, make an Icons directory...
            icon_dir = os.path.join( dot_transgaming_dir, "icons" )
            self.create_directory_for_migration(icon_dir)

            # Copy over the default icon...
            if os.access( self.P2PPATH + "/gui/tg_logo.png", os.F_OK ):
                shutil.copyfile( self.P2PPATH + "/gui/tg_logo.png", icon_dir + "/tg_logo.png")

            # Now it's time to create the games.ini file for our new directory...
            filePtr=open(dot_transgaming_dir + "/games.ini", 'w+')
            if filePtr:
                # Loop through the directory...
                for root, dirs, files in os.walk(dot_transgaming_dir, topdown=False):
                    for file in files:
                        # Are there any .exes?
                        if file.find(".exe") != -1:
                            # If these .exes are not IKernel, regsvr32 and are not in the c_drive/windows directory, we want it!
                            if file.find("Kernel") == -1 and file.find("regsvr32") == -1 and root.find("c_drive/windows") == -1:
                                exe_dir = root[(root.rfind('/')+1):]
                                path = root[(root.rfind("c_drive")+7):]
                                file_without_ext = file[:(file.rfind('.'))]
                                filePtr.write( "[" + exe_dir + " - " + file_without_ext + "]\n" )
                                filePtr.write( "path=C:" + path + "/" + file + "\n" )
                                filePtr.write( "icon=" + icon_dir + "/tg_logo.png\n" )
                                filePtr.write( "workdir=" + root + "\n" )
            filePtr.close()

        # Update the GUI to show the new folder...
        self.add_game_folder_and_shortcuts_to_gui("Dot TransGaming")

        # Remove the TransGaming_Drive symlink from the user's directory...
        if os.access( transgaming_drive, os.F_OK ):
            try:
                os.remove( transgaming_drive )
            except:
                pass
        done_migration_dialog = p2p_diag.P2PMessageDialog( self.parent_window, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("Migration of your old .transgaming directory is complete."))
        done = done_migration_dialog.run()
        done_migration_dialog.destroy()
        

    ###########################################################################################################
    # - Pthread API                                                                                           #
    ###########################################################################################################

    def get_pthread_setting(self, profile_config):
        pthreads_value = profile_config.get_option("pthreads", "PthreadsValue")

        if pthreads_value == None or pthreads_value == "default":
            # The profile is set to default...use the .cedegarc value for
            # pthreads_forced
            pthreads_value = self.config["pthreads_forced"]

            if pthreads_value == "on" or pthreads_value == "yes":
                pthreads_value = "yes"
            elif pthreads_value == "off" or pthreads_value == "no":
                pthreads_value = "no"
            else:
                pthreads_value = "auto"

        if pthreads_value == "auto":
            return None

        # If we made it here, then it's a "yes" or a "no"...return it!
        return pthreads_value


    ###########################################################################################################
    # - CommandLine API                                                                                       #
    ###########################################################################################################

    def commandline_renice( self, *args ):
        while 1:
            try:
                ret = os.waitpid( 0, os.WNOHANG )

                # Keep looping until the Cedega pid is up and running...
                while self.renice_wineserver == 1:
                   contents = os.popen( "ps -C wineserver -o pid --noheaders" )
                   pids = contents.readlines()
                   contents.close()

                   if pids:
                      for pid in pids:
                         os.system ( "renice 10 -p " + str(pid))
                      self.renice_wineserver = 0
                   time.sleep(1)

                if ret[0] == 0:
                    gtk.timeout_add( 1000, self.reap_children )
                    return

            except:
                # Install or game completed..
                self.CMDLINE_RENICING = 0
                # Get rid of the entries in the list of winex running
                # FIXME: There is no nice way to track wineserver PIDs vs.
                # running, so this isn't going to work very well if you have
                # more than 1 game running - could be looked at later but
                # also might not be a big deal considering most people only
                # run one game at a time (and there are also other "more
                # than one wine at a time issues such as with the "stop
                # winex" menu)
                self.WineXRunning = []

                # No more renicing if it hasn't happened yet...
                self.renice_wineserver = 0

                return


    def get_profile_load_list( self, game_folder=None, shortcut_name = None, noshortcut = None, install_gddb=None ):
      dfile = None
      engine = None
      no_shortcut = 0
      gddb = None

      try:      
          gddb = self.lookup_gddb_by_proggrp_shortcut( game_folder, shortcut_name )
      except:
          pass
      
      if install_gddb != None:
          gddb = install_gddb
      
      if game_folder != None and shortcut_name != None:
        try:
          dfile = self.get_shortcut_delta_name( game_folder, shortcut_name )
        except TypeError:
          no_shortcut = 1
          pass
        except KeyError:  # likely a new install
          no_shortcut = 1
          pass
          
        engine = None
        
        if no_shortcut == 0 and os.access(dfile, os.F_OK):
          sdelta = config.config_file( [dfile] )
          e = sdelta.get_option("Version", "Cedega")
          if e != None:
            engine = e
        else:
          dfile = None
        
      gdfile = self.P2PFOLDER + self.GLOBAL_DELTA_FILENAME
      if os.access(gdfile, os.F_OK):
        sdelta = config.config_file( [gdfile] )
        if engine == None:
          engine = sdelta.get_option("Version", "Cedega")
      else:
        gdfile = None
      
      if engine == None or engine == "None":
        engine = self.default_winex

      if engine == None or engine == "" or engine == "default" or engine == "None":
        engine = self.default_winex
      efile = self.get_default_profile_for_engine( engine )
      
      if noshortcut == None:
        return( [efile, gdfile, gddb, dfile] )
      
      if noshortcut == 2:
        return( [efile, gddb] )
      
      return( [efile, gdfile, gddb] )


    def update_gddb( self ):
        master_files = os.listdir( self.P2PPATH + "/gddb" )
        local_files = os.listdir( self.P2PFOLDER + "/.gddb" )

        # Run through local files, updating newer revisions where necessary
        # from master copy
        for x in local_files:
            if x.rfind(".gddb") != -1:
                try:
                    local_file = self.P2PFOLDER + "/.gddb/" + x
                    master_file = self.P2PPATH + "/gddb/" + x
                    g = gddb.load_gddb( local_file, silent=1 )
                    if g == None:
                        local_ver == -2^19
                    local_ver = float( g["header"]["revision"] )

                    if os.access( master_file, os.F_OK ):
                        g = gddb.load_gddb( master_file, silent=1 )
                        if g == None:
                            master_ver = -2^20
                        master_ver = float( g["header"]["revision"] )
                    else:
                        master_ver = -2^20

                    if master_ver > local_ver:
                        shutil.copy( master_file, local_file )
                except:
                    pass

        # Run through master files, copying over when necessary
        for x in master_files:
            if not x in local_files:
                if x != "":
                    try:
                        shutil.copy( self.P2PPATH + "/gddb/" + x, self.P2PFOLDER + "/.gddb/" + x )
                    except IOError:
                        pass


    def gddb_notes( self, game_folder, shortcut_name, gddb_file=None ):
        general = ""
        play = ""
        install = ""
        all = ""

        if gddb_file != None:
            all_conf_paths = self.get_profile_load_list( None, None, install_gddb=gddb_file )
            all_conf = config.config_file( all_conf_paths )
        else:
            gddb_file = self.lookup_gddb_by_proggrp_shortcut( game_folder, shortcut_name )
            all_conf_paths = self.get_profile_load_list( game_folder, shortcut_name )
            all_conf = config.config_file( all_conf_paths )

        if gddb_file != None:
            gddb_prog = gddb.load_gddb( gddb_file )

            buf = "\tGeneral Notes\n\n"
            gn = gddb.evaluate_run( gddb_prog["game"]["general"], all_conf )
            if (gn != None and len(gn) > 0):
                for x in gn:
                    buf += x + "\n\n"
            else:
                buf += "No known issues.\n"
            buf += "\n"

            general = buf

            buf = "\tPlaying\n\n"
            gn = gddb.evaluate_run( gddb_prog["game"]["run"], all_conf )
            if (gn != None and len(gn) > 0):
                for x in gn:
                    buf += x + "\n\n"
            else:
                buf += "No known issues.\n"
            buf += "\n"

            play = buf


            # Set 'install' notebook tab game notes
            installer_buf = "\tInstallation Notes\n\n"
            gn = gddb.evaluate_run( gddb_prog["game"]["installer"], all_conf )
            if gn != None and len(gn) > 0:
                for x in gn:
                    installer_buf += x + "\n\n"
            else:
                installer_buf += "No known issues.\n"
            buf += "\n"
            install = installer_buf


            # Set 'all' notebook tab game notes
#                textbuffer = p2p_gui.widgets["gn_all"].get_buffer( )
            buf = ""

            gn = gddb.evaluate_run( gddb_prog["game"]["general"], all_conf, True )
            buf += "\tGeneral Notes\n\n"
            if (gn != None and len(gn) > 0):
                for x in gn:
                    buf += x + "\n\n"
            else:
                buf += "No known issues.\n"
            buf += "\n"

            gn = gddb.evaluate_run( gddb_prog["game"]["run"], all_conf, True )
            buf += "\tPlay Notes\n\n"
            if (gn != None and len(gn) > 0):
                for x in gn:
                    buf += x + "\n\n"
            else:
                buf += "No known issues.\n"
            buf += "\n"

            gn = gddb.evaluate_run( gddb_prog["game"]["installer"], all_conf, True )
            buf += "\tInstallation Notes\n\n"
            if (gn != None and len(gn) > 0):
                for x in gn:
                    buf += x + "\n\n"
            else:
                buf += "No known issues.\n"
            buf += "\n"
            all = buf

            return [play, install, general, all]
      
      
###########################################################################################################
# - Misc.                                                                                                 #
###########################################################################################################


# module functions
def get_version_number_from_product_string(product_string):
    """ Helper function - takes a string like "WineX 3.1" or "Cedega 4.2" and returns the
        version number alone. """
    return product_string[product_string.rfind(" ")+1:]
