"""
Point2Play Source Code
Copyright 2003,2004,2005,2006 TransGaming Technologies Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""

import os, sys
from stat import *

# Make sure P2PPATH is both set and valid. Do this before we import any internal
# modules - otherwise we get exceptions (for example, log.py will look in the wrong
# place for its log.ini file. Then for the rest of the code we can assume P2PPATH
# is in the env.

complain = False
if os.environ.has_key('P2PPATH'):
    p2ppath = os.getenv("P2PPATH")
    if p2ppath[-1] != '/':
        p2ppath = p2ppath + "/" # tack on the slash
        os.environ["P2PPATH"] = p2ppath  # store it back in the environment variable
    # make sure it is a valid path - that is, it contains this file. Note:
    # I cannot use __name__ because it will be set to "__main__" when this is run.
    if not os.access( p2ppath + "Point2Play_gui.py", os.F_OK ):
        complain = True
else:
    complain = True

if complain:
    print "CRITICAL ERROR: Please set the environment variable P2PPATH to the installation directory before running Cedega."  # note - we can't use _("...") yet
    sys.exit(1)

import re, popen2, string
import gddb
import ISOHeader

# Need to import the stuff for python gettext 
import locale, gettext

try:
    import pygtk
    pygtk.require('2.0')
    import urllib, urlparse
except:
    pass

import log
import warnings
warnings.filterwarnings('ignore', category=DeprecationWarning, lineno=0)

try:
    import gtk, gobject, gtk.gdk
except:
    # we can't use _("..") yet.
    log.logger.critical("Unable to load GTK2 Python bindings: " + str(sys.exc_info()[1]))
    
    sys.exit(1)

import gtk.glade

try:
    p2pfolder = os.environ["P2PFOLDER"]
    assert p2pfolder
except:
    p2pfolder = os.environ["HOME"] + "/.cedega"

try: 
    locale.setlocale(locale.LC_ALL, '') 
except: 
    locale.setlocale(locale.LC_ALL, 'C') 

gtk.glade.bindtextdomain('p2p_localized_msg', os.path.expanduser( str( p2pfolder + "/.languages" )))
gettext.textdomain('p2p_localized_msg')
gettext.install('p2p_localized_msg', os.path.expanduser( str( p2pfolder + "/.languages")), unicode=1)

try:
    import misc_utils, ConfigParser, config, config_gui, winex_get_gui, winex_get, preferences_gui, p2p_diag, license, beginners_guide, winex_ver_gui, debug_gui, httplib, urllib, cedega_version_parse, p2p_gui, setup_wizard_gui, system_tests_gui, tgaccount_gui, new_install_gui, system_settings_gui, options_gui, about_gui, system_preferences_gui, install_cedega_engine, shortcut_properties_gui, entry_dialog, conversion_dialogue, system_detection, remove_engine_gui
except ImportError, e:
    log.logger.critical(_("Unable to load GUI components: " + str( sys.exc_info()[1])))
    sys.exit(1)  

import glob

# AlreadyInstalled GUI is internal only. don't bork if the code isn't there

ALREADYINSTALLEDGUI_PRESENT = 0

# How often to check/update the visual cue(s) for any running games (per game folder)
GAME_FOLDER_ACTIVITY_UPDATE_INTERVAL = 5000    # five seconds

#
# NOTE: Going to try to not use this anymore... for now.. 
#       Only want to support one way to do this, and this one is pretty
#       messy
#
#try:
#   import alreadyinstalled_gui
#   ALREADYINSTALLEDGUI_PRESENT = 1
#except:
#   pass

# URL for Volunteer request for translations for P2P
TRANSLATION_VOLUNTEER_URL = "http://www.transgaming.org/i18n"

# List of avail langs
TRANSLATION_AVAILLANGS_HOST = "transgaming.org"
TRANSLATION_AVAILLANGS_PATH = "/i18n/availlang.html"

# how long to show the busy cursor
BROWSER_LAUNCH_BUSY_TIME = 10000   # milliseconds

import Point2Play

try:
    import mount, cdrom
except:
    log.logger.critical(_("Unable to load optical drive utilities"))
    sys.exit(1)

import tempfile, os, select, signal, copy, shutil, socket

GAME_RENAMED = 1
GAME_NOT_RENAMED = 0

# Constants for return codes from the pthreads stack tester
NO_PTHREADS_NEEDED              = 1
PTHREADS_NEEDED_TOO_SMALL_STACK = 2
PTHREADS_NEEDED_OK              = 3

# Global variable for whether themes are enabled or not
THEMES_ENABLED = 0
# Auto WM Icon sync constants
#AUTOWMICONSYNC_ON         = 0
#AUTOWMICONSYNC_OFF        = 1

def isemptyfile(filename):
    try:
        size = os.path.getsize(filename)
    except:
        return True
    log.logger.info("Size of config file is: " + str(size))
    return size == 0

def kill_wine_timeout( procs, *args ):
    for proc in procs:
        try:
            os.kill( proc, signal.SIGKILL )
        except:
            pass

class PasswordDialog( gtk.Dialog ):
    def __init__(self):
        gtk.Dialog.__init__(self)
        self.add_button( gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL )
        self.add_button( gtk.STOCK_OK, gtk.RESPONSE_OK )
        vbox = self.get_children()[0]
        vbox.add( gtk.Label( _("Password") ) )
        self.entry = gtk.Entry()
        self.entry.set_visibility( 0 )
        vbox.add( self.entry )
        vbox.show_all()

    def run(self):
        resp = gtk.Dialog.run(self)
        if resp == gtk.RESPONSE_OK:
            return self.entry.get_text()
        else:
            return None


class RenameGameFolderDialog( gtk.Dialog ):
    def __init__(self, P2P_gui, game_folder ):
        gtk.Dialog.__init__(self)
        self.set_transient_for( p2p_gui.widgets["main"] )
        self.orig_game_folder = game_folder
        self.set_title( _("Rename") )
        self.P2P = P2P_gui.Point2Play
        vbox = self.get_children()[0]
        vbox.add( gtk.Label( _("Enter new name for %s:") % game_folder ) )
        self.text_entry = gtk.Entry()
        vbox.add( self.text_entry )
        button = self.add_button( gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL )
        button.set_name( "round_button" )
        button = self.add_button( gtk.STOCK_OK, gtk.RESPONSE_OK )
        button.set_name( "round_button" )
        vbox.show_all()

    def run( self ):
        while 1:
            resp = gtk.Dialog.run( self )
            if resp == gtk.RESPONSE_OK:
                new_name = self.text_entry.get_text()
                if new_name.find('/') >= 0 or new_name.find('\\') >= 0:
                    err_diag = p2p_diag.P2PMessageDialog( self, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("Game folders may not have slashes in their names (either forward or backward). Please try again with a new name."))
                    err_diag.run()
                    err_diag.destroy()
                    continue
                if self.orig_game_folder == new_name:
                    return GAME_NOT_RENAMED
                try:
                    self.P2P.rename_game_folder( self.orig_game_folder, new_name )
                    return GAME_RENAMED
                except Point2Play.renameGameFolderFailed:
                    err_diag = p2p_diag.P2PMessageDialog( self, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("Sorry, the game folder %s already exists. Try a different name.") % (new_name))
                    err_diag.run()
                    err_diag.destroy()
                    continue
                except:
                    err_diag = p2p_diag.P2PMessageDialog( self, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("Sorry, there was an unknown error while trying to rename the game folder %s to %s. Please try a different name.") % (self.orig_game_folder, new_name))
                    err_diag.run()
                    err_diag.destroy()
                    continue
            else:
                return GAME_NOT_RENAMED


# This is a class that duplicates functionality in Point2Play_gui. This functionality
# is needed by the commandline feature, and has therefore been packed up in an easy to
# access class so that it's GUI independent.
# ** NOTE ** The old functions are still there and are used by the GUI.  I plan to 
# revisit this soon and rework the GUI code so that everyone is using the same functions.
# Hopefully tomorrow.
class Reaper( gtk.Dialog ):
    def __init__(self):
        self.REAPING = 0
        self.renice_wineserver = 0

    def reap(self):
        if not self.REAPING:
            self.REAPING = 1
            gtk.timeout_add( 1000, self.reap_children )        

    def terminate_gtk_main(self):
        gtk.main_quit()

    def reap_children( self, *args ):
        while 1:
            try:
                ret = os.waitpid( 0, os.WNOHANG )

                if self.renice_wineserver == 1:
                   contents = os.popen( "ps -C wineserver -o pid --noheaders" )
                   pids = contents.readlines()
                   contents.close()


                   if pids:
                      for pid in pids:
                         os.system ( "renice 10 -p " + str(pid))

                      self.renice_wineserver = 0

                if ret[0] == 0:
                    gtk.timeout_add( 1000, self.reap_children )
                    return

            except:
                # Install or game completed..
                self.REAPING = 0

                # No more renicing if it hasn't happened yet...
                self.renice_wineserver = 0

                # This is a bit of a hack that I plan to revisit soon.  Since the command
                # line needs gtk.main called, but doesn't have a quit button to kill gtk_main,
                # a call has to be made at the end of installation.  So, when's the end of 
                # installation?  The closest thing is right here.  I put a bit of a delay on it
                # since the icons take a little time to finish up during their creation.
                gtk.timeout_add( 5000, self.terminate_gtk_main )
                return


# This is a class that duplicates functionality in Point2Play_gui. This functionality
# is needed by the commandline feature, and has therefore been packed up in an easy to
# access class so that it's GUI independent.
# ** NOTE ** The old functions are still there and are used by the GUI.  I plan to 
# revisit this soon and rework the GUI code so that everyone is using the same functions.
class InstallMonitor( gtk.Dialog ):

    def __init__(self, Point2Play ):
        self.monitor = None
        self.additional_icon_lines = None
        self.Point2Play = Point2Play

    def monitor_game_install( self, game ):
        if not self.monitor:
             self.monitor = [ game, ]
             gtk.timeout_add( 2000, self.monitor_cb )
        else:
            for i in self.monitor:
                if i == game:
                    return
            self.monitor.append( game )

    def monitor_cb( self ):
        if not self.monitor:
            return
        for i in self.monitor:
                path = self.Point2Play.gamedir + i + "/" + "games.ini"
                if self.additional_icon_lines and self.additional_icon_lines.has_key(i):
                   (fp, clines) = self.additional_icon_lines[i]
                   del self.additional_icon_lines[i]
                else:
                   clines = 0
                   if os.access(path + ".new", os.F_OK):
                       lfile = None
                       try:
                           lfile = os.open (self.Point2Play.gamedir + i + "/" + ".lock", os.O_RDWR | os.O_EXCL | os.O_CREAT)
                       except os.error:
                           gtk.timeout_add( 500, self.monitor_cb )
                           return
                       os.rename( path + ".new",  path + ".tmp" )
                       fp = file( path + ".tmp", "r" )
                       os.unlink( path + ".tmp" )
                       os.close (lfile)
                       os.remove (self.Point2Play.gamedir + i + "/" + ".lock")
                   else:
                       # We'll try again later
                       continue

                lines = fp.readlines()[clines:]
                #If we had a partial read for this game left over from before,
                #load it in
                if self.additional_icon_lines and self.additional_icon_lines.get(i):
                   lines = self.additional_icon_lines[i] + lines
                   del self.additional_icon_lines[i]

                clines = clines + self.monitor_parse_icon(i, lines)

                if clines < len( lines ):
                    if not self.additional_icon_lines:
                       self.additional_icon_lines = { i : (fp, clines) }
                    else:
                       self.additional_icon_lines[i] = (fp, clines)
        gtk.timeout_add( 4000, self.monitor_cb )


    def monitor_parse_icon(self, i, lines):
        nlines = 4
        clines = 0
        use_v2 = 0

        if len(lines) > 1 and lines[0][:3] == "V2L":
            nlines = int(lines[0][3:])+1
            use_v2 = 1
        while len( lines ) >= nlines:
            if use_v2:
                name = None
                executable = None
                icon = None
                workdir = None
                args = None
                for n in range(1, nlines):
                    ltype = lines[n][:1]
                    ldata = lines[n][1:]
                    if ltype == 'N': name = ldata.strip()
                    elif ltype == 'E': executable = ldata.strip()
                    elif ltype == 'I': icon = ldata.strip()
                    elif ltype == 'W': workdir = ldata.strip()
                    elif ltype == 'A': args = ldata.strip()
            else:
                name = lines[0].strip()
                executable = lines[1].strip()
                icon = lines[2].strip()
                workdir = lines[3].strip()
                args = None

            self.monitor_register_icon(i, name, executable, icon, workdir, args)
            lines = lines[nlines:]
            clines = clines + nlines

        return clines

    def monitor_register_icon(self, proggrp_name, game_name, executable, icon, workdir, args):
        if game_name.lower().find( "uninstall" ) == -1:
            try:
                # Convert the name into UTF8
                import codecs
                latin1=codecs.lookup("latin1")
                utf8=codecs.lookup("utf-8")
                game_name=utf8[0](latin1[1](game_name)[0])[0]
            except LookupError:
                pass
            # Create the new shortcut in memory

            if misc_utils.is_URL(executable):
                cedega_app = "N"
            else: 
                cedega_app = "Y"

            retval = self.Point2Play.create_new_shortcut( proggrp_name, game_name, executable, icon, workdir, args, cedega_app)
            if not retval:
               # There was a problem. Don't continue creating this icon
               # (Most likely the icon already exists)
               return

            # Scribble out the games array to disk
            self.Point2Play.write_shortcut_entries( proggrp_name )
            exe = os.path.basename( executable )
            # Refresh the game view in P2P
            #p2p_gui.widgets["main_game_folder_view"].set_model( self.get_game_folder_model( 1 ) )


class Point2Play_gui:
    def __init__(self, Point2Play_instance):
        self.Point2Play = Point2Play_instance # This is the data of the Point2Play
        self.Point2Play.RegisterP2PGui_Instance (self) # Register the gui instance with Point2Play object 
        self.Point2Play.RegisterParentWindow(p2p_gui.widgets["main"]) # Register the parent window with Point2Play object so it can pop up dialogs

        # Needed for the corner case when we start to process the icon file while 
        # its in mid icon write
        # FIXME: Probably still not 100% ideal to completely eliminate the race, as its not 100% atomic. 
        self.additional_icon_lines = None

        self.renice_wineserver = 0

        self.All_Iter = None

        # An array to keep track of winex version and winexprefix of running winexes so that
        # they can be signalled when the user needs to unmount a disk (4.0 and > winexes only)
        self.WineXRunning = []

        # We don't know about this until the user requests us to contact transgaming.org/i18n
        self.AvailLang = None 
        # Only warn about connecting to the internet once
        self.LangInternetConnectWarned = 0

        try:
           P2PPATH = os.environ["P2PPATH"] + "/"
        except:
           P2PPATH = ""

        if os.access(P2PPATH + "VERSION", os.F_OK):
            # Set our demo flag...
            self.TIMEDEMO = 1
        else:
            self.TIMEDEMO = 0

        # If this is a timedemo, then let's set things up...
        if self.TIMEDEMO:
            self.setup_timedemo()

        self.setup_embedded_package()

        self.MONITOR_CEDEGA_PROCESSES = False  # disable the process monitoring / visual cue code for now
        if self.MONITOR_CEDEGA_PROCESSES:
            # this dictionary keyed on game folder keeps track of which game folders have running games at the moment
            self.currently_active_game_folders = {}
            # this dictionary will be the same, but for the last check 
            # (so we can tell if we need to do anything to the GUI)
            self.formerly_active_game_folders = {} 

            self.game_folder_activity_timeout_retval = gtk.timeout_add( GAME_FOLDER_ACTIVITY_UPDATE_INTERVAL, self.update_game_folder_activity )

        self.setup_for_oem()
        self.setup_for_license()
        self.setup_pthread_checking()
        self.setup_menu_and_toolbar()  # also checks to see if the wizard needs to be run
        # check for and setup any new system-wide games.
        # Requires an engine, so make sure we do this after the wizard step.
        self.setup_systemwide_games()
        self.setup_views()
        self.setup_logo()
        self.setup_cdrom()

        # connect up the signals
        p2p_gui.widgets.widgets.signal_autoconnect(self)

        # turn off text search in the tree views - it looks too bizarre (a text entry widget pops up at the bottom)
        p2p_gui.widgets["main_game_folder_view"].set_enable_search(False)
        p2p_gui.widgets["main_shortcut_view"].set_enable_search(False)
 
        p2p_gui.widgets["main"].show()


        # Configure GDDB text-buffers
        self.documents = {}
        self.documents[ "general" ] = p2p_gui.widgets[ "gn_general" ].get_buffer()
        self.documents[ "install" ] = p2p_gui.widgets[ "gn_install" ].get_buffer()
        self.documents[ "run" ]     = p2p_gui.widgets[ "gn_play" ].get_buffer()
        self.documents[ "all" ]     = p2p_gui.widgets[ "gn_all" ].get_buffer()
        
        
        # Is it necessary to copy over GDDB files?
        if not os.access(self.Point2Play.P2PFOLDER + "/.gddb/", os.F_OK):
            os.mkdir( self.Point2Play.P2PFOLDER + "/.gddb" )
            self.update_gddb( )
        
        if self.Point2Play.config["last_gui_version_run"] == "" or self.Point2Play.config["last_gui_version_run"] != self.Point2Play.VERSION_STRING:
            self.update_gddb( )
        
        self.Point2Play.config[ "last_gui_version_run" ] = self.Point2Play.VERSION_STRING
        self.Point2Play.write( )
        
        # Conversion to new delta format -- open this dialogue if necessary
        if self.Point2Play._games != None and self.Point2Play.config["delta_migration_required"] == "True": # yes, the string
            num_to_convert = 0

            for x in self.Point2Play.profiles.iteritems():
                for y in x[1]:
                    num_to_convert += 1

            for proggrp in self.Point2Play._games:
                for gameicon in self.Point2Play._games[proggrp]:
                    # only count it if it isn't a game installed system-wide,
                    # because we added system-wide games after the switch to delta files.
                    try:
                        if proggrp not in os.listdir(self.Point2Play.SYSTEMWIDE_GAMEDIR):   
                            num_to_convert += 1                  
                    except:
                        pass # maybe there's no system wide game dir or we can't access it...
                             # either way, we don't need to do anything.

            if num_to_convert > 0:
                dlg = conversion_dialogue.ConversionDialogue( self, num_to_convert )
                dlg.widgets["conversion_gui"].set_transient_for(p2p_gui.widgets["main"])
                dlg.widgets["conversion_gui"].show_all()

            self.Point2Play.config["delta_migration_required"] = "False" # yes, the string
            self.Point2Play.write()

        else: # we're also doing this at the end of migration
            system_detection.detect_optical_drives(self.Point2Play)
    
    
    def setup_systemwide_games(self):
        """ Check for new system-wide games, and if there are any, set them up 
            so the user can see them.
            Note: we use the engine to set up new game folders so make
            sure this runs only once you have at least one engine installed. """

        new_games = self.Point2Play.new_systemwide_games()
        if new_games:
            # let user know we're about to setup the games
            if len(new_games) == 1:
                plural_string1 = ""
                plural_string2 = "has"
                plural_string3 = "it"
            else:
                plural_string1 = "s"
                plural_string2 = "have"
                plural_string3 = "them"
            
            new_games_string = string.join( new_games, "\n" )

            yes_no_diag = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_YES_NO, _("The following game%s %s been found on your system:\n\n%s\n\nWould you like to set %s up now?\nNote: It will take a few seconds.") % (plural_string1, plural_string2, new_games_string, plural_string3))
            resp = yes_no_diag.run()
            yes_no_diag.destroy()
            if resp == gtk.RESPONSE_YES:
                self.Point2Play.setup_new_systemwide_games(new_games)

    def update_gddb( self ):
        self.Point2Play.update_gddb( )


    ##
    ## This function is executed every time Cedega starts.  It checks to see
    ## if there is an installed engine, and if there is not, it will look
    ## for an embedded package within P2PPATH/cedega-engine-embedded.cpkg
    ## and install it silently.
    ##
    ## For the Mandriva deal (Sep 8/06), but implemented as a general case.
    ##
    def setup_embedded_package( self ):
        # Next we need to sort out the [transgaming] section of the (local) rc file
        default_cedega_version = self.Point2Play.default_winex

        if default_cedega_version == None:
            embedded_cedega_engine = self.Point2Play.P2PPATH + "cedega-engine-embedded.cpkg"
            if os.path.exists(embedded_cedega_engine):
                install_cedega_engine_helper = install_cedega_engine.InstallCedegaEngineHelper(self.Point2Play, self)
                install_cedega_engine_helper.install_cedega_engine( InstallMode=install_cedega_engine.INSTALL_MANUAL_PACKAGE, embedded_filename=embedded_cedega_engine, SilentInstall=1 )
            else:
                return
        else:
            return

        launch = self.Point2Play.launchLicenseDialog()
        if not self.Point2Play.config["accept_license"]:
            TRANSGAMING_EULA_filename = P2PPATH + "LICENSE"
            TRANSGAMING_EULA_fp = file(TRANSGAMING_EULA_filename, "r" )
            TRANSGAMING_EULA_diag = license.LicenseDialog( TRANSGAMING_EULA_fp )
            TRANSGAMING_EULA_diag.set_position(gtk.WIN_POS_CENTER)
            resp = TRANSGAMING_EULA_diag.run()
            TRANSGAMING_EULA_diag.destroy()
            if resp == license.LICENSE_DECLINE:
                sys.exit(1)
            self.Point2Play.config["accept_license"] = 1
            self.Point2Play.write()


    def setup_timedemo(self):
        # Next we need to sort out the [transgaming] section of the (local) rc file
        default_cedega_version = self.Point2Play.default_winex

        if default_cedega_version == None:
            embedded_cedega_engine = self.Point2Play.P2PPATH + "cedega-engine-timedemo.cpkg"
            install_cedega_engine_helper = install_cedega_engine.InstallCedegaEngineHelper(self.Point2Play, self)
            install_cedega_engine_helper.install_cedega_engine( InstallMode=install_cedega_engine.INSTALL_MANUAL_PACKAGE, embedded_filename=embedded_cedega_engine, SilentInstall=1 )

            if os.environ["TIMEDEMOFOLDER"] != os.environ["HOME"]:
                cedega_timedemo_setup = os.environ["TIMEDEMOFOLDER"] + "/cedega_timedemo_setup"
            else:
                cedega_timedemo_setup = os.environ["P2PFOLDER"] + "/cedega_timedemo_setup"

            if not os.access( cedega_timedemo_setup, os.F_OK ):
                (winexdir, dummy, dum2, dum3) = self.Point2Play.get_winex_path("default")
                cedega_engine_script = winexdir + "/bin/winex3"

                # Check and see if the engine script file needs its WINEXINSTALLDIR path updated...
                newlines = []
                # If the games.ini file exists....
                if os.access( cedega_engine_script, os.F_OK ):
                    # Open it...
                    filePtr=open( cedega_engine_script, 'r')
                    if filePtr:
                        lines = filePtr.readlines()
                        # Read it line by line...
                        for line in lines:
                            # Replace every reference to "point2play" to "cedega"
                            line = line.replace("XXXDEMOINSTALLDIRXXX", winexdir)
                            newlines.append(line)
                    filePtr.close()

                    # Now that we have the changed lines...write them...
                    filePtr=open( cedega_engine_script, 'w' )
                    if filePtr:
                        filePtr.writelines(newlines)
                    filePtr.close()

                os.symlink( cedega_engine_script, cedega_timedemo_setup )

        launch = self.Point2Play.launchLicenseDialog()
        if not self.Point2Play.config["accept_license"]:
            TRANSGAMING_EULA_filename = P2PPATH + "LICENSE"
            TRANSGAMING_EULA_fp = file(TRANSGAMING_EULA_filename, "r" )
            TRANSGAMING_EULA_diag = license.LicenseDialog( TRANSGAMING_EULA_fp )
            TRANSGAMING_EULA_diag.set_position(gtk.WIN_POS_CENTER)
            resp = TRANSGAMING_EULA_diag.run()
            TRANSGAMING_EULA_diag.destroy()
            if resp == license.LICENSE_DECLINE:
                sys.exit(1)
            self.Point2Play.config["accept_license"] = 1
            self.Point2Play.write()

        (days,total) = self.Point2Play.numdays()       #must be computed
        if days <= 0:
            greeting = "Welcome to Transgaming Cedega Demo package"
        else:
            substring = ""
            if days >= 2:
                substring = "s"
            if days < total:
                greeting = "You have been using this demo version of cedega for " + str(days) + " day" + substring + " out of " + str(total) + " days"
            else:
                greeting = "Your copy of the Cedega Time Demo has expired.  Please click Subscribe Now"

        #create a plain dialog
        remind = gtk.Dialog("Cedega Time Trial")
        #load the image
        image = gtk.Image()
        image.set_from_file(P2PPATH + "gui/tg_logo_splash.png")
        #add the image
        remind.vbox.pack_start(image)
        #add the reminder text
        label = gtk.Label(greeting)
        remind.vbox.pack_start(label)
        #add the register button
        remind.add_button("Subscribe Now", 68)
        remind.add_button("Play", 2)
        remind.set_default(None)
        remind.set_focus(None)
        #remove the separator and center the dialog
        remind.set_has_separator(0)
        remind.set_position(gtk.WIN_POS_CENTER)
        #make things visible
        remind.show_all()
        #decide wether to display dialogs
        response_id = remind.run()
        remind.destroy()
        queue = gtk.events_pending()
        while queue:
            gtk.main_iteration()
            queue = gtk.events_pending()
        if response_id == 68:
            oemfile = None
            try:
                oemfile = file(P2PPATH + "VERSION", "r")
            except:
                pass
            if (oemfile != None):
                oemname = oemfile.readline()
                oemfile.close()

            self.go_to_url("http://www.transgaming.com/?ref=" + oemname)
            sys.exit(0)
        else:
            #make sure we have the needed material to launch
            self.Point2Play.canlaunch()
        (days,total) = self.Point2Play.numdays()       #must be computed
        if days < 0:
            sys.exit(0)
        else:
            if days >= total:
                expired = gtk.Dialog("Time Limit Expired...")
                #load the image
                image = gtk.Image()
                image.set_from_file(P2PPATH + "gui/tg_logo_big.png")
                #add the image
                expiremessage = "This version of the Cedega Time Demo has expired."
                subscribemessage = "Please subscribe at www.transgaming.com to continue playing."
                expired.vbox.pack_start(image)
                label_space = gtk.Label("")
                label_space2 = gtk.Label("")
                label_expiration = gtk.Label(expiremessage)
                label_subscription = gtk.Label(subscribemessage)
                expired.vbox.pack_start(label_space)
                expired.vbox.pack_start(label_expiration)
                expired.vbox.pack_start(label_subscription)
                expired.vbox.pack_start(label_space2)
                #add the register button
                #expired.add_button("Close", 2)
                expired.add_button("Close", 68)
                expired.set_default(None)
                expired.set_focus(None)
                #remove the separator and center the dialog
                expired.set_has_separator(0)
                expired.set_position(gtk.WIN_POS_CENTER)
                #make things visible
                expired.show_all()
                response_id = expired.run()
                expired.destroy()
                if response_id == 68:
                    print "exiting..."
                    sys.exit(1)


    def setup_for_oem(self):
        # We need to determine if we are a TransGamer subscriber release
        # of P2P or an OEM release. OEM releases typically have different
        # behaviour in regards to how WineX installed versions are handled
        if os.access(P2PPATH + "OEM", os.F_OK):
           # Slurp in the OEM version crunk
           self.OEM_VERSION = 1     
           oem_fp = file( P2PPATH + "OEM", "r" )
           lines = oem_fp.readlines()
           self.OEM_TEXT = ""
           for line in lines:
              self.OEM_TEXT = self.OEM_TEXT + line
           oem_fp.close()
           
           # Now, if there is a TRANSGAMER_PAGE file as well, we slurp 
           # in that content for a TransGamer/TransGaming information page 
           # The file is set up:
           #   Line 1:    Tab Name
           #   Line 2:    URL link to more information
           #   Line 3:    Text on the Link Button
           #   Line 4->n: The actual supporting to text
           if (os.access(P2PPATH + _("TRANSGAMER_PAGE"), os.F_OK) ):
              oem_tg_tab_fp = file( P2PPATH + _("TRANSGAMER_PAGE"))
              self.OEM_TABNAME = oem_tg_tab_fp.readline()
              self.OEM_INFOURL = oem_tg_tab_fp.readline()
              self.OEM_LINKBTNTEXT = oem_tg_tab_fp.readline()
              self.OEM_TABTEXT = ""
              tg_tabtexts = oem_tg_tab_fp.readlines()
              for line in tg_tabtexts:
                 self.OEM_TABTEXT = self.OEM_TABTEXT + line

              oem_tg_tab_fp.close()
           else:   
              self.OEM_TABNAME = None 
              self.OEM_INFOURL = None
              self.OEM_LINKBTNTEXT = None
              self.OEM_TABTEXT = None
        else:
           self.OEM_VERSION = 0
           self.OEM_TEXT = None

        # The section below reflects the differences between the menus
        # in the OEM version and non-OEM version
        if self.OEM_VERSION:
           # in the original GUI, the OEM version had no Configure menu
           # and there was a Cedega How-To option under Help.
           # since the Configure stuff (joysticks, fonts)
           # is now in Global Preferences, we'll have to suppress it 
           # there. FIXME

           # Sync Icons is under Tools now, so we disable it.
           # ?? But why is this necessary for OEM? FIXME  -Emily
           p2p_gui.widgets["sync_icons_with_wm_menuitem"].hide()
           

    def setup_for_license(self):
        # License - accepting, denying, etc.
        if not self.Point2Play.config["accept_license"]:
            lic_file = P2PPATH + "LICENSE"
            lic_fp = file( lic_file, "r" )
            lic_diag = license.LicenseDialog( lic_fp )
            resp = lic_diag.run()
            lic_diag.destroy()
            if resp == license.LICENSE_DECLINE:
                raise license.licenseError(_("License not accepted."))
            self.Point2Play.config["accept_license"] = 1
            self.Point2Play.write()
        self.monitor = None
        self.REAPING = 0
        self.config = copy.copy( self.Point2Play.config )

    def setup_pthread_checking(self):
        # Now we need to check this distro to see if he needs pthreads,
        # and if he does need it, that his stack is big enough...
        #
        # otherwise we will popup a warning.. but only once.. no need
        # to harass the poor guy ;)

        pid = os.fork()
        if pid:
            (wpid, retval) = os.waitpid( pid, 0 )
            retval = os.WEXITSTATUS(retval)

            if retval == NO_PTHREADS_NEEDED or retval == PTHREADS_NEEDED_OK:
                # we flip this incase they get an evil distro later...    
                self.Point2Play.config["warned_about_pthreadstacksize"] = 0
                self.Point2Play.write()
            else:
                # Uh-oh.. we're on a distro that needs pthreads and the stacksize
                # is small.. warn the guy. but only once
                if self.Point2Play.config["warned_about_pthreadstacksize"] == 0:
                   self.Point2Play.config["warned_about_pthreadstacksize"] = 1
                   self.Point2Play.write()
                   self.display_pthreads_warning()
        else:
            """We are the child"""
            try:
                pthread_test = P2PPATH + "pthread_test"
                os.execl( pthread_test, ( "pthread_test") )
                sys.exit(1)
            except:
                sys.exit(1)

    def on_run_setup_wizard_menuitem_activate(self, widget, *args):
        self.run_wizard()

    def need_to_run_wizard(self):
        """ Returns True if we need to run the wizard. 
            Change this function to force the wizard to be re-run for new versions of P2P. """

        if self.Point2Play.config["wizard_version_last_run"] == "":   # it was never run
            return True

        # Do any version comparison stuff here 
        return False

    def run_wizard(self):
        setup_wizard = setup_wizard_gui.SetupWizardDialog(self.Point2Play)
        setup_wizard.widgets["wizard"].set_transient_for(p2p_gui.widgets["main"])
        must_quit = False   # true when the user refuses to install an engine

        while 1:
            response=setup_wizard.widgets["wizard"].run()
            # a dialog-box button was clicked - let's check the response.

            # RESPONSE_REJECT means the user Finished or Cancelled without installing an engine
            if response==gtk.RESPONSE_REJECT:
                must_quit = True
                break

            # Cancel or OK - we're done
            if response==gtk.RESPONSE_CANCEL or response==gtk.RESPONSE_OK:
                break

            # any other response means we are paging through the tabs and should continue

        setup_wizard.widgets["wizard"].destroy()

        if must_quit:
            p2p_gui.widgets["main"].destroy()
            sys.exit(0)

        # Refresh the p2p games view...
        p2p_gui.widgets["main_game_folder_view"].set_model( self.get_game_folder_model( 1 ) )


    def setup_menu_and_toolbar(self):
        # Look at the config file and set up the radio buttons appropriately
        # and set up menu initial sensitivity of buttons and menu-items

        # Check the CDROM eject monitoring setting

        p2p_gui.widgets["cdrom_monitoring_always_on_menuitem"].set_active(0)
        p2p_gui.widgets["cdrom_monitoring_always_off_menuitem"].set_active(0)
        p2p_gui.widgets["cdrom_monitoring_default_menuitem"].set_active(0)

        if int(self.Point2Play.config["cdrom_eject_monitoring"]) == Point2Play.CDROMEJECTMONITOR_ON:
           p2p_gui.widgets["cdrom_monitoring_always_on_menuitem"].set_active(1)
        elif int(self.Point2Play.config["cdrom_eject_monitoring"]) == Point2Play.CDROMEJECTMONITOR_OFF:
           p2p_gui.widgets["cdrom_monitoring_always_off_menuitem"].set_active(1)
        else:
           p2p_gui.widgets["cdrom_monitoring_default_menuitem"].set_active(1)

# set up the proper auto wm icon sync checkbox
#        if int(self.Point2Play.config["autowmiconsync"]) == AUTOWMICONSYNC_ON:
#           p2p_gui.widgets["sync_icons_auto_on_menuitem"].set_active(1)
#        elif int(self.Point2Play.config["autowmiconsync"]) == AUTOWMICONSYNC_OFF:
#           p2p_gui.widgets["sync_icons_auto_off_menuitem"].set_active(1)

        # Make play, debug and add icon insensitive at the start
        p2p_gui.widgets["play_menuitem"].set_sensitive(0)
        p2p_gui.widgets["play_button"].set_sensitive(0)
        p2p_gui.widgets["properties_button"].set_sensitive(0)
        p2p_gui.widgets["debug_menuitem"].set_sensitive(0)
        p2p_gui.widgets["add_shortcut_menuitem"].set_sensitive(0)

        wizard_just_run = False

        if self.need_to_run_wizard():
            self.run_wizard()
            wizard_just_run = True

        # Disable Cedega-> Get Latest Version and Cedega -> Get Another Version
        # if the username and password are not configured
        if not len(self.Point2Play.config["password"]) or not len(self.Point2Play.config["username"]):
            if not self.OEM_VERSION:
                p2p_gui.widgets["check_for_updates_menuitem"].set_sensitive(0)
                p2p_gui.widgets["download_another_version_menuitem"].set_sensitive(0)

        self.num_engines_changed() # not really "changed" but good for initial setup

        # These menu items should be disabled until a game is selected
        p2p_gui.widgets["delete_shortcut_menuitem"].set_sensitive( 0 )
        p2p_gui.widgets["delete_game_folder_menuitem"].set_sensitive( 0 )
        p2p_gui.widgets["rename_game_folder_menuitem"].set_sensitive( 0 )
        p2p_gui.widgets["properties_menuitem"].set_sensitive( 0 )

    def setup_views(self):
        self.game_folder_view_model = self.get_game_folder_model(1)
        p2p_gui.widgets["main_game_folder_view"].set_model(self.game_folder_view_model)
        self.shortcut_view_model = gtk.ListStore( gtk.gdk.Pixbuf, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING)
        p2p_gui.widgets["main_shortcut_view"].set_model( self.shortcut_view_model )
        p2p_gui.widgets["main_game_folder_view"].get_selection().connect( 'changed', self.game_folder_select_cb )
        p2p_gui.widgets["main_game_folder_view"].connect( 'button_press_event', self.game_folder_mouseclick_cb )
        p2p_gui.widgets["main_game_folder_view"].get_selection().set_mode( 'single' )
        p2p_gui.widgets["main_shortcut_view"].get_selection().connect( 'changed', self.shortcut_select_cb )
        # Set up a mouse click handler, we want to filter out the crap and get doubleclicks.
        # People like to double click to start their games :)
        p2p_gui.widgets["main_shortcut_view"].connect( 'button_press_event', self.shortcut_mouseclick_cb )

        # Now we attach rendering widgets to the game folder view tree. Note: we aren't showing 
        # the headers so the names of the columns are just here for readability.

        cell = gtk.CellRendererText()
        column = gtk.TreeViewColumn( _("Game Folder"), cell, text=p2p_gui.GAME_FOLDER_NAME_COL)
        p2p_gui.widgets["main_game_folder_view"].append_column( column )

        cell = gtk.CellRendererPixbuf()
        column = gtk.TreeViewColumn( _("Activity Indicator"), cell, pixbuf=p2p_gui.GAME_FOLDER_ACTIVITY_COL )
        p2p_gui.widgets["main_game_folder_view"].append_column( column )

        # And rendering widgets for the shortcut view tree...
        cell = gtk.CellRendererPixbuf()
        column = gtk.TreeViewColumn( _("Icon"), cell, pixbuf=p2p_gui.SHORTCUT_ICON_COL )
        p2p_gui.widgets["main_shortcut_view"].append_column( column )

        cell = gtk.CellRendererText()
        column = gtk.TreeViewColumn( _("Shortcut"), cell, text=p2p_gui.SHORTCUT_NAME_COL)
        p2p_gui.widgets["main_shortcut_view"].append_column( column )

    def setup_logo(self):
        """ Set up the upper-right-hand corner logo and the window manager icon. """

        Logo_file = P2PPATH + "gui/cedega_logo.png"
        p2p_gui.widgets["main_logo_space"].set_from_file( Logo_file )
        p2p_gui.widgets["main_logo_space"].show()

        # Set our window manager icon properly
        SpikePNG="%s%s/tg_logo.png" % (P2PPATH, "gui")
        p2p_gui.widgets["main"].set_icon(gtk.gdk.pixbuf_new_from_file(SpikePNG))


    def setup_cdrom(self):
        self.cdroms = cdrom.getOpticalDrivesList()

        if not len( self.cdroms ):
            p2p_gui.widgets["mount_button"].set_sensitive( 0 )
        else:
            self.CDDialog = mount.CDDialog( self.cdroms, self )


    # Simple callbacks to keep track of which cdrom eject monitoring option has been chosen
    def cdrom_eject_monitor_default_cb( self, widget):
       if widget.active:
          self.Point2Play.config["cdrom_eject_monitoring"] = Point2Play.CDROMEJECTMONITOR_DEFAULT
          self.Point2Play.write()

    def cdrom_eject_monitor_always_on_cb( self, widget):
       if widget.active:
          self.Point2Play.config["cdrom_eject_monitoring"] = Point2Play.CDROMEJECTMONITOR_ON
          self.Point2Play.write()

    def cdrom_eject_monitor_always_off_cb( self, widget):
       if widget.active:
          self.Point2Play.config["cdrom_eject_monitoring"] = Point2Play.CDROMEJECTMONITOR_OFF
          self.Point2Play.write()

#    def autowmiconsync_monitor_cb(self, selected_item, widget):
#       if widget.active:
#          self.Point2Play.config["autowmiconsync"] = selected_item
#          self.Point2Play.write()


    # Manually executed - syncronizes all the shortcuts and game folders as they currently exist with
    # the user's window manager.
    def sync_icons_wmmanager (self, widget, *args):
       # Loop through the list of game folders
       for game_folder in self.Point2Play._games:
          # For each game folder, loop through each shortcut
          for shortcut_entry in self.Point2Play._games[game_folder]:
             os.system(os.path.join(P2PPATH, "p2p_wmicon_helper") + " --gamegroup '%s' --iconname '%s'  --icon '%s'" % (game_folder, shortcut_entry["Name"], shortcut_entry["Icon"]) )
       return


    # This function simply registers a running WineX process (the WinePrefix directory as well as the
    # version information) to fasciliate such things as the new unmounting/eject functionality for
    # multiCD installers in WineX 4.0
    def RegisterWineXRunning (self, winexversion, winexprefix):
       self.WineXRunning.append( { "winexversion" : winexversion,
                                   "winexprefix"  : winexprefix
                                 }
                               )

    # Retrieve the list of running winexes
    def GetWineXRunningList ( self ):
       return self.WineXRunning

    # Reset the list of running Cedegas...
    def ResetWineXRunningList ( self ):
       self.WineXRunning = []


    ##### Visual Cue for Running Games code follows:

    def mark_game_folder_active(self, game_folder):
        """ Show a game folder has just become active."""

        if self.currently_active_game_folders.has_key(game_folder):
            # already active! no need to do anything.
            return

        # Disable the timeout so it doesn't update the GUI while we're in the 
        # middle of starting the game (then it might not notice it's active for another 
        # GAME_FOLDER_ACTIVITY_UPDATE_INTERVAL)

        gtk.timeout_remove(self.game_folder_activity_timeout_retval)
        self.currently_active_game_folders[game_folder] = True

        self.update_game_activity_markers()

        # start the timer again
        self.game_folder_activity_timeout_retval = gtk.timeout_add( GAME_FOLDER_ACTIVITY_UPDATE_INTERVAL, self.update_game_folder_activity )

    def stop_games_by_game_folder_cb(self, widget, *args):
        """ Kill the processes associated with the given game folder
            and update the GUI. """

        # disable the timeout so it doesn't update the GUI while we're in the 
        # middle of this
        gtk.timeout_remove(self.game_folder_activity_timeout_retval)

        (game_folder, shortcut_name) = self.get_selected_names()
        self.kill_wine_procs_helper(game_folder)        

        try:
            del self.currently_active_game_folders[game_folder]
        except:
            pass # it should have been in there, but oh well.

        # update the GUI without checking if the process is completely dead yet,
        # so we can have immediate feedback
        self.update_game_activity_markers()

        # start the timer again
        self.game_folder_activity_timeout_retval = gtk.timeout_add( GAME_FOLDER_ACTIVITY_UPDATE_INTERVAL, self.update_game_folder_activity )


    def update_game_activity_markers(self):
        """ This function is called when there is a change in game folder activity:
            a game might have been started or killed, whether manually
            or from the console.

            1. Save the current selection in the game folder view and refresh it
               to display the active icons.
            2. Make the logo move or stand still accordingly.

        """

        log.logger.info("In update_game_activity_markers...")

        # save the path in the left-hand pane so we can restore the 
        # selection after we've updated the view
        sel = True
        try:
            game_folder_sel = p2p_gui.widgets["main_game_folder_view"].get_selection().get_selected()
            game_folder_model, game_folder_iter = game_folder_sel
            game_folder_path = game_folder_model.get_path(game_folder_iter)
        except:
            sel = False

        p2p_gui.widgets["main_game_folder_view"].set_model( self.get_game_folder_model(1) )

        # restore previous game folder selection if there was one
        if sel:
            game_folder_sel = p2p_gui.widgets["main_game_folder_view"].get_selection()
            game_folder_sel.select_path(game_folder_path)

        # see if we need to get the logo moving or not
        if self.currently_active_game_folders:  # anything in there?
            Logo_animation_file = P2PPATH + "gui/cedega_logo_animation.gif"
            animation = gtk.gdk.PixbufAnimation(Logo_animation_file)
            p2p_gui.widgets["main_logo_space"].set_from_animation(animation)
        else:
            # make it stand still
            Logo_file = P2PPATH + "gui/cedega_logo.png"
            p2p_gui.widgets["main_logo_space"].set_from_file( Logo_file )
#        p2p_gui.widgets["main_logo_space"].show()


    def update_game_folder_activity(self):
        """ 
        If something has changed (we notice a game has started or stopped somehow
        other than an explicit start or kill in the GUI), we will
        update the activity icons in the second column of the
        game folder listview in the main window. This function is
        run periodically by the GUI every GAME_FOLDER_ACTIVITY_UPDATE_INTERVAL 
        milliseconds. We only want to refresh the GUI if it is necessary,
        so we keep a copy of the last activity list.

        We determine whether a game folder has active processes
        by searching for the "socket" file in the game folder's wineserver-*
        directory, which is of this form:
                wineserver-[server-name]-[user-name])

        """

        log.logger.info("In update_game_folder_activity...")
        # check to see if anything has changed

        self.formerly_active_game_folders = self.currently_active_game_folders
        self.currently_active_game_folders = {}
        for game_folder in self.Point2Play.games():  # check each game folder
            if glob.glob(self.Point2Play.gamedir + game_folder + '/wineserver-*/socket'):
                log.logger.info(game_folder + " is active.")
                self.currently_active_game_folders[game_folder] = True

        if self.currently_active_game_folders == self.formerly_active_game_folders:
            log.logger.info("No change in activity.")
            return True

        self.update_game_activity_markers()

        # And to make sure the function keeps getting called every GAME_FOLDER_ACTIVITY_UPDATE_INTERVAL:
        return True

    # Display the Pthreads warning dialog for distros without enough stack
    def display_pthreads_warning(self):
        alert = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("You are running a distribution of Linux on which %s requires the usage of pthreads on (ie. very recent glibc)\n\nUnfortunately the maximum stack size on your distribution is not large enough for some games, and therefore you might have issues.\n\nThis is due to the way these games were programmed and a limitation with your distribution, not a limitation of %s technology.\n\nWhen using WineX/%s 3.2 or newer, you may not need to use pthreads which\ncan help avoid these problems. Earlier versions of WineX, however,\nmust have the pthreads option used.\n\nPlease refer to the Cedega release notes for more details.") % (Point2Play.WINEX_DISPLAY_TITLE, Point2Play.WINEX_DISPLAY_TITLE, Point2Play.WINEX_DISPLAY_TITLE ) )
        alert.run()
        alert.destroy()

    # Display the release notes via URL...
    def release_notes_cb(self, widget, *args):
        self.go_to_url("http://www.transgaming.com/cedega_release_notes")

    # The How-To will now be presented via URL...
    def p2p_howto_cb(self, widget, *args):
        self.go_to_url("http://www.transgaming.com/p2p_howto")

    # Display the Cedega HowTo Guide in a popup; will have to do until
    # we get a real help system
    def cedega_howto_cb(self, widget, *args):
       try:
           Bg_file = os.environ["P2PPATH"] + _("/WineX_howto.txt")
       except:
           Bg_file = _("WineX_howto.txt")
       Bg_fp = file( Bg_file, "r" )
       Bg_diag = beginners_guide.BeginnersGuideDialog( Bg_fp, _("HowTo") )
       resp = Bg_diag.run()
       Bg_diag.destroy()

    def make_support_request_cb( self, widget, *args ):
        self.go_to_url("http://www.transgaming.com/support.php")

    def vote_cb( self, widget, *args):
        self.go_to_url("http://www.transgaming.com/pollslist.php")

    def gamesDB_cb(self, widget, *args):
        self.go_to_url("http://transgaming.org/gamesdb/")

    def go_to_url ( self, url ):
        """ Visit the given URL. If a browser is specified by the Global Settings, use it."""

        misc_utils.go_to_url(url, self.Point2Play, p2p_gui)
        self.setBusyCursorBriefly() # some feedback to let the user know something's happening

    # This button pops up a webbrowser and takes us to a certain URL (only used for
    # OEM builds, the url is sucked in from $P2PPATH/TRANSGAMER_PAGE
    def go_to_tg_url(self, widget):
        self.go_to_url( self.OEM_INFOURL )
        return

    def setBusyCursorBriefly(self):
        """ Set the cursor to the busy cursor and set a timer for 20 seconds. Used in go_to_url()
            This is to show some feedback with the cursor since launching a browser can take
            a few seconds. We can't actually tell when to turn it back to the normal cursor so
            we show busy for a short amount of time in the hopes that it will let users know
            that *something* is happening. """

        c = gtk.gdk.Cursor(gtk.gdk.WATCH)
        try:
            p2p_gui.widgets["main"].window.set_cursor(c)
        except:
            pass
        self.timer = gtk.timeout_add (BROWSER_LAUNCH_BUSY_TIME, self.busycursor_timeout)

    def busycursor_timeout(self):
        """ Time's up so revert cursor back to normal. """
        c = gtk.gdk.Cursor(gtk.gdk.LEFT_PTR)
        p2p_gui.widgets["main"].window.set_cursor(c)
        return False  # time's up      

    def install_older_engine_cb (self, widget):
        """ Let the user pick an older engine to install. 
            Pops up a list of all existing versions. """

        install_cedega_engine_helper = install_cedega_engine.InstallCedegaEngineHelper(self.Point2Play, self)
        install_cedega_engine_helper.install_cedega_engine( InstallMode=install_cedega_engine.INSTALL_DOWNLOAD_PREVIOUS_VERSION )
        self.num_engines_changed()


    def setup_tg_acct_cb( self, widget ):
        """ Edit TransGamer account information. """

        tg_account_dialog = tgaccount_gui.TGAcctDialog( self.Point2Play )
        tg_account_dialog.widgets["transgamer_account"].set_transient_for(p2p_gui.widgets["main"] )
        tg_account_dialog.widgets["transgamer_account"].run()

        # Disable "Get Latest WineX" and "Install Other WineX Version" buttons if
        # the username and password are not configured, otherwise enable!
        if not self.OEM_VERSION:
            if not len(self.Point2Play.config["password"]) or not len(self.Point2Play.config["username"]):
               p2p_gui.widgets["check_for_updates_menuitem"].set_sensitive(0)
               p2p_gui.widgets["download_another_version_menuitem"].set_sensitive(0)
            else:
               p2p_gui.widgets["check_for_updates_menuitem"].set_sensitive(1)
               p2p_gui.widgets["download_another_version_menuitem"].set_sensitive(1)

        tg_account_dialog.widgets["transgamer_account"].destroy()

    def check_for_updates_cb( self, widget, *args ):
        """ Checks for what packages need updating """
        p2p_gui.check_for_updates(self, 0) # 0 here means we're not calling from the wizard

    def allGamesRowIsSelected(self):
        """ Returns true if the row in the left-hand pane marked All Games is selected. """
        sel_model, sel_iter = p2p_gui.widgets["main_game_folder_view"].get_selection().get_selected()
        if sel_iter:
            game_folder = sel_model.get_value( sel_iter, p2p_gui.GAME_FOLDER_NAME_COL )
            if game_folder == p2p_gui.ALL_GAMES_STRING:
                return True
        return False


    def get_selected_names(self):
        """ Returns a list of the names selected in the left and right panes 
            of the main window: game folder name and shortcut name.

            Important note: if we've selected a shortcut while "All Games" is selected,
            the game folder returned will be the shortcut's actual game folder, but
            if no shortcut is selected, the "All Games" string will be returned as the
            game folder name.

            Note: returns None for any/each item if there is no selection.
            Previously known as get_prog_game_names(self) 
              (which returned the values in reversed order)
        """

        shortcut_name = None
        game_folder = None

        # look for a selection in the right-hand pane
        shortcut_sel = p2p_gui.widgets["main_shortcut_view"].get_selection().get_selected()
        if shortcut_sel:
            shortcut_model, shortcut_iter = shortcut_sel
            if shortcut_iter:
                shortcut_name = shortcut_model.get_value( shortcut_iter, p2p_gui.SHORTCUT_NAME_COL )

                # getting the game folder name from the shortcut_view itself ensures it is the actual
                # game folder, not the ALL GAMES string.
                game_folder = shortcut_model.get_value (shortcut_iter, p2p_gui.SHORTCUT_GAME_FOLDER_COL )
                log.logger.info("Selected shortcut name is " + shortcut_name + " within (actual) game folder " + game_folder)
                return (game_folder, shortcut_name)

        # if we got here, no shortcut was selected
        game_folder_sel = p2p_gui.widgets["main_game_folder_view"].get_selection().get_selected()
        if game_folder_sel:
            game_folder_model, game_folder_iter = game_folder_sel
            if game_folder_iter:                
                game_folder = game_folder_model.get_value( game_folder_iter, p2p_gui.GAME_FOLDER_NAME_COL )
        return (game_folder, None)
       

    def play_game (self, debug_channel=None, output_file=None, output_console=None, run_in_gdb=None):
        """ Helper function for the play button/menuitem and debug menuitem. """

        (game_folder, shortcut_name) = self.get_selected_names()
        if self.MONITOR_CEDEGA_PROCESSES:
            self.mark_game_folder_active(game_folder)
        shortcut_properties = self.Point2Play.get_shortcut_properties(game_folder, shortcut_name)
        if shortcut_properties:
            if shortcut_properties["CedegaApp"] == "Y":
               try:
                  retval = self.Point2Play.winex( 0, game_folder, shortcut_properties["Path"], shortcut_properties["Workdir"], cmdlineoverride=shortcut_properties["CommandLineArguments"], debug_channel=debug_channel, output_file=output_file, output_console=output_console, run_in_gdb=run_in_gdb, OverrideGameProfile=None, shortcut_name=shortcut_name)
               except Point2Play.Point2PlayError, e:
                  err_diag = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, e.__str__() )
                  err_diag.run()
                  err_diag.destroy()
                  return
               if not self.REAPING:
                  self.REAPING = 1
                  gtk.timeout_add( 1000, self.reap_children )

            else:
               # not a cedega app
               # is it a URL?
               if misc_utils.is_URL(shortcut_properties["Path"]):
                   self.go_to_url(shortcut_properties["Path"])
               else:
                   if shortcut_properties["Workdir"]:
                       try:
                           os.chdir(shortcut_properties["Workdir"])
                       except:
                           err_diag = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("Unable to change to directory %s.") % ( shortcut_properties["Workdir"] ) )
                           err_diag.run()
                           err_diag.destroy()
                           return

                   if shortcut_properties["CommandLineArguments"]:
                       # construct a list for the arguments - split on whitespace
                       whitespace_regexp = re.compile(r'\s+')
                       arglist = whitespace_regexp.split(shortcut_properties["CommandLineArguments"])
                       arglist.insert(0, shortcut_properties["Path"])  # the arglist must have the command as its first item
                   else:
                       arglist = [ shortcut_properties["Path"] ]
                   log.logger.info(arglist)
                   try:
                       os.spawnve(os.P_NOWAIT, shortcut_properties["Path"], arglist, os.environ)
                   except:
                       err_diag = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("Unable to run command %s %s.") % ( shortcut_properties["Path"], shortcut_properties["CommandLineArguments"] ) )
                       err_diag.run()
                       err_diag.destroy()
                       return            

    def play_cb( self, widget, *args ):
        """ Play button/menuitem callback. """
        self.play_game()

    def debug_cb (self, widget, *args ):
        """ Pops up the debug dialog box, to allow the user to choose what 
            debug messages, etc. they want to use for this run of the game. """

        debug_diag = debug_gui.DebugProgramDialog(self.Point2Play)
        debug_diag.widgets["debug_dialog"].set_transient_for(p2p_gui.widgets["main"])
        response = debug_diag.run()
        debug_diag.widgets["debug_dialog"].destroy()

        if response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_DELETE_EVENT:
            return

        self.play_game(self.Point2Play.config["debugger_last_channel"], self.Point2Play.get_debugger_output_filename(), self.Point2Play.config["debugger_output_to_console"])

    def reap_children( self, *args ):
        while 1:
            try:
                ret = os.waitpid( 0, os.WNOHANG )

                if self.renice_wineserver == 1:
                   contents = os.popen( "ps -C wineserver -o pid --noheaders" )
                   pids = contents.readlines()
                   contents.close()


                   if pids:
                      for pid in pids:
                         os.system ( "renice 10 -p " + str(pid))

                      self.renice_wineserver = 0

                if ret[0] == 0:
                    gtk.timeout_add( 1000, self.reap_children )
                    return

            except:
                # Install or game completed..
                self.REAPING = 0

                # No more renicing if it hasn't happened yet...
                self.renice_wineserver = 0

                return

    def install_menu_cb( self, widget, *args ):
        """ Callback to install a game. """
        log.logger.info("Default winex version is: " + self.Point2Play.default_winex)

        (game_folder, shortcut_name) = self.get_selected_names()
        # see if we can pre-populate the entry for game folder
        install_dialog = new_install_gui.InstallDialog(self.Point2Play, self, game_folder)
        install_dialog.widgets["install"].set_transient_for(p2p_gui.widgets["main"])
        ( install, game_folder, dos_cwd, big_exe, cd_eject_monitoring, winexver_touse, custom_installer_config, debug_channel, output_file, output_console, run_in_gdb, gddb_file ) = install_dialog.run()
        
        if gddb_file != None and gddb_file.rfind( "/installer" ) != -1:
            self.Point2Play.last_gddb = os.path.basename(gddb_file[ :gddb_file.rfind("/installer") ])
        install_dialog.widgets["install"].destroy()

        if not game_folder:
           # Bail out... no installing going on here...
           return

        log.logger.info ("Got install: " + install + " and game: " + game_folder + " and winexver_touse: " + winexver_touse)
           
        # Only go on if the game already exists
        if self.Point2Play._games and self.Point2Play._games.get(game_folder):
            path = self.Point2Play.gamedir + game_folder + "/"
            exe = self.Point2Play.lookup_game_exe(game_folder, shortcut_name)

        # Open up the custom_installer_config file so that it's open for "option" access...then send it into the
        # function and wait for the correct setting to pop out...
        load_list = self.Point2Play.get_profile_load_list( None, None, install_gddb=gddb_file )
        load_list.append( self.Point2Play.USER_DEFINED_PROFILES_PATH + custom_installer_config )
        installer_config_conf = config.config_file( load_list )
        pthreads_option = self.Point2Play.get_pthread_setting(installer_config_conf)
        
        while gtk.events_pending():
            gtk.main_iteration()
        if install and len( install ) > 0 and game_folder and len( game_folder ) > 0:
            retval = self.Point2Play.winex( 1, game_folder, install, None, self.Point2Play.gamedir + game_folder + "/" + "games.ini.new", winexver_touse, ( dos_cwd, big_exe, pthreads_option ), ModifiedInstallConfigs=custom_installer_config, debug_channel=debug_channel, output_file=output_file, output_console=output_console, run_in_gdb=run_in_gdb, MonitorCdromEject=cd_eject_monitoring, gddb_file=gddb_file  )
            if retval:
                # Add new program group and update p2p so it shows up (so that you can delete a
                # game dir created by a crashed installer)
                self.Point2Play.create_new_game_folder( game_folder )
                # Refresh the game view in P2P
                p2p_gui.widgets["main_game_folder_view"].set_model( self.get_game_folder_model( 1 ) )
                if not self.REAPING:
                    self.REAPING = 1
                    gtk.timeout_add( 1000, self.reap_children )
                self.monitor_game_install( game_folder )


    def rename_game_folder_cb( self, widget, *args ):
        """ Rename a game folder. """
        (game_folder, shortcut_name) = self.get_selected_names()
        if game_folder: # we should have one, or we don't really have a way to get here
            diag = RenameGameFolderDialog( self, game_folder )
            resp = diag.run()
            new_game_folder = diag.text_entry.get_text()
            diag.destroy()
            if resp == GAME_RENAMED:
                self.Point2Play.write_shortcut_entries( new_game_folder )
                p2p_gui.widgets["main_game_folder_view"].set_model( self.get_game_folder_model( 1 ) )
                shortcut_model = gtk.ListStore( gtk.gdk.Pixbuf, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING )
                p2p_gui.widgets["main_shortcut_view"].set_model( shortcut_model )

    def remove_game_folder_cb( self, widget, *args ):
        """ Remove the selected game folder. """
        (game_folder, shortcut_name) = self.get_selected_names()
        if not game_folder:
            return

        fullpath = self.Point2Play.P2PFOLDER + game_folder
        follow_symlink = False

        # Ask for confirmation
        if os.path.islink(fullpath):
            actualpath = os.readlink(fullpath)
            # ask the user if he/she wants to delete the actual game or just the symlink.
            confirm_dialog = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_WARNING, gtk.BUTTONS_YES_NO, _("The game folder %s is a symbolic link to the folder %s. Do you wish to delete the actual game as well (click Cancel for no action)?") % (game_folder, actualpath)  )
            # There's no such thing as gtk.BUTTONS_YES_NO_CANCEL, so I'll just add a Cancel myself...
            confirm_dialog.add_button(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL)
            response = confirm_dialog.run()
            confirm_dialog.destroy()
            if response == gtk.RESPONSE_CANCEL:
                return
            if response == gtk.RESPONSE_YES:
                follow_symlink = True
        else:
            confirm_dialog = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], gtk.DIALOG_DESTROY_WITH_PARENT, gtk.MESSAGE_WARNING, gtk.BUTTONS_YES_NO, _("About to delete %s\nAre you sure?") % game_folder  )
            response = confirm_dialog.run()
            confirm_dialog.destroy()
            if response == gtk.RESPONSE_NO:
                return
        try:
            self.Point2Play.remove_game_folder( game_folder, follow_symlink )
        except:
            err_diag = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, str( sys.exc_info()[1]))
            err_diag.run()
            err_diag.destroy()
            return

        self.Point2Play.write_shortcut_entries( game_folder )
        p2p_gui.widgets["main_game_folder_view"].set_model( self.get_game_folder_model( 1 ) )
        p2p_gui.widgets["main_shortcut_view"].set_model( gtk.ListStore( gtk.gdk.Pixbuf, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING ) )

        # that could have been the last game folder we had, in which case we need to grey out a bunch of things.
        if not len( self.Point2Play.games() ):
            p2p_gui.widgets["play_button"].set_sensitive( 0 )
            p2p_gui.widgets["properties_button"].set_sensitive( 0 )
            p2p_gui.widgets["debug_menuitem"].set_sensitive( 0 )
            p2p_gui.widgets["play_menuitem"].set_sensitive( 0 )
            p2p_gui.widgets["delete_shortcut_menuitem"].set_sensitive( 0 )
            p2p_gui.widgets["delete_game_folder_menuitem"].set_sensitive( 0 )
            p2p_gui.widgets["rename_game_folder_menuitem"].set_sensitive( 0 )
            p2p_gui.widgets["properties_menuitem"].set_sensitive( 0 )
            p2p_gui.widgets["add_shortcut_menuitem"].set_sensitive( 0 )

    def mount_cb( self, *args ):
        if self.CDDialog:
            if self.CDDialog.get_cdrom_count() > 1:
               self.CDDialog.show()
            else:
               self.CDDialog.mount_single_drive()
        else:
            self.CDDialog = mount.CDDialog( self.cdroms, self )
            if self.CDDialog.get_cdrom_count() == 1:
               self.CDDialog.mount_single_drive()


    def remove_shortcut_cb( self, selection, *args ):
        """ Callback to remove a shortcut (formerly known as "icon"). """

        (game_folder, shortcut_name) = self.get_selected_names()
        if not game_folder or not shortcut_name:
            return  # shouldn't happen, but we'll check anyway

        yes_no_diag = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_YES_NO, _("Are you sure you want to delete %s?") % shortcut_name )
        resp = yes_no_diag.run()
        yes_no_diag.destroy()
        if resp == gtk.RESPONSE_YES:
            # save the path in the left-hand pane so we can restore the 
            # selection after we've updated the views
            game_folder_sel = p2p_gui.widgets["main_game_folder_view"].get_selection().get_selected()
            game_folder_model, game_folder_iter = game_folder_sel
            game_folder_path = game_folder_model.get_path(game_folder_iter)

            try:
                self.Point2Play.remove_shortcut( game_folder, shortcut_name )
            except:
                err_diag = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("Unable to remove %s from %s") % (shortcut_name, game_folder) + " (%s)." % str( sys.exc_info()[1] ) )
                err_diag.run()
                err_diag.destroy()
            else: # success
                self.Point2Play.write_shortcut_entries( game_folder )

                # At this point, game_folder is the name of the actual game folder that
                # contained the shortcut. However, the user could have been deleting that 
                # shortcut while viewing All Games. We want to restore the
                # previous selection of the left-hand pane, so this next step will 
                # return the same game folder as before or the All Games row.
                game_folder = game_folder_model.get_value( game_folder_iter, p2p_gui.GAME_FOLDER_NAME_COL )

                p2p_gui.widgets["main_shortcut_view"].set_model( self.get_shortcut_model(game_folder) )
                p2p_gui.widgets["properties_button"].set_sensitive( 0 )
                p2p_gui.widgets["play_button"].set_sensitive( 0 )

                # restore previous game folder selection
                game_folder_sel = p2p_gui.widgets["main_game_folder_view"].get_selection()
                game_folder_sel.select_path(game_folder_path)

    def add_game_folder_title_cb( self, widget, *args):
        """ Add a game folder to the left pane of the main window.
            Creates a blank game folder - basically just creates a directory under
            the user's .cedega directory. This does not install anything. People
            can use it to group non-cedega games or whatever they want to launch. """

        dlg = entry_dialog.EntryDialog(_("Create Game Folder"), _("Please enter the new game folder name:"))
        dlg.widgets["entry_dialog"].set_transient_for(p2p_gui.widgets["main"])
        response = dlg.widgets["entry_dialog"].run()
        new_title = ""
        if response == gtk.RESPONSE_OK:
            new_title = dlg.widgets["entry"].get_text()
        dlg.widgets["entry_dialog"].destroy()
        if new_title.find('/') >= 0 or new_title.find('\\') >= 0:
            err_diag = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("Sorry, game folders may not have slashes in their names (either forward or backward)."))
            err_diag.run()
            err_diag.destroy()
            return

        if new_title != "":
            new_directory = self.Point2Play.P2PFOLDER + new_title
            if os.access( new_directory, os.F_OK ):
                # that directory already exists - complain.
                err_diag = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("Unable to create new program title %s. It already exists.") % ( new_title ) )
                err_diag.run()
                err_diag.destroy()
                return
            # if we got here, run the winex3 script so that it creates 
            # all the necessary files and proper directory hierarchy in the new directory
            # This takes a few seconds so we need to wait on it
            p2p_gui.widgets["main"].window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))

            install_profile =  self.Point2Play.create_install_profile()
            try:
                retval = self.Point2Play.winex( 1, new_title, "", None, self.Point2Play.gamedir + new_title + "/" + "games.ini.new", self.Point2Play.default_winex, OverrideGameProfile=install_profile)
            except Point2Play.Point2PlayError, e:
                err_diag = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, e.__str__() )
                err_diag.run()
                err_diag.destroy()
                # restore normal cursor and delete the install profile
                p2p_gui.widgets["main"].window.set_cursor(gtk.gdk.Cursor(gtk.gdk.LEFT_PTR))
                self.Point2Play.delete_install_profile()
                return

            self.Point2Play.delete_install_profile()

            if retval:
                # We should have a pid now to wait on.
                pid = retval
                (p, v) = os.waitpid(pid, os.WNOHANG)
                while p == 0 and v == 0:
                    (p, v) = os.waitpid(pid, os.WNOHANG)
                    while gtk.events_pending():
                        gtk.main_iteration()  

            # Refresh the left window pane now that we're done
            self.Point2Play.create_new_game_folder( new_title )
            model = self.get_game_folder_model(1)
            p2p_gui.widgets["main_game_folder_view"].set_model( model )

            # the new entry was added alphabetically - loop through the rows
            # to find it and select it. 
            model.foreach(misc_utils.select_string_in_left_pane, new_title)

            # restore normal cursor
            p2p_gui.widgets["main"].window.set_cursor(gtk.gdk.Cursor(gtk.gdk.LEFT_PTR))



    def add_shortcut_cb( self, widget, *args ):
        """ Create a new shortcut. """

        # save the selected game folder so we can restore selection later
        game_folder_sel = p2p_gui.widgets["main_game_folder_view"].get_selection().get_selected()
        if game_folder_sel:
            game_folder_model, game_folder_iter = game_folder_sel
            game_folder = game_folder_model.get_value( game_folder_iter, p2p_gui.GAME_FOLDER_NAME_COL )
            # save the selection so we can restore it after the view is updated
            game_folder_path = game_folder_model.get_path(game_folder_iter)

            properties_dialog = shortcut_properties_gui.ShortcutPropertiesDialog(self.Point2Play, self, -1)  # -1 means we're creating a new one
            properties_dialog.widgets["shortcut_properties"].set_transient_for(p2p_gui.widgets["main"])
            add = properties_dialog.run()
            properties_dialog.widgets["shortcut_properties"].destroy()
            if add:
                try: 
                    retval = self.Point2Play.create_new_shortcut( game_folder, add["Name"], add["Path"], add["Icon"], add["Workdir"], add["CommandLineArguments"], add["CedegaApp"], add["gddb"] ) 
                except:
                    err_diag = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("Unable to add %s to %s (%s).") % ( add["Name"], game_folder, str( sys.exc_info()[1] ) ) )
                    err_diag.run()
                    err_diag.destroy()
                    return
                if retval:
                    self.Point2Play.write_shortcut_entries( game_folder )
                    p2p_gui.widgets["main_game_folder_view"].set_model( self.get_game_folder_model( game_folder ) )
                    shortcut_model = self.get_shortcut_model(game_folder)
                    p2p_gui.widgets["main_shortcut_view"].set_model( shortcut_model )

                    # associate the exe with the profile
                    exe = os.path.basename( add["Path"] )
                    self.Point2Play.associate_exe_and_profile(exe, add["DirName"], add["Profile"])                    
                    self.Point2Play.write()
                    # restore the previously-selected game folder
                    new_game_folder_sel = p2p_gui.widgets["main_game_folder_view"].get_selection()
                    new_game_folder_sel.select_path(game_folder_path)

                    # select the new shortcut in the right pane
                    shortcut_model.foreach(misc_utils.select_string_in_right_pane, add["Name"])
                else:
                    log.logger.warning("Could not create a new shortcut - something may have corrupted your ~/.cedega/ directory.")

    def is_wine_process_running_in_game_folder(self, game_folder, cmds):
        """ Given an array "cmds" containing a wine-related command and its separated-out
            command-line arguments, check to see if the command has anything 
            at all to do with the specified game folder. For example, the game folder could
            be "My Very Own Max Payne" but the argument might not contain that; it might
            be "C:/Program Files/Max Payne Demo/MaxPayne'sDemo.exe".
            So to do this properly we need to look in the self.games() info for the Path
            entity. """

        for arg in cmds:
            for shortcut in self.Point2Play._games[game_folder]:
                log.logger.info("Comparing against " + shortcut["Path"])
                if arg == shortcut["Path"]:
                    return True
        return False


    def kill_wine_procs_helper(self, game_folder = None):
        """ Kill all wine processes owned by this user. If a game folder is specified,
            kill only the wine processes running in that folder. """

        proc_list = os.listdir("/proc")

        procs_to_kill = []

        for proc in proc_list:
            try:
                proc_num = int(proc)
                # It's a number, ie. a process
                try:
                    cmdline = file("/proc/" + proc + "/cmdline")
                    cmds = cmdline.read().split("\0")
                    if os.path.basename(cmds[0]) == "wine" or os.path.basename( cmds[0] ) == "wineserver" or os.path.basename( cmds[0] ) == "wine_relocated" or os.path.basename( cmds[0] ) == "winedbg" or os.path.basename( cmds[0] ) == "wineconsole" or os.path.basename(cmds[0]) == "wine-preloader":
                        if game_folder:
                            if self.is_wine_process_running_in_game_folder(game_folder, cmds):
                                procs_to_kill.append( proc_num ) 
                        else:
                            procs_to_kill.append( proc_num )
                except:
                    pass
            except:
                pass

        for proc in procs_to_kill:
            try:
                os.kill( proc, signal.SIGTERM )
            except:
                pass
        gtk.timeout_add( 5, kill_wine_timeout, procs_to_kill )


    def kill_wine_cb( self, widget, *args ):

        log.logger.info("In kill_wine_cb")

        if self.MONITOR_CEDEGA_PROCESSES:
            # disable the timeout so it doesn't update the GUI while we're in the 
            # middle of this
            gtk.timeout_remove(self.game_folder_activity_timeout_retval)

        self.kill_wine_procs_helper()

        if self.MONITOR_CEDEGA_PROCESSES:
            self.currently_active_game_folders = {}  # nothing should be active now
            # give immediate feedback
            self.update_game_activity_markers()

            # restart timout
            self.game_folder_activity_timeout_retval = gtk.timeout_add( GAME_FOLDER_ACTIVITY_UPDATE_INTERVAL, self.update_game_folder_activity )

    def about_menu_cb( self, widget, *args ):
        dlg = about_gui.AboutDialog()
        dlg.widgets["about"].set_transient_for(p2p_gui.widgets["main"])
        dlg.widgets["about"].run()
        dlg.widgets["about"].destroy()

    def contact_menu_cb( self, widget, *args ):
        dialog = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"],
                                    gtk.DIALOG_DESTROY_WITH_PARENT,
                                    p2p_diag.MESSAGE_TG_LOGO,
                                    gtk.BUTTONS_OK,
                                    _("Contact: info@transgaming.com") )
        dialog.run()
        dialog.destroy()

    def support_menu_cb( self, widget, *args ):
        dialog = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"],
                                    gtk.DIALOG_DESTROY_WITH_PARENT,
                                    p2p_diag.MESSAGE_TG_LOGO,
                                    gtk.BUTTONS_OK,
                                    _("Support: support@transgaming.com") )
        dialog.run()
        dialog.destroy()

    def game_folder_select_cb( self, selection ):
        """ Called when a game folder is selected. """

        sel = selection.get_selected()
        if sel:
            game_folder_ops_sensitive = 0
            model, iter = sel
            if iter:
                game_folder = model.get_value( iter, p2p_gui.GAME_FOLDER_NAME_COL )
                model = self.get_shortcut_model( game_folder )
                if game_folder != p2p_gui.ALL_GAMES_STRING:
                    game_folder_ops_sensitive = 1

                # If this is a symlink and the user deleted the actual game folder that this was
                # pointing to in the same session, it will still be in the left pane, but we'll have
                # problems. This is the best place to catch it. Note: dangling symlinks are not
                # shown at startup.
                fullpath = self.Point2Play.P2PFOLDER + game_folder
                if os.path.islink(fullpath):
                    actual_folder = os.readlink(fullpath)
                    if actual_folder[0:1] != "/":
                        actual_folder = self.Point2Play.gamedir + actual_folder
                    if not os.access(actual_folder, os.F_OK):
                        err_diag = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_WARNING, gtk.BUTTONS_YES_NO, _("This game folder is a symbolic link whose target no longer exists. Is it OK to delete it?") )
                        response = err_diag.run()
                        err_diag.destroy()
                        if response == gtk.RESPONSE_YES:
                            self.Point2Play.remove_game_folder(game_folder, follow_symlink=False)
                            self.Point2Play.write_shortcut_entries( game_folder )
                            p2p_gui.widgets["main_game_folder_view"].set_model( self.get_game_folder_model( 1 ) )
                        return # if they say No, still don't show the shortcuts.
            else:
                model = gtk.ListStore( gtk.gdk.Pixbuf, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING )

            p2p_gui.widgets["main_shortcut_view"].set_model( model )
            p2p_gui.widgets["play_button"].set_sensitive( 0 )
            p2p_gui.widgets["properties_button"].set_sensitive( 0 )
            p2p_gui.widgets["debug_menuitem"].set_sensitive( 0 )
#            p2p_gui.widgets["configure_button"].set_sensitive( 0 )
            p2p_gui.widgets["delete_game_folder_menuitem"].set_sensitive( game_folder_ops_sensitive )
            p2p_gui.widgets["rename_game_folder_menuitem"].set_sensitive( game_folder_ops_sensitive )
            p2p_gui.widgets["properties_menuitem"].set_sensitive( 0 )
            p2p_gui.widgets["add_shortcut_menuitem"].set_sensitive( game_folder_ops_sensitive )
            p2p_gui.widgets["gamenotes_notebook"].hide()



    def shortcut_select_cb( self, selection ):
        """ Called when a shortcut is selected. """
      
        (game_folder, shortcut_name) = self.get_selected_names()
        if shortcut_name != None and game_folder != None:
            exe = self.Point2Play.lookup_game_exe(game_folder, shortcut_name)
            gddb_file = self.Point2Play.lookup_gddb_by_proggrp_shortcut( game_folder, shortcut_name )
            all_conf_paths = self.Point2Play.get_profile_load_list( game_folder, shortcut_name )
            all_conf = config.config_file( all_conf_paths )
            if gddb_file != None:
                notes = self.Point2Play.gddb_notes( game_folder, shortcut_name )
                self.documents[ "general" ].set_text( notes[2] )
                self.documents[ "run" ].set_text( notes[0] )
                self.documents[ "all" ].set_text( notes[3] )
                self.documents[ "install" ].set_text( notes[1] )
                p2p_gui.widgets[ "gamenotes_notebook" ].show_all()
                p2p_gui.widgets[ "gamenotes_notebook" ].set_current_page(gddb.PLAY_PAGE)
            else:
                p2p_gui.widgets[ "gamenotes_notebook" ].hide()

            if len( self.Point2Play.versions ):
                play_sensitive = 1
            else:
                play_sensitive = 0
            sensitive = 1
        else:
            play_sensitive = 0
            sensitive = 0

        p2p_gui.widgets["play_button"].set_sensitive( play_sensitive )
        p2p_gui.widgets["debug_menuitem"].set_sensitive( play_sensitive )
        p2p_gui.widgets["properties_button"].set_sensitive( sensitive )
        p2p_gui.widgets["delete_shortcut_menuitem"].set_sensitive( sensitive )
        p2p_gui.widgets["properties_menuitem"].set_sensitive( sensitive )
        p2p_gui.widgets["add_shortcut_menuitem"].set_sensitive( sensitive )
        p2p_gui.widgets["play_menuitem"].set_sensitive( play_sensitive )


    def game_folder_mouseclick_cb ( self, TreeView, MouseEvent ):
       """ Unselects anything on the right pane when you click on something in the left.
           Sets up a right-click context menu for the selected game folder (unless it is
           the All Games string. """

       if MouseEvent.button == 1:
          # unselect anything in the right pane (individual game icons)
          p2p_gui.widgets["main_shortcut_view"].get_selection().unselect_all()
          # fake a changed event on left pane (game group)
          self.game_folder_select_cb(p2p_gui.widgets["main_game_folder_view"].get_selection()) 
       elif MouseEvent.button == 3:
           x = int(MouseEvent.x)
           y = int(MouseEvent.y)
           time = MouseEvent.time
           pthinfo = TreeView.get_path_at_pos(x, y)
           if pthinfo != None:
               path, col, cellx, celly = pthinfo
               TreeView.grab_focus()
               TreeView.set_cursor( path, col, 0)
               self.build_context_menu_for_game_folders(MouseEvent.button, time)
           return 1

    def shortcut_mouseclick_cb ( self, TreeView, MouseEvent ):
        """ Double-click functionality for playing something pointed to by a shortcut.
            Right-click context menu functionality for shortcuts. """

        # is it the primary (left) button?
        if MouseEvent.button == 1:
            # is it a double click?
            if MouseEvent.type == gtk.gdk._2BUTTON_PRESS:
                # woo hoo let the play_cb do its stuff
                self.play_cb( None, None )
        elif MouseEvent.button == 3:
            x = int(MouseEvent.x)
            y = int(MouseEvent.y)
            time = MouseEvent.time
            pthinfo = TreeView.get_path_at_pos(x, y)
            if pthinfo != None:
                path, col, cellx, celly = pthinfo
                TreeView.grab_focus()
                TreeView.set_cursor( path, col, 0)
                self.build_context_menu_for_shortcuts(MouseEvent.button, time)
            return 1

    def edit_overrides_cb(self, button):
        """ Edit the engine overrides for the selected shortcut. """

        (game_folder, shortcut_name) = self.get_selected_names()
        shortcut_properties = self.Point2Play.get_shortcut_properties(game_folder, shortcut_name)
        if shortcut_properties:
            exe = os.path.basename(shortcut_properties["Path"])
            curr_gddb = shortcut_properties["gddb"]
            log.logger.info("Got shortcut name " + shortcut_name + " game folder: " + game_folder)
            log.logger.info("In edit_profile_cb, with current gddb entry of " + str(curr_gddb))
            gddb_path = ""
            if curr_gddb:
                for x in gddb.scan( self.Point2Play.P2PFOLDER + "/.gddb" ):
                    if x[1] == curr_gddb:
                        gddb_path = self.Point2Play.P2PFOLDER + "/.gddb/" + x[1]
                        break

            settings_dialog = options_gui.OptionsDialog(self.Point2Play, p2p_gui.widgets["main"], exe, game_folder, exe, shortcut_name, gddb_path=gddb_path)
            settings_dialog.widgets["options"].set_transient_for(p2p_gui.widgets["main"])
            settings_dialog.run()
            settings_dialog.widgets["options"].destroy()

    def build_context_menu_for_shortcuts(self, button, time):
        """ Build the right-click menu for shortcuts. """

        if len(self.Point2Play.versions):
            sensitive = True
        else:
            sensitive = False

        entries = [
            (_(" Edit Properties "), self.properties_cb, True),
            (_(" Edit Settings for this Shortcut "), self.edit_overrides_cb, sensitive),
            (_(" Delete "), self.remove_shortcut_cb, True)
        ]

        menu = gtk.Menu()
        for title,callback,sensitivity in entries:
            item = gtk.ImageMenuItem(title)
            if callback:
                item.connect("activate",callback)
            item.set_sensitive(sensitivity)
            item.show()
            menu.append(item)
        menu.popup(None,None,None,button,time)

    def build_context_menu_for_game_folders(self, button, time):
        """ Build the right-click menu for game folders."""

        if self.allGamesRowIsSelected():
            return
        
        entries = [
            (_(" Add Shortcut "), self.add_shortcut_cb, True),
            (_(" Delete"), self.remove_game_folder_cb, True),
            (_(" Rename"), self.rename_game_folder_cb, True)            
        ]

        if self.MONITOR_CEDEGA_PROCESSES:
            # If the game folder has (an) active process(es), add the option to kill them:
            (game_folder, shortcut_name) = self.get_selected_names()
            if self.currently_active_game_folders.has_key(game_folder):
                entries.append( (_(" Kill active processes "), self.stop_games_by_game_folder_cb, True) )

        menu = gtk.Menu()
        for title,callback,sensitivity in entries:
            item = gtk.ImageMenuItem(title)
            if callback:
                item.connect("activate",callback)
            item.set_sensitive(sensitivity)
            item.show()
            menu.append(item)
        menu.popup(None,None,None,button,time)

    def properties_cb(self, button):
        """ Shortcut properties callback. """

        (game_folder, shortcut_name) = self.get_selected_names()
        if not game_folder or not shortcut_name:
            return
        shortcut_index = self.Point2Play.get_shortcut_index(game_folder, shortcut_name)

        # save the path in the views so we can restore the selection after we've
        # updated the view

        game_folder_model, game_folder_iter = p2p_gui.widgets["main_game_folder_view"].get_selection().get_selected()
        shortcut_model, shortcut_iter = p2p_gui.widgets["main_shortcut_view"].get_selection().get_selected()
        shortcut_path = shortcut_model.get_path(shortcut_iter)
        game_folder_path = game_folder_model.get_path(game_folder_iter)

        if shortcut_index >= 0:
            properties_dialog = shortcut_properties_gui.ShortcutPropertiesDialog(self.Point2Play, self, shortcut_index)
            properties_dialog.widgets["shortcut_properties"].set_transient_for(p2p_gui.widgets["main"])
            new_props = properties_dialog.run()
            properties_dialog.widgets["shortcut_properties"].destroy()

            if new_props:
                try:
                    self.Point2Play.modify_shortcut_properties(game_folder, shortcut_index, new_props)
                except:
                    err_diag = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, str( sys.exc_info()[1]))
                    err_diag.run()
                    err_diag.destroy()
                    return
                # update main window views in case something has changed
                p2p_gui.widgets["main_game_folder_view"].set_model(self.get_game_folder_model(game_folder))
                p2p_gui.widgets["main_shortcut_view"].set_model(self.get_shortcut_model(game_folder))

                # associate the exe with the profile
                exe = os.path.basename( new_props["Path"] )
                self.Point2Play.associate_exe_and_profile(exe, new_props["DirName"], new_props["Profile"])
                self.Point2Play.write()
                
                # restore previous selection
                game_folder_sel = p2p_gui.widgets["main_game_folder_view"].get_selection()
                game_folder_sel.select_path(game_folder_path)
                shortcut_sel = p2p_gui.widgets["main_shortcut_view"].get_selection()
                shortcut_sel.select_path(shortcut_path)

        else: # couldn't find that shortcut for editing - shouldn't happen!
            err_diag = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("Unable to modify shortcut properties for %s.") % shortcut_name )
            err_diag.run()
            err_diag.destroy()

    def get_game_folder_model( self, all = 0 ):
        model = gtk.ListStore( gobject.TYPE_STRING, gtk.gdk.Pixbuf)
        if self.Point2Play._games:
            if len( self.Point2Play.games() ) > 1:
                if all:
                    self.All_Iter = model.append()
                    model.set_value( self.All_Iter, p2p_gui.GAME_FOLDER_NAME_COL, p2p_gui.ALL_GAMES_STRING )
                    pbuf = gtk.gdk.Pixbuf( gtk.gdk.COLORSPACE_RGB, 1, 8, p2p_gui.GFA_ICON_WIDTH, p2p_gui.GFA_ICON_HEIGHT )
                    pbuf.fill( 0 )
                    model.set_value( self.All_Iter, p2p_gui.GAME_FOLDER_ACTIVITY_COL, pbuf)
            for game_folder in self.Point2Play.games():
                iter = model.append()
                model.set_value( iter, p2p_gui.GAME_FOLDER_NAME_COL, game_folder )
                if self.MONITOR_CEDEGA_PROCESSES and self.currently_active_game_folders.has_key(game_folder):
                    pbuf = gtk.gdk.pixbuf_new_from_file( P2PPATH + "gui/star.png" )
                else:
                    pbuf = gtk.gdk.Pixbuf( gtk.gdk.COLORSPACE_RGB, 1, 8, p2p_gui.GFA_ICON_WIDTH, p2p_gui.GFA_ICON_HEIGHT )
                    pbuf.fill( 0 )
                model.set_value( iter, p2p_gui.GAME_FOLDER_ACTIVITY_COL, pbuf)
        return model

    def get_shortcut_model( self, game_folder, model = None ):
        """ Returns a list model for the shortcuts.

        Parameters:
            game_folder and model (if we're appending to an already existing model)
        """
        if not model: # Are we appending or creating a new list
            model = gtk.ListStore( gtk.gdk.Pixbuf, gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING )
        if self.Point2Play._games: # Do we have any games to list?
            if game_folder == p2p_gui.ALL_GAMES_STRING: # Special case of all shortcuts in all game folders
                for a_game_folder in self.Point2Play.games():
                    self.get_shortcut_model( a_game_folder, model )
            else:
                for shortcut_entry in self.Point2Play._games[game_folder]: # for each shortcut entry
                    iter = model.append()             # Append iterator
                    model.set_value( iter, p2p_gui.SHORTCUT_NAME_COL, shortcut_entry["Name"] )
                    model.set_value( iter, p2p_gui.SHORTCUT_GAME_FOLDER_COL, game_folder )
                    try: # set up the icon if there is one
                        pbuf = gtk.gdk.pixbuf_new_from_file( shortcut_entry["Icon"] )
                    except: # no icon - put some blank space there instead
                        pbuf = gtk.gdk.Pixbuf( gtk.gdk.COLORSPACE_RGB, 1, 8, p2p_gui.ICON_WIDTH, p2p_gui.ICON_HEIGHT )
                        pbuf.fill( 0 )
                    if pbuf.get_width() != p2p_gui.ICON_WIDTH or pbuf.get_height() != p2p_gui.ICON_HEIGHT:
                        pbuf = pbuf.scale_simple( p2p_gui.ICON_WIDTH, p2p_gui.ICON_HEIGHT, gtk.gdk.INTERP_HYPER )
                    model.set_value( iter, p2p_gui.SHORTCUT_ICON_COL, pbuf )
                    model.set_value( iter, p2p_gui.SHORTCUT_DIR_COL, shortcut_entry["DirName"])
        return model

    def conf_panel_cb( self, selection ):
        sel = selection.get_selected()
        if sel:
            model, iter = sel

            if iter:
                print "Selected ", model.get_value( iter, 0 )
            else:
                print "Unselected"

    def prefs_menu_cb( self, widget, *args ):
        sel = p2p_gui.widgets["main_game_folder_view"].get_selection().get_selected()
        if sel:
            model, iter = sel
            if iter:
                game = model.get_value( iter, 0 )        
                dialog = preferences_gui.PreferencesDialog( self.Point2Play, game )
                dialog.set_title( _("Preferences %s") % game )
                dialog.run()
                dialog.destroy()

    def monitor_game_install( self, game ):
        if not self.monitor:
             self.monitor = [ game, ]
             gtk.timeout_add( 2000, self.monitor_cb )
        else:
            for i in self.monitor:
                if i == game:
                    return
            self.monitor.append( game )


    def monitor_cb( self ):
        if not self.monitor:
            return

        for i in self.monitor:
                path = self.Point2Play.gamedir + i + "/" + "games.ini"
                if self.additional_icon_lines and self.additional_icon_lines.has_key(i):
                   (fp, clines) = self.additional_icon_lines[i]
                   del self.additional_icon_lines[i]
                else:
                   clines = 0
                   if os.access(path + ".new", os.F_OK):
                       lfile = None
                       try:
                           lfile = os.open (self.Point2Play.gamedir + i + "/" + ".lock", os.O_RDWR | os.O_EXCL | os.O_CREAT)
                       except os.error:
                           gtk.timeout_add( 500, self.monitor_cb )
                           return
                       os.rename( path + ".new",  path + ".tmp" )
                       fp = file( path + ".tmp", "r" )
                       os.unlink( path + ".tmp" )
                       os.close (lfile)
                       os.remove (self.Point2Play.gamedir + i + "/" + ".lock")
                   else:
                       # We'll try again later
                       continue

                lines = fp.readlines()[clines:]

                #If we had a partial read for this game left over from before, 
                #load it in
                if self.additional_icon_lines and self.additional_icon_lines.get(i):
                   lines = self.additional_icon_lines[i] + lines
                   del self.additional_icon_lines[i]

                clines = clines + self.monitor_parse_icon(i, lines)

                if clines < len( lines ):
                    if not self.additional_icon_lines:
                       self.additional_icon_lines = { i : (fp, clines) }
                    else:
                       self.additional_icon_lines[i] = (fp, clines)

        gtk.timeout_add( 5000, self.monitor_cb )


    def monitor_parse_icon(self, i, lines):
        nlines = 4
        clines = 0
        use_v2 = 0

        if len(lines) > 1 and lines[0][:3] == "V2L":
            nlines = int(lines[0][3:])+1
            use_v2 = 1
        while len( lines ) >= nlines:
            if use_v2:
                name = None
                executable = None
                icon = None
                workdir = None
                args = None
                for n in range(1, nlines):
                    ltype = lines[n][:1]
                    ldata = lines[n][1:]
                    if ltype == 'N': name = ldata.strip()
                    elif ltype == 'E': executable = ldata.strip()
                    elif ltype == 'I': icon = ldata.strip()
                    elif ltype == 'W': workdir = ldata.strip()
                    elif ltype == 'A': args = ldata.strip()
            else:
                name = lines[0].strip()
                executable = lines[1].strip()
                icon = lines[2].strip()
                workdir = lines[3].strip()
                args = None

            self.monitor_register_icon(i, name, executable, icon, workdir, args)
            lines = lines[nlines:]
            clines = clines + nlines

        return clines

    def monitor_register_icon(self, proggrp_name, game_name, executable, icon, workdir, args):
        if game_name.lower().find( "uninstall" ) == -1:
            try:
                # Convert the name into UTF8
                import codecs
                latin1=codecs.lookup("latin1")
                utf8=codecs.lookup("utf-8")
                game_name=utf8[0](latin1[1](game_name)[0])[0]
            except LookupError:
                pass
            # Create the new program icon in memory 

            if misc_utils.is_URL(executable):
                cedega_app = "N"
            else: 
                cedega_app = "Y"

            retval = self.Point2Play.create_new_shortcut( proggrp_name, game_name, executable, icon, workdir, args, cedega_app )
            if not retval:
               # There was a problem. Don't continue creating this icon
               # (Most likely the icon already exists)
               return

            # Save all the shortcut info to disk
            self.Point2Play.write_shortcut_entries( proggrp_name )
            exe = os.path.basename( executable )

#            if self.Point2Play.config["autowmiconsync"] == AUTOWMICONSYNC_ON:
#               # Let's try to create WM icons now (if the user requested for the two menu systems to be synchronized)    
#               # If the user has autosyncing enabled.
#               os.system(os.path.join(P2PPATH, "p2p_wmicon_helper") + " --gamegroup '%s' --iconname '%s' --workdir '%s' --icon '%s' --args '%s'" % (proggrp_name, game_name, workdir, icon, args))

            # Refresh the game view in P2P
            p2p_gui.widgets["main_game_folder_view"].set_model( self.get_game_folder_model( 1 ) )


    # NEW:
    def do_tests_dialog(self, selection):
        """ selection is a list of tests that have been selected. 
            Select the checkboxes on the dialog. """
        tests_dialog = system_tests_gui.SystemTestsDialog(self.Point2Play, selection)
        tests_dialog.widgets["system_tests"].set_transient_for(p2p_gui.widgets["main"])
        tests_dialog.widgets["system_tests"].run()

    def test_run_all_cb(self, widget, *args):
        log.logger.info("Run all tests selected")
        selection = ( "test_cdrom", "test_sound", "test_gl", "test_pthreads", )
        self.do_tests_dialog(selection)

    def test_cdrom_cb(self, widget, *args):
        log.logger.info("Run CDROM test")
        selection = ( "test_cdrom", )
        self.do_tests_dialog(selection)

    def test_sound_cb(self, widget, *args):
        log.logger.info("Run sound test")
        selection = ( "test_sound", )
        self.do_tests_dialog(selection)

    def test_video_cb(self, widget, *args):
        log.logger.info("Run video test")
        selection = ( "test_gl", )
        self.do_tests_dialog(selection)

    def test_pthreads_cb(self, widget, *args):
        log.logger.info("Run pthreads test")
        selection = ( "test_pthreads", )
        self.do_tests_dialog(selection)

    # OLD:

    def tests_run_cb(self, widget, *args):
        test_sel = p2p_gui.widgets["tests_testlist"].get_selection().get_selected()
        if test_sel:
            test_model, test_iter = test_sel
            if test_iter:
                test_name = test_model.get_value( test_iter, 0 )
                self.Point2Play.tests.runTest(test_name, self)

    def tests_selection_cb(self, widget, *args):
        self.test_area_labels_clear()
        test_sel = p2p_gui.widgets["tests_testlist"].get_selection().get_selected()
        if test_sel:
            test_model, test_iter = test_sel
            if test_iter:
                test_name = test_model.get_value( test_iter, 0 )
                self.Point2Play.tests.setTestDescription(test_name, self)
                p2p_gui.widgets["tests_run_button"].show()


    def test_area_labels_clear(self):
        while (len(p2p_gui.widgets["test_area_label_boxs"]) > 0):
            box = p2p_gui.widgets["test_area_label_boxs"][-1]
            p2p_gui.widgets["test_area_table"].remove(box)
            p2p_gui.widgets["test_area_labels"].remove(box.get_child())
            box.remove(box.get_child())
            del p2p_gui.widgets["test_area_label_boxs"][-1]

        p2p_gui.widgets["test_area_table"].resize(1, 2)

        if (len(p2p_gui.widgets["test_area_label_boxs"]) > 0):
            raise Exception("internal error (boxs exists)")
        if (len(p2p_gui.widgets["test_area_labels"]) > 0):
            raise Exception("internal error (labels exist)")

    def test_area_labels_add(self, colour_left, colour_right, text_left, text_right):
        numrows = p2p_gui.widgets["test_area_table"].get_property("n-rows")
        if (numrows > 1 or (len(p2p_gui.widgets["test_area_label_boxs"]) > 0 and len(p2p_gui.widgets["test_area_labels"]) > 0)):
            p2p_gui.widgets["test_area_table"].resize(numrows + 1, 2)
        else:
            numrows = numrows - 1

        p2p_gui.widgets["test_area_labels"].append(gtk.Label(text_left))
        p2p_gui.widgets["test_area_labels"][-1].set_padding(2, 5)
        p2p_gui.widgets["test_area_labels"][-1].set_line_wrap(gtk.TRUE)

        p2p_gui.widgets["test_area_label_boxs"].append(gtk.EventBox())
        p2p_gui.widgets["test_area_label_boxs"][-1].modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(colour_left))
        p2p_gui.widgets["test_area_label_boxs"][-1].modify_bg(gtk.STATE_ACTIVE, gtk.gdk.color_parse(colour_left))
        p2p_gui.widgets["test_area_label_boxs"][-1].modify_bg(gtk.STATE_INSENSITIVE, gtk.gdk.color_parse(colour_left))
        p2p_gui.widgets["test_area_label_boxs"][-1].modify_bg(gtk.STATE_PRELIGHT, gtk.gdk.color_parse(colour_left))
        p2p_gui.widgets["test_area_label_boxs"][-1].modify_bg(gtk.STATE_SELECTED, gtk.gdk.color_parse(colour_left))

        p2p_gui.widgets["test_area_label_boxs"][-1].add(p2p_gui.widgets["test_area_labels"][-1])

        p2p_gui.widgets["test_area_labels"].append(gtk.Label(text_right))
        p2p_gui.widgets["test_area_labels"][-1].set_padding(2, 5)
        p2p_gui.widgets["test_area_labels"][-1].set_line_wrap(gtk.TRUE)

        p2p_gui.widgets["test_area_label_boxs"].append(gtk.EventBox())
        p2p_gui.widgets["test_area_label_boxs"][-1].modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(colour_right))
        p2p_gui.widgets["test_area_label_boxs"][-1].modify_bg(gtk.STATE_ACTIVE, gtk.gdk.color_parse(colour_right))
        p2p_gui.widgets["test_area_label_boxs"][-1].modify_bg(gtk.STATE_INSENSITIVE, gtk.gdk.color_parse(colour_right))
        p2p_gui.widgets["test_area_label_boxs"][-1].modify_bg(gtk.STATE_PRELIGHT, gtk.gdk.color_parse(colour_right))
        p2p_gui.widgets["test_area_label_boxs"][-1].modify_bg(gtk.STATE_SELECTED, gtk.gdk.color_parse(colour_right))
        p2p_gui.widgets["test_area_label_boxs"][-1].add(p2p_gui.widgets["test_area_labels"][-1])

        p2p_gui.widgets["test_area_table"].attach( p2p_gui.widgets["test_area_label_boxs"][-2], 0, 1,\
            numrows, numrows+1, gtk.EXPAND | gtk.FILL, gtk.EXPAND | gtk.FILL, 0, 0)
        p2p_gui.widgets["test_area_table"].attach( p2p_gui.widgets["test_area_label_boxs"][-1], 1, 2,\
            numrows, numrows+1, gtk.EXPAND | gtk.FILL, gtk.EXPAND | gtk.FILL, 0, 0)

        p2p_gui.widgets["test_area_label_boxs"][-1].show()
        p2p_gui.widgets["test_area_label_boxs"][-2].show()

        p2p_gui.widgets["test_area_labels"][-1].show()
        p2p_gui.widgets["test_area_labels"][-2].show()

    def proxy_check_toggled( self, widget, *args ):
        self.config["proxy_check"] = widget.get_active()
        p2p_gui.widgets["proxy_host"].set_sensitive( self.config["proxy_check"] )
        p2p_gui.widgets["proxy_port"].set_sensitive( self.config["proxy_check"] )
        

    # Translation tab related stuff...
    def translate_volunteer_btn_clicked (self, widget):
        self.go_to_url ( TRANSLATION_VOLUNTEER_URL )

    def system_settings_cb(self, widget, *args):
        system_settings_dialog = system_settings_gui.SystemSettingsDialog(self.Point2Play, self)
        system_settings_dialog.widgets["settings"].set_transient_for(p2p_gui.widgets["main"])
        system_settings_dialog.widgets["settings"].run()
        system_settings_dialog.widgets["settings"].destroy()

    def options_cb(self, widget, *args):
        manage_profiles_dialog = options_gui.OptionsDialog(self.Point2Play, p2p_gui.widgets["main"], options_gui.GLOBAL_CONF_STRING)
        manage_profiles_dialog.widgets["options"].set_transient_for(p2p_gui.widgets["main"])
        manage_profiles_dialog.run()
        manage_profiles_dialog.widgets["options"].destroy()

    def system_preferences_cb(self, widget, *args):
        system_preferences_dialog = system_preferences_gui.SystemPreferencesDialog(self.Point2Play, self)
        system_preferences_dialog.widgets["system_preferences"].set_transient_for(p2p_gui.widgets["main"])
        system_preferences_dialog.widgets["system_preferences"].run()
        system_preferences_dialog.widgets["system_preferences"].destroy() 

    def manual_cedega_install_cb(self, widget, *args):
        # We have a downloaded engine file...let's install it!
        install_cedega_engine_helper = install_cedega_engine.InstallCedegaEngineHelper(self.Point2Play, self)
        install_cedega_engine_helper.install_cedega_engine( InstallMode=install_cedega_engine.INSTALL_MANUAL_PACKAGE )
        self.num_engines_changed()

    def remove_engines_cb(self, widget, *args):
        if len( self.Point2Play.versions ):
            remove_engines_dialog = remove_engine_gui.RemoveEnginesDialog(self.Point2Play, self)
            remove_engines_dialog.widgets["remove_engines_dialog"].set_transient_for(p2p_gui.widgets["main"])
            remove_engines_dialog.widgets["remove_engines_dialog"].run()
            remove_engines_dialog.widgets["remove_engines_dialog"].destroy()

            # Now, it's entirely possible that the user has deleted *all* the engines. 
            # If so, we want to make sure the right actions are disabled in the GUI.
            self.num_engines_changed()

    def num_engines_changed(self):
        """ Called whenever the number of engines changes.
            If there are none, we need to disable a bunch of functionality.
            If there were none and now there is at least one, we need to 
            enable a bunch of functionality. """

        if len (self.Point2Play.versions):
            sensitive = 1
        else:
            sensitive = 0

        p2p_gui.widgets["install_menuitem"].set_sensitive( sensitive )
        p2p_gui.widgets["install_button"].set_sensitive( sensitive )
        p2p_gui.widgets["remove_installed_engines_menuitem"].set_sensitive( sensitive )
        p2p_gui.widgets["options_menuitem"].set_sensitive( sensitive )
        p2p_gui.widgets["add_game_folder_menuitem"].set_sensitive( sensitive )
        p2p_gui.widgets["stop_toolbutton"].set_sensitive( sensitive )
        p2p_gui.widgets["stop_menuitem"].set_sensitive( sensitive )

        if not sensitive:  # we disable play when there's no engine, but
                           # do not necessarily enable it when there is an engine present.
            p2p_gui.widgets["play_menuitem"].set_sensitive( sensitive )
            p2p_gui.widgets["play_button"].set_sensitive( sensitive )
        else:
            self.shortcut_select_cb(None) # check to see if a shortcut is selected


    def quit_cb(self, widget, *args):
        p2p_gui.widgets["main"].destroy()
        gtk.main_quit()


if __name__ == '__main__':
    import license, sys

    argc = len( sys.argv )
    help_req = 0      # Print out the help string and exit
    runcmd_req = 0    # User wants to run a game directly
    cmdline_req = 0
    installcmd_req = 0

    try:
        gdf = os.environ["GLOBAL_DELTA_FILENAME"]
        assert gdf
    except:
        gdf = ".global.delta"
    global_delta_file_location = p2pfolder + "/" + gdf
    log.logger.info("Global delta file location: = " + global_delta_file_location)

    if not os.access(global_delta_file_location, os.F_OK):
        # the global delta file needs to be created
        log.logger.info("Creating the global delta file.")
        try:
            f = file( global_delta_file_location, "w+" )
            f.write( "WINE REGISTRY Version 2\n" )
            f.close( )
        except IOError:
            log.logger.warning("For Cedega to work properly, the directory " + p2pfolder + "must be writable.")
            sys.exit(1)

    try:  # figure out the path to the .cedegarc file or use a plausible location
        cedegarc_path = os.getenv('CEDEGA_RC_PATH')
        log.logger.info("Setting cedegarc file to the env var CEDEGA_RC_PATH: " + cedegarc_path)
        assert cedegarc_path
        log.logger.info("... which is nonempty, so that's good")
    except:
        cedegarc_path = os.getenv('HOME') + "/.cedegarc"
        log.logger.info("Setting cedegarc file to $HOME/.cedegarc - environment variable CEDEGA_RC_PATH was either unset or empty.")

    log.logger.info("Set cedegarc file to " + cedegarc_path)

    # --run option specific stuff
    runcmd_game_folder = ""    # Game folder to use
    runcmd_shortcut_name = ""   # Shortcut to use
    runcmd_cmdlinearg = ""  # Commandline arguments

    # --install option specific stuff
    installcmd_gamename = ""
    installcmd_exename = ""

    exe_found = 0

    # -- flags
    use_big_exe = 0
    monitor_cdrom_eject = 0

    # -- values to be passed into the winex function
    dos_cwd_path = ""
    config_file_to_use = ""
    winver = ""
    pthreads_value = ""
    debug_channels = ""
    
    no_gddb_install = 0
    gddb_file = None
    verify_gddb_files = 0
    gamenote_podfile = None

    for i in range(1, argc ):
        if sys.argv[i] == "-theme" or sys.argv[i] == "--theme":
            THEMES_ENABLED = 1
            continue

        if sys.argv[i] == "-run" or sys.argv[i] == "--run":
            # Check first to be sure we have enough parameters 
            if (argc - i < 3):
                print _("\nERROR: -run command requires at least a Game Name and Icon Name\n")
                sys.exit(1)
                break

            # Good to go
            runcmd_req = 1
            cmdline_req = 1

            # Grab the game folder and shortcut name
            runcmd_game_folder = sys.argv[i+1]
            runcmd_shortcut_name = sys.argv[i+2]

            if (argc - i > 2):
                for j in range(i+3, argc):
                    runcmd_cmdlinearg = runcmd_cmdlinearg + " " + sys.argv[j]
            # All done now, no more options to process
            break


        if sys.argv[i] == "-install" or sys.argv[i] == "--install":
            # Check first to be sure we have enough parameters 
            if (argc - i < 3):
                print _("\nERROR: The -install command requires at least a Game Name and Setup EXE Name.\n")
                sys.exit(1)
                break

            cmdline_req = 1
            installcmd_req = 1

            if sys.argv[i+2].lower().find(".exe") != -1:
                # Store the folder name and the executable name...
                installcmd_gamename = sys.argv[i+1]
                installcmd_exename = sys.argv[i+2]
                exe_found = 1
                if (argc - i > 2):
                    for j in range(i+3, argc):
                        runcmd_cmdlinearg = runcmd_cmdlinearg + " " + sys.argv[j]
                # All done now, no more options to process

            else:
                print _("\nERROR: Invalid .exe.\n")
                sys.exit(1)

            break


        if sys.argv[i] ==  "-version" or sys.argv[i] == "--version":
            print _("Cedega GUI Version: %s" % Point2Play.VERSION_STRING)
            
            # Now open up .cedegarc and find the default engine version
            cedegarc_file_parsed = ConfigParser.ConfigParser()
            cedegarc_file_parsed.readfp( file(cedegarc_path, "r") )
            default_winex_vers = cedegarc_file_parsed.get("winex", "default")
            print("Cedega Engine Version (current default): %s" % default_winex_vers)
            sys.exit(0)
            break

        if sys.argv[i] ==  "-config" or sys.argv[i] == "--config":
            config_file_to_use = sys.argv[i+1]
            continue

        if sys.argv[i] ==  "-use-dos-cwd" :
            dos_cwd_path = sys.argv[i+1] + "/"
            continue

        if sys.argv[i] ==  "-use-big-exe" :
            use_big_exe = 1
            continue

        if sys.argv[i] ==  "-use-pthreads" :
            pthreads_value = sys.argv[i+1]

            # IF the entered value is either "on" or "off", convert it to "yes" or "no"...
            if pthreads_value == "on" or pthreads_value == "On" or pthreads_value == "ON":
                pthreads_value = "yes"
            elif pthreads_value == "off" or pthreads_value == "Off" or pthreads_value == "OFF":
                pthreads_value = "no"
            continue

        if sys.argv[i] ==  "-monitor-cdrom-eject" :
            monitor_cdrom_eject = 1
            continue

        if sys.argv[i] ==  "-debugmsg" or sys.argv[i] == "--debugmsg":
            debug_channels = sys.argv[i+1]
            continue

        if sys.argv[i] == "-gddb" or sys.argv[i] == "--gddb":
            gddb_file = sys.argv[i + 1]
        
        if sys.argv[i] == "-no-detect-disc" or sys.argv[i] == "--no-detect-disc":
            no_gddb_install = 1

        if sys.argv[i] ==  "-winver" :
            runcmd_cmdlinearg = runcmd_cmdlinearg + " -winver " + sys.argv[i+1]
            continue

        if sys.argv[i] ==  "-help" or sys.argv[i] == "--help":
            help_req = 1   
            break
        
        if sys.argv[i] == "-verify-gddb" or sys.argv[i] == "--verify-gddb":
            verify_gddb_files = 1
	
        if sys.argv[i] == "-podify" or sys.argv[i] == "--podify":
            if (argc - i > 1):
                gamenote_podfile = sys.argv[i+1]
                continue
            else:
                print "ERROR: Please provide filename to save gamenotes to in pod format\n"
                sys.exit(1)
		

        if sys.argv[i].lower().find(".exe") != -1 and exe_found == 0:
            # Check first to be sure we have enough parameters
            if (argc - i < 1):
                print _("\nERROR: Please provide a valid .exe to launch.\n")
                help_req = 1
                break

            # Good to go
            cmdline_req = 1
            installcmd_req = 1

            # Grab the programname and gamename
            installcmd_gamename = "Dot TransGaming"
            installcmd_exename = sys.argv[i]

            # If there are any more options following the program name and game name,
            # feed them as commandline options
            if (argc - i > 1):
                for j in range(i+1, argc):
                    runcmd_cmdlinearg = runcmd_cmdlinearg + " " + sys.argv[j]

            # All done now, no more options to process
            break
        
    # Get rid of the extra space-character that might have been placed at the beginning 
    # of the commandline-args in all those loops above.
    if runcmd_cmdlinearg and runcmd_cmdlinearg[0] == ' ':
        runcmd_cmdlinearg = runcmd_cmdlinearg[1:]

    # If we need to display the help, do so and exit
    if (help_req):
        print _("\nCedega Version %s" % Point2Play.VERSION_STRING)
        print _("Copyright 2001 - 2007 TransGaming Technologies Inc.")

        print _("Syntax:\n")

        print _("cedega -option /PATH_TO_FOO/foo.exe -switch\n")

        print _("This will run program foo.exe in the directory PATH_TO_FOO and pass")
        print _("the command line argument -switch to foo.exe.  Using this method the")
        print _("directory $HOME/.cedega/Dot TransGaming will be used as run/install")
        print _("location. If the \"Dot TransGaming\" directory is not present it will")
        print _("be created.\n")

        print _("Options\n")

        print _("cedega [-version] [-winver <Version>] [-debugmsg <Channels>]")
        print _("[-use-pthreads <Enable/Disable/Yes/No>] [-use-dos-cwd <Path>]")
        print _("[-monitor-cdrom-eject] [-no-detect-disc] [-gddb <GDDBprofile>]")
        print _("{[-install <Game Folder> <installerapp> [cmdline]] | ")
        print _(" [-run <Game Folder> <Shortcut> [cmdline]] | <executable> [cmdline]}\n")

        print _("-version             - Display the current Cedega GUI version and the\n\t\t       default engine version then exits.")
        print _("-winver              - Run using specified winver: \n\t\t       win95, win98, nt40, win351, winme, win2k, winxp.")
        print _("-debugmsg            - Create a debug trace using specified debug channels.")
        print _("-use-pthreads        - Enable/Disable pthreads.")
        print _("-use-dos-cwd         - Set the working directory to <Path>.")
        print _("-monitor-cdrom-eject - Enable CD/DVD Eject Monitoring.")
        print _("-no-detect-disc      - Do not attempt to detect game disc or use optimal\n\t\t       settings (used with -install).")
        print _("-gddb                - Use the settings specified in the <GDDBprofile>.gddb\n\t\t       file in the default location.")
        print _("-install             - Install the specified application into <Game Folder>.")
        print _("-run                 - Run specified <Shortcut> from the specified <Game Folder>.")
        print _("--help               - Show this help menu.\n")

        print _("Please see the Cedega How-To or the man page for more information.\n")

        sys.exit(0)

    try:
        P2PPATH=os.environ["P2PPATH"] + "/"
    except:
        P2PPATH=""

    # Load in the theme only if the user hasn't turned it off
    if THEMES_ENABLED:
        gtk.rc_parse( P2PPATH+"Point2Play.rc")

    demo = 0
    if os.access(P2PPATH + "VERSION", os.F_OK ):
        demo = 1
        config_file = os.environ["TIMEDEMOFOLDER"] + "/.cedega_timedemorc"
    else:
        config_file = cedegarc_path
    
    if verify_gddb_files == 1:
        Point2Play_ref = Point2Play.Point2Play( config_file )    
        gddb_scan = gddb.scan( Point2Play_ref.P2PFOLDER + "/.gddb/" )
        cf = config.config_file( [Point2Play_ref.get_default_profile_for_engine(Point2Play_ref.default_winex)] )
        notes = None
        if gamenote_podfile != None:
            try:
                notes = open(gamenote_podfile, 'w')
            except:
                print "ERROR: could not open file '" + str(gamenote_podfile) + "' for writing. NOT extracting gamenotes!\n"
                notes = None

        def dump_notes( file, heading, notes ):
            buf = ""
            if (notes != None and len(notes) > 0):
               buf += "=item " + heading + "\n\n"
               buf += "=over\n\n"
               for x in notes:
                 buf += "=item *\n" + x + "\n\n"
               buf += "=back\n\n"
            file.writelines(buf)

        def podify_gamenotes( file, game, gnotes, inotes, rnotes ):
            file.write("=head3 " + game + "\n\n")
            file.write("=over\n\n")
            dump_notes(file, "General Notes", gnotes)
            dump_notes(file, "Installation Notes", inotes)
            dump_notes(file, "Play Notes", rnotes)
            file.write("=back\n\n")
               
        for x in gddb_scan:
            print "Verifying " + str(x[0])
            g = gddb.load_gddb( Point2Play_ref.P2PFOLDER + "/.gddb/" + x[1] )
            if g != None:
                gen_notes = gddb.evaluate_run( g["game"]["general"], cf, True )
                inst_notes = gddb.evaluate_run( g["game"]["installer"], cf, True )
                run_notes = gddb.evaluate_run( g["game"]["run"], cf, True )
                if notes != None:
                    podify_gamenotes( notes, str(x[0]), gen_notes, inst_notes, run_notes )

        if notes != None:
            notes.close()
            print "GDDB Gamenotes extracted to " + gamenote_podfile + ".\n"

        sys.exit( 0 )

    if not cmdline_req:
        if not os.path.isfile( config_file ) or isemptyfile(config_file):
            if demo:
                shutil.copyfile( P2PPATH+"cedega_timedemorc", config_file )
            else:  
                shutil.copyfile( P2PPATH+"cedegarc", config_file )
        try:
            if demo:
                Point2Play_ref = Point2Play.Point2Play( config_file )
            Point2Play_gui_ref = Point2Play_gui( Point2Play.Point2Play( config_file) )
            feg = p2p_gui.widgets["main"]
        except license.licenseError, e:
            pass
        else:
            feg.connect( 'destroy', gtk.main_quit )
            gtk.main()
    else:
        Point2Play_ref = Point2Play.Point2Play( config_file )

        # We now have both..  let's do some magic!!  :-)
        found_game = 0
        # We need to check if the game name they gave us is good.
        # We'll get an exception if the name is bad (we just try to
        # reference it, basically a NOP)
    
    if gddb_file:
        if os.access(Point2Play_ref.P2PFOLDER + "/.gddb/" + gddb_file + ".gddb", os.F_OK):
            gddb_file = Point2Play_ref.P2PFOLDER + "/.gddb/" + gddb_file + ".gddb"
        else:
            print _("\nWARNING: gddb file not found: ") + gddb_file + "\n"
            gddb_file = None

    if runcmd_game_folder:
        try:
            Point2Play_ref._games[runcmd_game_folder]
        except:
            print _("\nERROR: No game group \'") + runcmd_game_folder + _("\' exists.\n")
            sys.exit(1)

#        if config_file_to_use:
#            err = Point2Play_ref.verify_profile_exists( config_file_to_use )
            # If there was an error finding the profile, do not proceed.
#            if err == -1:
#                sys.exit(1)
 
        for shortcut_properties in Point2Play_ref._games[runcmd_game_folder]:
            if shortcut_properties["Name"] == runcmd_shortcut_name:
                found_game = 1
                # if the user specifies something on the commandline, it will take higher
                # precedence than the commandline args found in the shortcut properties,
                # but we will notify the user if we are doing this.
                if runcmd_cmdlinearg:
                    log.logger.info("Your command line args: " + runcmd_cmdlinearg)
                    log.logger.info("Shortcut's command line args: " + shortcut_properties["CommandLineArguments"])
                    if runcmd_cmdlinearg != shortcut_properties["CommandLineArguments"]:
                        log.logger.warning("You have specified command-line arguments that are different from those currently stored in the shortcut's properties. Ignoring the stored ones....")
                    cmdlineargs = runcmd_cmdlinearg
                else:
                    cmdlineargs = shortcut_properties["CommandLineArguments"]

                if shortcut_properties["CedegaApp"] == "Y":
                    try:
                        retval = Point2Play_ref.winex( 0, runcmd_game_folder, shortcut_properties["Path"], shortcut_properties["Workdir"], cmdlineoverride=cmdlineargs, debug_channel=debug_channels, gddb_file=gddb_file, shortcut_name=shortcut_properties["Name"] )
                        # Do we need to renice the wineserver?
                        if not Point2Play_ref.CMDLINE_RENICING:
                            Point2Play_ref.CMDLINE_RENICING = 1
                            Point2Play_ref.commandline_renice()

                    except Point2Play.Point2PlayError, e:
                        print _("ERROR: There was an error running your game: ") +  e.__str__() 

                else: # it's a native app or url
                    if misc_utils.is_URL(shortcut_properties["Path"]):
                        misc_utils.go_to_url(shortcut_properties["Path"], Point2Play_ref)
                    else:
                        if shortcut_properties["Workdir"]:
                            try:
                                os.chdir(shortcut_properties["Workdir"])
                            except:
                                print _("Unable to change to directory %s.") % ( shortcut_properties["Workdir"] )
                                sys.exit(1)
                        # prepare the argument list for the spawn cmd
                        if cmdlineargs:
                            whitespace_regexp = re.compile(r'\s+')
                            arglist = whitespace_regexp.split(shortcut_properties["CommandLineArguments"])
                            arglist.insert(0, shortcut_properties["Path"])  # the arglist must have the command as its first item
                        else:
                            arglist = [ shortcut_properties["Path"] ]
                        log.logger.info("Argument list for native app's arglist: ")
                        print arglist
                        try: # run the command
                            os.spawnve(os.P_NOWAIT, shortcut_properties["Path"], arglist, os.environ)
                        except:
                            print _("Unable to run command %s %s.") % ( shortcut_properties["Path"], shortcut_properties["CommandLineArguments"])
                            sys.exit(1)
                        
                break # no need to keep looping since we've found the shortcut
        
        if found_game == 0:
            print _("ERROR: No shortcut \'") + runcmd_shortcut_name + _("\' in game folder \'") + runcmd_game_folder + "\'"

    if installcmd_req:
        # Instantiate the InstallMonitor class...
        InstallMonitor_ref = InstallMonitor( Point2Play_ref )
        TheReaper_ref = Reaper()

        # Create the temporary "install_config" file...
        if not config_file_to_use:
            if not Point2Play_ref.default_winex:
                print _("Error: No Cedega engine is currently installed.\nPlease run the setup wizard to install a Cedega engine.")
                sys.exit(1)

            config_file_to_use = str("cedega_" + Point2Play_ref.default_winex)
        else:
            err = Point2Play_ref.verify_profile_exists( config_file_to_use )
            # If there was an error finding the profile, do not proceed.
            if err == -1:
                sys.exit(1) 
        
        if no_gddb_install != 1:
            cdroms = cdrom.getFstabOpticalDrivesList()
            if not cdroms:
                cdroms = cdrom.getDbusOpticalDrivesList()
            
            volumes = []
            for cdrom in cdroms:
                if cdrom.isDriveMounted():
                    iso = ISOHeader.ISOHeader( cdrom.getDev() )
                    v = getattr( iso, "volume_id" )
                    volumes.append( [v, cdrom.mount_path] )
            detected = gddb.detect( Point2Play_ref.P2PFOLDER + "/.gddb", None, volumes )
        
            if gddb_file == None and detected != None:
                gddb_file = detected[0]
        

        while gtk.events_pending():
            gtk.main_iteration()

        # Run Cedega, configured for installation!
        retval = Point2Play_ref.winex( 1, installcmd_gamename, installcmd_exename, None, Point2Play_ref.gamedir + installcmd_gamename + "/" + "games.ini.new", Point2Play_ref.default_winex, ( dos_cwd_path, use_big_exe, pthreads_value ), cmdlineoverride=runcmd_cmdlinearg, ModifiedInstallConfigs="install_config", debug_channel=debug_channels, OverrideGameProfile=config_file_to_use, MonitorCdromEject=monitor_cdrom_eject, gddb_file=gddb_file )

        if retval:
            # Add new program group and update p2p so it shows up (so that you can delete a
            # game dir created by a crashed installer)
            Point2Play_ref.create_new_game_folder( installcmd_gamename )
            # Don't fear the reaper...set him off so that we will know when installation is done...
            TheReaper_ref.reap()

            # Keep a monitor going so that the icons and games.ini get 
            # created...
            InstallMonitor_ref.monitor_game_install( installcmd_gamename )




        # Start looping the callbacks..
        gtk.main()


    if demo:
        (days, total) = Point2Play_ref.numdays()
        if days >0:
            #create a plain dialog
            remind = gtk.Dialog("Cedega Time Trial")
            #load the image
            image = gtk.Image()
            image.set_from_file(P2PPATH + "gui/tg_logo_splash.png")
            #add the image
            remind.vbox.pack_start(image)
            #add the register button
            remind.add_button("Subscribe Later", 2)
            remind.add_button("Subscribe Now", 68)
            remind.set_default(None)
            remind.set_focus(None)
            #remove the separator and center the dialog
            remind.set_has_separator(0)
            remind.set_position(gtk.WIN_POS_CENTER)
            #make things visible
            remind.show_all()
            #decide wether to display dialogs
            response_id = remind.run()
            remind.destroy()
            if response_id == 68:
                oemfile = None
                try:
                    oemfile = file(P2PPATH + "VERSION", "r")
                except:
                    pass
                if (oemfile != None):
                    oemname = oemfile.readline()
                oemfile.close()
                Point2Play_gui_ref.go_to_url("http://www.transgaming.com/?ref=" + oemname)
