"""
Point2Play Source Code
Copyright 2004, TransGaming Technologies Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.
"""

# This is a simple class that is used to process out version information from a particular 
# full version string. It has helper functions to determine if the version
# is more recent than is supported by p2p, and will return the version string to display
# minus the version information
class ParseVersion:
   # This is the most recent feature that this release of Point2Play supports
   LARGEST_SUPPORTED_CEDEGA_VERSION = 600


   # Initial parsing of version string - get out the credentials and get
   # most recent version that is listed (prior versions of cedega sometimes
   # had multiple credentials)
   def __init__ (self, version_string):
      # The feature version credential version as an integer
      self.version_int = -1
      # The version string without all of the version credentials
      self.parsed_version_str = version_string
      # Whether version credentials were found or not
      self.credentials_found = 0

      # Get first set of greater and less than 
      greatpos = self.parsed_version_str.find(">")
      lesspos = self.parsed_version_str.find("<")

      # While we still have groups of ">xxxf<" version strings
      while greatpos > lesspos and greatpos != -1 and lesspos != -1:
         self.credentials_found = 1

         # Record the version number if it is larger than the previously
         # stored on (if any)
         ver = int( self.parsed_version_str[lesspos+1:greatpos-1] ) 
         if ver > self.version_int:
            self.version_int = ver

         # Filter out the version credential
         self.parsed_version_str = self.parsed_version_str[greatpos+1:]

         # Get the next set (if there is any)
         greatpos = self.parsed_version_str.find(">")
         lesspos = self.parsed_version_str.find("<")
         
      
      # At this point we should be settled


   # Check if this particular version has a feature set more recent than
   # what we have support for (and display a warning if so)
   def check_cedega_version_support (self):
      if self.version_int > self.LARGEST_SUPPORTED_CEDEGA_VERSION:
         return 0
      else:
         return 1
       
     
   # Get the processed version string back (without the version credentials)
   def get_parsed_version_string(self):
      return self.parsed_version_str


   # What version credential is there for this version string?
   def get_version_credential(self):
      return self.version_int

   # Were there were any version credentials at all 
   def any_credentials_parsed(self):
      return (self.credentials_found)
