"""
Point2Play Source Code
Copyright 2003, 2004 TransGaming Technologies Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""
import types, os, gddb

class config_file:
    def __init__( self, filename, legacy=0, appdefaults=None ):
        self._config_file__sections = []
        self._config_file__trailing_comments = []
        self.legacy = legacy
        
        if appdefaults != None:
            self.appdefaults = appdefaults
        else:
            self.appdefaults = ""
        
        self.last_section_line = ""

        for f in filename:
          if f != None:
            if f.rfind(".gddb") != -1:
                fname = f
                gddb_installer = 0
                
                if f.rfind( "/run" ) != -1:
                    fname = f[ :f.rfind("/run") ]
                elif f.rfind( "/installer" ) != -1:
                    fname = f[ :f.rfind("/installer") ]
                    gddb_installer = 1
                gpt = gddb.load_gddb( fname )
                if gpt != None:
                    if gddb_installer != 0:
                        sp = gddb.evaluate_run( gpt["game"]["installer"], self, False, True )
                    else:
                        gddb.evaluate_run( gpt["game"]["run"], self )
            else:
                self.load( f )

    
    def get_appdefaults( self ):
        return self.appdefaults


    def load( self, filename ):
        comment = []   # The list of 'current' comments
        options = []   # The list of 'current' options
        section = None # The 'current' section
        appedefaults = ""

        # Note: This call can throw an IOError, especially if attempting to install
        # a game for the first time. This will be caught by Point2Play.winex()
        f = file( filename )  # Create file object from filename

        lines = f.readlines() # Get a list of lines
        """Check to see if first line is proper WINE REGISTRY --- this needs fixing"""
        if not lines or lines[0].rstrip().lower() != "wine registry version 2":
            print "Bad registry"
            return None

        del lines[0] # Remove the first line
        
        is_appdefault = 0
        
        """
        Next we parse the rest of the config file lines... because we have
        to maintain the comments the parsing isn't as simple as could be

        Update:
        The file isn't a .INI style file..  it contains subsections which
        make a 'stateless' parser impossible.
        """
        for line in lines:
            line = line.strip() # Strip trailing newlines
            if len( line ) == 0:         # Empty line?
                continue
            if line[0] == ";" or line[0] == "#": # Are we a comment line?
                comment.append( line )
            elif line[0] == "[":                 # Are we a section header?
                section_name = line[1:len(line)-1]
                # print "Got new section path: %s" % ( section_name )
                node = None
                while 1:
                    index = section_name.find( "\\\\" )
                    if index == -1:
                        key = section_name
                        node = self.get_sub_section( node, key, 1 )
                        break
                    else:
                        is_appdefault = 1
                        key = section_name[:index]
                        section_name = section_name[index+2:]
                        node = self.get_sub_section( node, key, 1 )
                        if line != self.last_section_line:
                            self.appdefaults += line + "\n"
                            self.last_section_line = line
                section = node
                options = section["options"]
                section["comments"].extend(comment)
                comment = []
            elif  line[0] == "\"":               # Are we an option?
                option_name, value = line.split( "=", 1 )
                option_name = option_name.strip()
                """Strip out the quotes in the names"""
                if option_name[0] == '"' and option_name[ len( option_name ) - 1 ] == '"':
                    option_name = option_name[ 1:len( option_name )-1 ]
                value = value.strip()
                """Strip out the quotes in the values"""
                if value[0] == '"' and value[ len ( value ) - 1 ] == '"':
                    value = value[ 1:len( value ) - 1 ]
                """Create an option dict with option name, value and the comments on top"""
                if self.legacy != 0:
                    options.append( { "name": option_name, "value": value, "comments": comment } )
                elif is_appdefault != 1:
                    self.set_option( section_name, option_name, value )

                if is_appdefault == 1:
                    self.appdefaults += line + "\n"
                    
                comment = [] # Clear comments
            else:
                pass
              
        self._config_file__trailing_comments = comment
        f.close()

    def write( self, filename, dconfig = None):
        f = file( filename, "w+" )
        # print self.sections()
        f.write( "WINE REGISTRY Version 2\n" )

        try:
            GLOBAL_DELTA_FILENAME = os.getenv('GLOBAL_DELTA_FILENAME')
        except:
            GLOBAL_DELTA_FILENAME = ".global.delta"
        
        written_enginever = 0
        
        if dconfig != None:
          for x in self.sections():
            s = self.get_section( x )
            printedsection = False
            
            for o in s[ "options" ]:
              s2 = dconfig.get_section( x )
              o2 = dconfig.get_option( x, o["name"] )
              
              if (o2 != o["value"]) or (o["name"] == "Cedega" and os.path.basename(filename) == GLOBAL_DELTA_FILENAME):
                  if printedsection == False:
                      printedsection = True
                      f.write( "\n\n[" + x + "]\n" )
                    
                  f.write( "\"" + o[ "name" ] )
                  f.write( "\" = \"" + o[ "value" ] + "\"\n" )

          return
        
        import traceback
        
        for section in self._config_file__sections:
            self.write_sub_section(f, section["name"], section)
        for comment in self._config_file__trailing_comments:
            f.write( comment + "\n" )
        # print self.sections()
        
        f.write( self.appdefaults + "\n" )
        
        f.flush() 
        os.fsync(f.fileno())
        f.close()

    def write_sub_section( self, f, path, section ):
        for comment in section[ "comments" ]:
            f.write( comment + "\n" )
        if len(section[ "options" ]):
            f.write( "[" + path + "]\n" )
            for options in section[ "options" ]:
                for comment in options[ "comments" ]:
                    f.write( comment + "\n" )
                f.write( "\"" + options[ "name" ] + "\" = \"" + options[ "value" ] + "\"\n" )
#                print( "\"" + options[ "name" ] + "\" = \"" + options[ "value" ] + "\"" )
            f.write( "\n" )
        for sub_sec in section[ "sections" ]:
            self.write_sub_section(f, path + "\\\\" + sub_sec["name"], sub_sec )

    def sections(self):
        sec_list = []
        for section in self._config_file__sections:
            sec_list.append( section[ "name" ] )
        return list( sec_list )

    def has_section( self, section ):
        for sec in self._config_file__sections:
            if sec[ "name" ] == section:
                return 1
        return 0

    def add_section( self, name, options, comments = () ):
        self._config_file__sections.append( { "name": name, "options": options, "comments": comments, "sections": [] } )

    def get_section( self, section, super_section = None):
        if super_section != None:
            appdefs = self.get_sub_section( None, "AppDefaults" )
            if appdefs == None:
                return None
            super_sec = self.get_sub_section( appdefs, super_section )
            if super_sec == None:
                return None
            sections = super_sec["sections"]
        else:
            sections = self._config_file__sections
        for sec in sections:
            if sec[ "name" ].lower() == section.lower():
                return sec
        return None

    def get_sub_section( self, root, section, make_it = 0 ):
        if root == None:
            sections = self._config_file__sections
        else:
            sections = root["sections"]
        for sec in sections:
            if sec[ "name" ].lower() == section.lower():
                return sec
        if make_it:
            sec = { "name": section, "comments": [], "options": [], "sections": [] }
            sections.append(sec)
            return sec
        return None

    def get_option( self, section, option, super_section = None, toplevel_ok = 0 ):
        if super_section != None:
            super_sec = self.get_section( section, super_section )
            if super_sec != None:
                for opt in super_sec[ "options" ]:
                    if opt[ "name" ] == option:
                        return opt[ "value" ]
            else:
                if toplevel_ok == 0:
                    return None
        sec = self.get_section( section )
        if sec != None:
            for opt in sec[ "options" ]:
                if opt["name"] == option:
                  return opt["value"] 
        else:
            return None

    def set_option( self, section, option, value, super_section = None, toplevel_ok = 0 ):
#        print "[%s]\n%s=%s\n" % ( section, option, value )
        if super_section != None:
            appdefs = self.get_sub_section( None, "AppDefaults", 1 )
            super_sec = self.get_sub_section( appdefs, super_section, 1 )
            sec = self.get_sub_section( super_sec, section, 1 )
            
            for opt in sec["options"]:
                if opt["name"] == option:
                    sec["options"][sec["options"].index(opt)] = { "name": option,
                                 "value": value,
                                 "comments": opt["comments"] }
                    return
            sec["options"].append( { "name": option,
                                     "value": value,
                                     "comments": [] } )
        else:
            sec = self.get_section( section )
            if sec == None:
                   sec = { "name": section, "comments": [], "options": [ { "name": option, "value": value, "comments": [] } ], "sections": [] }
                   self._config_file__sections.insert( 1, sec )
            else:
                entry_added = 0

                # Go through the list of options already set for this particular section
                opt = None
                for opt in sec["options"]:
                    # If we get a match, we just replace what was there with the new value
                    if opt["name"].lower() == option.lower():
                        entry_added = 1
                        sec["options"][sec["options"].index(opt)] = { "name": option, "value": value, "comments": opt["comments"] }

                # If there isn't already this particular option in this section, we need to add it in
                if entry_added == 0:
                   sec["options"].append( { "name": option, "value": value, "comments": [] } )


    def drive_index( self, drive ):
        i = 0
        for sec in self._config_file__sections:
            if sec["name"].lower() == "drive " + drive.lower():
                return i
            i = i + 1
        return -1

    def get_drive( self, drive ):
        index = self.drive_index( drive )
        if index >= 0:
            return self._config_file__sections[index]["options"]

    def set_drive( self, drive, path, type, label, filesystem ):
        section = { "name"   : "Drive " + drive.upper(),
                    "options": ( { "name"     : "Path",
                                   "value"    : path,
                                   "comments" : () },
                                 { "name"     : "Type",
                                   "value"    : type,
                                   "comments" : () },
                                 { "name"     : "Label",
                                   "value"    : label,
                                   "comments" : () },
                                 { "name"     : "Filesystem",
                                   "value"    : filesystem,
                                   "comments" : () } ),
                    "comments": [],
                    "sections": [] }
        index = self.drive_index( drive )
        if self.drive_index( drive ) >= 0:
            self._config_file__sections[index] = section
        else:
            self._config_file__sections.append( section )

