#!/usr/bin/env python

"""
Point2Play Source Code
Copyright (c) 2005 by TransGaming Technologies, Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.
"""

import pygtk
pygtk.require('2.0')
import gtk
import gtk.glade
import log, os
import config, system_detection

# If Point2Play is being launched by the packaged launch script, then it needs to be able
# to be launched from anywhere.  In this case, P2PPATH will already be set, and we can use
# that as a path to locate our files.

p2ppath = os.getenv("P2PPATH")
GLADE_FILE = p2ppath + "/gui/" + __name__ + ".glade"
	
class WidgetsWrapper:
    def __init__(self):
        self.widgets = gtk.glade.XML(GLADE_FILE, None, "p2p_localized_msg")

    # this function lets us do widgets['widget_name'].action()
    def __getitem__(self, key):
        return self.widgets.get_widget(key)

class ConversionDialogue:
    def __init__(self, p2p_gui_r, num_to_convert):
        self.widgets = WidgetsWrapper( )
        self.widgets.widgets.signal_autoconnect(self)
        self.p2p_gui = p2p_gui_r
        self.num_to_convert = num_to_convert

        self.set_progress( 0, self.num_to_convert )
        
        version_list = self.p2p_gui.Point2Play.sort_winex_versions()
        
        self.widgets[ "version" ].set_popdown_strings( version_list )
        default_winex = self.p2p_gui.Point2Play.default_winex
        if default_winex != None:
            self.widgets[ "version_entry" ].set_text( default_winex )
        
        self.p2p_gui.Point2Play.populate_default_profiles_directory(copy_all=1)


    def on_button_start_clicked(self, button):
        converted = 0
        self.widgets[ "button_stop" ].set_sensitive( 0 )
        
        self.p2p_gui.Point2Play.default_winex = self.widgets[ "version_entry" ].get_text()
        self.p2p_gui.Point2Play.populate_default_profiles_directory()
        
        for x in self.p2p_gui.Point2Play.profiles.iteritems():
            for y in x[1]:
                exe = y[ y.rfind( "/" ) + 1: ]
                self.widgets[ "label_converting" ].set_text( y )
                p1 = config.config_file( [ (self.p2p_gui.Point2Play.USER_DEFINED_PROFILES_PATH + x[0]) ], 1 )

                v = p1.get_option("Version", "Cedega")
                if v == None or v == "" or v == "default":
                    v = self.p2p_gui.Point2Play.default_winex
                p2_path = self.p2p_gui.Point2Play.get_default_profile_for_engine( v )
                p2 = config.config_file( [p2_path] )

                try:
                  p1.write( self.p2p_gui.Point2Play.lookup_shortcut_delta(y), p2 )
                except IOError:
                  pass		# we're likely writing to an old game which doesn't actually exist anymore
                except KeyError:
                  pass		# ""

                converted += 1
                self.set_progress( converted, self.num_to_convert )

            
        default_config = config.config_file( [self.p2p_gui.Point2Play.get_default_profile_for_engine(self.p2p_gui.Point2Play.default_winex)], 1 )
        last_profile = None

        for proggrp in self.p2p_gui.Point2Play._games:
           appdefault = default_config.get_sub_section( None, "AppDefaults" )
           if len( appdefault[ "sections" ][ 0 ][ "sections" ] ):
               for gameicon in self.p2p_gui.Point2Play._games[proggrp]:
                 exe = gameicon["Path"][ gameicon["Path"].rfind( "/" ) + 1: ]
                 exe_profile = self.p2p_gui.Point2Play.lookup_game_profile_association( exe, proggrp )
                 
                 try:
                     delta_file = self.p2p_gui.Point2Play.get_shortcut_delta_name( proggrp, gameicon["Name"] )
                 except KeyError:
                     break	# invalid game
                    
                 if exe_profile != None and exe_profile != "":
                     ad_profile = config.config_file( [self.p2p_gui.Point2Play.USER_DEFINED_PROFILES_PATH + exe_profile], 1 )
                 else:
                     ad_profile = default_config
                     
                 appdefault = ad_profile.get_sub_section( None, "AppDefaults" )
                 if len( appdefault[ "sections" ][ 0 ][ "sections" ] ):
                   for d in appdefault[ "sections" ]:
                     if exe.lower() == d[ "name" ].lower():
                       if not os.access(delta_file, os.F_OK):
                         f = file( delta_file, "w+" )
                         f.write( "WINE REGISTRY Version 2\n" )
                         f.close()

                       p1 = config.config_file( [delta_file] )
                           
                       for e in d[ "sections" ]:
                         for f in e[ "options" ]:
                           p1.set_option( e["name"], f["name"], f["value"] )

                       p1.write( delta_file )

                 # Move command-line arguments from the profile to the games.ini file
                 cmdline = ad_profile.get_option("transgaming", "cmdline")
                 if cmdline:
                     gameicon["CommandLineArguments"] = cmdline
                     self.p2p_gui.Point2Play.write_programs(proggrp)

                 self.widgets[ "label_converting" ].set_text( exe )
                 self.set_progress( converted, self.num_to_convert )
                 converted += 1

        self.set_progress( self.num_to_convert, self.num_to_convert )

        self.widgets[ "label_converting" ].set_text( "Finished." )
        self.widgets[ "button_stop" ].set_sensitive( 1 )
        self.widgets[ "button_start" ].set_sensitive( 0 )
#        f = file( self.p2p_gui.Point2Play.P2PFOLDER + ".global.delta", "w+" )
#        f.write( "WINE REGISTRY Version 2\n" )
#        f.close( )                            

        system_detection.detect_optical_drives(self.p2p_gui.Point2Play)


    def on_button_stop_clicked(self, button):
        self.widgets[ "conversion_gui" ].destroy()
    
    
    def set_progress( self, converted, total ):
        prog_percent = float(converted) / float(total)
        self.widgets[ "label_status" ].set_text( str(converted) + "/" + str(total) + " games converted" )
        self.widgets[ "progressbar1" ].set_fraction( prog_percent )

        while gtk.events_pending():
            gtk.main_iteration()
        
