"""
Point2Play Source Code
Copyright (c) 2005 by TransGaming Technologies, Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""

import pygtk
pygtk.require('2.0')
import gtk
import gtk.glade
import log 
import os, re, popen2
import ClientCookie, urllib, winex_get
import p2p_gui

p2ppath = os.getenv("P2PPATH")
GLADE_FILE = p2ppath + "/gui/" + __name__ + ".glade"

# This module can download MozControl or MS Core Fonts, showing a progress bar.
# The scripts need to have wget use --dot-style=micro. MozControl is one big
# download so the progress bar will reflect the progress of that download,
# but MS Core Fonts seem to be required to be separate. So in the dialog 
# for those, the progress bar will say "Downloading n of m files" as it 
# goes along.

LABELS = [ _("Downloading Microsoft(R) Core Fonts"), _("Downloading the MozControl Package"), _("Downloading Cedega Engine"), _("Downloading Cedega GUI"), _("Downloading GDDB Updates") ]

LOGFILES = [ "tg-font-install.log", "tg-mozctl-install.log", "tg-engine-install.log", "tg-gui-install.log", "tg-gddb-update.log" ]

GDDB_URL_BASE = "http://downloads.transgaming.com/gddb/"


class WidgetsWrapper:
    def __init__(self):
        self.widgets = gtk.glade.XML(GLADE_FILE, None, "p2p_localized_msg")

    # Gives us the ability to do: widgets['widget_name'].action()
    def __getitem__(self, key):
        return self.widgets.get_widget(key)

class DownloadDialog:
    def __init__(self, caller_gui, Point2Play, target, calling_from_wizard, gddb_list=None):
        self.caller_gui = caller_gui
        self.Point2Play = Point2Play
        self.target = target

        self.gddb_list = gddb_list
        self.download_index = -1

        self.widgets = WidgetsWrapper()
        self.widgets["download_label"].set_label(LABELS[target])
        self.widgets.widgets.signal_autoconnect(self)
        self.exit_status = -1
        self.text = ""
        Point2Play.setup_environment_for_proxy()

        self.logfile = Point2Play.WINEXGLOBALPREFIX + LOGFILES[target]
        self.logfilehandler = file(self.logfile, 'w')

        # WARNING! Don't try to pipe the script commands to tee to capture the output - otherwise
        # we'll get the exit status of tee! We put the output into the log file manually.

        if target == p2p_gui.MOZCONTROL:
            # Run the tg-mozctl-install script. We force it since we'll only get here 
            # if the user insists.

            # WARNING: don't switch around the command line options! tg-mozctl-install only expects
            # one command-line option. The "-i_agree_to_license" is special, so it can be first or second,
            # but options like -force-update or -check-update can only go in the first position. 
            p = popen2.Popen4(Point2Play.MOZCTLINSTALLER_PATH + "tg-mozctl-install -force-update -i_agree_to_license", 1)
        elif target == p2p_gui.MSCOREFONTS:
            # Run the install script for MS Core Fonts
            p = popen2.Popen4(Point2Play.FONTINSTALLER_PATH + "tg-font-install -run_from_p2p", 1)
        elif target == p2p_gui.GDDB:
            winex_get.login_transgaming( Point2Play.config["username"], Point2Play.config["password"] )
            if self.gddb_list == None or len(self.gddb_list) == 0:
                self.gddb_list = p2p_gui.gddb_update_list( Point2Play, update_all_files=True )
            self.download_next( )
            self.timer = gtk.timeout_add( 100, gddb_progress_timeout, self, target, self )
            
        if target != p2p_gui.GDDB:
            self.timer = gtk.timeout_add( 100, progress_timeout, self, target, p )


    def download_next(self):
        """
        Queues up next downloadable GDDB entry.
        Returns True if successful, or False if there is no more to download.
        """
        self.download_index += 1
        if self.download_index >= len(self.gddb_list):
            return False
        self.gddb_fp = ClientCookie.urlopen( GDDB_URL_BASE + self.gddb_list[self.download_index] )
        
        self.text = _("Downloading") + " " + self.gddb_list[self.download_index] + " (" + str(self.download_index + 1) + " of " + str(len(self.gddb_list)) + ")"
        fraction = int( ( float(self.download_index + 1) / float(len(self.gddb_list))) * 100 )
        self.widgets["progressbar"].set_fraction( float(fraction) / 100 )
        self.widgets["progressbar"].set_text( self.text + " - " + str(fraction) + "%")
        
        self.gddb_buffer = ""
        
        return True

    def cancel_cb(self, button):
        log.logger.info("Download cancel clicked")
        gtk.timeout_remove(self.timer)
        self.timer = 0
        self.exit_status = -1
        self.widgets["download_dialog"].response(0)

    def done(self, exit_status):
        log.logger.info("Download done - exit status is " + str(exit_status))
        self.logfilehandler.close()
        gtk.timeout_remove(self.timer)
        self.timer = 0
        self.exit_status = exit_status
        if self.exit_status == os.EX_OK:
            self.widgets["download_dialog"].response(1)
        else:
            self.widgets["download_dialog"].response(0)

def progress_timeout(obj, target, p):
    line = p.fromchild.readline()
    obj.logfilehandler.write(line)

    if line:
        log.logger.info(line)
        match = re.search("FILE\s+(\d+)\s+of\s+(\d+)", line)
        if match:
            obj.text = "File " + match.group(1) + " of " + match.group(2) + " - "
            log.logger.info(obj.text)
        # look for the percentages wget should be outputting
        match = re.search("([0-9]+)\%", line)  
        if match:
            value = match.group(1)
            fraction = float(int(value)) / 100.0
            obj.widgets["progressbar"].set_fraction(fraction)
            obj.widgets["progressbar"].set_text(obj.text + value + "%")
            while gtk.events_pending():  # update
                gtk.main_iteration()
    else:  # EOF - check exit status
        status = p.wait()
        log.logger.info("Wait returned " + str(status))
        exit_status = os.WEXITSTATUS(status)
        log.logger.info("The exit status is " + str(exit_status))
        obj.done(exit_status)
        return gtk.FALSE

    return gtk.TRUE


def gddb_progress_timeout(obj, target, p):
    """ 
    GTK timeout callback to download new files and update the dialogue
    box for GDDB update mechanism.
    """
    data = p.gddb_fp.read( 2048 )
    if data == None or len(data) == 0:
        wp = file( obj.Point2Play.P2PFOLDER + "/.gddb/" + obj.gddb_list[obj.download_index], "w+b" )
        wp.write( obj.gddb_buffer )
        wp.close( )
        # save buffer
        if p.download_next() == False:
            obj.done( os.EX_OK )
            return gtk.FALSE
    else:
        obj.gddb_buffer += data
    
    while gtk.events_pending():
        gtk.main_iteration()
    
    return gtk.TRUE



# Note about wait ()
# Wait for completion of a child process, and return a tuple containing its
# pid and exit status indication: a 16-bit number, whose low byte is the
# signal number that killed the process, and whose high byte is the exit
# status (if the signal number is zero); the high bit of the low byte is set
# if a core file was produced. (Not on MS-DOS.)
