#!/usr/bin/python
import sys
import antlr
import boolexpr
import config
import os
import ConfigParser
import GDDBParser
import GDDBLexer
import re
import log 

( OLD_PARSER, NEW_32_PARSER, NEW_64_PARSER ) = range( 0, 3 )

GENERAL_PAGE=0
PLAY_PAGE=1
INSTALL_PAGE=2
ALL_PAGE=3

use_parser = OLD_PARSER

try:
    sys.path.append( os.environ["P2PPATH"] )
    import gddb_parser
    use_parser = NEW_32_PARSER
    log.logger.info( "Using 32 bit extension GDDB parser" )
except ImportError, e:
    try:
        import gddb_parser64
        use_parser = NEW_64_PARSER
        log.logger.info( "Using 64 bit extension GDDB parser" )
    except ImportError, e2:
        use_parser = OLD_PARSER
        log.logger.info( "Using Python GDDB parser" )


#print boolexpr.evalbool(0, len(stack)-1, stack)[0]  
eFALSE, eTRUE, eAND, eOR, eEND, eINCREMENT, eDECREMENT, eTEST = range( 8 )

NONE_STRING = "<None>"

lexer = 0
sysinfo = None

def breakdown_if( expr, conf, sysinfo ):
  newstack = []
  
  for x in expr:
      if x[0] == eTEST:
        section = x[1][ x[1].find(".") + 1: ]
        option = x[1][ :x[1].find(".") ]
        var1 = None
        
        if option == "sysinfo":
            var1 = sysinfo[ section ]
        else:
            var1 = conf.get_option( option, section )
        
        if var1 == None:
            raise Exception( "Invalid attribute " + section )
            
        op   = x[2]
        var2 = x[3]
        result = eTRUE
        
        if op == "contains":
            if var1.lower().find( var2.lower() ) >= 0:
                result = eTRUE
            else:
                result = eFALSE
        
        if op == "==":
            if float(var1) == (var2):
                result = eTRUE
            else:
                result = eFALSE
        
        if op == ">":
            if float(var1) > float(var2):
                result = eTRUE
            else:
                result = eFALSE
        
        if op == ">=":
            if float(var1) >= float(var2):
                result = eTRUE
            else:
                result = eFALSE
        
        if op == "<":
            if float(var1) < float(var2):
                result = eTRUE
            else:
                result = eFALSE
        
        if op == "<=":
            if float(var1) <= float(var2):
                result = eTRUE
            else:
                result = eFALSE
        
        if op == "=~":
            p = re.compile( str(var2), re.IGNORECASE )
            if p.match(str(var1)) != None:
                result = eTRUE
            else:
                result = eFALSE
        
        newstack.append( result )
      
      else:
        newstack.append( x[0] )
      
  return( newstack )
  


##
## evaluate_run_r
##
## Internal (recursive) function which evaluates a 'run' or 'installer'
## clause, following conditional paths.
##
def evaluate_run_r( e, conf, sysinfo, all_conditionals_true=False, returnShortcutProperties=False ):
  gamenotes = []
  shortcut_properties = []
  
  for x in e:
      if x == None:
          pass
          
      elif x[0] == "set":
          section = x[2][ x[2].find(".") + 1: ]
          option = x[2][ :x[2].find(".") ]
          
          conf.set_option( option, section, x[3] )
      
      elif x[0] == "if":
            try:
                broken = breakdown_if( x[1], conf, sysinfo )
            except KeyError, ke:
                raise Exception( "Problem with GDDB file around line " + str(x[3]) + ":  Invalid attribute " + ke.__str__() )
            except Exception, e:
                raise Exception( "Problem with GDDB file around line " + str(x[3]) + ":  " + e.__str__() )
            
            eval = 0
            if all_conditionals_true == True:
                eval = 1
            else:
                eval = boolexpr.evalbool( 0, len(broken) - 1, broken )[ 0 ]
            if eval == 1:
                if returnShortcutProperties == True:
                    (s, g) = evaluate_run_r( x[2], conf, sysinfo, all_conditionals_true=all_conditionals_true, returnShortcutProperties=returnShortcutProperties )
                    shortcut_properties += s
                    gamenotes += g
                else:
                    gamenotes += evaluate_run_r( x[2], conf, sysinfo, all_conditionals_true=all_conditionals_true, returnShortcutProperties=returnShortcutProperties )

      elif x[0] == "gamenote":
          gamenotes.append( x[1] )
      
      elif x[0] == "shortcut" and returnShortcutProperties == True:
          shortcut_properties.append( [x[1], x[2], x[3]] )
  
  if returnShortcutProperties == True:
      return [shortcut_properties, gamenotes]
  
  return gamenotes



def evaluate_run( e, conf, all_conditionals_true=False, returnShortcutProperties=False ):
    global sysinfo
    if sysinfo == None:
        sysinfo = {}
        cedegarc = ConfigParser.ConfigParser()

        try: # figure out the path to the .cedegarc file or use a plausible location
            cedegarc_path = os.getenv('CEDEGA_RC_PATH')
            assert cedegarc_path
        except:
            cedegarc_path = os.getenv('HOME') + "/.cedegarc"
       
        cedegarc.readfp( file( cedegarc_path, "r" ) )
        sysinfo_items = cedegarc.items( "system" )
        for (x, y) in sysinfo_items:
            sysinfo[ x ] = y
    
    try:
        g = evaluate_run_r( e, conf, sysinfo, all_conditionals_true, returnShortcutProperties )
        return g
    except Exception, e:
        import p2p_diag, gtk
        err_diag = p2p_diag.P2PMessageDialog( None, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, "GDDB	evaluation problem: " + str(e) )
        err_diag.run()
        err_diag.destroy()
        


def load_gddb( fn, silent=None ):

  if not os.access( fn, os.F_OK ):
      return None
  
  try:
      if use_parser == NEW_32_PARSER:
          r = gddb_parser.load( fn )
      elif use_parser == NEW_64_PARSER:
          r = gddb_parser64.load( fn )
      else:
          L = GDDBLexer.Lexer( fn )
          P = GDDBParser.Parser( L )
          global lexer
          lexer = L
          r = P.gddbfile()
  except antlr.ANTLRException, ex:
      if silent == None:
          import p2p_diag, gtk
          err_diag = p2p_diag.P2PMessageDialog( None, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, "Problem with GDDB file " + fn + "at line " + str(ex.line) + ": " + str(ex) )
          err_diag.run()
          err_diag.destroy()
      return None
  except Exception, e:
      if silent == None:
          import p2p_diag, gtk
          err_diag = p2p_diag.P2PMessageDialog( None, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, "I'm sorry, you have a corrupted GDDB file '" + fn + "' - not evaluating it: " + str(e) )
          err_diag.run()
          err_diag.destroy()
      return None
      
  
  return r


def scan( dir ):
    cmd = 'grep "game \\"" ' + dir + '/*.gddb | sed \'s/:game "/ /g\' | sed \'s/"$//g\''
    p = os.popen( cmd )
    l = p.readline()
    
    r = []
    
    while l != None and l != "":
        l = l[ :-1 ]

        game = l[ l.find(' ') + 1: ]
        gddb_file = os.path.basename( l[ :l.find(' ') ] )
        r.append( [game, gddb_file] )
        
        l = p.readline()

    return r


def detect( dir, setup_exe, volumes ):
    try:
        gddb_files = os.listdir( dir )
    except:
        return []
        

    for f in gddb_files:
        if f.rfind(".gddb") != -1:
            g = load_gddb( dir + "/" + f, silent=1 )
            
            if g != None:
                if g.has_key("game") and setup_exe != None:
                    if g["game"].has_key("exe"):
                        if os.path.basename(setup_exe) == g["game"]["exe"]:
                            return dir + "/" + f
                
                if g["game"].has_key("volume"):
                    for v in volumes:
                        for w in g["game"]["volume"]:
                            if v[0].lower() == w.lower():
                                return [ (dir + "/" + f), v, g["game"]["exe"], v[1] ]
    
    return None
            
