#!/usr/bin/env python

"""
Point2Play Source Code
Copyright (c) 2005 by TransGaming Technologies, Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written
permission of TransGaming Technologies Inc.

"""
import pygtk
pygtk.require('2.0')
import gtk
import gtk.glade
import log, copy
import os, shutil
from types import *
import gobject
import p2p_diag
import  p2p_gui
import cedega_version_parse
import ConfigParser
import winex_ver_gui, winex_get_gui, winex_get
import license
import manual_cedega_install
import Point2Play
import tempfile, socket, sys

INSTALL_DOWNLOAD_CHECK_UPDATES = 0
INSTALL_DOWNLOAD_PREVIOUS_VERSION = 1
INSTALL_MANUAL_PACKAGE = 2

class InstallCedegaEngineHelper:
    def __init__(self, Point2Play, caller_gui, CallingFromWizard=0):
        self.Point2Play = Point2Play
        self.caller_gui = caller_gui
        self.calling_from_wizard = CallingFromWizard 

        # In most cases, the calling gui will be the main gui...
        self.parent = p2p_gui.widgets["main"]

        # However...sometimes, it will be the wizard
        if CallingFromWizard:
            self.parent = caller_gui.widgets["wizard"]

        self.reset()


    def reset(self):
        self.filename = ""
        self.dirname = ""
        self.cwd = ""
        self.version = ""
        self.credential = "401"
        self.InternetInstall = 0
        self.manifest = ""
        self.containedfile = ""
        self.silent_install = 0

    def init_temp_directory(self):
        self.temp_directory = tempfile.mkdtemp()


    def remove_temp_directory(self):
        if os.access( self.temp_directory, os.F_OK ):
            shutil.rmtree( self.temp_directory )


    def extract_manifest_from_engine_package(self, filename):
        # We need to make sure there are no loading errors...
        error = 0

        # Unpack the engine package...	
        if (os.system( "tar xf '%s' -C '%s'" % ( filename, self.temp_directory) ) != 0):
            # Hrmm...that didn't work?  Maybe it was a gzipped tar file...
            if (os.system( "tar xzf '%s' -C '%s'" % ( filename, self.temp_directory) ) != 0):
                # Nope...well, signal the problem...
                # remove any unpacked files but do not delete cpkg as that is not necessarily temporary
                self.remove_temp_directory()
                error_dnld = p2p_diag.P2PMessageDialog(self.parent, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("Error extracting a file from the download archive - either you ran out of disk space or the file was somehow corrupted during download. Please try again"))
                resp = error_dnld.run()
                error_dnld.destroy()
                # Update the config file
                self.Point2Play.write()
                os.chdir( self.cwd )
                return

        # Next we need to sort out the [manifest] section of "manifest" file
        self.manifest = ConfigParser.ConfigParser()
        self.manifest.readfp( file( self.temp_directory + "/manifest", "r" ) )

        # Read in the manifest values...
        self.filename = os.path.join(self.temp_directory, self.manifest.get( "manifest", "containedfile" ))
        if not self.filename:
            error = 1

        self.version = self.manifest.get( "manifest", "version" )
        if not self.version:
            error = 1
        
        self.credential = self.manifest.get( "manifest", "credential" )
        if not self.credential:
            error = 1

        self.version_string = self.manifest.get( "manifest", "version_string" )
        if not self.version_string:
            error = 1

        if error:
            error_manifest = p2p_diag.P2PMessageDialog(self.parent, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("Error extracting a value from the engine archive - the file seems to be corrupt. Please try again"))
            resp = error_manifest.run()
            error_manifest.destroy()
						

    def install_cedega_engine(self, InstallMode, embedded_filename=None, SilentInstall=0):
        # Ok, we're looking to install a cedega engine package...are we doing
        # this through the 'net or manually?
        self.init_temp_directory()

        if InstallMode == INSTALL_DOWNLOAD_CHECK_UPDATES:
            self.get_latest_version()
        elif InstallMode == INSTALL_DOWNLOAD_PREVIOUS_VERSION:
            # ** ARPAnet Install **
            self.download_previous_cedega_engine()
        else:
            # ** Manual Install **
            if SilentInstall:
                # In a silent install, we won't show the license or regular dialogs.  Only allow error dialogs.
                # This is currently used for embedded timedemo packages.
                self.silent_install = 1

            # First of all, ensure that we have a temp directory to copy our manifest into...
            self.manually_install_cedega_engine(embedded_filename)

        # All 3 installation methods above unpack cpkg contents to tempdir, so may as well delete it only once here.
        self.remove_temp_directory()


    def create_new_cedega_directory(self, version_number):
        if version_number:
            self.dirname = os.path.join( self.Point2Play.gamedir, ".winex_ver", "winex-%s" % version_number)
            if os.path.islink( self.dirname ) or os.path.isfile( self.dirname ):
                os.unlink( self.dirname )
            elif os.path.isdir( self.dirname ):
                shutil.rmtree( self.dirname )
            os.makedirs( self.dirname )
            self.cwd = os.getcwd()
            os.chdir( self.dirname )


    def check_if_already_installed(self, version, version_string):
        """ Checks to see if the given version is already installed.
            If it is, we ask the user if he/she wishes to overwrite it.
            The return value will be:
                If it is already installed
                    If the user wishes to overwrite: False (we pretend it isn't there)
                    If the user does not wish to overwrite: True
                If not, False
            """

        for ver_loop in self.Point2Play.versions:
            if ver_loop["version"] == version:
                err_diag = p2p_diag.P2PMessageDialog(self.parent, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_YES_NO, _("You already have %s installed. Overwrite?") % (str(version_string)) )
                resp = err_diag.run()
                err_diag.destroy()
                if resp == gtk.RESPONSE_NO:
                    return True
                else:
                    return False
        return False

    def manually_install_cedega_engine(self, embedded_filename=None):
        # If an embedded filename was passed in, then we don't need to open the dialog for
        # the user to browse, since we already know the name and location of the file.
        if not embedded_filename:
            manual_cedega_install_dialog = manual_cedega_install.ManualCedegaInstallDialog(self.Point2Play, self)
            manual_cedega_install_dialog.widgets["manual_cedega_install"].set_transient_for(self.parent)

            # Launch the dialog, and retrieve the name and location of the engine package...
            filename = manual_cedega_install_dialog.run()
        else:
            filename = embedded_filename

        if filename and not os.access(filename, os.F_OK):
            err_diag = p2p_diag.P2PMessageDialog(self.parent, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("%s does not appear to exist. No engine installed.") % (filename))
            err_diag.run()
            err_diag.destroy()
            manual_cedega_install_dialog.widgets["manual_cedega_install"].destroy()
            return

        if filename and not self.silent_install:
            EULA_filename = self.caller_gui.Point2Play.P2PPATH + "LICENSE"
            EULA_fp = file(EULA_filename, "r" )
            EULA_diag = license.LicenseDialog( EULA_fp )
            resp = EULA_diag.run()
            EULA_diag.destroy()

            if resp == license.LICENSE_DECLINE:
                manual_cedega_install_dialog.widgets["manual_cedega_install"].destroy()
                return 1

        if not embedded_filename:
            manual_cedega_install_dialog.widgets["manual_cedega_install"].destroy()

        # Did the user give us a filename?
        if filename:
            self.cwd = os.getcwd()

            # Extract the manifest file...
            self.extract_manifest_from_engine_package(filename)

            # Check and see if this version has already been installed...
            if self.check_if_already_installed(self.version, self.version_string):
                return

            # Create the directory in .winex_ver to house the new version...
            self.create_new_cedega_directory( self.version )

            # There is no going back now, we need to remove the version from the list (if it was there before)
            for i in range(0, len( self.Point2Play.versions ) ):
                if self.Point2Play.versions[i]["version"] == self.version:
                    del self.Point2Play.versions[i]
                    break

            # Unpack the engine contents...
            self.unpackage_cedega_engine()



    def get_latest_version(self):
        # Show license
        EULA_filename = self.caller_gui.Point2Play.P2PPATH + "LICENSE"
        EULA_fp = file(EULA_filename, "r" )
        EULA_diag = license.LicenseDialog( EULA_fp )
        resp = EULA_diag.run()
        EULA_diag.destroy()

        if resp == license.LICENSE_DECLINE:
            p2p_gui.add_message_to_message_string( _("Cedega: License declined.") )
            return 1

        password = self.caller_gui.Point2Play.config["password"]

        self.caller_gui.Point2Play.setup_environment_for_proxy()

        try:
            winex_get.login_transgaming( self.caller_gui.Point2Play.config["username"], password )
        except socket.gaierror, e:
            p2p_gui.add_error_to_error_string( _("Cedega: Unable to connect to server\n%s") % e[1] )
            return 1 
        except:
            p2p_gui.add_error_to_error_string( _("Cedega: ") + str( sys.exc_info()[1] ) )
            return 1
        else:
            try:
                ver_list = winex_get.winex_versions()
            except socket.gaierror, e:
                p2p_gui.add_error_to_error_string( _("Cedega: Unable to connect to server\n%s\n") % e[1] )
                return 1
            except:
                p2p_gui.add_error_to_error_string( _("Cedega: ") + str( sys.exc_info()[1] ) )
                return 1
            if len( ver_list ):
                latest = ver_list[len( ver_list ) - 1]
                versionparse = cedega_version_parse.ParseVersion(latest["version"])
                latest["version"] = versionparse.get_parsed_version_string()
                self.version = latest["version"]

                # Only a version with credentials is Cedega
                if versionparse.get_version_credential() > 0:
                    productname = Point2Play.WINEX_DISPLAY_TITLE
                else:
                    productname = "WineX"

                self.version_string = ("%s Engine Version %s" % (productname, self.version))

                # If the version of Cedega to be downloaded has features that are more recent than this
                # version of Point2Play supports, we should warn the user.
                if not versionparse.check_cedega_version_support():
                    err_diag = p2p_diag.P2PMessageDialog( self.parent, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_YES_NO, _("The version of Cedega you wish to download (%s) contains features that this version of Cedega doesn't support. It is suggested you upgrade your version of Cedega.  Do you want to download this version of Cedega anyway?") % versionparse.get_parsed_version_string() )
                    resp = err_diag.run()
                    err_diag.destroy()
                    if resp == gtk.RESPONSE_NO:
                        p2p_gui.add_error_to_error_string( _("Cedega: Download aborted by user.") )
                        return 1

                file_diag = winex_get_gui.WinexGetDialog( self.caller_gui.Point2Play, ver_list, latest["version"] )
                file_diag.set_transient_for( self.parent )
                file_diag.set_title( _("Download %s Engine %s") % (productname, latest["version"]) )
                if not file_diag.arewe_good_to_go():
                    if file_diag.errorstring:
                        errorstring = file_diag.errorstring
                    else:
                        errorstring = _("Cedega: unknown error.")
                    p2p_gui.add_error_to_error_string( errorstring )
                    return 1

                self.filename = file_diag.run()
                gtk.input_remove( file_diag.input_handler )
                file_diag.destroy()
                if self.filename:
                    self.create_new_cedega_directory(latest["version"])

                    # There is no going back now, we need to remove the version from the list (if it was there before)
                    for i in range(0, len( self.Point2Play.versions ) ):
                        if self.Point2Play.versions[i]["version"] == self.version:
                            del self.Point2Play.versions[i]
                            break

                    if (self.filename.find("cpkg") != -1):
                        # Unpackage the downloaded engine package and access the manifest...
                        self.extract_manifest_from_engine_package( self.filename )

                    # Untar the cedega package...
                    self.unpackage_cedega_engine()
                    self.cleanup_slash_tmp()


    def download_previous_cedega_engine(self):
        """ Pops up the Install Older Engines dialog."""

        get_engine_dlg = winex_ver_gui.InstallOlderEngineDialog( self.Point2Play )
        if get_engine_dlg.had_error:
            return
        get_engine_dlg.widgets["install_older_engine_dialog"].set_transient_for(self.parent)
        res = get_engine_dlg.widgets["install_older_engine_dialog"].run()

        # Get the data from the popup
        self.version = get_engine_dlg.version
        self.version_string = get_engine_dlg.version_string
        versionparse = get_engine_dlg.sel_versionparse

        get_engine_dlg.widgets["install_older_engine_dialog"].destroy()

        if res == gtk.RESPONSE_CANCEL or res == gtk.RESPONSE_DELETE_EVENT:
            return

        if not self.version or not self.version_string:
            return

        # Has the user installed this version before? (and if so, does he/she
        # wish to overwrite?
        if self.check_if_already_installed(self.version, self.version_string):
            return

        # If the version of Cedega to be downloaded has features that are more recent than this
        # version of Point2Play supports, we should warn the user.
        if not versionparse.check_cedega_version_support():
            err_diag = p2p_diag.P2PMessageDialog(self.parent, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_YES_NO, _("The version of Cedega you wish to download (%s) contains features that this version of Cedega doesn't support. It is suggested you upgrade your version of Cedega.  Do you want to download this version of Cedega anyway?") % versionparse.get_parsed_version_string() )
            resp = err_diag.run()
            err_diag.destroy()
            if resp == gtk.RESPONSE_NO:
                return

        EULA_filename = self.Point2Play.P2PPATH + "LICENSE"
        EULA_fp = file(EULA_filename, "r" )
        EULA_diag = license.LicenseDialog( EULA_fp )
        resp = EULA_diag.run()
        EULA_diag.destroy()

        if resp == license.LICENSE_DECLINE:
            return

        # OK its good to go! let 'er rip
        file_diag = winex_get_gui.WinexGetDialog( self.Point2Play, get_engine_dlg.ver_list, self.version )
        if not file_diag.arewe_good_to_go():
            return

        file_diag.set_transient_for(self.parent)
        file_diag.set_title( _("Downloading %s") % ( self.version_string ) )
        self.filename = file_diag.run()
        gtk.input_remove( file_diag.input_handler )

        file_diag.destroy()
        if self.filename:
            self.create_new_cedega_directory(self.version)

            # There is no going back now.
            # Remove the version from the list (if it was there before)
            for i in range(0, len( self.Point2Play.versions ) ):
                if self.Point2Play.versions[i]["version"] == self.version:
                    del self.Point2Play.versions[i]
                    break

            if (self.filename.find("cpkg") != -1):
                # Unpackage the downloaded engine package and access the manifest...
                self.extract_manifest_from_engine_package( self.filename )

            # Untar the cedega package...
            self.unpackage_cedega_engine()

            self.cleanup_slash_tmp()

    def cleanup_slash_tmp(self):
        """ Get rid of the engine left behind in the /tmp dir. """

        files_in_tmp = os.listdir("/tmp")
        for curr_tmpfile in files_in_tmp:
            # check to see if there's a cpkg file with the same version we've just installed
            if curr_tmpfile.find(".cpkg") != -1 and curr_tmpfile.find(str(self.version)) != -1:
                log.logger.info("Cleanup: removing " + curr_tmpfile + " from /tmp")
                os.unlink("/tmp/" + curr_tmpfile)
                return

    def unpackage_cedega_engine( self ):
        # Try to untar the downloaded tarball of WineX - if it fails we bail out with an error message
        # The engine file should be tarred and gzipped...
        if (os.system( "tar xzf '%s'" % self.filename ) != 0):
            # Hrmm....that didn't work....see if it wasn't tarred...
            if (os.system( "tar xf '%s'" % self.filename ) != 0):
                os.unlink ( self.filename )
                error_dnld = p2p_diag.P2PMessageDialog(self.parent, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("Error extracting %s from the download archive - either you ran out of disk space or the file was somehow corrupted during download. Please try again") % self.version_string )
                resp = error_dnld.run()
                error_dnld.destroy()
                # Update the config file
                self.Point2Play.write()
                os.chdir( self.cwd )
                return

        # Add it to the list of WineX versions since the untaring seems to have worked
        os.chdir( self.cwd )
        self.Point2Play.versions.append( { "version":self.version, "path":self.dirname, "local": 1, "script":None, "retailinstalldir":None, "version_credential":self.credential} )
        os.unlink( self.filename )

        if self.version_string.find("Engine") == -1:  # interpolate it
            if self.version_string[0:6] == "Cedega":
                self.version_string = "Cedega Engine " + self.version_string[7:]
            else:  # shouldn't happen but we'll have it anyway
                self.version_string = "WineX Engine " + self.version_string[6:]

        # with a question about whether they want the new version the default or not.
        if self.Point2Play.default_winex != None and self.Point2Play.default_winex != "None":
            if not self.silent_install:
                install_done_diag = p2p_diag.P2PMessageDialog(self.parent, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_YES_NO, _("Installation of %s complete. Would you like to set this as the default engine to be used by Cedega?\n\nNote: The default engine version will be used for all applications run in Cedega unless otherwise specified in the shortcut properties dialog. The default engine version can be changed at any time in the Edit -> Global Settings dialog.") % ( self.version_string ) )
                resp = install_done_diag.run()
                install_done_diag.destroy()

            # If they said yes, we change the default
            if resp == gtk.RESPONSE_YES:
                self.Point2Play.set_default_winex(self.version)
            else:
                # There are no others installed - so we set the default no matter what
                # still need to notify the user though that the download is complete.

                # Copy over the new package's configuration profile.
                self.Point2Play.match_and_copy_default_cedega_profiles()
                if not self.silent_install:
                    install_done_diag = p2p_diag.P2PMessageDialog(self.parent, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("Installation of %s complete.") % (self.version_string))
                    resp = install_done_diag.run()
                    install_done_diag.destroy()

            # Enable widgets now that there is a game installed
            p2p_gui.widgets["install_menuitem"].set_sensitive( 1 )
            p2p_gui.widgets["install_button"].set_sensitive( 1 )

        else:
            # There are no others installed - so we set the default no matter what
            # still need to notify the user though that the download is complete.
            self.Point2Play.set_default_winex(self.version)
            if not self.silent_install:
                install_done_diag = p2p_diag.P2PMessageDialog(self.parent, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("Installation of %s complete.") % (self.version_string))
                resp = install_done_diag.run()
                install_done_diag.destroy()

        # Update the config file
        self.Point2Play.write()
