"""
Point2Play Source Code
Copyright 2003, TransGaming Technologies Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""

import os, gtk, p2p_diag

def drive_mounted( device_path ):
    fp = file( "/proc/mounts" )
    lines = fp.readlines()
    for i in lines:
        (dev, path, rest ) = i.split( " ", 2 )
        if dev == device_path:
            return 1
    return 0

class CDDialog( gtk.Dialog ):
    def __init__(self, cdroms, Point2PlayGui_instance):
        gtk.Dialog.__init__( self )
        self.Point2Play_gui = Point2PlayGui_instance

        # Remember the count of cdrom drives
        self.cdrom_drives_count = len(cdroms)

        # If we only have one cdrom drive, don't show ourselves at all.
        # Also we'll retain the cdrom information for this case (instead
        # of using the callback method as below, since that involves a popup
        # window, which we are trying to get rid of in the single CDROM drive case)
        if len(cdroms) <= 1:
           self.hide()
           self.singlecd_devpath = cdroms[0].getDev()
           self.singlecd_mountpath = cdroms[0].getMountPath()
           return
      

        table = gtk.Table( 2, len( cdroms ) )
        for i in range( 0, len( cdroms ) ):
           table.attach( gtk.Label( cdroms[i].getDev() + "  " + cdroms[i].getMountPath() ), 0, 1, i, i + 1 )
           if cdroms[i].isDriveMounted():
               button = gtk.Button( _("Unmount") )
           else:
               button = gtk.Button( _("Mount") )
           button.set_name( "round_button" )
           button.connect( 'clicked', self.mount_cb, cdroms[i].getMountPath(), cdroms[i].getDev() )
           table.attach( button, 1, 2, i, i + 1 )
        self.get_children()[0].add( table )
        button = self.add_button( gtk.STOCK_CLOSE, gtk.RESPONSE_CLOSE )
        button.set_name( "round_button" )
        self.connect( 'response', self.response_cb )
        self.connect( 'delete-event', self.delete_cb )
        self.set_title( _("Mount CDs") )
        table.show_all()

    def get_cdrom_count(self):
       return ( self.cdrom_drives_count )
       
    # Used in the case of only one CD drive - there is no popup in this case, therefore the code to un/mount
    # a cd, which is previously defined in button callbacks, isn't done. So we need to call it manually in this
    # case
    def mount_single_drive(self):
       return self.mount_cb(None, self.singlecd_mountpath, self.singlecd_devpath )

    def mount_cb( self, widget, mountpath, devpath ):
        if drive_mounted( devpath ):
            (retval, message ) = self.mount_cd( "-u", mountpath, devpath )
            label = _("Mount")
        else:
            ( retval, message ) = self.mount_cd( "-m", mountpath, devpath )
            label = _("Unmount")
        if retval:
            err_diag = p2p_diag.P2PMessageDialog( None, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_CANCEL, message )
            err_diag.run()
            err_diag.destroy()
        else:
            if widget:
               widget.set_label( label )
            return label   


    def response_cb( self, widget, response ):
        if response == gtk.RESPONSE_CLOSE:
            self.hide()
        elif response == gtk.RESPONSE_DELETE_EVENT:
            self.hide()

    def delete_cb( self, widget, *args ):
        widget.hide()
        return 1
 

    def mount_cd(self, action, path, devpath ):
       if action == "-m":
          cmd = "/bin/mount %s 2>&1" % path
       else:
          # The behvaiour for unmounting is different depending on
          # whether: 
          #  A) A winex is currently running
          #  B) that winex is a 4.0 featured winex
          # Then we make use of the new unmount/eject infrastructure
          # 
          # else we just unmount ourselves
          ejectused_40feature = 0

          # To ensure that we know whether Cedega is running, let's check and see
          # if a there are wineserver processes present...
          winerunning = os.popen("ps ax | grep wineserver", "r")
          output = winerunning.readlines()
          num_wine_lines = len(output) 
          winerunning.close()

          # greping for a process that isn't running can still produce up to 2 lines of
          # output...when Cedega is running, there are 2 wineserver process and then the
          # extra entries from the grep, adding up to a sum greater than 2.
          if num_wine_lines > 2:
              winex_running = self.Point2Play_gui.GetWineXRunningList()
              for ver in winex_running:
                  if ver["winexversion"]["version_credential"] >= 40:
                      ws_ejectfile = os.path.join(ver["winexprefix"], "wineserver_p2p_eject_device")
                      try:
                          devfile = file( ws_ejectfile, "w" )
                          devfile.write ( devpath + "\n" )
                          devfile.close()
                  
                          ejectused_40feature = 1
                      except:  
                          return ( 1, _("Error writing information required to unmount/eject CD to %s") % ws_ejectfile )
          
              if ejectused_40feature:
                  # Signal the wineserver(s)
                  try:
                      os.system ("kill -USR2 `ps ax | grep wineserver | awk '{print $1}'`")
                  except:
                      return (1, _("Error signaling the wineserver - shouldn't happen") )

                  # Nothing more to do.
                  return (0, "")
              else:
                  # If the wineserver is running, but our version is less than 40, unmount the CD in the
                  # old fashion way...
                  self.Point2Play_gui.ResetWineXRunningList()

                  # No 4.0 featured winex servers, do the old fashioned way
                  cmd = "/bin/umount %s 2>&1" % path
          else:
              # If we're doing unmount during non-installation/non-running periods, clear the self.WineXRunning list so
              # that it's fresh and up to date for the next run...
              self.Point2Play_gui.ResetWineXRunningList()

              # No 4.0 featured winex servers, do the old fashioned way
              cmd = "/bin/umount %s 2>&1" % path
		 
       fp = os.popen( cmd, "r" );
       lines = fp.readlines()
       retval = fp.close()
       if retval:
          err_str = "".join( lines )
          return ( retval, err_str )
       else:
          return ( 0, "" )

