#!/usr/bin/env python

"""
Point2Play Source Code
Copyright (c) 2005 by TransGaming Technologies, Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""

import pygtk
pygtk.require('2.0')
import gtk
import gtk.glade
import gobject
import log
import os, p2p_diag, config_gui, debug_gui, Point2Play, p2p_gui, options_gui
import config
import gddb
import gui_utils
import ISOHeader

INSTALL_INSTALL = 1
INSTALL_CANCEL  = 2
INSTALL_MOUNT   = 3
INSTALL_CONFIG  = 4
INSTALL_DEBUG   = 5

# IMPORTANT! We've overriden run(). Use that instead of the GtkDialog run().

p2ppath = os.getenv("P2PPATH")
GLADE_FILE = p2ppath + "/gui/" + __name__ + ".glade"

def get_active_text(combobox):
    model = combobox.get_model()
    active = combobox.get_active()
    if active < 0:
        try:   # for some reason, it doesn't set the entry as active if we've edited it. sheesh.
            entry = combobox.child
            return entry.get_text()
        except:
            return ""
    return model[active][0]

class WidgetsWrapper:
    def __init__(self):
        self.widgets = gtk.glade.XML(GLADE_FILE, None, "p2p_localized_msg")

    # Gives us the ability to do: widgets['widget_name'].action()
    def __getitem__(self, key):
        return self.widgets.get_widget(key)

class InstallDialog:
    def __init__(self, Point2Play, parent, game_name):
        self.widgets = WidgetsWrapper()
        self.Point2Play = Point2Play
        self.game_name = game_name
        self.parent = parent
        self.widgets.widgets.signal_autoconnect(self)
        self.build_program_title_combo()
        # We want to listen in on both the combo box entry and the filename entry boxes,
        # and gray out the continue button if it is empty
        self.widgets["filename_entry"].connect( 'changed', self.sensitive_cb)
        self.widgets["install_program_title_new"].connect( 'changed', self.sensitive_cb)

        self.widgets["continue_button"].set_sensitive( 0 )
        self.widgets["cdrom_monitoring"].set_active(self.Point2Play.get_cdrom_monitoring())
        
        # Setup GDDB combo entry box
        # (for next commit)
        self.gddb_scan = gddb.scan( self.Point2Play.P2PFOLDER + "/.gddb" )
        gddb_names = []
        gui_utils.setup_comboboxentry_completion(self.widgets["gddb_entry"], [])
        
        self.widgets[ "gddb_entry" ].append_text( "<None>" )
        for x in self.gddb_scan:
            self.widgets[ "gddb_entry" ].append_text( x[0] )
            
        # Handle config file stuff.  Here, we are creating a temporary config file called "install_config"
        # in the configuration_profiles folder.  It will be the equivalent of the default cedega currently
        # in use.  This "install_config" file will be deleted once installation is finished.
        self.install_profile =  self.Point2Play.create_install_profile()

    def populate_gddb( self, gddb_file ):
        notes = self.Point2Play.gddb_notes( None, None, gddb_file )

        gddb_prog = gddb.load_gddb( gddb_file )
        all_conf_paths = self.Point2Play.get_profile_load_list( None, None, install_gddb=gddb_file )
        all_conf = config.config_file( all_conf_paths )
        
        textbuffer = self.widgets["gn_general"].get_buffer( )        
        textbuffer.set_text( notes[2] )
        
        textbuffer = self.widgets["gn_play"].get_buffer( )       
        textbuffer.set_text( notes[0] )
        
        textbuffer = self.widgets["gn_install"].get_buffer( )          
        textbuffer.set_text( notes[1] )
      
        textbuffer = self.widgets["gn_all"].get_buffer( )
        textbuffer.set_text( notes[3] )

        self.widgets["gddb_notebook"].set_current_page(gddb.INSTALL_PAGE)


    def gddb_selected_cb( self, c ):
        text = self.widgets["gddb_entry"].get_active_text()
        self.real_gddb_selected = 0
        for (game, fn) in self.gddb_scan:
            if text == game:
                self.populate_gddb( self.Point2Play.P2PFOLDER + "/.gddb/" + fn )
                self.real_gddb_selected = 1
                break
        
        if self.real_gddb_selected == 0:
                textbuffer = self.widgets["gn_install"].get_buffer( )
                textbuffer.set_text( "" )
        


    def gddb_cb(self, button):
        volumes = []
        for cdrom in self.parent.cdroms:
            log.logger.info("Autodetecting an optical drive... about to check if its path is mounted.")
            if cdrom.isDriveMounted():
                iso = ISOHeader.ISOHeader( cdrom.getDev() )
                v = getattr( iso, "volume_id" )
                log.logger.info("Optical drive at " + cdrom.getDev() + ": " + str(v))
                volumes.append( [v, cdrom.mount_path] )
        
        detected = gddb.detect( self.Point2Play.P2PFOLDER + "/.gddb", None, volumes )
        gamename = ""
        if detected != None:

            count = 1
            for x in self.gddb_scan:
                if os.path.basename(x[1]) == os.path.basename(detected[0]):
                    self.widgets[ "gddb_entry" ].set_active( count )
                    gamename = x[0]
                    break
                
                count += 1
            self.widgets["filename_entry"].set_text( detected[1][1] + "/" + detected[2] )
            
            
            if len(get_active_text(self.widgets["install_program_title_new"])) < 1:
                count = 0;
                self.widgets[ "install_program_title_new" ].prepend_text( gamename )
                self.widgets[ "install_program_title_new" ].set_active( 0 )
            
            self.populate_gddb( detected[0] )


    def get_full_gddb_path( self ):
        gddb_path = ""
        for x in self.gddb_scan:
            if x[0] == self.widgets["gddb_entry"].get_active_text():
                gddb_path = self.Point2Play.P2PFOLDER + "/.gddb/" + x[1]
                break

        if not os.access( gddb_path, os.F_OK ):
            return None
        
        return gddb_path + "/installer"

        
    def build_program_title_combo(self):
        if self.Point2Play._games:
            games = list( self.Point2Play.games() )
            games.append ("")
            games.sort()
            self.widgets["install_program_title_new"].set_model(gtk.ListStore(gobject.TYPE_STRING))
            for i in games:
                self.widgets["install_program_title_new"].append_text(i)

            # If we have a game name, and its not "All", lets have that game
            # preselected
            if self.game_name and self.game_name != p2p_gui.ALL_GAMES_STRING:
               #self.widgets["install_program_title_combo"].entry.set_text ( self.game_name )
               self.widgets["install_program_title_new"].set_active(games.index(self.game_name))

    def get_install_profile_cedega_version(self):
        # Reload the file ( the values may have been changed by the options dialog ) and store the values.
        load_list = self.Point2Play.get_profile_load_list( None, None )
        load_list.append( (self.Point2Play.USER_DEFINED_PROFILES_PATH + self.install_profile) )
        game_config = config.config_file( load_list )

        # Return the current version of the install_config profile...
        return game_config.get_option("Version", "Cedega")
    
    def debug_cb(self, button):
        """ Set up the debugger popup. """
        debug_diag = debug_gui.DebugProgramDialog(self.Point2Play, standalone=0)
        debug_diag.widgets["debug_dialog"].set_transient_for(self.widgets["install"])
        response = debug_diag.run()
        debug_diag.widgets["debug_dialog"].destroy()
        if response != gtk.RESPONSE_CANCEL and response != gtk.RESPONSE_DELETE_EVENT:
            self.useDebugOptions = True
        
    
    def browse_cb(self,button):
        """ Pop up a file selection dialog for the installer. """
        self.filesel = gtk.FileChooserDialog(_("Select an Installer"))
        self.filesel.set_transient_for(self.widgets["install"])
        self.filesel.connect("response", self.filesel_response_cb)
        self.filesel.add_button(gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL)
        self.filesel.add_button(gtk.STOCK_OK,gtk.RESPONSE_OK)
        #self.filesel.ok_button.connect("clicked", self.browse_ok_cb)
        #self.filesel.hide_fileop_buttons()
        self.filesel.set_resizable(1)
        try:
            self.filesel.add_shortcut_folder(self.Point2Play.P2PFOLDER)
        except:
            pass

        # Ok, let's see if the user entered a path to browse...
        filename = self.widgets["filename_entry"].get_text()
        if not len( filename ):
            # The user has entered nothing, so we'll default to HOME...
            filename = os.environ["HOME"]

            # If there is a cdrom/dvd mounted, we'll go there first...
            for cdrom in self.parent.cdroms:
                if (cdrom.isDriveMounted()):
                    filename = cdrom.mount_path
        else:
            filename = filename[ :filename.rfind("/") ]

        self.filesel.set_current_folder(filename)

        filter1 = gtk.FileFilter()
        filter1.add_pattern("*.[Ee][Xx][Ee]")
        filter1.set_name("EXE Files")
        self.filesel.add_filter(filter1)

        filter2 = gtk.FileFilter()
        filter2.add_pattern("*")
        filter2.set_name("All Files")
        self.filesel.add_filter(filter2)

        self.filesel.run()

        # remove .cedega from the shortcuts list before we destroy 
        # the file chooser - this is system-wide!
        try:
            self.filesel.remove_shortcut_folder(self.Point2Play.P2PFOLDER)
        except:
            pass
        self.filesel.destroy()

    def filesel_response_cb(self, filesel, response):
        """ Put the fileselection popup data in the right place """
        if response==gtk.RESPONSE_OK:
            self.widgets["filename_entry"].set_text(self.filesel.get_filename())

    def mount_cb(self, button):
        self.parent.mount_cb(button)
        if self.parent.CDDialog.get_cdrom_count() > 1:
            self.parent.CDDialog.set_transient_for( self.widgets["install"] )
            self.parent.CDDialog.run()

    def help_cb(self,button):
        # FIXME - correct this text
        alert = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("Use this dialog to install a game, patch or demo. The program\ntitle can be whatever you choose. You need to specify a\ntitle and an installer (full path) before you can proceed."))

        alert.run()
        alert.destroy()

    def options_cb(self, button):
        """ Options callback """
        log.logger.info("Edit Install Configuration Profile")
        # Send in the install dialogue's config file so that options saves them before the user runs "continue"...
#        print "Self.install_profile is " + self.install_profile
#        print "get_profile_load_list: " + str(self.Point2Play.get_profile_load_list())
        manage_profiles_dialog = options_gui.OptionsDialog(self.Point2Play, self.widgets["install"], self.install_profile, None, None, None, 1, gddb_path=self.get_full_gddb_path() )
        manage_profiles_dialog.widgets["options"].set_transient_for(p2p_gui.widgets["main"])
        manage_profiles_dialog.run()
        manage_profiles_dialog.widgets["options"].destroy()

    def sensitive_cb( self, button ):
        if (len(self.widgets["filename_entry"].get_text()) > 0 and len(get_active_text(self.widgets["install_program_title_new"])) > 0 ):
            self.widgets["continue_button"].set_sensitive(1)

    def delete_install_config(self):
        log.logger.info("** new_install_gui.py - delete_install_config : Enter...")
        # Clean up the "install_config" file...by deleting it!
        # First, check to see that it still exists...
        install_config = self.Point2Play.USER_DEFINED_PROFILES_PATH + "install_config"
        if os.access(install_config, os.F_OK):
            log.logger.info("install_config found....delete it!")
            os.remove(install_config)
        log.logger.info("** new_install_gui.py - delete_install_config : Exit...")

    def run( self ):
        """ Runs and returns a tuple of eleven pieces of data when Continue is clicked """

        while 1:
            self.useDebugOptions = False
            response = gtk.Dialog.run( self.widgets["install"] )
            if response == gtk.RESPONSE_CANCEL or response == gtk.RESPONSE_DELETE_EVENT: 
                # Delete the install dialog's config profile...
                self.Point2Play.delete_install_profile()
                return ( None, None, 0, 0, 0, None, None, None, None, None, None, None )
            elif response == gtk.RESPONSE_OK:
                # "Continue" - start installing
                full_gddb_path = self.get_full_gddb_path()
                gddb_selection = self.widgets["gddb_entry"].get_active_text()
                if not full_gddb_path and gddb_selection != gddb.NONE_STRING and gddb_selection != "":
                    err_diag = p2p_diag.P2PMessageDialog( self.widgets["install"], 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("Could not find the specified GDDB Profile %s.\nPlease select one from the list (or choose %s).") % (gddb_selection, gddb.NONE_STRING) )
                    err_diag.run()
                    err_diag.destroy()
                    continue                    

                fname = self.widgets["filename_entry"].get_text()                
                new_game_folder = get_active_text(self.widgets["install_program_title_new"])
                if new_game_folder.find('/') >= 0 or new_game_folder.find('\\') >= 0:
                    err_diag = p2p_diag.P2PMessageDialog( self.widgets["install"], 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("Sorry, game folders may not have slashes in their names (either forward or backward)."))
                    err_diag.run()
                    err_diag.destroy()
                    continue

                new_game_folder_path = self.Point2Play.P2PFOLDER + new_game_folder
                if os.access(new_game_folder_path, os.F_OK) and not (os.access(new_game_folder_path, os.W_OK | os.R_OK | os.X_OK)) :
                    err_diag = p2p_diag.P2PMessageDialog( self.widgets["install"], 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("Sorry, the game folder %s already exists but has the wrong permissions. Please make sure it is user-readable, user-writable, and user-executable.") % (new_game_folder_path ))
                    err_diag.run()
                    err_diag.destroy()
                    continue

                if self.useDebugOptions:
                    dchan = self.Point2Play.config["debugger_last_channel"]
                    dfname = self.Point2Play.get_debugger_output_filename()
                    dcon = self.Point2Play.config["debugger_output_to_console"]
                else:
                    dchan = None
                    dfname = None
                    dcon = None
                
                if( os.access( fname, os.F_OK ) ):
                    versions = self.Point2Play.get_sorted_named_versions_list()
                    cedega_ver = self.get_install_profile_cedega_version()
                    return ( fname, new_game_folder, self.widgets["dos_cwd"].get_active(), self.widgets["big_exe"].get_active(), self.widgets["cdrom_monitoring"].get_active(), cedega_ver, self.install_profile, dchan, dfname, dcon, 0, full_gddb_path )
                else:
                    err_diag = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_CANCEL, _("The installer %s does not exist.") % (fname) )
                    err_diag.run()
                    err_diag.destroy()

