#!/usr/bin/env python

"""
Point2Play Source Code
Copyright (c) 2004 by TransGaming Technologies, Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""

import pygtk
pygtk.require('2.0')
import gtk, gobject
import gtk.glade
import log
import os, re
import gui_utils, license, p2p_gui, p2p_diag, system_detection
import config


# IMPORTANT! We've overriden run(). Use that instead of the GtkDialog run().

p2ppath = os.getenv("P2PPATH")
GLADE_FILE = p2ppath + "/gui/" + __name__ + ".glade"

GENERAL_PAGE=0
AUDIO_PAGE=1
GRAPHICS_PAGE=2
JOYSTICKS_PAGE=3
FONTS_PAGE=4
SYSTEM_PAGE=5

JOYSTICKDETAILS_CHANGED = 1
JOYSTICKDETAILS_CANCELED = 0

CANCEL_CLICKED = 0
OK_CLICKED = 1
DEFAULTS_CLICKED = 2
APPLY_CLICKED = 3

GLOBAL_CONF_STRING = _("Global Settings")

general_data_widgets_list = ["Windows", "Managed", "UseXVidMode", "UseXRandR",
    "DXGrab", "renice_wineserver", "ShowFPS", "ShowMem", "SHMWineserver",
    "scheduler_value", "PthreadsValue", "Desktop", "Browser", "MozillaControl", 
    "Cedega", "FreeType"]

audio_data_widgets_list = ["alsa_radio", "oss_radio", "pcm0", "ctl0",
    "alsaUseMMap", "dsp0", "mixer0", "ossFullDuplex",
    "ossUseMMap"]

graphics_data_widgets_list = ["VideoRam", "AGPVertexRam", "NV_VAR", "ARB_VBO",
    "AnisotropicTextureFiltering", "VertexShaders", "PixelShaders", "VertexShadersLevel",
    "GLSL", "DynamicVBO", "IndexVBO", "FBO", "FloatTextures", "PixelShadersLevel", "ClipSpaceFix", 
    "FixedProgram", "FragmentOffset", "RectangleTextures",
    "GLExtensionBuffer", "FixedGLExtensionBuffer", "OcclusionQueries"]


# all of the config-type widgets ...
data_widgets_list = general_data_widgets_list + audio_data_widgets_list + graphics_data_widgets_list

button_list = ["add_joystick_button", "remove_joystick_button", "joystick_properties_button",
                "add_font_button", "remove_font_button", "font_properties_button" ]

class WidgetsWrapper:
    def __init__(self):
        self.widgets = gtk.glade.XML(GLADE_FILE, None, "p2p_localized_msg")

    # Gives us the ability to do: widgets['widget_name'].action()
    def __getitem__(self, key):
        return self.widgets.get_widget(key)

class OptionsDialog:
    def __init__(self, Point2Play, parent, profile="", game=None, exe=None, shortcut_name = None, installer=None, gddb_path=None):
        self.Point2Play = Point2Play
        self.widgets = WidgetsWrapper()
        self.parent = parent
        self.profile = profile
        self.profile_full_path = self.Point2Play.USER_DEFINED_PROFILES_PATH + self.profile
        self.joysticks = []
        self.current_joystick = None
        self.model = self.joystick_list_model()
        self.widgets["joystick_treeview"].set_model( self.model )
        self.widgets["joystick_treeview"].get_selection().connect('changed', self.joystick_view_select_cb )
        self.game = game
        self.exe = exe
        self.shortcut_name = shortcut_name

        renderer = gtk.CellRendererText()
        column = gtk.TreeViewColumn( _("Joystick"), renderer, text=0)
        self.widgets["joystick_treeview"].append_column( column )

        self.widgets["remove_joystick_button"].set_sensitive(0)
        self.widgets["joystick_properties_button"].set_sensitive(0)

        # If for some reason, the incoming profile is MIA, we'll just use the default...
        if not self.profile:
            self.profile = self.Point2Play.get_profile_from_full_path(self.Point2Play.current_user_cedega_profile)
            self.profile_full_path = self.Point2Play.current_user_cedega_profile
        else:
            self.profile_full_path = self.Point2Play.USER_DEFINED_PROFILES_PATH + self.profile

        if self.profile == "":  # one-off, such as for the install dialog
            pass # FIXME - maybe copy the default profile? where does it get copied to so install uses it?

        # load the profile data in - to do so you need to figure out the full path.
        # Profiles can come from two different places now - Point2Play.USER_DEFINED_PROFILES_PATH or
        # Point2Play.DEFAULT_PROFILES_PATH. So it should be in one of those places.
        # Here's how to create the object that reads and writes the old-style game config files:
        #    self.game_config = config.config_file(profile_filename_with_path)
        log.logger.info("Load the values from this profile : " + self.Point2Play.USER_DEFINED_PROFILES_PATH + self.profile)
        
        self.gddb_path = gddb_path
        if self.profile == "install_config":
            load_list = Point2Play.get_profile_load_list(None, None, install_gddb=gddb_path)
            load_list.append( self.profile_full_path )
            self.game_config = config.config_file( load_list )
        else:
            if gddb_path == "":
                self.gddb_path = None
            self.game_config = config.config_file( Point2Play.get_profile_load_list(game, self.shortcut_name, install_gddb=self.gddb_path) )

        self.widgets.widgets.signal_autoconnect(self)

        self.populate_cedega_versions_combo()

        psl_cell = gtk.CellRendererText()
        self.widgets["PixelShadersLevel"].pack_start(psl_cell, True)
        self.widgets["PixelShadersLevel"].add_attribute(psl_cell, 'text', 0)
        vsl_cell = gtk.CellRendererText()
        self.widgets["VertexShadersLevel"].pack_start(vsl_cell, True)
        self.widgets["VertexShadersLevel"].add_attribute(vsl_cell, 'text', 0)


        self.set_shader_values(gui_utils.POST6) # just to start out...
        self.load_profile_data_into_widgets(self.game_config, data_widgets_list)

        # Load up the joysticks and populate the tree view...
        self.populate_joystick_treeview()

        # Ensure that the scheduler combo's value has the proper
        # influence on its child options...
        self.choose_scheduler_setting_cb()

        # Ensure that the ShowFPS checkbox has the proper influence on its child option
        self.showFPS_toggled_cb(None)

        # Grey out the sound card driver type that is not being used
        self.on_soundcarddrivertype_radio_toggled(None)

        # Set tabbing order        
        #self.setup_focus_chains()

        # set up title and top label
        if self.profile == GLOBAL_CONF_STRING:
            # note: I'm separating out the space to aid in translation
            self.widgets["options"].set_title(_("Edit" + " " + GLOBAL_CONF_STRING))
            self.widgets["profile_label"].set_text(GLOBAL_CONF_STRING)
        else:
            labeltext = GLOBAL_CONF_STRING + " + " + _("GDDB") + ": "
            if gddb_path == None or gddb_path == "":
                labeltext = labeltext + _("None")
            else:
                labeltext = labeltext + os.path.basename(gddb_path)[:-5] # get rid of .gddb

            if self.shortcut_name:
                self.widgets["options"].set_title(_("Edit Settings for") + " " + self.shortcut_name)
                labeltext = labeltext + _(" + settings for ") + self.shortcut_name
            else:
                if self.profile == "install_config":
                    self.widgets["options"].set_title(_("Edit Settings for Installation"))
                    labeltext = labeltext + _(" + Install Settings")
                else: # does this ever happen?
                    self.widgets["options"].set_title(_("Edit Settings"))
            self.widgets["profile_label"].set_text(labeltext)

        self.widgets["Cedega"].connect('changed', self.engine_or_glsl_changed_cb)
        self.widgets["GLSL"].connect('toggled', self.engine_or_glsl_changed_cb)
        self.widgets["FBO"].connect('toggled', self.fbo_toggled_cb)
        self.engine_or_glsl_changed_cb(None)
        self.fbo_toggled_cb(None)

    def fbo_toggled_cb(self, button):
        """ FBO is toggled - if it's off, grey out FloatTextures.
            If someone turns FBO on, enable FloatTextures as well. """

        if self.widgets["FBO"].get_active():
            self.widgets["FloatTextures"].set_sensitive(True)
            if button:  # if this function was called as a callback
                        # (since we also call it directly on opening the dialog)
                self.widgets["FloatTextures"].set_active(True)
        else:
            self.widgets["FloatTextures"].set_sensitive(False)
            self.widgets["FloatTextures"].set_active(False)


    def engine_or_glsl_changed_cb(self, button):
        """ Called when the engine version changes or GLSL is toggled.

            if (credential < 600)
                uncheck GLSL and disable it
                remove/disable the 2.0 and above selections from the ShadersLevel combos
                (only 1.x selections are valid)
            else
                enable GLSL
                if GLSLChecked
                    add 2.0 selections to PixelShadersLevel and VertexShadersLevel
                    combos
                else
                    remove/disable the 2.0 and above selections from the
                    ShadersLevel combos (only 1.x selections are valid)

        """

        log.logger.info("engine_or_glsl_changed_cb called.")
        current_engine_version = self.widgets["Cedega"].get_text()
        version_credential = self.Point2Play.get_version_credential(current_engine_version)
        psl_iter = self.widgets["PixelShadersLevel"].get_active_iter()
        if not psl_iter:
            log.logger.info("No pixel shader level found. Returning.")
            return
        psl_model = self.widgets["PixelShadersLevel"].get_model()
        psl = psl_model.get_value(psl_iter, 0)

        if current_engine_version == None or current_engine_version == "" or psl == None or psl == "":
            # this means we're in the middle of a change - the callback will be called again
            # when the final value is in place.
            log.logger.info("No current engine found.")
            return  
           
        log.logger.info("Engine version: " + current_engine_version + "; version credential: " + str(version_credential) + "; PSL: " + str(psl))
        
        if int(version_credential) < 600:
            self.widgets["GLSL"].set_sensitive(False)
            self.widgets["GLSL"].set_active(False)
            log.logger.info("Now put the pre-6.0 values into the shader level comboboxes.")
            self.set_shader_values(gui_utils.PRE6)
        else:
            self.widgets["GLSL"].set_sensitive(True)
            if self.widgets["GLSL"].get_active(): 
                log.logger.info("Now put the post-6.0 values into the shader level comboboxes.")
                self.set_shader_values(gui_utils.POST6)
            else:
                log.logger.info("Now put the pre-6.0 values into the shader level comboboxes.")
                self.set_shader_values(gui_utils.PRE6)


    def set_shader_values(self, credential_level):
        """ Place the appropriate values in the Comboboxes for PixelShadersLevel 
            and VertexShadersLevel. Use credential_level as a key into the
            dictionaries pixel_shader_levels and vertex_shader_levels.
        """ 
        log.logger.info("set_shader_values called.")

        try:
            psl_model = self.widgets["PixelShadersLevel"].get_model()
            vsl_model = self.widgets["VertexShadersLevel"].get_model()
            psl_active = self.widgets["PixelShadersLevel"].get_active()
            vsl_active = self.widgets["VertexShadersLevel"].get_active()
            psl = psl_model[psl_active][0]
            vsl = vsl_model[vsl_active][0]

            log.logger.info("In set_shader_values() with credential level of " + str(credential_level))
            log.logger.info("Selection was vsl: " + str(vsl) + "; psl: " + str(psl))
            restore_old_values = True
        except:
            log.logger.info("Not restoring old values...")
            restore_old_values = False

        psl_liststore = gtk.ListStore(gobject.TYPE_STRING)
        vsl_liststore = gtk.ListStore(gobject.TYPE_STRING)

        psls = gui_utils.pixel_shader_levels[credential_level]
        vsls = gui_utils.vertex_shader_levels[credential_level]

        # populate the liststores
        for level in psls:
            psl_liststore.append([level])
        for level in vsls:
            vsl_liststore.append([level])            

        self.widgets["PixelShadersLevel"].set_model(psl_liststore)
        self.widgets["VertexShadersLevel"].set_model(vsl_liststore)

        if restore_old_values:
            log.logger.info("Restore old values...")
            # restore selection if there was one
            pos = 0
            psl_success = False
            vsl_success = False
            for l in psls:
                log.logger.info("Comparing pixel shader level " + str(l) + " to " + str(psl))
                if l == psl:
                    self.widgets["PixelShadersLevel"].set_active(pos)
                    log.logger.info("Set pslevel to " + str(pos))
                    psl_success = True
                    break
                pos = pos + 1

            log.logger.info("Now to vertex shaders level...")
            pos = 0
            for l in vsls:
                log.logger.info("Comparing vertex shader level " + str(l) + " to " + str(vsl))
                if l == vsl:
                    self.widgets["VertexShadersLevel"].set_active(pos)
                    log.logger.info("Set vslevel to " + str(pos))
                    vsl_success = True
                    break
                pos = pos + 1

            # A previously-selected option may be no longer available.
            # For example, 2.0 was selected but is no longer in the pull-down.
            # If this happens, select the top-most available (position 0)

            if not psl_success:
                log.logger.info("Previously selected PixelShadersLevel no longer available... selecting next highest.")
                self.widgets["PixelShadersLevel"].set_active(0)
            if not vsl_success:
                log.logger.info("Previously selected VertexShadersLevel no longer available... selecting next highest.")
                self.widgets["VertexShadersLevel"].set_active(0)

    def showFPS_toggled_cb(self, button):
        """ This is called whenever the checkbox called ShowFPS (Activate TransGaming HUD)
            is toggled. Only when it is checked can we allow Show Memory Usage to be 
            checked or unchecked. """
        self.widgets["ShowMem"].set_sensitive(self.widgets["ShowFPS"].get_active())

    def setup_focus_chains(self):
        self.widgets["general_tab"].set_focus_chain(
            [ self.widgets["Windows"], self.widgets["combo3"],
              self.widgets["Cedega"], self.widgets["cedega_version_combo"],
              self.widgets["UseXVidMode"], self.widgets["UseXRandR"], 
              self.widgets["DXGrab"], self.widgets["Managed"], 
              self.widgets["MozillaControl"], self.widgets["renice_wineserver"], 
              self.widgets["SHMWineserver"], self.widgets["scheduler_value"], self.widgets["PthreadsValue"], 
              self.widgets["force_pthreads"], self.widgets["general_desktop_sizes"],
              self.widgets["Desktop"], self.widgets["Browser"], 
              self.widgets["FreeType"], 
              self.widgets["ShowFPS"], self.widgets["ShowMem"]
            ])
        self.widgets["audio_tab"].set_focus_chain(
            [ self.widgets["alsa_radio"],  self.widgets["ctl0"], 
              self.widgets["pcm0"], self.widgets["alsaFullDuplex"], 
              self.widgets["alsaUseMMap"], self.widgets["oss_radio"], 
              self.widgets["dsp0"], self.widgets["mixer0"],
              self.widgets["ossFullDuplex"], self.widgets["ossUseMMap"]
            ])
        self.widgets["graphics_tab"].set_focus_chain(
            [ self.widgets["VideoRam"], self.widgets["AGPVertexRam"],   
              self.widgets["PixelShaders"], self.widgets["PixelShadersLevel"], 
              self.widgets["graphics_shader_version"],
              self.widgets["VertexShaders"], self.widgets["VertexShadersLevel"], self.widgets["NV_VAR"], 
              self.widgets["ARB_VBO"], self.widgets["DynamicVBO"], 
              self.widgets["IndexVBO"], self.widgets["FixedGLExtensionBuffer"], 
              self.widgets["GLExtensionBuffer"], self.widgets["FixedProgram"], 
              self.widgets["combo5"],
              self.widgets["FragmentOffset"], self.widgets["combo6"],
              self.widgets["RectangleTextures"], self.widgets["combo7"],
              self.widgets["ClipSpaceFix"], self.widgets["AnisotropicTextureFiltering"],
              self.widgets["FBO"], self.widgets["FloatTextures"]
            ])
    
    def load_default_audio_data_into_widgets(self, game_config):
        """ Load the hardware/system settings that were autodetected or manually set in
            the wizard or the Hardware/System Settings dialog."""

        dict = system_detection.get_alsa_oss_info_from_cedegarc(self.Point2Play)
        if dict["alsa_radio"]:
            self.widgets["alsa_radio"].set_active(1)
            self.widgets["alsaUseMMap"].set_active(dict["alsaUseMMap"])
        else:
            self.widgets["oss_radio"].set_active(1)

        self.on_soundcarddrivertype_radio_toggled(None)

    def load_profile_data_into_widgets(self, game_config, data_widgets_list, LoadDefaults = False):
        gui_utils.load_gameconfig_data_into_widgets(self, data_widgets_list, game_config, None)

        # do something special with Cedega
        if "Cedega" in data_widgets_list:
            if self.profile == GLOBAL_CONF_STRING and not LoadDefaults:
                current_default_engine = self.Point2Play.default_winex
                self.widgets["Cedega"].set_text(str(current_default_engine))

    def populate_cedega_versions_combo(self):
        version_list = self.Point2Play.sort_winex_versions()
        if self.profile != GLOBAL_CONF_STRING:
            version_list.insert(0, "default")  # prepend

        if version_list: # should always be true
            self.widgets["cedega_version_combo"].set_popdown_strings(version_list)

    def apply_cb(self, button):
        """ Applies the options on the current notebook page """
        notebook_page = self.widgets["notebook"].get_current_page()

        if notebook_page == GENERAL_PAGE:
            # Save the altered ( or not ) options to the profile's file.
            gui_utils.save_gameconfig_data(self, general_data_widgets_list, self.game_config, self.profile_full_path, None,  self.Point2Play, self.game, self.exe, shortcut=self.shortcut_name)
            if self.profile == GLOBAL_CONF_STRING:
                # we may have changed the default engine
                self.save_default_engine_version()
                
        elif notebook_page == AUDIO_PAGE:
            gui_utils.save_gameconfig_data(self, audio_data_widgets_list, self.game_config, self.profile_full_path, None,  self.Point2Play, self.game, self.exe, shortcut=self.shortcut_name)
        elif notebook_page == GRAPHICS_PAGE:
            gui_utils.save_gameconfig_data(self, graphics_data_widgets_list, self.game_config, self.profile_full_path, None,  self.Point2Play, self.game, self.exe, shortcut=self.shortcut_name)
        elif notebook_page == JOYSTICKS_PAGE:
            self.save_joysticks()

        log.logger.info("Options applied on page " + str(notebook_page))

    def save_default_engine_version(self):
        """ Change the current default engine."""
        version_number = self.widgets["Cedega"].get_text()
        if version_number == "4.2ForSteam":
            version_number = "4.2forsteam" # make it lower case - otherwise this gets lost
        self.Point2Play.set_default_winex(version_number)
        self.Point2Play.write()

    def on_soundcarddrivertype_radio_toggled(self, button):
        """ When one of the radio buttons on the Audio page is toggled,
            we grey out the options under the other. For example, if OSS is selected,
            grey out the ALSA options."""

        alsa = 0
        oss = 0      
        if self.widgets["alsa_radio"].get_active():
            alsa = 1
        else:
            oss = 1
            
        self.widgets["dsp0"].set_sensitive(oss)
        self.widgets["mixer0"].set_sensitive(oss)
        self.widgets["ossUseMMap"].set_sensitive(oss)
        self.widgets["ossFullDuplex"].set_sensitive(oss)
        self.widgets["ctl0"].set_sensitive(alsa)
        self.widgets["pcm0"].set_sensitive(alsa)
        self.widgets["alsaUseMMap"].set_sensitive(alsa)

    def defaults_cb(self, button):
        """ Set GUI elements to default values for the current page. 
            If we are editing the global settings, we load in the profile for the currently 
            selected engine. Otherwise we load in the profile for the global default engine
            (Cedega dropdown will be set to "default"). 

            If we are on the Audio page, we need to bring in some information from the
            cedegarc file."""

        notebook_page = self.widgets["notebook"].get_current_page()
        default_config_file = None
        
        if self.profile == GLOBAL_CONF_STRING:
            engine = self.widgets["Cedega"].get_text()
        else:
            engine = 'default'
        gd_file = None
        if self.profile != GLOBAL_CONF_STRING:
            gd_file = self.Point2Play.P2PFOLDER + self.Point2Play.GLOBAL_DELTA_FILENAME
        default_config_file = config.config_file( [self.Point2Play.get_default_profile_for_engine(engine), gd_file, self.gddb_path] )

        if default_config_file:
            if notebook_page == GENERAL_PAGE:
                self.load_profile_data_into_widgets( default_config_file, general_data_widgets_list, LoadDefaults=True)
                # restore engine in Cedega dropdown
                self.widgets["Cedega"].set_text(engine)
            if notebook_page == AUDIO_PAGE:
                if self.profile == GLOBAL_CONF_STRING:
                    self.load_default_audio_data_into_widgets(default_config_file) # get hardware/system settings from the cedegarc
                else:
                    self.load_profile_data_into_widgets( default_config_file, audio_data_widgets_list)
            if notebook_page == GRAPHICS_PAGE:
                self.load_profile_data_into_widgets( default_config_file, graphics_data_widgets_list)
            if notebook_page == JOYSTICKS_PAGE:
                self.set_joysticks_to_defaults()
        log.logger.info("Defaults applied on page " + str(notebook_page))


    def ok_cb(self, button):
        log.logger.info("Options OK clicked")
        # Save the widgets...
        gui_utils.save_gameconfig_data(self, data_widgets_list, self.game_config, self.profile_full_path, None, self.Point2Play, self.game, self.exe, gddb_file=self.gddb_path, shortcut=self.shortcut_name)

        # if we've changed the default engine
        if self.profile == GLOBAL_CONF_STRING:
            # we may have changed the default engine
            self.save_default_engine_version()

        # Save the joysticks...
        self.save_joysticks()

        # Eject!
        self.widgets["options"].response(OK_CLICKED)

    def cancel_cb(self, button):
        log.logger.info("Options cancel clicked")
        self.widgets["options"].response(CANCEL_CLICKED)

    def run(self):
        """ Allows continued running of the dialog when Apply and Defaults are clicked. """

        while 1:
            response = gtk.Dialog.run( self.widgets["options"] )
            if response == CANCEL_CLICKED or response == gtk.RESPONSE_DELETE_EVENT:
                return 0
            elif response == OK_CLICKED:
                return 1
            else:
                pass  # apply or defaults clicked... keep running.


    ###########################################################################################################
    # - Scheduler API                                                                                          #
    ###########################################################################################################

    def turn_scheduler_on(self):
        # Ok, since the scheduler is on, we have to toggle some options.
        self.widgets["SHMWineserver"].set_active(1)
        self.widgets["SHMWineserver"].set_sensitive(0)
        self.widgets["renice_wineserver"].set_active(0)
        self.widgets["renice_wineserver"].set_sensitive(0)

        self.game_config.set_option("Wineserver", "Scheduler", "Yes")
#        self.game_config.write(self.profile_full_path)


    def turn_scheduler_off(self):
        # Ok, since the scheduler is on, we have to toggle some options.
        self.widgets["SHMWineserver"].set_sensitive(1)
        self.widgets["renice_wineserver"].set_sensitive(1)

        self.game_config.set_option("Wineserver", "Scheduler", "No")
#        self.game_config.write(self.profile_full_path)

    def choose_scheduler_setting_cb(self, text=None):
        # What is teh value in teh combo box?
        scheduler_value = self.widgets["scheduler_value"].get_text()

        # If yes....
        if scheduler_value == "Yes":
            # We wax on the scheduler...
            self.turn_scheduler_on()
        # If no...
        elif scheduler_value == "No":
            # We wax off the scheduler....
            self.turn_scheduler_off()
        # If default.....
        elif scheduler_value == "Default":
            # Hrmmm...we're in default mode...let's take a look at the kernel
            # version and decide which way to handle this....
            kernel_version = self.Point2Play.config["kernel"]

            # Kernels 2.6 and above will have the scheduler on by default...
            if kernel_version[0:3] >= "2.6":
                self.turn_scheduler_on()
            else:
                self.turn_scheduler_off()

    ###########################################################################################################
    # - Joystick API                                                                                          #
    ###########################################################################################################

    def joystick_list_model(self):
        model = gtk.ListStore( gobject.TYPE_STRING )
        return model

    def load_joysticks(self):
        # We don't know how many joysticks we have, or what they're called, so let's just
        # suck up the entire joysticks section and sift through...
        if self.game_config.has_section("joystick"):
            section = self.game_config.get_section("joystick", None)

            # Sifting...
            for js in section["options"]:
                # Extract the juicy values, and populate our array...
                self.joysticks.append( { "name":js["name"], "details":js["value"] } )

    def save_joysticks(self):
        # To make things clean and easy, let's erase all the stored joysticks in the
        # profile...this saves us cycling through and trying to determine which have
        # been deleted, which are added and the like...
        self.clear_joysticks_in_profile()

        # Now that we have a clear slate, let's write in our stored values...
        for js in self.joysticks:
            self.game_config.set_option( "joystick", js["name"], js["details"], None )

        # Save the changes permanently...
#        self.game_config.write(self.profile_full_path)

    def clear_joysticks_in_profile(self):
        # Get the joystick options from the profile...
        if self.game_config.has_section("joystick"):
            joystick_section = self.game_config.get_section("joystick", None)

            # Cycle through the list of joysticks in the profile, and delete them...
            for i in range(0, len(joystick_section["options"])):
                del joystick_section["options"][0]

    def set_joysticks_to_defaults(self):
        # First, clear the treeview...
        self.model.clear()
        self.joysticks = []

        # Next, reload values...
        self.populate_joystick_treeview()

    def populate_joystick_treeview(self):
        # First thing, load up the joysticks from the profile...
        self.load_joysticks()

        # Now cycle through the list and populate the tree view...
        for joystick in self.joysticks:
            iter = self.model.append()
            self.model.set_value(iter, 0, joystick["name"]) 

    def joystick_view_select_cb(self, selection):
        sel = selection.get_selected()
        if sel:
            model, iter = sel
            if iter:
                self.widgets["remove_joystick_button"].set_sensitive(1)
                self.widgets["joystick_properties_button"].set_sensitive(1)
                self.current_joystick = model.get_value(iter,0)

    def add_joystick_cb(self, button):
        log.logger.info("Options joystick add clicked")
        # FIXME - this stuff uses .point2playrc but we want per-game joystick configurations to be possible.

        # Launch the edit joystick dialog....
        NewJoystickD = NewEditJoystickDialog(self.parent, None, None)
        res = NewJoystickD.run()

        # Only if we get a new joystick back do we add it to the list
        if res == JOYSTICKDETAILS_CHANGED:
            # Add the new joystick to the list
            self.joysticks.append ( { "name":NewJoystickD.name_entry.get_text(), "details":NewJoystickD.details_entry.get_text() } )

        iter = self.model.append()
        self.model.set_value(iter, 0, NewJoystickD.name_entry.get_text())

        NewJoystickD.destroy()

        if len(self.model):
            self.widgets["remove_joystick_button"].set_sensitive(1)
            self.widgets["joystick_properties_button"].set_sensitive(1)

    def remove_joystick_cb(self, button):
        log.logger.info("Options joysticks remove clicked")
        sel = self.widgets["joystick_treeview"].get_selection().get_selected()
        if sel:
            model, iter = sel
            if iter:

                # If the number of joysticks is 0, then disable the buttons...
                if ( len(self.model) == 0 ):
                    self.widgets["remove_joystick_button"].set_sensitive(0)
                    self.widgets["joystick_properties_button"].set_sensitive(0)

                joystick = self.model.get_value( iter, 0 )

                # Ask the user if he is sure...
                yes_no_diag = p2p_diag.P2PMessageDialog( self.parent, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_YES_NO, _("Are you sure you want to delete %s?") % joystick )
                resp = yes_no_diag.run()
                yes_no_diag.destroy()
                if resp == gtk.RESPONSE_YES:
                    # Right , user wants to delete the joystick!
                    for i in range(0, len( self.joysticks ) ):
                        if self.joysticks[i]["name"] == joystick:
                            del self.joysticks[i]
                            break

                    # Remove the selected entry from the gui list...
                    model.remove(iter)

                    # If the number of joysticks is 0, then disable the buttons...
                    if ( len(self.model) == 0 ):
                        self.widgets["remove_joystick_button"].set_sensitive(0)
                        self.widgets["joystick_properties_button"].set_sensitive(0)
                    return
        
        # FIXME

    def joystick_properties_cb(self, button):
        log.logger.info("Options joysticks properties clicked")
        sel = self.widgets["joystick_treeview"].get_selection().get_selected()
        if sel:
            model, iter = sel
            if iter:
                list_item = self.model.get_value( iter, 0 )
                for i in range(0, len( self.joysticks ) ):
                    if self.joysticks[i]["name"] == list_item:
                        joystick = self.joysticks[i]
                        break
        
                NewJoystickD = NewEditJoystickDialog(self.parent, joystick["name"], joystick["details"])
                res = NewJoystickD.run()

                # Has anything changed?
                if res == JOYSTICKDETAILS_CHANGED:
                    joystick["name"] = NewJoystickD.name_entry.get_text()
                    joystick["details"] = NewJoystickD.details_entry.get_text()

                    # Update the treeview...
                    self.model.set_value(iter, 0, joystick["name"])

                # Destroy the dialog...
                NewJoystickD.destroy()


    def add_font_cb(self, button):
        log.logger.info("Options fonts add clicked")
        # FIXME

    def remove_font_cb(self, button):
        log.logger.info("Options fonts remove clicked")
        # FIXME
        
    def fonts_properties_cb(self, button):
        log.logger.info("Options fonts properties clicked")
        # FIXME


# Helper class for the new joystick details entry dialog box
class NewEditJoystickDialog( gtk.Dialog ):
    def __init__(self, JSDiag, old_name, old_details ):
        gtk.Dialog.__init__(self)

        # Initial setup stuff
        self.set_transient_for( JSDiag )
        self.set_title( _("Joystick Details"))
        vbox = self.get_children()[0]

        # First the box for the Joystick Name:
        vbox.add( gtk.Label( _("Name:") ) )
        self.name_entry = gtk.Entry()
        vbox.add( self.name_entry )

        # Now the Box for the details about the joystick
        vbox.add( gtk.Label( _("Configuration of the joystick axis. The joystick name and\naxis functions can be determined with jstest.\n") ) )
        self.details_entry = gtk.Entry()
        vbox.add( self.details_entry )


        # Add in the old name and details (if supplied to us)
        if (old_name):
           self.name_entry.set_text ( old_name )
        if (old_details):
           self.details_entry.set_text ( old_details )

        button = self.add_button( gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL )
        button.set_name( "round_button" )
        button = self.add_button( gtk.STOCK_OK, gtk.RESPONSE_OK )
        button.set_name( "round_button" )
        vbox.show_all()


    def run( self ):
        while 1:
            resp = gtk.Dialog.run( self )
            if resp == gtk.RESPONSE_OK:
                try:
                    return JOYSTICKDETAILS_CHANGED
                except ( Point2Play.Point2PlayError, IOError ), e:
                    err_diag = p2p_diag.P2PMessageDialog( self.parent, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, str(e) )
                    err_diag.run()
                    err_diag.destroy()
            else:
                return JOYSTICKDETAILS_CANCELED

