#!/usr/bin/env python

"""
Point2Play Source Code
Copyright (c) 2006 by TransGaming Technologies, Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""

import pygtk
pygtk.require('2.0')
import gtk, string
import gtk.glade, gtk.gdk
import log, p2p_diag, p2p_gui, Point2Play, os, shutil
import gobject


p2ppath = os.getenv("P2PPATH")
GLADE_FILE = p2ppath + "/gui/" + __name__ + ".glade"
	
class WidgetsWrapper:
    def __init__(self):
        self.widgets = gtk.glade.XML(GLADE_FILE, None, "p2p_localized_msg")

    # Gives us the ability to do: widgets['widget_name'].action()
    def __getitem__(self, key):
        return self.widgets.get_widget(key)

class RemoveEnginesDialog:
    def __init__(self, Point2Play, parent):
        self.Point2Play = Point2Play
        self.parent = parent
        self.widgets = WidgetsWrapper()
        self.widgets.widgets.signal_autoconnect(self)
        self.populate_listview()
        self.widgets["ok_button"].set_sensitive(0) # initially greyed out
        
    def populate_listview(self):
        model = gtk.ListStore( gobject.TYPE_STRING )
        version_list = self.Point2Play.sort_winex_versions()
        for version in version_list:
            iter = model.append()
            model.set_value( iter, 0, version )
        self.widgets["engine_view"].set_model(model)
        self.widgets["engine_view"].get_selection().connect( 'changed', self.engine_select_cb )
        self.widgets["engine_view"].get_selection().set_mode( 'multiple' )
        # Now we attach a rendering widget to the view tree
        cell = gtk.CellRendererText()
        column = gtk.TreeViewColumn( _("Version"), cell, text=0)
        self.widgets["engine_view"].append_column( column )
        self.widgets["heading_label"].set_text(_("\n Please select the engine version(s) that you wish to remove from your system. \n"))

    def engine_select_cb(self, button):
        """ Enable the OK button if something is selected."""
        self.widgets["ok_button"].set_sensitive(1)

    def ok_cb(self, button):
        """ Remove the selected engines after an Are you sure? dialog. """
        model, selected = self.widgets["engine_view"].get_selection().get_selected_rows()
        iters = [model.get_iter(path) for path in selected]
        removal_list = []
        for iter in iters:
            removal_list.append(model.get_value(iter, 0))

        removal_liststring = string.join( removal_list, "\n" )

        yesno_diag = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_YES_NO, _("Are you sure you wish to remove the following version(s) of the Cedega engine from your computer?\n\n%s") % (removal_liststring) )
        resp = yesno_diag.run()
        yesno_diag.destroy()
        if resp == gtk.RESPONSE_NO:
            self.widgets["remove_engines_dialog"].response(gtk.RESPONSE_CANCEL)
            return

        # if we got here, remove them
        for version in removal_list:
            log.logger.info("Removing " + version)
            dirname = os.path.join( self.Point2Play.gamedir, ".winex_ver", "winex-%s" % version )
            if os.path.islink( dirname ) or os.path.isfile( dirname ):
               os.unlink( dirname )
            elif os.path.isdir( dirname ):
               shutil.rmtree( dirname )

            # Find out the index in the list of versions
            winexver_pos = -1
            for i in range(0, len( self.Point2Play.versions ) ):
               if self.Point2Play.versions[i]["version"] == version:
                  winexver_pos = i
                  break
            if winexver_pos == -1: # shouldn't happen - we just got the version list after all
                continue # go to the next? FIXME

            # Next, get rid of it from the config file
            del self.Point2Play.versions[winexver_pos]

            # if we match the default, we gotta switch it...
            if self.Point2Play.default_winex == version:
                # OK, we're the default, either we blank it out or
                # set it to the first one available...
                if (len(self.Point2Play.versions) > 0):
                      self.Point2Play.set_default_winex(self.Point2Play.versions[0]["version"])
                else:     
                    self.Point2Play.set_default_winex(None)


            if self.Point2Play.default_winex: # if we haven't deleted all of the engines...
                # get rid of the default profile for that engine
                default_profile = self.Point2Play.get_default_profile_for_engine( version )
                try:
                   log.logger.info("Deleting the engine's default profile, " + default_profile)
                   os.unlink( default_profile )
                except:
                   log.logger.info("The engine version " + version + " did not appear to have a default profile.")    

                # This next step is very important - otherwise we'll
                # get strange errors about malformed configs
                # when deleting one or more non-default engines.
                self.Point2Play.update_current_default_cedega_profiles(self.Point2Play.default_winex)

        # save and update the list
        self.Point2Play.write()

        info_diag = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("Engine(s) removed successfully.\n\n") )
        info_diag.run()
        info_diag.destroy()
