#!/usr/bin/env python

"""
Point2Play Source Code
Copyright (c) 2004 by TransGaming Technologies, Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""

import pygtk
pygtk.require('2.0')
import gtk
import gtk.glade
import log
import os
import gui_utils
import system_detection, system_tests_gui, p2p_gui, p2p_diag, gui_utils, install_cedega_engine, config

p2ppath = os.getenv("P2PPATH")
GLADE_FILE = p2ppath + "/gui/" + __name__ + ".glade"

NUM_PAGES=5

tg_data_widgets_list = ["username", "password", "proxy_check", "proxy_host", "proxy_port",
                     "proxy_username", "proxy_password"]

data_widgets_list = tg_data_widgets_list + system_detection.system_data_widgets_list

class WidgetsWrapper:
    def __init__(self):
        self.widgets = gtk.glade.XML(GLADE_FILE, None, "p2p_localized_msg")

    # Gives us the ability to do: widgets['widget_name'].action()
    def __getitem__(self, key):
        return self.widgets.get_widget(key)

class SetupWizardDialog:
    def __init__(self, Point2Play):
        self.Point2Play = Point2Play
        self.widgets = WidgetsWrapper()
        #self.setup_focus_chains()
        self.hide_drawing_areas()
        self.hide_feedback_buttons()

        if not self.Point2Play.Point2Play_gui.TIMEDEMO:
            self.Point2Play.check_if_migration_needed()

        if self.Point2Play.config["wizard_version_last_run"] == "": # never run before
            self.set_defaults()
            self.detect_system_info()
            self.need_to_check_proxy_info_changed = False
        else: 
            # if the wizard was run before, see if there's any old info 
            # we can put into the blanks
            self.load_widget_data()
            self.old_proxy_host = self.Point2Play.config["proxy_host"]
            self.old_proxy_port = self.Point2Play.config["proxy_port"]
            self.old_proxy_username = self.Point2Play.config["proxy_username"]
            self.old_proxy_password = self.Point2Play.config["proxy_password"]
            self.need_to_check_proxy_info_changed = True
        self.widgets.widgets.signal_autoconnect(self)

        self.setup_labels()

    def setup_focus_chains(self):
        # make the focus chain on the transgamer acct page go 
        # left-top down, then right-top down.
        self.widgets["transgamer_settings_container"].set_focus_chain(
          [ self.widgets["username"],  self.widgets["password"],
            self.widgets["check_updates_checkbutton"],
            self.widgets["proxy_check"], self.widgets["proxy_host"],
            self.widgets["proxy_port"], self.widgets["proxy_username"],
            self.widgets["proxy_password"]
          ])

        self.widgets["tests_container"].set_focus_chain(
          [ self.widgets["wizard_4_run_test"],
            self.widgets["checkbutton_opengl_direct_rendering"],
            self.widgets["checkbutton_3d_accel"],
            self.widgets["checkbutton_threads"],
            self.widgets["checkbutton_copyprot"],
            self.widgets["checkbutton_oss"],
            self.widgets["checkbutton_alsa"],
            self.widgets["tests_moreinfo_opengl_button"],
            self.widgets["tests_moreinfo_3d_button"],
            self.widgets["tests_moreinfo_pthreads_button"],
            self.widgets["tests_moreinfo_copyprot_button"],
            self.widgets["tests_moreinfo_oss_button"],
            self.widgets["tests_moreinfo_alsa_button"]
          ])

    def set_defaults(self):
        # maybe there's some old username/password info
        gui_utils.load_config_data_into_widgets(self, tg_data_widgets_list, self.Point2Play)

        # force a call for initial insensitivity of proxy stuff
        self.proxy_check_cb(self.widgets["proxy_check"])
        self.widgets["back_button"].set_sensitive(0) # since we start on page 1

    def load_widget_data(self):
        gui_utils.load_config_data_into_widgets(self, data_widgets_list, self.Point2Play)
        self.load_passwords_into_widgets()
        self.proxy_check_cb(self.widgets["proxy_check"])

    def load_passwords_into_widgets(self):
        self.widgets["password"].set_text( str(self.Point2Play.config["password"]) )
        self.widgets["proxy_password"].set_text( str(self.Point2Play.config["proxy_password"]) )

    def detect_system_info(self):
        # not all the data widgets will be autodetected. It's OK to leave some out.
        system_info = system_detection.make_detected_system_info_dictionary()
        for w in system_detection.system_data_widgets_list:
            self.widgets[w].set_text( str(system_info[w]) )

    def detect_system_info_cb(self, button):
        self.detect_system_info()
        system_detection.detect_optical_drives(self.Point2Play, force=True)

    def on_back_button_clicked(self, button):
        """ Goes to the previous page in the notebook and sets up buttons appropriately. """
        current_page = self.widgets["notebook"].get_current_page()

        # if we're leaving the last page, restore the go-forward button its normal label
        if current_page == (NUM_PAGES - 1):
          self.widgets["forward_button"].set_label(_("Forward"))
          # note: don't move this code to __init__ and make the forward_button use that image.
          # The image will be destroyed, and if the user goes forward then back again, we'll
          # have a gaping hole. It seems it really does need to be created each time we get here.
          image = gtk.Image()
          image.set_from_stock(gtk.STOCK_GO_FORWARD, gtk.ICON_SIZE_BUTTON)
          self.widgets["forward_button"].set_image(image)

        # if we're leaving the second page, make the back button for page 1 insensitive
        if current_page == 1:
          self.widgets["back_button"].set_sensitive(0)

        # go to the previous page in the notebook
        self.widgets["notebook"].prev_page()

    def on_forward_button_clicked(self, button):
        """ Goes forward a page and sets up the buttons appropriately. """

        current_page = self.widgets["notebook"].get_current_page()

        # we're leaving the first page so we can make the back button sensitive again
        if current_page == 0:
            self.widgets["back_button"].set_sensitive(1)

        if current_page == 1:
            if self.widgets["check_updates_checkbutton"].get_active():
                # first store the info on this page in the Point2Play object.
                gui_utils.save_widget_data(self, tg_data_widgets_list, self.Point2Play)
                p2p_gui.check_for_updates(self, 1) # 1 means we're calling from the wizard
                self.widgets["check_updates_checkbutton"].set_active(0)

        # if we're on the last page the user has just clicked "Apply".
        # otherwise go to the next page
        if current_page == (NUM_PAGES - 1):
            self.finish_setup()
        else:
            if current_page == 1:

                if not self.Point2Play.versions:
                    # the user has not installed an engine
                    alert = p2p_diag.P2PMessageDialog( self.widgets["wizard"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("Please note that to work properly, Cedega requires at least one engine to be installed. Until that happens, functionality will be restricted."))
                    alert.run()
                    alert.destroy()
                else:
                    # now that there's an engine we can do the optical drive detection
                    # do we want to force it to re-detect if there is drive information? FIXME
                    system_detection.detect_optical_drives(self.Point2Play, force=False)
            self.widgets["notebook"].next_page()

            # change the forward button to Apply if we're on the last page
            if (current_page + 1) == (NUM_PAGES - 1):
                self.widgets["forward_button"].set_label(_("Finish"))
                # note: don't move this code to __init__ and make the forward_button use that image.
                # The image will be destroyed, and if the user goes forward then back again, we'll
                # have a gaping hole. It seems it really does need to be created each time we get here.
                image = gtk.Image()
                image.set_from_stock(gtk.STOCK_APPLY, gtk.ICON_SIZE_BUTTON)
                self.widgets["forward_button"].set_image(image)

    def local_update_cb(self, button):
        """ User wants to install a local package of the Cedega engine."""
        log.logger.warning("Install local update...")
        install_cedega_engine_helper = install_cedega_engine.InstallCedegaEngineHelper(self.Point2Play, self, CallingFromWizard=1)
        install_cedega_engine_helper.install_cedega_engine( InstallMode=install_cedega_engine.INSTALL_MANUAL_PACKAGE )



    def on_cancel_button_clicked(self, button):
        """ Handles cancel. Make sure an engine has been installed - otherwise ask 
            the user to continue or quit entirely. """
        self.widgets["wizard"].response(gtk.RESPONSE_CANCEL)

    def finish_setup(self):
        """ Finish has been clicked."""

        self.Point2Play.config["wizard_version_last_run"] = self.Point2Play.VERSION_STRING
        log.logger.info("Stored wizard_version_last_run = " + self.Point2Play.VERSION_STRING)

        gui_utils.save_widget_data(self, data_widgets_list, self.Point2Play)
        # if they set up the account information, enable some widgets in the main gui
        if (self.widgets["username"].get_text() and self.widgets["password"].get_text()):
            p2p_gui.widgets["check_for_updates_menuitem"].set_sensitive(1)
            p2p_gui.widgets["download_another_version_menuitem"].set_sensitive(1)

        if self.need_to_check_proxy_info_changed and self.proxy_info_changed():
            p2p_gui.notify_user(_("Changes to your proxy information may not take effect until Cedega is restarted."), self.widgets["wizard"])

        # Run the match_and_copy_default_cedega_profiles() function again ( it's always called at startup ).  We do
        # this because this function looks at the currently installed Cedega versions and makes a user-editable copy
        # of those default profiles only.  ( i.e. There should be a default config.cedega_3.2 file in the default
        # profiles folder, but if the user doesn't have it installed, a user editable copy will not be created. ) So
        # back to why this is here.  When the user first runs Point2Play out of the package, __init__ is called and
        # so is this function...but at the time, there are no versions of Cedega installed yet, therefore it doesn't
        # copy over any profiles.  So, we call this here, as the user SHOULD have a version of Cedega installed by the
        # time it's closed.
        self.Point2Play.match_and_copy_default_cedega_profiles()
        self.store_soundcard_driver_info()
        self.widgets["wizard"].response(gtk.RESPONSE_OK)


    # System Tests stuff...

    # When a new test is run, the visible drawing areas will be "wiped clean".  Erased
    # from existence?  Not quite that heavy...just hidden until the current tests are finished.
    def hide_drawing_areas(self):
        self.widgets["setup_wiz_opengl_drawing_area"].hide()
        self.widgets["setup_wiz_three_d_accel_drawing_area"].hide()
        self.widgets["setup_wiz_pthreads_drawing_area"].hide()
        self.widgets["setup_wiz_copy_protection_drawing_area"].hide()
        self.widgets["setup_wiz_oss_sound_drawing_area"].hide()
        self.widgets["setup_wiz_alsa_sound_drawing_area"].hide()

    # Much like the drawing areas, when a new test is run, the visible feedback buttons are
    # hidden, so that only the buttons reflecting the currently run test(s) show up.
    def hide_feedback_buttons(self):
        self.widgets["setup_wiz_opengl_feedback_button"].hide()
        self.widgets["setup_wiz_three_d_accel_feedback_button"].hide()
        self.widgets["setup_wiz_pthreads_feedback_button"].hide()
        self.widgets["setup_wiz_copy_protection_feedback_button"].hide()
        self.widgets["setup_wiz_oss_sound_feedback_button"].hide()
        self.widgets["setup_wiz_alsa_sound_feedback_button"].hide()


    # This is an important function.  It runs down a list of checkboxes and "checks" whether or not
    # they are checked or not.  If they are, then the test corresponding to the checkbox is added
    # to a list of tests to be run.
    def on_wizard_run_test_cb(self, button):
        # do the tests selected (we can't rely on the initial selection because
        # the user might have changed the checkboxes
        tests = ()

        if (self.widgets["checkbutton_opengl_direct_rendering"].get_active()):
            tests += ('test_gl',)

        if (self.widgets["checkbutton_3d_accel"].get_active()):
            tests += ('test_three_d',)

        if self.widgets["checkbutton_threads"].get_active():
             tests += ('test_pthread',)

        if self.widgets["checkbutton_copyprot"].get_active():
            tests += ('test_cdrom',)

        if (self.widgets["checkbutton_oss"].get_active()):
            tests += ('test_oss_sound',)

        if (self.widgets["checkbutton_alsa"].get_active()):
            tests += ('test_alsa_sound',)

        self.hide_feedback_buttons()
        self.hide_drawing_areas()

        for test in tests:
            exec('import ' + test)
            self.Point2Play.tests.runTest(test, self)


    # This is the function that is responsible for providing feedback from a run test.  After the test is
    # conducted, the test sends back some form of textual feedback and a result color.  This color is
    # either green, yellow or red, for Passed, Warning, and Failed respectively.  By passing in the test
    # name and color, we can locate the test's relative drawing area and color it with the result color.
    # The result color can also be sent to setup_feedback_button() to allow it to determine what the
    # button's label and message text should be.
    def update_drawing_area(self, test_name, test_feedback, test_status, result_color):
        if test_name == "test_gl":
            self.widgets["setup_wiz_opengl_drawing_area"].modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(result_color))
            self.widgets["setup_wiz_opengl_drawing_area"].show()
            self.setup_feedback_button(self.widgets["setup_wiz_opengl_feedback_button"], result_color)
        if test_name == "test_three_d":
            self.widgets["setup_wiz_three_d_accel_drawing_area"].modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(result_color))
            self.widgets["setup_wiz_three_d_accel_drawing_area"].show()
            self.setup_feedback_button(self.widgets["setup_wiz_three_d_accel_feedback_button"], result_color)
        if "test_pthread" == test_name:
            self.widgets["setup_wiz_pthreads_drawing_area"].modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(result_color))
            self.widgets["setup_wiz_pthreads_drawing_area"].show()
            self.setup_feedback_button(self.widgets["setup_wiz_pthreads_feedback_button"], result_color)
        if "test_cdrom" == test_name:
            self.widgets["setup_wiz_copy_protection_drawing_area"].modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(result_color))
            self.widgets["setup_wiz_copy_protection_drawing_area"].show()
            self.setup_feedback_button(self.widgets["setup_wiz_copy_protection_feedback_button"], result_color)
        if "test_oss_sound" == test_name:
            self.widgets["setup_wiz_oss_sound_drawing_area"].modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(result_color))
            self.widgets["setup_wiz_oss_sound_drawing_area"].show()
            self.setup_feedback_button(self.widgets["setup_wiz_oss_sound_feedback_button"], result_color)
        if "test_alsa_sound" == test_name:
            self.widgets["setup_wiz_alsa_sound_drawing_area"].modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(result_color))
            self.widgets["setup_wiz_alsa_sound_drawing_area"].show()
            self.setup_feedback_button(self.widgets["setup_wiz_alsa_sound_feedback_button"], result_color)


    # As mentioned above, this function takes in a result color, which allows the button to determine the
    # results of the test, and thus display the correct feedback button label and messagebox text.
    def setup_feedback_button(self, button, result_color):
        # After the test is run, we can tell how it went based on its result color. So we'll check the color
        # and set the label of the button to match the color accordingly.
        if result_color == "green":
            button.set_label("Passed")
        elif result_color == "yellow":
            button.set_label("Warning")
        else:
            button.set_label("Failed")

        # Now that the button's text has been properly determined, let's reveal our masterpiece!
        button.show()

    # Feedback button handler functions...
    def on_setup_wiz_opengl_feedback_button_clicked(self, button):
        system_tests_gui.on_opengl_feedback_button_clicked(self, button)

    def on_setup_wiz_three_d_accel_feedback_button_clicked(self, button):
        system_tests_gui.on_three_d_accel_feedback_button_clicked(self, button)

    def on_setup_wiz_pthreads_feedback_button_clicked(self, button):
        system_tests_gui.on_pthreads_feedback_button_clicked(self, button)

    def on_setup_wiz_copy_protection_feedback_button_clicked(self, button):
        system_tests_gui.on_copy_protection_feedback_button_clicked(self, button)

    def on_setup_wiz_oss_sound_feedback_button_clicked(self, button):
        system_tests_gui.on_oss_sound_feedback_button_clicked(self, button)

    def on_setup_wiz_alsa_sound_feedback_button_clicked(self, button):
        system_tests_gui.on_alsa_sound_feedback_button_clicked(self, button)

    # More Info button handler functions...
    def on_tests_moreinfo_alsa_clicked(self, button):
        system_tests_gui.on_tests_moreinfo_alsa_clicked(self,button)

    def on_tests_moreinfo_oss_clicked(self, button):
        system_tests_gui.on_tests_moreinfo_oss_clicked(self, button)

    def on_tests_moreinfo_copyprot_clicked(self, button):
        system_tests_gui.on_tests_moreinfo_copyprot_clicked(self, button)

    def on_tests_moreinfo_pthreads_clicked(self, button):
        system_tests_gui.on_tests_moreinfo_pthreads_clicked(self, button)

    def on_tests_moreinfo_3d_clicked(self, button):
        system_tests_gui.on_tests_moreinfo_3d_clicked(self, button)

    def on_tests_moreinfo_opengl_clicked(self, button):
        system_tests_gui.on_tests_moreinfo_opengl_clicked(self, button)

    def proxy_port_changed_cb(self, button):
        """ If the user tries to enter a non-number, clear the field """
        try:
            x = int(self.widgets["proxy_port"].get_text())
        except:
            self.widgets["proxy_port"].set_text("")
        
    def proxy_check_cb(self, button):
        active = self.widgets["proxy_check"].get_active()
        for w in ["proxy_host", "proxy_port", "proxy_username", "proxy_password"]:
            self.widgets[w].set_sensitive(active)

    def proxy_info_changed(self):
        if self.old_proxy_host != self.widgets["proxy_host"].get_text():
            log.logger.info(self.old_proxy_host + " -- " + self.widgets["proxy_host"].get_text())
            return True
        if str(self.old_proxy_port) != self.widgets["proxy_port"].get_text():
            log.logger.info(str(self.old_proxy_port) + " -- " + str(self.widgets["proxy_port"].get_text()))
            return True
        if self.old_proxy_username != self.widgets["proxy_username"].get_text():
            log.logger.info(self.old_proxy_username + " -- " + self.widgets["proxy_username"].get_text())
            return True
        if self.old_proxy_password != self.widgets["proxy_password"].get_text():
            log.logger.info(self.old_proxy_password + " -- " + self.widgets["proxy_password"].get_text())
            return True
        return False

    def setup_labels(self):
        """ These strings have been removed from the glade file due to problems with translations.
            We've found that multi-line labels and labels with HTML markers don't seem to work properly
            unless they are put into the GUI this way. Not sure why. """
        self.widgets["wiz1info2_label"].set_text(_("Welcome to the Cedega setup wizard.  This wizard is designed to help configure your computer and Cedega for best performance with your games.\n\nThe wizard will lead you through the following stages:\n\nTransGamer Account Setup\n   - Please have your TransGamer username and password ready\nChecking for available updates\n   - If you are not installing a local update, an active internet connection is required\nSystem Autodetection\n   - The wizard will attempt to detect various computer settings and options\nSystems Tests\n   - The wizard will run a number of systems tests to determine if your computer is correctly set up for use with Cedega\n\nPlease note that attempting to quit from the Setup Wizard without having installed a\nCedega engine will close Cedega.\n\n--\nThank you for choosing Cedega."))
        self.widgets["system_information_label"].set_text(_("<b>System Information</b>"))
        # this is in the glade file but I seem to have to specify it again after the set_text
        self.widgets["system_information_label"].set_use_markup(True) 

        self.widgets["video_card_label"].set_text(_("<b>Video Card</b>"))
        self.widgets["video_card_label"].set_use_markup(True)

        self.widgets["hardware_information_label"].set_text(_("<b>Hardware Information</b>"))
        self.widgets["hardware_information_label"].set_use_markup(True)

        self.widgets["sound_card_label"].set_text(_("<b>Sound Card</b>"))
        self.widgets["sound_card_label"].set_use_markup(True)

        self.widgets["final_message"].set_text(_("Congratulations you have successfully completed the Cedega setup wizard.  Should you wish to rerun this wizard in the future it can be accessed in the Tools menu -> Run Setup Wizard. Configuration and systems settings may be modified at any time using the Edit menu.\n\nPlease refer to the How To Guide for more detailed information on configuring and using Cedega.\n\nLet the games begin..."))

    def store_soundcard_driver_info(self):
        """ Store the soundcard driver info in the user's .global.delta file as well
            as in the cedegarc file so that it will be available in the Audio tab
            of the global settings the first time we go there. This should be called
            only the first time we use the wizard. If the user changes global information
            after that, it will not be changed, but they can revert to the detected
            values by clicking "Default", to bring in the data from the cedegarc again."""

        # by the time we call this function, the data has been stored in the cedegarc.
        # Get the values we need.
        dict = system_detection.get_alsa_oss_info_from_cedegarc(self.Point2Play)

        # open the global delta file:
        profile = self.Point2Play.P2PFOLDER + self.Point2Play.GLOBAL_DELTA_FILENAME
        gdconfig = config.config_file( self.Point2Play.get_profile_load_list(None, None) )
        if dict["alsa_radio"]:
            gdconfig.set_option( "transgaming", "alsa_radio", "Y", None)
            gdconfig.set_option( "transgaming", "oss_radio", "N", None)
            if dict["alsaUseMMap"]:
                gdconfig.set_option( "winealsa", "UseMMap", "Y", None)
            else:
                gdconfig.set_option( "winealsa", "UseMMap", "N", None)
            gdconfig.set_option( "WinMM", "Drivers", "winealsa.drv", None)
        else:
            gdconfig.set_option( "transgaming", "alsa_radio", "N", None)
            gdconfig.set_option( "transgaming", "oss_radio", "Y", None)
            gdconfig.set_option( "WinMM", "Drivers", "wineoss.drv", None)

        dconfig = config.config_file( self.Point2Play.get_profile_load_list(None, None, 2) )
        gdconfig.write(profile, dconfig)
