#!/usr/bin/env python

"""
Point2Play Source Code
Copyright (c) 2006 by TransGaming Technologies, Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""

import pygtk
pygtk.require('2.0')
import gtk
import gtk.glade, gtk.gdk
import os, log, p2p_diag, p2p_gui, options_gui, Point2Play
import gobject
import misc_utils, time, signal
import gddb
import gui_utils

p2ppath = os.getenv("P2PPATH")
GLADE_FILE = p2ppath + "/gui/" + __name__ + ".glade"
	
class WidgetsWrapper:
    def __init__(self):
        self.widgets = gtk.glade.XML(GLADE_FILE, None, "p2p_localized_msg")

    # Gives us the ability to do: widgets['widget_name'].action()
    def __getitem__(self, key):
        return self.widgets.get_widget(key)

class ShortcutPropertiesDialog:
    def __init__(self, Point2Play, parent, shortcut_index):
        self.Point2Play = Point2Play
        self.parent = parent
        self.shortcut_index = shortcut_index
        (self.game_folder, self.initial_shortcut_name) = self.parent.get_selected_names()
        self.widgets = WidgetsWrapper()
        self.widgets.widgets.signal_autoconnect(self)
        self.widgets["shortcut_properties"].set_title(_("Shortcut Properties for %s") % self.game_folder )
        self.current_profile = ""
        self.icon = ""
        self.widgets["application_directory_label"].set_text(self.game_folder)
        
        if self.shortcut_index >= 0: # we're editing properties, not creating a new shortcut
            props = self.Point2Play.get_shortcut_properties_by_index(self.game_folder, self.shortcut_index)
            self.populate_widgets(props)
        else: # grey out the OK and extract icon buttons until we fill in necessary blanks
            self.widgets["ok_button"].set_sensitive(0)
            self.widgets["extract_icon_button"].set_sensitive(0)
            self.widgets["settings_button"].set_sensitive(0)

        # if there are no engines available, disable the extract icon button and settings button
        if not len(self.Point2Play.versions):
            self.widgets["extract_icon_button"].set_sensitive(0)
            self.widgets["settings_button"].set_sensitive(0)

        self.widgets["shortcut_name"].connect( 'changed', self.sensitive_cb)
        self.widgets["path_to_program"].connect( 'changed', self.sensitive_cb)
        
        self.gddb_scan = gddb.scan( self.Point2Play.P2PFOLDER + "/.gddb" )
        gddb_names = []
        count_x = 1

        gui_utils.setup_comboboxentry_completion(self.widgets["profile_combo"], [])
        
        self.widgets[ "profile_combo" ].append_text( gddb.NONE_STRING )
        
        for x in self.gddb_scan:
            self.widgets[ "profile_combo" ].append_text( x[0] )
            
            if self.shortcut_index >= 0: # if editing properties
                if props.has_key('gddb') and x[1] == props["gddb"]:
                    self.widgets[ "profile_combo" ].set_active( count_x )
            
            count_x += 1

        # this has been transported from the glade file because of locale problems
        # (being multi-line and using markup each seem to mess things up)
        self.widgets["bottom_section_label"].set_text(_("<b>Advanced Settings Overrides</b>"))
        self.widgets["settings_button_label"].set_text(_("(These settings override the Global and GDDB settings)"))
        self.widgets["bottom_section_label"].set_use_markup(True)
            

    def sensitive_cb(self, widget):
        """ We require the Name and Path to Program entries to be filled before we can create
            the shortcut or extract the icon from the executable. """

        sensitive = ( len(self.widgets["shortcut_name"].get_text()) > 0) & ( len(self.widgets["path_to_program"].get_text()) > 0)
        self.widgets["ok_button"].set_sensitive(sensitive)
        self.widgets["extract_icon_button"].set_sensitive(sensitive)
        self.widgets["settings_button"].set_sensitive(sensitive)

    def populate_widgets(self, props):
        """ Fill in the blanks for a shortcut we are editing (rather
            than creating) """
        self.icon = props["Icon"]
        self.widgets["icon_button_image"].set_from_file(self.icon)
        self.widgets["shortcut_name"].set_text(props["Name"])
        self.widgets["working_directory"].set_text(props["Workdir"])
        self.widgets["path_to_program"].set_text(props["Path"])
        self.widgets["command_line_options"].set_text(props["CommandLineArguments"])
        if props["CedegaApp"] == "N":
            self.widgets["non_cedega_app_checkbutton"].set_active(1)

    def lookup_current_profile(self):
        if self.shortcut_index >= 0:
            exe = self.Point2Play.lookup_game_exe(self.game_folder, self.initial_shortcut_name)
            self.current_profile = self.Point2Play.lookup_game_profile_association(exe, self.game_folder)

    def help_cb(self, button):      
        alert = p2p_diag.P2PMessageDialog( self.widgets["shortcut_properties"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("The Shortcut Properties Dialog allows users to specify settings such as name and icons to be used for each available shortcut.\n\nPath to Program - This is the path to the program executable to be associated with the shortcut.\n\nNative Linux Application - Check this box if the application is a native Linux application that you wish to launch via the Cedega GUI.\n\nExtract icon from program - Attempts to retrieve an icon embedded in the executable file.\n\nWorking Directory - Specify a working directory for the executable.\n\nCommand Line Options - Specify \"options\" or \"switches\" to be passed directly to the program executable.\n\nGDDB Profile - Specify an available GDDB Profile to use with this shortcut.\n\nEdit Settings - Manual override for Global Settings and GDDB Profiles.\n"))
        alert.run()
        alert.destroy()

    def get_current_profile(self):
        profile_index = self.widgets["profile_combo"].get_active()
        profile = ""
        if profile_index > 0:
            profile = self.Point2Play.get_nth_profile(profile_index - 1)
        return profile

    def edit_cb(self, button):
        if self.shortcut_index >= 0:
            gddb_path = ""
            for x in self.gddb_scan:
                if x[0] == self.widgets["profile_combo"].get_active_text():
                    gddb_path = self.Point2Play.P2PFOLDER + "/.gddb/" + x[1]
                    break
            exe = self.Point2Play.lookup_game_exe(self.game_folder, self.initial_shortcut_name)
            manage_profiles_dialog = options_gui.OptionsDialog(self.Point2Play, self.widgets["properties"], exe, self.game_folder, exe, self.initial_shortcut_name, gddb_path=gddb_path)
            manage_profiles_dialog.widgets["options"].set_transient_for(self.widgets["properties"])
            manage_profiles_dialog.run()
            manage_profiles_dialog.widgets["options"].destroy()
        else:
            alert = p2p_diag.P2PMessageDialog( self.widgets["shortcut_properties"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("Please create the shortcut before editing the engine settings."))
            alert.run()
            alert.destroy()

    def browse_helper(self, widget_name, get_directory):
        """ Use the data in widget_name as a starting point for the
            file-browser, and if there is none (yet), start at
            the .cedega/GAMETITLE directory."""

        file_diag = p2p_diag.P2PFileSelection(get_directory)
        if get_directory:
            file_diag.set_title(_("Select a Working Directory for this Shortcut"))
        else:
            file_diag.set_title(_("Select an Executable for this Shortcut"))

        file_diag.set_resizable(1)
        filename = self.widgets[widget_name].get_text()
        starting_dir = ""
        if len(filename):  # figure out a likely starting-point
            if misc_utils.is_URL(filename):
                if filename[0:7] == "file://": # if it's a file URL, use that directory
                    starting_dir = os.path.dirname(filename[7:])
            else: # a path
                if misc_utils.is_windows_path(filename):  # convert to unix
                    filename = Point2Play._windows_to_unix_path(os.path.dirname(filename), self.Point2Play.gamedir + self.game_folder + "/c_drive") + "/" + os.path.basename(filename)
                starting_dir = os.path.dirname(filename)
        if not len(starting_dir): # if filename was empty or a URL, plonk us in the .cedega/GAMETITLE dir
            starting_dir = os.path.join( self.Point2Play.gamedir, self.game_folder) + "/"

        if starting_dir[0] == '/':  # the directory has to be absolute
            file_diag.set_current_folder( starting_dir )

        try:
            file_diag.add_shortcut_folder(self.Point2Play.P2PFOLDER)
        except:
            pass

        if not get_directory:  # then we're looking for an EXE file...
            filter1 = gtk.FileFilter()
            filter1.add_pattern("*.[Ee][Xx][Ee]")
            filter1.set_name("EXE Files")
            file_diag.add_filter(filter1)

            filter2 = gtk.FileFilter()
            filter2.add_pattern("*")
            filter2.set_name("All Files")
            file_diag.add_filter(filter2)

        resp = file_diag.run()
        filename = file_diag.get_filename()
        if resp == gtk.RESPONSE_OK:
            self.widgets[widget_name].set_text( filename )

        # remove .cedega from the shortcuts list before we destroy 
        # the file chooser - this is system-wide!
        try:
            file_diag.remove_shortcut_folder(self.Point2Play.P2PFOLDER)
        except:
            pass

        file_diag.destroy()


    def browse_for_working_directory_cb(self, button):
        self.browse_helper("working_directory", 1)

    def browse_for_path_cb(self, button):
        self.browse_helper("path_to_program", 0)

    def browse_for_icon_cb(self, button):
        file_diag = p2p_diag.P2PFileSelection(0)
        file_diag.set_resizable(1)
        dir = os.path.join( self.Point2Play.gamedir, self.game_folder, "icons" ) + "/"
        file_diag.set_current_folder( dir )
        resp = file_diag.run()
        filename = file_diag.get_filename()
        if resp == gtk.RESPONSE_OK:
            self.icon = filename
        file_diag.destroy()
        self.widgets["icon_button_image"].set_from_file(self.icon)
        
    def extract_icon_cb(self, button):
        """ Extract icon from the specified program executable and put it in the game icon
            directory with the specified name. 

            We want to launch the engine with these args:  extract_icon.exe executable name
            The workdir MUST be the game - otherwise the icon extractor can't find it.            

        """

        exe = self.widgets["path_to_program"].get_text()
        if self.widgets["non_cedega_app_checkbutton"].get_active() or misc_utils.is_URL(exe):
            err_diag = p2p_diag.P2PMessageDialog( self.widgets["shortcut_properties"], 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("Sorry, can't extract a game icon from a native Linux application.") )
            err_diag.run()
            err_diag.destroy()
            return

        # set busy cursor
        self.widgets["shortcut_properties"].window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))

        if misc_utils.is_windows_path(exe):
            unix_path = Point2Play._windows_to_unix_path(os.path.dirname(exe), self.Point2Play.gamedir + self.game_folder + "/c_drive")
            exe = unix_path + "/" + os.path.basename(exe)
        else:
            unix_path = os.path.dirname(exe)

        name = self.widgets["shortcut_name"].get_text()

        # The name of an executable or icon might have spaces or
        # quotes (single or double) in it, so we quote each one with 
        # double-quotes and make sure to backslashify any
        # double-quotes occurring therein.
        # Note: we must use double-quotes to enclose them because they
        # pass through a shell script at some point, and bash (unlike perl)
        # does not allow you to escape single quotes with backslashes
        # within a singly-quoted string.

        args = '"' + misc_utils.backslashify_quotes(os.path.basename(exe)) + '" "' + misc_utils.backslashify_quotes(name) + '"'
        self.launch_icon_extractor(unix_path, args, name)

        # set normal cursor
        self.widgets["shortcut_properties"].window.set_cursor(gtk.gdk.Cursor(gtk.gdk.LEFT_PTR))

    def launch_icon_extractor(self, workdir, args, name):
        """ workdir: the game directory
            args: arguments to pass to the icon extractor
            name: desired name of the icon
 
            Launches the icon extractor."""

        icon_extractor_subpath = "/extract-icon/extract-icon.exe"
        icon_extractor_path = os.environ["P2PPATH"] + icon_extractor_subpath
        if not os.access( icon_extractor_path, os.F_OK ):
            icon_extractor_path = os.getcwd() + icon_extractor_subpath
            if not os.access( icon_extractor_path, os.F_OK ):
                err_diag = p2p_diag.P2PMessageDialog( self.widgets["shortcut_properties"], 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("Sorry, can't find the icon extractor.") )
                err_diag.run()
                err_diag.destroy()
                return

        try:
            retval = self.Point2Play.winex(0, self.game_folder, icon_extractor_path, workdir=workdir, cmdlineoverride=args)
        except Point2Play.Point2PlayError, e:
            err_diag = p2p_diag.P2PMessageDialog( self.widgets["shortcut_properties"], 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, e.__str__() )
            err_diag.run()
            err_diag.destroy()
            return

        if retval:
            # We should have a pid now to wait on.
            pid = retval
            (p, v) = os.waitpid(pid, os.WNOHANG)
            # in case the thing takes too long:
            self.timeout_function = gtk.timeout_add( 50000, self.icon_extractor_timeout)
            self.keep_running = 1
            while p == 0 and v == 0 and self.keep_running: 
                # still running - just sit there for a moment. It shouldn't take long.
                (p, v) = os.waitpid(pid, os.WNOHANG)
                while gtk.events_pending():
                    gtk.main_iteration()  
                    # note: clicking cancel won't cancel the icon extractor.
                    # this loop is just here so we can see the nice busy-wait cursor.
            gtk.timeout_remove(self.timeout_function)

            if self.keep_running:  # in other words, if we didn't time out
                # look for the icon the extractor extracted ... 
                # currently only .xpm and .png are being created - FIXME if more are added.

                icon_path = self.Point2Play.gamedir + self.game_folder + "/icons/" + name
                if os.access( icon_path + ".xpm", os.F_OK ):
                    self.widgets["icon_button_image"].set_from_file(icon_path + ".xpm")
                    self.icon = icon_path + ".xpm"
                    return
                if os.access( icon_path + ".png", os.F_OK ):
                    self.widgets["icon_button_image"].set_from_file(icon_path + ".png")
                    self.icon = icon_path + ".png"
                    return
            else:
                log.logger.warning("Icon extractor timed out.")
                try:
                    os.kill( pid, signal.SIGTERM )
                except:
                    pass # well, it'll die when the gui is shut down anyway

        # if we made it here, the icon extractor did not extract an icon
        err_diag = p2p_diag.P2PMessageDialog( self.widgets["shortcut_properties"], 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("Sorry, can't seem to extract a game icon.") )
        err_diag.run()
        err_diag.destroy()
        self.widgets["extract_icon_button"].set_sensitive(0)


    def icon_extractor_timeout(self, *args):
        self.keep_running = False

    def run(self):
        """ Runs and returns when OK is clicked. """
        while 1:
            response = gtk.Dialog.run( self.widgets["shortcut_properties"] )
            if response == gtk.RESPONSE_OK:
                shortcut_name = self.widgets["shortcut_name"].get_text()
                if shortcut_name.find('/') >= 0 or shortcut_name.find('\\') >= 0:
                    alert =  p2p_diag.P2PMessageDialog( self.widgets["shortcut_properties"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("Sorry, slashes (either forward or backward) are not allowed in shortcut names.") )
                    alert.run()
                    alert.destroy()
                    continue

                if self.shortcut_index >= 0: 
                    if shortcut_name != self.initial_shortcut_name:
                        # the user has changed the shortcut name - make sure the new shortcut name is unique.
                        if self.Point2Play.shortcut_name_already_exists(self.game_folder, shortcut_name):
                            alert = p2p_diag.P2PMessageDialog( self.widgets["shortcut_properties"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("Sorry, the new shortcut name you've chosen is already in use within this game folder.") )
                            alert.run()
                            alert.destroy()
                            continue
                else: # creating a shortcut - make sure the name is not in use
                    if self.Point2Play.shortcut_name_already_exists(self.game_folder, shortcut_name):
                        alert = p2p_diag.P2PMessageDialog( self.widgets["shortcut_properties"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("Sorry, this shortcut name already exists within this game folder. Please choose another name.") )
                        alert.run()
                        alert.destroy()
                        continue

                # return path, name, icon, workdir

#                profile = self.get_current_profile()
                cmdlineargs = self.widgets["command_line_options"].get_text()
                if self.widgets["non_cedega_app_checkbutton"].get_active():
                    cedega_app = "N"
                else:
                    cedega_app = "Y"
                path = self.widgets["path_to_program"].get_text()
                if misc_utils.is_URL(path) and cedega_app == "Y":
                    # this shortcut appears to point to a URL, so make sure cedega_app = "N"
                    cedega_app = "N"
                
                gddb_path = ""
                specified_gddb_profile = self.widgets["profile_combo"].get_active_text()
                log.logger.info("Specified gddb profile is " + specified_gddb_profile)
                if specified_gddb_profile != "":
                    for x in self.gddb_scan:
                        if x[0] == specified_gddb_profile:
                            gddb_path = x[1]
                            break

                    if gddb_path == "" and specified_gddb_profile != gddb.NONE_STRING:
                        # Couldn't find the specified gddb profile.
                        err_diag = p2p_diag.P2PMessageDialog( self.widgets["shortcut_properties"], 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("Could not find the specified GDDB Profile %s.\nPlease select one from the list (or choose %s).") % (specified_gddb_profile, gddb.NONE_STRING) )
                        err_diag.run()
                        err_diag.destroy()
                        continue

                return ( { "Path" : path,
                           "Name" : self.widgets["shortcut_name"].get_text(),
                           "Icon" : self.icon,
                           "Workdir" : self.widgets["working_directory"].get_text(),
                           "Profile" :  "",
                           "DirName" : self.game_folder,
                           "CommandLineArguments" : cmdlineargs,
                           "gddb" : gddb_path,
                           "CedegaApp" : cedega_app  } )

            if response == gtk.RESPONSE_DELETE_EVENT or response == gtk.RESPONSE_CANCEL:
                return
            
