#!/usr/bin/env python

"""
Point2Play Source Code
Copyright (c) 2004 by TransGaming Technologies, Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""

import pygtk
pygtk.require('2.0')
import gtk
import gtk.glade
import log, copy, misc_utils
import os
from types import *
import gui_utils
import gobject
import p2p_diag, p2p_gui
import httplib, urllib
import shutil

LANGUAGEDETAILS_CHANGED = 1
LANGUAGEDETAILS_CANCELED = 0

CANCEL_CLICKED = 0
OK_CLICKED = 1
DEFAULTS_CLICKED = 2
APPLY_CLICKED = 3


# URL for Volunteer request for translations for P2P
TRANSLATION_VOLUNTEER_URL = "http://www.transgaming.org/i18n"

# List of avail langs
TRANSLATION_AVAILLANGS_HOST = "transgaming.org"
TRANSLATION_AVAILLANGS_PATH = "/i18n/availlang.html"

p2ppath = os.getenv("P2PPATH")
GLADE_FILE = p2ppath + "/gui/" + __name__ + ".glade"
	
class WidgetsWrapper:
    def __init__(self):
        self.widgets = gtk.glade.XML(GLADE_FILE, None, "p2p_localized_msg")

    # Gives us the ability to do: widgets['widget_name'].action()
    def __getitem__(self, key):
        return self.widgets.get_widget(key)

class SystemPreferencesDialog:
    def __init__(self, Point2Play, parent):
        self.Point2Play = Point2Play
        self.parent = parent
        self.widgets = WidgetsWrapper()

        self.languages = []
        self.AvailLang = None
        self.LangInternetConnectWarned = 0
        #self.model = self.language_list_model()
        #self.widgets["language_treeview"].set_model( self.model )
        self.widgets["language_treeview"].get_selection().connect('changed', self.language_view_select_cb )

        cell = gtk.CellRendererText()
        column1 = gtk.TreeViewColumn( _("Locale"), cell, text=0 )
        column2 = gtk.TreeViewColumn( _("Version"), cell, text=1 )
        column3 = gtk.TreeViewColumn( _("Credits"), cell, text=2 )
        self.widgets["language_treeview"].append_column( column1 )
        self.widgets["language_treeview"].append_column( column2 )
        self.widgets["language_treeview"].append_column( column3 )


        self.model = self.language_list_model()
        self.widgets["language_treeview"].set_model( self.model )

        self.widgets.widgets.signal_autoconnect(self)

        # Update the list of installed translations...
        self.update_availlang_list()


    #####################################################################################
    # - Languages and Localization
    #####################################################################################

    def language_list_model(self):
        model = gtk.ListStore( gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING )
        return model


    def install_lang_cb(self, button):
        log.logger.info("Installing a translation...")

        # Connect to the ARPANET and get the text from the page with the language resources
        if not self.LangInternetConnectWarned:
            warn_diag = p2p_diag.P2PMessageDialog(p2p_gui.widgets["main"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_YES_NO, _("Cedega will need to contact TransGaming's server to determine which languages are currently available.  No personal information will be transmitted.  Is this OK?") )
            resp = warn_diag.run()
            warn_diag.destroy()
            if resp == gtk.RESPONSE_NO:
                return
            else:
                self.LangInternetConnectWarned = 1

        # Launch the edit joystick dialog....
        AvailableLanguagesD = AvailableLanguagesDialog(self.parent )
        res = AvailableLanguagesD.run()

        if res == LANGUAGEDETAILS_CHANGED:
            self.languages.append ( { "Locale":AvailableLanguagesD.selected_language["Locale"], "Version":AvailableLanguagesD.selected_language["Version"], "Credits":AvailableLanguagesD.selected_language["Credits"] } )

            # Add a new row to the list of installed areas...
            iter = self.model.append()
	    
            # Fill in the language info
            self.model.set_value(iter, 0, AvailableLanguagesD.selected_language["Locale"])
            self.model.set_value(iter, 1, AvailableLanguagesD.selected_language["Version"])
            self.model.set_value(iter, 2, AvailableLanguagesD.selected_language["Credits"])

            # Update the list of AvailLang available on the internet
            self.AvailLang = AvailableLanguagesD.AvailLang

            # Let's make a directory for our new translation file, and download it....
            self.make_language_directory( AvailableLanguagesD.selected_language["Locale"] )

        # Kill the available languages dialog...
        AvailableLanguagesD.destroy()

    def uninstall_lang_cb(self, button):
        log.logger.info("Uninstall...!")
        sel = self.widgets["language_treeview"].get_selection().get_selected()
        if sel:
            model, iter = sel
            if iter:
                lang_id = model.get_value( iter, 0 )
                dirname = self.Point2Play.P2PFOLDER + ".languages/" + str(lang_id)

                # Remove the directory with the corresponding translation file...
                shutil.rmtree( dirname )

                # Delete from installed list
                if self.Point2Play.LocalesInstalled:
   
                    if len(self.Point2Play.LocalesInstalled) == 1:
                        self.Point2Play.LocalesInstalled = None
                    else:
                        del self.Point2Play.LocalesInstalled[lang_id]

                self.Point2Play.write()

                # Remove the localization information from the treeview...
                model.remove(iter)


    def make_language_directory(self, locale):
        #os.mkdir( self.Point2Play.P2PFOLDER + ".languages/" + locale )
        #os.mkdir( self.Point2Play.P2PFOLDER + ".languages/" + locale + "/LC_MESSAGES")
        # Create the directories needed
        dirname = os.path.join (self.Point2Play.P2PFOLDER, ".languages", locale, "LC_MESSAGES")

        if os.path.islink( dirname ) or os.path.isfile( dirname ):
            os.unlink( dirname )
        elif os.path.isdir( dirname ):
            shutil.rmtree( dirname )

        #os.mkdir( dirname )
        #this check is necessary for case when installing a newer version of already installed language
        if not os.path.exists( self.Point2Play.P2PFOLDER + ".languages/" + locale ):
            os.mkdir( self.Point2Play.P2PFOLDER + ".languages/" + locale )
        
        os.mkdir( self.Point2Play.P2PFOLDER + ".languages/" + locale + "/LC_MESSAGES")

        # Connect to the IntarWeb and get the text from the page with the language resources
        filename = os.path.join(dirname, "p2p_localized_msg.mo")
        lang_id = locale
        try:
            urllib.urlretrieve(self.AvailLang[lang_id]["mopath"], filename)
        except socket.gaierror, e:
            err_diag = p2p_diag.P2PMessageDialog(p2p_gui.widgets["main"], 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("Unable to connect to server\n%s")% e[1] )
            err_diag.run()
            err_diag.destroy()
            return
        except:
            err_diag = p2p_diag.P2PMessageDialog(p2p_gui.widgets["main"], 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, "%s" % str( sys.exc_info()[1] ) )
            err_diag.run()
            err_diag.destroy()
            return

        # Add to the installed list
        if not self.Point2Play.LocalesInstalled:
            self.Point2Play.LocalesInstalled = { lang_id : { "p2pver":self.AvailLang[lang_id]["p2pver"], "revver":self.AvailLang[lang_id]["revver"],
"credits":self.AvailLang[lang_id]["credits"] } }
        else:
            self.Point2Play.LocalesInstalled[lang_id] = { "p2pver":self.AvailLang[lang_id]["p2pver"], "revver":self.AvailLang[lang_id]["revver"], "credits":self.AvailLang[lang_id]["credits"] }

        # Remove from the avail list
        if len(self.AvailLang) == 1:
            self.AvailLang = None
        else:
            del self.AvailLang[lang_id]

        # Update the avail and installed language lists
        #self.update_installedlang_list()
        #self.update_availlang_list()

        # Update the config file
        self.Point2Play.write()


    # This function is used to update the list of available languages to be installed
    # on the "Languages" tab.  It is called whenever the available list of languages is
    # modified (ie. when the user requests to update the list)
    def update_availlang_list (self):
        self.model.clear()

        if self.Point2Play.LocalesInstalled:
            localeids = self.Point2Play.LocalesInstalled.keys()
            for lid in localeids:
                # Column 1 is the Locale
                # Column 2 is the P2P targetted Version + revision
                # Column 3 is the credits

                iter = self.model.append()
                self.model.set_value(iter, 0, lid)
                self.model.set_value(iter, 1, self.Point2Play.LocalesInstalled[lid]["p2pver"] + " Rev." + self.Point2Play.LocalesInstalled[lid]["revver"])
                self.model.set_value(iter, 2, self.Point2Play.LocalesInstalled[lid]["credits"])




    def language_view_select_cb(self, selection):
        sel = selection.get_selected()
        if sel:
            model, iter = sel
            if iter:
                self.current_joystick = model.get_value(iter,0)



###########################################################################
# - AvailableLanguagesDialog                                              #
###########################################################################


# Helper class for the new joystick details entry dialog box
class AvailableLanguagesDialog( gtk.Dialog ):
    def __init__(self, parent_gui):
        gtk.Dialog.__init__(self)

        self.set_size_request(600,480)

        # Initial setup stuff
        #self.set_transient_for( parent_gui )
        self.set_title( _("Available Translations"))
        vbox = self.get_children()[0]

        self.Point2Play_gui = parent_gui
        self.Point2Play = parent_gui.Point2Play
        self.widgets = {}

        self.selected_language = []

        """Build languages page"""
        lang_table = gtk.Table(2,3)


        # Fields for the Installed Language listbox: Locale, Version, Needs Updating, Credits
        self.widgets["availlang_liststore"] = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING)
        self.widgets["availlang_listview"] = gtk.TreeView(self.widgets["availlang_liststore"])

        #Put it in a scrolledwindow
        availlang_sw = gtk.ScrolledWindow()
        availlang_sw.add ( self.widgets["availlang_listview"] )

        # Now we attach rendering widgets to the game tree
        cell = gtk.CellRendererText()
        column1 = gtk.TreeViewColumn( _("Locale"), cell, text=0 )
        column2 = gtk.TreeViewColumn( _("Version"), cell, text=1 )
        column3 = gtk.TreeViewColumn( _("Credits"), cell, text=2 )
        self.widgets["availlang_listview"].append_column( column1 )
        self.widgets["availlang_listview"].append_column( column2 )
        self.widgets["availlang_listview"].append_column( column3 )

        vbox.pack_start(gtk.Label(_("Installed Languages:")), gtk.FALSE, gtk.FALSE)
        vbox.pack_start(availlang_sw)

        vbox.add( lang_table )

        self.get_list_availlangs_cb(self.widgets)

        button = self.add_button( gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL )
        button.set_name( "round_button" )
        button = self.add_button( gtk.STOCK_OK, gtk.RESPONSE_OK )
        button.set_name( "round_button" )
        vbox.show_all()

    # This function contacts the server to get the list of all available languages, and converts into a list
    # of updates that the user might want to install (> revision than currently installed, or more closely matches
    # the P2P version)
    def get_list_availlangs_cb (self, widgets):
        try:
            http_conn = httplib.HTTPConnection(TRANSLATION_AVAILLANGS_HOST)
            http_conn.request("GET", TRANSLATION_AVAILLANGS_PATH )
            resp = http_conn.getresponse()
            list_avail_lang = resp.read()
            http_conn.close()
        except socket.gaierror, e:
            err_diag = p2p_diag.P2PMessageDialog(p2p_gui.widgets["main"], 0, gtk.MESSAGE_ERROR,gtk.BUTTONS_OK, _("Unable to connect to server\n%s") % e[1] )
            err_diag.run()
            err_diag.destroy()
            return
        except:
            err_diag = p2p_diag.P2PMessageDialog(p2p_gui.widgets["main"], 0, gtk.MESSAGE_ERROR,gtk.BUTTONS_OK, "%s" % str( sys.exc_info()[1] ) )
            err_diag.run()
            err_diag.destroy()
            return

        # Get our current P2P version in a format usable by us
        if self.Point2Play.VERSION_STRING.find(" ") != -1:
            version = self.Point2Play.VERSION_STRING[:self.Point2Play.VERSION_STRING.find(" ")]
        else:
            version = self.Point2Play.VERSION_STRING

        (p2pver_major, p2pver_minor, p2pver_minorminor) = self.split_out_versionnum(version)

        list_avail_lang_split = list_avail_lang.splitlines()
        self.AvailLang = None  # Starting over

        for lang in list_avail_lang_split:
            curinstall_major = -1
            curinstall_minor = -1
            curinstall_minorminor = -1
            newlang_major = -1
            newlang_minor = -1
            newlang_minorminor = -1
            put_lang_in_list = 0

            (lang_locale, lang_p2pver, lang_revver, lang_mopath, lang_popath, lang_credit) = lang.split(";", 5)
            (newlang_major, newlang_minor, newlang_minorminor) = self.split_out_versionnum(lang_p2pver)

            # Do we have a version of this locale installed already?
            if self.Point2Play.LocalesInstalled and self.Point2Play.LocalesInstalled.get(lang_locale):
                # Get the info for the currently installed version
                (curinstall_major, curinstall_minor, curinstall_minorminor) = self.split_out_versionnum(self.Point2Play.LocalesInstalled[lang_locale]["p2pver"])

                # Is it newer than the current one installed?
                res = misc_utils.which_version_newer(curinstall_major, curinstall_minor, curinstall_minorminor, newlang_major, newlang_minor, newlang_minorminor)
                if (res == 1 or (res == -1 and lang_revver > self.Point2Play.LocalesInstalled[lang_locale]["revver"])):
                    put_lang_in_list = 1
            else:
                # We'll have it available by default
                put_lang_in_list = 1

            if put_lang_in_list:
                if not self.AvailLang:
                    self.AvailLang = { lang_locale : { "p2pver":lang_p2pver, "revver":lang_revver, "mopath":lang_mopath, "popath":lang_popath, "credits":lang_credit  } }
                else:
                    self.AvailLang[lang_locale] = { "p2pver":lang_p2pver, "revver":lang_revver, "mopath":lang_mopath, "popath":lang_popath, "credits":lang_credit }

        # Update the list of available languages
        self.update_availlang_list()

        # Add the installed language to the point2playrc file
        self.Point2Play.write()

        return


    # This function is used to update the list of available languages to be installed
    # on the "Languages" tab.  It is called whenever the available list of languages is
    # modified (ie. when the user requests to update the list)
    def update_availlang_list (self):
        self.widgets["availlang_liststore"].clear()
        if self.AvailLang:
            localeids = self.AvailLang.keys()
            for lid in localeids:
                # Column 1 is the Locale
                # Column 2 is the P2P targetted Version + revision
                # Column 3 is the credits

                iter = self.widgets["availlang_liststore"].append()
                self.widgets["availlang_liststore"].set_value(iter, 0, lid)
                self.widgets["availlang_liststore"].set_value(iter, 1, self.AvailLang[lid]["p2pver"] + " Rev." + self.AvailLang[lid]["revver"])
                self.widgets["availlang_liststore"].set_value(iter, 2, self.AvailLang[lid]["credits"])


    def split_out_versionnum(self, ver_complete):
        major=-1
        minor=-1
        minorminor=-1

        num_periods = ver_complete.count(".")
        if num_periods == 0:
            major = ver_complete
        elif num_periods == 1:
            (major,minor) = ver_complete.split(".", 1)
        elif num_periods >= 2:
            (major,minor,minorminor) = ver_complete.split(".", 2)
            # Get rid of any MinorMinorMinor version #s (ie. the .1 part of 1.3.2.1)
            if minorminor.find(".") > 0:
                minorminor = minorminor[0][minorminor.find(".")-1]
        return (major, minor, minorminor)



    def run( self ):
        while 1:
            resp = gtk.Dialog.run( self )
            if resp == gtk.RESPONSE_OK:
                try:
                    sel = self.widgets["availlang_listview"].get_selection().get_selected()
                    if sel:
                        model, iter = sel
                        if iter:
                            language = { "Locale":self.widgets["availlang_liststore"].get_value( iter, 0 ), "Version":self.widgets["availlang_liststore"].get_value( iter, 1 ), "Credits":self.widgets["availlang_liststore"].get_value( iter, 2 ) }
                            self.selected_language = language
                    return LANGUAGEDETAILS_CHANGED
                except ( Point2Play.Point2PlayError, IOError ), e:
                    err_diag = p2p_diag.P2PMessageDialog( self.parent, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, str(e) )
                    err_diag.run()
                    err_diag.destroy()
            else:
                return LANGUAGEDETAILS_CANCELED

