#!/usr/bin/env python

"""
Point2Play Source Code
Copyright (c) 2004 by TransGaming Technologies, Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""

import pygtk
pygtk.require('2.0')
import gtk, gobject
import gtk.glade
import log
import os, re
import gui_utils, system_detection, p2p_gui, Point2Play

p2ppath = os.getenv("P2PPATH")
GLADE_FILE = p2ppath + "/gui/" + __name__ + ".glade"

class WidgetsWrapper:
    def __init__(self):
        self.widgets = gtk.glade.XML(GLADE_FILE, None, "p2p_localized_msg")

    # Gives us the ability to do: widgets['widget_name'].action()
    def __getitem__(self, key):
        return self.widgets.get_widget(key)

class SystemSettingsDialog:
    def __init__(self, Point2Play, parent):
        self.Point2Play = Point2Play
        self.widgets = WidgetsWrapper()
        self.parent = parent
        self.widgets.widgets.signal_autoconnect(self)
        self.set_system_info(Point2Play.config)

    def ok_cb(self, button):
        gui_utils.save_widget_data(self, system_detection.system_data_widgets_list, self.Point2Play)
        self.widgets["settings"].response(1)

    def cancel_cb(self, button):
        self.widgets["settings"].response(0)

    def autodetect_cb(self, button):
        system_info = system_detection.make_detected_system_info_dictionary()
        self.set_system_info(system_info)
        system_detection.detect_optical_drives(self.Point2Play, force=True)

    def copy_to_clipboard_cb(self, button):
        """ Copy all the information into the clipboard in a format appropriate for
            sending us bug-reports."""
        clipboard = gtk.clipboard_get(gtk.gdk.SELECTION_CLIPBOARD)
        stringified_data = ""
        for w in system_detection.system_data_widgets_list:
            stringified_data = stringified_data + w + ": " + self.widgets[w].get_text() + "\n"
        # also add the gui version
        stringified_data = stringified_data + "GUI version: " + Point2Play.VERSION_STRING + "\n"
        clipboard.set_text(stringified_data)

    def set_system_info(self, dictionary):
        """ Pass either a dictionary with default values given by detect_system_info() or
            a Point2Play config object """

        for w in system_detection.system_data_widgets_list:
            log.logger.info("w " + w + " is " + dictionary[w])
            self.widgets[w].set_text(dictionary[w])


