#!/usr/bin/env python

"""
Point2Play Source Code
Copyright (c) 2004 by TransGaming Technologies, Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""

import pygtk
pygtk.require('2.0')
import gtk
import gtk.glade
import log
import os
import p2p_diag
import p2p_gui
import string

p2ppath = os.getenv("P2PPATH")
GLADE_FILE = p2ppath + "/gui/" + __name__ + ".glade"
	
class WidgetsWrapper:
    def __init__(self):
        self.widgets = gtk.glade.XML(GLADE_FILE, None, "p2p_localized_msg")

    # Gives us the ability to do: widgets['widget_name'].action()
    def __getitem__(self, key):
        return self.widgets.get_widget(key)

class SystemTestsDialog:
    def __init__(self, Point2Play, selection):
        """ Selection is the checkboxes to be checked initially """
        self.Point2Play = Point2Play
        self.widgets = WidgetsWrapper()
        self.widgets.widgets.signal_autoconnect(self)
        self.set_checkboxes(selection)
        self.hide_feedback_buttons()
        self.hide_drawing_areas()
        self.check_bitness()

    def check_bitness(self):
        """ If the user has a 64-bit machine and does not have the proper 32-bit
            emulation libraries installed, we show a warning to the user."""

        if self.Point2Play.get_machine_bitness() == "64":
            log.logger.info("64 bit machine ... check output of ldd tests/gl_test to see if the 32-bit emulation libs are installed.")

            pipeFile = os.popen("ldd " + p2ppath + "/tests/gl_test") # the choice of gl_test is arbitrary
            cmd_output = pipeFile.readlines() # slurp output into a list
            status = pipeFile.close()
            if status:
                # warn the user
                err_diag = p2p_diag.P2PMessageDialog( self.widgets["system_tests"], 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("You appear to be missing one or more dependencies for the system tests. This could be because your 64-bit system is missing the 32-bit emulation libraries. Without them, the tests and other auxiliary programs will not run.\n\n\n%s\n") % string.join(cmd_output))
                err_diag.run()
                err_diag.destroy()
        
    # When a new test is run, the visible drawing areas will be "wiped clean".  Erased
    # from existence?  Not quite that heavy...just hidden until the current tests are finished.
    def hide_drawing_areas(self):
        self.widgets["opengl_drawing_area"].hide()
        self.widgets["three_d_accel_drawing_area"].hide()
        self.widgets["pthreads_drawing_area"].hide()
        self.widgets["copy_protection_drawing_area"].hide()
        self.widgets["oss_sound_drawing_area"].hide()
        self.widgets["alsa_sound_drawing_area"].hide()


    # Much like the drawing areas, when a new test is run, the visible feedback buttons are
    # hidden, so that only the buttons reflecting the currently run test(s) show up.
    def hide_feedback_buttons(self):
        self.widgets["opengl_feedback_button"].hide()
        self.widgets["three_d_accel_feedback_button"].hide()
        self.widgets["pthreads_feedback_button"].hide()
        self.widgets["copy_protection_feedback_button"].hide()
        self.widgets["oss_sound_feedback_button"].hide()
        self.widgets["alsa_sound_feedback_button"].hide()


    def set_checkboxes(self, selection):
        if "test_cdrom" in selection:
            self.widgets["copy_protection_checkbutton"].set_active(1)
        if "test_pthreads" in selection:
            self.widgets["pthreads_checkbutton"].set_active(1)
        if "test_sound" in selection:
            self.widgets["oss_sound_checkbutton"].set_active(1)
            self.widgets["alsa_sound_checkbutton"].set_active(1)
        if "test_gl" in selection:
            self.widgets["three_D_acceleration_checkbutton"].set_active(1)
            self.widgets["opengl_direct_rendering_checkbutton"].set_active(1)


    def on_opengl_feedback_button_clicked(self, button):
        on_opengl_feedback_button_clicked(self, button)

    def on_three_d_accel_feedback_button_clicked(self, button):
        on_three_d_accel_feedback_button_clicked(self, button)

    def on_pthreads_feedback_button_clicked(self, button):
        on_pthreads_feedback_button_clicked(self, button)

    def on_copy_protection_feedback_button_clicked(self, button):
        on_copy_protection_feedback_button_clicked(self, button)

    def on_oss_sound_feedback_button_clicked(self, button):
        on_oss_sound_feedback_button_clicked(self, button)

    def on_alsa_sound_feedback_button_clicked(self, button):
        on_alsa_sound_feedback_button_clicked(self, button)
    
    def on_tests_moreinfo_alsa_clicked(self, button):
        on_tests_moreinfo_alsa_clicked(self, button)

    def on_tests_moreinfo_oss_clicked(self, button):
        on_tests_moreinfo_oss_clicked(self, button)

    def on_tests_moreinfo_copyprot_clicked(self, button):
        on_tests_moreinfo_copyprot_clicked(self, button)

    def on_tests_moreinfo_pthreads_clicked(self, button):
        on_tests_moreinfo_pthreads_clicked(self, button)

    def on_tests_moreinfo_3d_clicked(self, button):
        on_tests_moreinfo_3d_clicked(self, button)

    def on_tests_moreinfo_opengl_clicked(self, button):
        on_tests_moreinfo_opengl_clicked(self, button)


    # This is the function that is responsible for providing feedback from a run test.  After the test is
    # conducted, the test sends back some form of textual feedback and a result color.  This color is 
    # either green, yellow or red, for Passed, Warning, and Failed respectively.  By passing in the test
    # name and color, we can locate the test's relative drawing area and color it with the result color.
    # The result color can also be sent to setup_feedback_button() to allow it to determine what the 
    # button's label and message text should be.
    def update_drawing_area(self, test_name, test_feedback, test_status, result_color):
        if test_name == "test_gl":
            self.widgets["opengl_drawing_area"].modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(result_color))
            self.widgets["opengl_drawing_area"].show()
            self.setup_feedback_button(self.widgets["opengl_feedback_button"], result_color)
        if test_name == "test_three_d":
            self.widgets["three_d_accel_drawing_area"].modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(result_color))
            self.widgets["three_d_accel_drawing_area"].show()
            self.setup_feedback_button(self.widgets["three_d_accel_feedback_button"], result_color)
        if "test_pthread" == test_name:
            self.widgets["pthreads_drawing_area"].modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(result_color))
            self.widgets["pthreads_drawing_area"].show()
            self.setup_feedback_button(self.widgets["pthreads_feedback_button"], result_color)
        if "test_cdrom" == test_name:
            self.widgets["copy_protection_drawing_area"].modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(result_color))
            self.widgets["copy_protection_drawing_area"].show()
            self.setup_feedback_button(self.widgets["copy_protection_feedback_button"], result_color)
        if "test_oss_sound" == test_name:
            self.widgets["oss_sound_drawing_area"].modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(result_color))
            self.widgets["oss_sound_drawing_area"].show()
            self.setup_feedback_button(self.widgets["oss_sound_feedback_button"], result_color)
        if "test_alsa_sound" == test_name:
            self.widgets["alsa_sound_drawing_area"].modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse(result_color))
            self.widgets["alsa_sound_drawing_area"].show()
            self.setup_feedback_button(self.widgets["alsa_sound_feedback_button"], result_color)


    # As mentioned above, this function takes in a result color, which allows the button to determine the 
    # results of the test, and thus display the correct feedback button label and messagebox text.  
    def setup_feedback_button(self, button, result_color):
        # After the test is run, we can tell how it went based on its result color. So we'll check the color
        # and set the label of the button to match the color accordingly.
        if result_color == "green":
            button.set_label("Passed")
        elif result_color == "yellow":
            button.set_label("Warning")
        else:
            button.set_label("Failed")

        # Now that the button's text has been properly determined, let's reveal our masterpiece!
        button.show() 


    def cancel_cb(self, button):
        self.widgets["system_tests"].response(0)
        self.widgets["system_tests"].destroy()


    # This is an important function.  It runs down a list of checkboxes and "checks" whether or not
    # they are checked or not.  If they are, then the test corresponding to the checkbox is added
    # to a list of tests to be run.
    def run_selected_tests_cb(self, button):
        # do the tests selected (we can't rely on the initial selection because
        # the user might have changed the checkboxes
        tests = ()

        if (self.widgets["opengl_direct_rendering_checkbutton"].get_active()):
            tests += ('test_gl',)

        if (self.widgets["three_D_acceleration_checkbutton"].get_active()):
            tests += ('test_three_d',)

        if self.widgets["pthreads_checkbutton"].get_active():
             tests += ('test_pthread',)

        if self.widgets["copy_protection_checkbutton"].get_active():
            tests += ('test_cdrom',)

        if (self.widgets["oss_sound_checkbutton"].get_active()):
            tests += ('test_oss_sound',)

        if (self.widgets["alsa_sound_checkbutton"].get_active()):
            tests += ('test_alsa_sound',)

        self.hide_feedback_buttons()
        self.hide_drawing_areas()
        
        for test in tests:
            exec('import ' + test)
            self.Point2Play.tests.runTest(test, self)


# These are global so that both the system tests popup and the
# setup wizard can use them.

def on_opengl_feedback_button_clicked(window, button):
    if button.get_label() == "Passed":
        alert = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("Your video card appears to be using direct hardware rendering.\n"))
    elif button.get_label() == "Warning":
        alert = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("Your OpenGL drivers do not appear to be setup correctly. Please check the documentation for your Linux distribution and your graphics card drivers to ensure proper installation.\n"))
    else:
        alert = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("Your OpenGL drivers do not appear to be setup correctly. Please check the documentation for your Linux distribution and your graphics card drivers to ensure proper installation.\n"))
    alert.run()
    alert.destroy()


def on_three_d_accel_feedback_button_clicked(window, button):
    if button.get_label() == "Passed":
        alert = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("Your graphics card is correctly setup for hardware accelerated 3D rendering.\n"))
    elif button.get_label() == "Warning":
        alert = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("Your graphics card does not appear to be setup correctly.\nPlease check the documentation for your Linux distribution\nand your graphics card drivers to ensure proper installation.\n"))
    else:
        alert = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("Your graphics card does not appear to be setup correctly.\nPlease check the documentation for your Linux distribution\nand your graphics card drivers to ensure proper installation.\n"))
    alert.run()
    alert.destroy()


def on_pthreads_feedback_button_clicked(window, button):
    if button.get_label() == "Passed":
        alert = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("POSIX Threads are available on your system.\nBe sure to enable pthreads in the Configuration section (this\nshould be enabled by default).\n"))
    elif button.get_label() == "Warning":
        alert = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("POSIX Threads (pthreads) are available but the stack size is smaller than recommended.\nSome games may require pthreads to be disabled.\nPlease check the Cedega release notes to see which games pthreads should be disabled for.\n"))
    else:
        alert = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("Your system does not have POSIX Threads (pthreads).\nBe sure to disable pthreads in the Configuration section (they should be disabled by default).\n"))
    alert.run()
    alert.destroy()

def on_copy_protection_feedback_button_clicked(window, button):
    if button.get_label() == "Passed":
        alert = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("Your CD/DVD-ROM devices should be correctly configured for copy protection.\n"))
    elif button.get_label() == "Warning":
        alert = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("A problem was detected with your CD/DVD-ROM devices. Some or all copy-protected games\nmay not work correctly with your drives.  Check the permissions on your CD/DVD-ROM device (in /dev).\nThe device can be found in the /etc/fstab file.\n\nIn a terminal perform:\n\nls -la /dev/CDROM (where CDROM is the device for your system)\n\nMake sure that all users have rx permissions.  If users do not have rx permissions then add the permissions by running:\n\nchmod a+rx /dev/CDROM.\n\nThe device may be a symbolic link to a second device, for example:\n\nlrwxrwxrwx  1 root root 3 Mar 23 05:50 /dev/cdrom -> hdc \n\nHere we can see that /dev/cdrom is a link to /dev/hdc.  Be sure to check permissions on the symbolically linked devices as well.\n"))
    else:
        alert = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("A problem was detected with your CD/DVD-ROM devices.  Some or all copy protected games\nmay not work correctly with your drives.  Check the permissions on your CD/DVD-ROM device (in /dev). The device can be found in the /etc/fstab file.\n\nIn a terminal perform:\n\nls -la /dev/CDROM (where CDROM is the device for your system)\n\nMake sure that all users have rx permissions.  If users do not have rx permissions then add the permissions by running\n\nThe device may be a symbolic link to a second device, for example:\n\nlrwxrwxrwx  1 root root 3 Mar 23 05:50 /dev/cdrom -> hdc \n\nHere we can see that /dev/cdrom is a link to /dev/hdc.  Be sure to check permissions on the symbolically linked devices as well.\n"))
    alert.run()
    alert.destroy()

def on_oss_sound_feedback_button_clicked(window, button):
    if button.get_label() == "Passed":
        alert = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("The OSS Sound System is setup and working on your system.  You may choose to use the Cedega OSS Sound plugin to output.\n"))
    elif button.get_label() == "Warning":
        alert = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("The OSS Sound system does not appear to be working or available.  Please check your Linux distribution documentation on how to set up your sound card.  If your sound card is working in other applications please make sure that no other applications are using your sound card and that ARTS and ESD have been disabled.\n"))
    else:
        alert = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("The OSS Sound system does not appear to be working or available.  Please check your Linux distribution documentation on how to set up your sound card.  If your sound card is working in other applications please make sure that no other applications are using your sound card and that ARTS and ESD have been disabled.\n"))
    alert.run()
    alert.destroy()


def on_alsa_sound_feedback_button_clicked(window, button):
    if button.get_label() == "Passed":
        alert = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("The ALSA Sound System is setup and working on your system.  You may choose to use the Cedega ALSA Sound plugin to output sound.  It is recommended that users who have ALSA available use this over OSS.\n"))
    elif button.get_label() == "Warning":
        alert = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _(" ALSA Sound system does not appear to be working or available.  Please check your Linux distribution documentation on how to set up ALSA.  If your sound card is working in other applications please make sure that no other applications are using your sound card and that ARTS and ESD have been disabled.\n"))
    else:
        alert = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _(" ALSA Sound system does not appear to be working or available.  Please check your Linux distribution documentation on how to set up ALSA.  If your sound card is working in other applications please make sure that no other applications are using your sound card and that ARTS and ESD have been disabled.\n"))
    alert.run()
    alert.destroy()


def on_tests_moreinfo_alsa_clicked(window, button):
    alert = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("ALSA Sound Test\n\nThis test attempts to play a sound file at 44 kHz with\nthe Advanced Linux Sound Architecture (ALSA).\n\nThis is by no means a comprehensive audio test,\nbut if you are able to hear the sound, you should at\nleast have basic sound support in Cedega with the\nappropriate driver.\n\nThis test should only take a few seconds, during which\nyou should hear a sound play.\n"))
    alert.run()
    alert.destroy()


def on_tests_moreinfo_oss_clicked(window, button):
    alert = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("OSS Sound Test\n\nThis test attempts to play a sound file at 44 kHz with\nthe Open Sound System (OSS).\n\nThis is by no means a comprehensive audio test,\nthough if you are able to hear the sound, you should at\nleast have basic sound support in Cedega with the\nappropriate driver.\n\nThis test should only take a few seconds, during which\nyou should hear a sound play.\n"))
    alert.run()
    alert.destroy()

def on_tests_moreinfo_copyprot_clicked(window, button):
    alert = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("Copy Protection Test\n\nThis test locates all available CD-ROM/DVD-ROM devices on\nyour computer that will be accessible by Cedega.\nIt also determines if these devices are capable of\nsupporting Copy-Protected games.\n\nThis test should only take a few seconds.\n"))
    alert.run()
    alert.destroy()


def on_tests_moreinfo_pthreads_clicked(window, button):
    alert = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("POSIX Threads (Pthreads) Test\n\nThis tests scans your system to determine if POSIX Threads (pthreads) are available. If the pthreads are found and available the test will further analyze to determine the available stack size.\n"))
    alert.run()
    alert.destroy()


def on_tests_moreinfo_3d_clicked(window, button):
    alert = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("3D Acceleration Test\n\nThis test will run a benchmark on your graphics card to determine the performance.\nDuring this test you should see 3 rotating gears. This test may take several minutes to perform, depending on your hardware.\n"))
    alert.run()
    alert.destroy()


def on_tests_moreinfo_opengl_clicked(window, button):
    alert = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("OpenGL Test\n\nThis test will determine if your OpenGL drivers are using direct hardware rendering.\nDirect rendering allows the OpenGL renderer to bypass a number of time consuming steps involving the X server and directly access the graphics hardware. This is necessary for most games to be playable.\n"))
    alert.run()
    alert.destroy()

