"""
Point2Play Source Code
Copyright 2003, TransGaming Technologies Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""

# each test case should have a variable which is a tuple of tuples. these subtables should contain:
# (DESCRIPTION, FUNCTION_TO_CALL, (BOOL) REQUIRE_GDKWINDOW)
# should return a list (any size) of lists (size 3) which contains: test name, test result, row colour


try:
    import pygtk
    pygtk.require('2.0')
except:
    pass

import os, sys

p2ppath = os.getenv("P2PPATH")
PATH_TO_TESTS = p2ppath + "/tests/"

class testsError(Exception):
    """base calss for test interface errors"""
    def __init__(self, message = ''):
        self.message = message

    def __str__(self):
        return `self.message`

class noTestsFound(testsError):
    """occurs when no tests were found during enumtests"""

    def __str__(self):
        return _('no tests were found')

try:
    import gtk, gobject
except:
    raise testsError(_("unable to load GTK2 Python bindings") )

class tests:
    """interfaces to all the test-*.py files, calls them, etc"""

    def __init__(self):
        self.test_pyfiles = []
        self.test_tests = []

        try:
            for file in os.listdir(PATH_TO_TESTS):
                if (file[0:5] == 'test_' and file[-3:] == '.py'):
                    self.test_pyfiles.append(file)
        except OSError:
            raise noTestsFound
        if (len(self.test_pyfiles) < 1): raise noTestsFound

        sys.path.append(PATH_TO_TESTS)
        for file in self.test_pyfiles:
            try:
                exec('import ' + file[0:-3])
                exec('tests = ' + file[0:-3] + '.TESTS')
                for test in tests:
                    test = file[0:-3], test[0], test[1], test[2]
                    self.test_tests.append(test)
            except AttributeError:
                pass

        if (len(self.test_tests) < 1): raise noTestsFound

    def getTestList(self):
        if (len(self.test_tests) < 1): raise noTestsFound
        return self.test_tests

    def getTestListGTKModel(self):
        if (len(self.test_tests) < 1): raise noTestsFound
        model = gtk.ListStore(gobject.TYPE_STRING)
        for test in self.test_tests:
            iter = model.append()
            model.set_value(iter, 0, test[1])
        return model

    def setTestDescription(self, testname, Point2Play_gui):
        for test in self.test_tests:
            if (test[1] == testname):
                exec('import ' + test[0])
                try:
                    exec('result = ' + test[0] + '.' + test[2] + '_description()')
                except AttributeError:
                    result = ''
                Point2Play_gui.widgets["test_description"].set_text(result)

    def runTest(self, testname, system_tests_gui):

        for test in self.test_tests:
            if (test[0] == testname):
                exec('import ' + test[0])
                if test[3]:
                    while gtk.events_pending():
                        gtk.main_iteration()
                    try: 
                        # this try block is here because the user might try to close the window
                        # in the middle of the tests. The test will continue but after it finishes,
                        # we would be trying to write in a window that no longer exists.
                        gdkwin = system_tests_gui.widgets["tests_drawing_area"].window 
                    except:
                        return # go home early

                    exec('result = ' + test[0] + '.' + test[2] + '(gdkwin)')
                else:
                    exec('result = ' + test[0] + '.' + test[2] + '()')

                # If the return is NULL, do nothing:
                if result == None:
                   return

                for aresult in result:
                    system_tests_gui.update_drawing_area(testname, aresult[0], aresult[1], aresult[2])
                # dialog.run()
                # dialog.destroy()


if (__name__ == "__main__"):
    try:
        systemtests = tests()
    except noTestsFound:
        print 'warning: no tests found'
        raise
    print systemtests.getTestList()
