#!/usr/bin/python

"""
Point2Play Source Code
Copyright 2003, TransGaming Technologies Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""

import Point2Play, system_detection, cdrom
import os

TESTS = (_("Test CD/DVD Drive"), "tgtest_cdrom", False),

def tgtest_cdrom():
    """ Runs tests to ensure the optical drives listed in the fstab 
       or detected through dbus are usable and that these drives will be
       visible to the Cedega engine. """

    res = []

    empty_res = []
    useable = _("Not available")
    colour = "red"
    empty_res.append(('No CD/DVD drive(s) found', useable, colour))

    drives_list = cdrom.getOpticalDrivesList()

    for optical_drive in drives_list:
	    # We wrap this call up, since it can exception on us if the optical drive
	    # device file doesn't exist (like, for example, on my laptop, i don't
	    # actually have a cdrom)
        if optical_drive.isCDDeviceReadable(): 
            useable = _("Available")
            colour = "green"
        else: 
            if optical_drive.isDeviceExists():
                useable = _("Available, But\nNo Copy\nProtection")
                colour = "yellow"
            else:
                # OK we got an exception. CD/DVD not available. danger will robinson!
                useable = _("Not available")
                colour = "red"

        res.append(('cdrom: ' + optical_drive.getDev(), useable, colour))
 
    if res:
        return res
    else:
        return empty_res

def tgtest_cdrom_description():
    return _('This test locates all available\nCD-ROM devices on your computer\nthat will be accessible by %s.\nIt also determines if these devices\nare capable of supporting Copy\nProtected games.\n\nThis test should only take\na few seconds at the most.\n\nIf this test fails, consult the\n%s documentation for more\ninformation.') % (Point2Play.WINEX_DISPLAY_TITLE, Point2Play.WINEX_DISPLAY_TITLE)

if (__name__ == "__main__"):
    for test in TESTS:
        print "running test ", test[0]
        eval(test[1] + '()')

