#!/usr/bin/python

"""
Point2Play Source Code
Copyright 2003, TransGaming Technologies Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""

TESTS = (_("Test If POSIX Threads\n(pthreads) Are Required"), "tgtest_pthreadstacksize", False),

import os,sys,p2p_diag,gtk, Point2Play

# Constants for return codes from the pthreads stack tester
NO_PTHREADS_NEEDED              = 1
PTHREADS_NEEDED_TOO_SMALL_STACK = 2
PTHREADS_NEEDED_OK              = 3

def tgtest_pthreadstacksize():
    """Runs the same test that we do at startup, using the same C program.
    This just allows the user to see whether we think they need pthreads or
    not """

    pid = os.fork()
    if pid:
       (wpid, retval) = os.waitpid( pid, 0 )
       retval = os.WEXITSTATUS(retval)

       if retval == NO_PTHREADS_NEEDED:
          res_text = _('Not required')
          res_colour = 'green'
       elif retval == PTHREADS_NEEDED_OK:
          res_text = _('Pthreads functional')
          res_colour = 'green'
       elif retval == PTHREADS_NEEDED_TOO_SMALL_STACK:
          res_text = _('Some Games Might Have Problems')
          res_colour = 'yellow'
    else:
       """We are the child"""
       try:
           try:
              pthread_test = os.environ["P2PPATH"] + "/pthread_test"
           except:
              pthread_test = "pthread_test"
           os.execl( pthread_test, ( "pthread_test") )
           sys.exit(1)
       except:
           sys.exit(1)

    res = ((_('Pthread status:'), res_text, res_colour),)
    return res

def tgtest_pthreadstacksize_description():
    return _('This test checks whether POSIX\nThreads (pthreads) are\nrequired on your distribution.')

# Display the Pthreads warning dialog for distros without enough stack
def display_pthreads_warning():
    alert = p2p_diag.P2PMessageDialog( None, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("You are running a distribution of Linux on which %s requires the usage of pthreads on (ie. very recent glibc)\n\nUnfortunately the maximum stack size on your distribution is not large enough for some games, and therefore you might have issues.\n\nThis is due to the way these games were programmed and a limitation with your distribution, not a limitation of %s technology.\n\nWhen using %s 3.2 or newer, you may not need to use pthreads which\ncan help avoid these problems. Earlier versions of %s, however,\nmust have the pthreads option used.\n\nPlease refer to the Point2Play release notes for more details.") % (Point2Play.WINEX_DISPLAY_TITLE, Point2Play.WINEX_DISPLAY_TITLE, Point2Play.WINEX_DISPLAY_TITLE, Point2Play.WINEX_DISPLAY_TITLE) )

    alert.run()
    alert.destroy()



if (__name__ == "__main__"):
    for test in TESTS:
        print "running test ", test[0]
        eval(test[1] + '()')

