#!/usr/bin/python

"""
Point2Play Source Code
Copyright 2003,2004, TransGaming Technologies Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""

import wave
import os, os.path, Point2Play

import ossaudiodev


TESTS = (_("Sound Support Test"), "tgtest_sound", True),

def tgtest_sound(window):
    """runs a test to ensure that sound output is working with the OSS and ALSA Sound systems"""
    OSSResult = None
    ALSAResult = None
    audio_out = None

    try:
        try:
            P2PPATH=os.environ["P2PPATH"] + "/"
        except:
            P2PPATH=""
        soundpath = P2PPATH+'tests/testsound1.wav'
        aplaypath = P2PPATH+'tests/aplay'
        if not os.path.exists(soundpath) or not os.path.exists(aplaypath):
           return ((_('Sound Support Test'), _("Unable to find necessary files\nCan not test sound card"), 'red'),)
        TestSound1 = wave.open(soundpath, 'rb')
        numframesleft = TestSound1.getnframes()
    except:
        return ((_('Sound Support Test'), _("Unable to find necessary files\nCan not test sound card"), 'red'))

    # OSS Portion of the test
    #
    try:
        audio_out = ossaudiodev.open('/dev/dsp', 'w')
    except IOError, e:
        OSSResult = ((_('OSS Sound Support'), _('Unable to open /dev/dsp'), 'red'))

    if audio_out:
       try:
          audio_out.setparameters(TestSound1.getframerate(), 16,
                                  TestSound1.getnchannels(), ossaudiodev.AFMT_S16_LE, False)
                                  #FIXME: use getsampwidth for 16 and AFMT_S16_LE
       except:
          audio_out.close()
          OSSResult = ((_('OSS Sound Support'), _('Unable to play sound, your\nSound card may not be configured correctly'), 'red'))

       try:
          audio_out.reset()
          while numframesleft > 0:
             buffer = TestSound1.readframes(1000)
             audio_out.write(buffer)
             numframesleft -= 1000
       except:
          audio_out.close
          OSSResult = ((_('OSS Sound Support'), _('Test failed part way through'), 'red'))

       audio_out.close()
       OSSResult = ((_('OSS Sound Support'), _('Completed successfully'), 'green'))


    # ALSA Portion of the test
    #
    fp = os.popen( aplaypath + " -N " + soundpath + " 2>&1", "r" );
    lines = fp.readlines()
    retval = fp.close()
    if retval:
       err_str = "".join( lines )
       ALSAResult = (( _('ALSA Sound Support'), _('Unable to complete sound test:\n') + err_str, 'red'))
    else:
       ALSAResult = ((_('ALSA Sound Support'), _('Completed successfully'), 'green'))


    # Done, return the results of both tests
    res = []
    res.append(OSSResult)
    res.append(ALSAResult)
    return res

def tgtest_sound_description():
    return _('This test attempts to play a sound file at\n44 khz with the Open Sound System (OSS) and\nAdvanced Linux Sound Architecture (ALSA)\nSound Systems.\n\nThis is by no means a comprehensive audio test,\nthough if you are able to hear the sound you\nshould at least have basic sound support in %s\nwith the appropriate driver (ALSA and/or OSS).\n\nThis test should only take a few\nseconds, during which you should\nhear a sound play.\n\nIf this test fails, consult your\nLinux distribution documentation.') % (Point2Play.WINEX_DISPLAY_TITLE)


if (__name__ == "__main__"):
    for test in TESTS:
        print "running test ", test[0]
        print eval(test[1] + '()')

