#!/usr/bin/python

"""
Point2Play Source Code
Copyright 2003,2004,2005 TransGaming Technologies Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""

import os, Point2Play
import system_detection
TESTS = (_("Test For Hardware 3D\nGraphics Acceleration"), "tgtest_three_d", True),

def tgtest_three_d(window):
    """run the glxtg test suite (from the .so file), that returns the FPS that glxgears
    can give us, so this module will then make an analysis based on that as to if the user has
    hardware acceleration"""

    # First we need to construct the path to the test program
    try:
       gltest_path = os.environ["P2PPATH"] + "/tests/gl_test"
    except:
       gltest_path = "tests/gl_test"

    # Execute the test program, getting the results into a file-like variable
    res_fromprog = os.popen (gltest_path, "r")

    # Get the first line (what we care about).. and throw away the rest, since
    # we only care about the first line (actually, there should only BE one line)
    output = res_fromprog.readline()
    res_fromprog.readlines()

    # Split up the output of the test program into meaningful chunks
    results = output.split()
    # if results is empty, then the user probably stopped the test prematurely
    if results == []:
       res = ( (_('Please let the\ntest complete'), '?', 'red'), )

       return res

    # If we got here, that means everything worked out properly.
    # Extract the results so the test can be completed
    result_fps = int(results[0])
    result_isdirect = int(results[1])
    result_glmajor = int(results[2])
    result_glminor = int(results[3])

    # Determine if our rendering is slow or OK
    if (result_fps > 800):
        fps_text = _("OK")
        fps_colour = "green"
    else:
        fps_text = _("Slow")
        fps_colour = "red"

    # Next determine if the user is accelerated or not and has the right version of OpenGL
    if ((result_isdirect) and (result_glmajor >= 1) and (result_glminor >= 2)):
        req_text = _('OK')
        req_colour = "green"
    elif (result_isdirect):
        req_text = _('Incorrect OpenGL version\n%i.%i (requires 1.2)') % (result_glmajor, result_glminor)
        req_colour = "red"
    else:
        req_text = _('Not Available')
        req_colour = "red"

    res = ((_('OpenGL Direct Rendering'), req_text, req_colour), 
           (_('3D Rendering Speed'), fps_text, fps_colour),)
    return res


def tgtest_three_d_description():
    return _('Performs several tests for hardware\nacceleration. This test will attempt to\ndetermine if your graphics card is\nsuitable for running 3D-intense games\nwith %s, and whether it has been\nset up correctly.  The test will also run a\nbenchmark on your card to try and gauge\nthe performance - during this test you\nshould see 3 rotating gears.\n\nThis test may take several minutes\nto perform depending on your hardware.\n\nIf this test fails, consult your\nLinux distribution documentation.') % (Point2Play.WINEX_DISPLAY_TITLE)

if (__name__ == "__main__"):
    for test in TESTS:
        print "running test ", test[0]
        eval(test[1] + '()')

