#!/usr/bin/env python

"""
Point2Play Source Code
Copyright (c) 2005 by TransGaming Technologies, Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""

import pygtk
pygtk.require('2.0')
import gtk
import gtk.glade
import log, copy, gui_utils, p2p_gui
import os
from types import *

p2ppath = os.getenv("P2PPATH")
GLADE_FILE = p2ppath + "/gui/" + __name__ + ".glade"
	
# note I'm leaving out "password" and "proxy_password" because they need encoding and decoding
# (which is done properly in Point2Play.py). Here I just need to make sure they're not in this
# data_widgets_list or bad things will happen.
data_widgets_list = ["username", "proxy_host", "proxy_port", "proxy_username", "proxy_check"]

class WidgetsWrapper:
    def __init__(self):
        self.widgets = gtk.glade.XML(GLADE_FILE, None, "p2p_localized_msg")

    # Gives us the ability to do: widgets['widget_name'].action()
    def __getitem__(self, key):
        return self.widgets.get_widget(key)

class TGAcctDialog:
    def __init__(self, Point2Play):
        self.Point2Play = Point2Play
        self.widgets = WidgetsWrapper()

        gui_utils.load_config_data_into_widgets(self, data_widgets_list, self.Point2Play)
        self.load_passwords_into_widgets()

        # force a call for initial insensitivity
        self.proxy_check_cb(self.widgets["proxy_check"])

        self.widgets.widgets.signal_autoconnect(self)

    def load_passwords_into_widgets(self):
        self.widgets["password"].set_text(self.Point2Play.config["password"])
        self.widgets["proxy_password"].set_text(self.Point2Play.config["proxy_password"])

    def cancel_cb(self, button):
        log.logger.info("Account cancel clicked")
        self.widgets["transgamer_account"].response(0)

    def proxy_port_changed_cb(self, button):
        """ If the user tries to enter a non-number, clear the field """
        try:
            x = int(self.widgets["proxy_port"].get_text())
        except:
            self.widgets["proxy_port"].set_text("")
        
    def proxy_check_cb(self, button):
        active = self.widgets["proxy_check"].get_active()
        for w in ["proxy_host", "proxy_port", "proxy_username", "proxy_password"]:
            self.widgets[w].set_sensitive(active)

    def ok_cb(self, button):
        log.logger.info("Account OK clicked")
        if self.proxy_info_changed():
            p2p_gui.notify_user(_("Changes to your proxy information may not take effect until Cedega is restarted."), self.widgets["transgamer_account"])
        self.Point2Play.config["password"] = self.widgets["password"].get_text()
        self.Point2Play.config["proxy_password"] = self.widgets["proxy_password"].get_text()
        gui_utils.save_widget_data(self, data_widgets_list, self.Point2Play)
        self.widgets["transgamer_account"].response(1)

    def proxy_info_changed(self):
        if self.Point2Play.config["proxy_host"] != self.widgets["proxy_host"].get_text():
            return True
        if str(self.Point2Play.config["proxy_port"]) != self.widgets["proxy_port"].get_text():
            return True
        if self.Point2Play.config["proxy_username"] != self.widgets["proxy_username"].get_text():
            return True
        if self.Point2Play.config["proxy_password"] != self.widgets["proxy_password"].get_text():
            return True
        return False

