"""
Point2Play Source Code
Copyright 2003, TransGaming Technologies Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""

try:
    import pygtk
    pygtk.require('2.0')
except:
    pass

import gtk, winex_get, os, os.path, time, p2p_diag, Point2Play, sys

START_BLOCK_SIZE=1024

def format_time( secs ):
    disp_secs = secs % 60
    mins = secs / 60
    time_str = "%ds" % disp_secs
    if mins:
       disp_mins = mins % 60
       hours = mins / 60
       time_str = "%dm %s" % ( disp_mins, time_str )
       if hours:
           time_str = "%dh %s" % ( hours, time_str )
    return time_str

class WinexGetDialog( gtk.Dialog ):
    def __init__( self, P2P, list, version ):
        gtk.Dialog.__init__(self)
        self.add_button( gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL )
        self.good_to_go = 0
        self.errorstring = ""
        self.fp = None

        try:
           self.fp = winex_get.winex_get( list, version)
        except AttributeError: 
           # this is the type of exception raised when the ClientCookie.urlopen routine tries to 
           # read from a file pointer that is NoneType. We're not exactly sure what the server is
           # doing exactly, so this might need to be expanded. urlopen should have thrown HTTPError
           # if we didn't get the server response of 200 (OK).
           err_diag = p2p_diag.P2PMessageDialog( self, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("The server is currently busy - please wait a few minutes and try again."))
           err_diag.run()
           err_diag.destroy()
           self.errorstring = _("Cedega: server was busy.")
           return 
        except:
           err_diag = p2p_diag.P2PMessageDialog( self, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("There was an error during the attempt to download: %s") % str( sys.exc_info()[1] ) )
           self.errorstring = str(sys.exc_info()[1])
           err_diag.run()
           err_diag.destroy()
           return 

        if self.fp:
            # Parse headers to find bytes
           try:
               self.length = int(self.fp.info()['content-length']) 
           except:
               self.length = 0
           self.block_size = START_BLOCK_SIZE
 
           vbox = self.get_children()[0]
           vbox.add( gtk.Label( os.path.basename( self.fp.geturl() ) ) )
           self.byte_label = gtk.Label("")
           vbox.add( self.byte_label )
           self.bps = gtk.Label( _("BPS: Unknown") )
           hbox = gtk.HBox()
           hbox.pack_start( self.bps, gtk.FALSE, gtk.FALSE )
           vbox.add( hbox )

           if self.length > 0:
               hbox = gtk.HBox()
               self.eta = gtk.Label( _("ETA: Unknown") )
               hbox.pack_start( self.eta, gtk.FALSE, gtk.FALSE )
               vbox.add( hbox )
               self.byte_label.set_text( _("0/%d bytes") % self.length )
               self.pb = gtk.ProgressBar()
               vbox.add( self.pb )
               self.pb.set_text( "0%" )
           else:
               self.byte_label.set_text( _("0 bytes") )
           self.temp_name = "/tmp/%s.%d" % ( str(os.path.basename(self.fp.geturl())), os.getpid() ) 
           self.wp = file(self.temp_name,"w+b")
           self.show_all()
           self.good_to_go = 1

    def arewe_good_to_go (self):
        return self.good_to_go

    def run( self ):
        self.total_read = 0
        self.old_bytes = 0
        self.old_time = time.time()
        self.xfer_rate = 0
        self.input_handler = gtk.input_add_full( self.fp, gtk.gdk.INPUT_READ, self.file_cb )
        resp = gtk.Dialog.run( self )
        if resp == gtk.RESPONSE_DELETE_EVENT or resp == gtk.RESPONSE_CANCEL:
            os.unlink( self.temp_name )
            return None
        return self.temp_name

    def file_cb( self, *args ):
        try:
           data = self.fp.read( self.block_size )
           if data:
              self.wp.write(data)
              size = len( data )
              self.total_read = self.total_read + size
              time_delta = time.time() - self.old_time
              if time_delta > 5:
                  data_delta = self.total_read - self.old_bytes
                  self.xfer_rate = float( data_delta/1024 ) / time_delta
                  if self.xfer_rate > 64:
                      self.block_size = 16384
                  elif self.xfer_rate > 32:
                      self.block_size = 8192
                  elif self.xfer_rate > 16:
                      self.block_size = 4096
                  elif self.xfer_rate > 8:
                      self.block_size = 2048
                  self.bps.set_text( _("BPS: %2.2f KB/s") % self.xfer_rate )
                  self.old_time = time.time()
                  self.old_bytes = self.total_read
              if self.length > 0:
                  percent = float(self.total_read) / float(self.length)
                  self.pb.set_fraction( percent )
                  self.pb.set_text( str( int( percent * 100 ) ) + "%" )
                  self.byte_label.set_text( str( self.total_read/1024 ) + " / " + str( self.length/1024 )+ "K" )
                  if self.xfer_rate > 0:
                      if time_delta > 1:
                          time_left = ( self.length - self.total_read ) / (self.xfer_rate*1024)
                          self.eta.set_text( _("ETA: %s") % format_time( int( time_left ) ) )
                  else:
                      self.eta.set_text( _("ETA: Unknown") )
              else:
                  self.byte_label.set_text( str( self.total_read/1024 ) + "K" )
              while gtk.events_pending():
                  gtk.main_iteration()
           else:
              self.wp.close()
              gtk.Dialog.response( self, 2 )

           return 1
        except:
           # BOOM.. an exception.. 99% sure its due to the fact we ran out of disk space...

           # Display error message...
           alert = p2p_diag.P2PMessageDialog( self, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_OK, _("Error saving %s download. This is most likely due to the disk being full.") % Point2Play.WINEX_DISPLAY_TITLE)
           alert.run()
           alert.destroy()

           # Then close everything down
           self.wp.close( )
           gtk.Dialog.response(self, gtk.RESPONSE_CANCEL)

           return 0
           

