"""
Point2Play Source Code
Copyright 2003, TransGaming Technologies Inc.
All rights reserved
Redistribution of Point2Play is forbidden without the expressed written 
permission of TransGaming Technologies Inc.

"""

try:
    import pygtk
    pygtk.require('2.0')
except:
    pass

try:
    import gtk, gobject
except:
    raise winex_verGUIError( _("Unable to load GTK2 Python bindings") )

import os, socket, sys, p2p_diag, winex_get, winex_get_gui, Point2Play, cedega_version_parse
import gtk.glade, gtk.gdk, log, p2p_gui

p2ppath = os.getenv("P2PPATH")
GLADE_FILE = p2ppath + "/gui/" + __name__ + ".glade"

class winex_verGUIError( Exception ):
    def __init__( self, message ):
        self.message = message

    def __str__( self ):
        return `self.message`

class WidgetsWrapper:
    def __init__(self):
        self.widgets = gtk.glade.XML(GLADE_FILE, None, "p2p_localized_msg")

    # Gives us the ability to do: widgets['widget_name'].action()
    def __getitem__(self, key):
        return self.widgets.get_widget(key)


class InstallOlderEngineDialog:
    def __init__( self, Point2Play_instance ):

        self.Point2Play_instance = Point2Play_instance
        self.had_error = False
        self.tglogin()
        if self.had_error:
            return

        self.widgets = WidgetsWrapper()
        self.widgets.widgets.signal_autoconnect(self)

        self.WineXVersions = { "blah": 0 } 

        self.widgets["instructions_label"].set_text( _("Please select which engine version you would\nlike to install from the list below:") )
        self.populate_listview()

        # values that the caller will require:
        self.version_string = None
        self.version = None
        self.sel_versionparse = None

    def tglogin(self):
        """ Log in and get the list of versions from the server. """

        password = self.Point2Play_instance.config["password"]
        self.Point2Play_instance.setup_environment_for_proxy()

        try:
            winex_get.login_transgaming( self.Point2Play_instance.config["username"], password )
        except socket.gaierror, e:
            err_diag = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("Unable to connect to server")+"\n%s" % e[1] )
            err_diag.run()
            err_diag.destroy()
            self.had_error = True
            return
        except:
            err_diag = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, "%s" % str( sys.exc_info()[1] ) )
            err_diag.run()
            err_diag.destroy()
            self.had_error = True    
            return
        # get the list of available versions from the TG webserver.
        try:
            self.ver_list = winex_get.winex_versions()
        except socket.gaierror, e:
            err_diag = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], None, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, _("Unable to connect to server")+"\n%s" % e[1] )
            err_diag.run()
            err_diag.destroy()
            self.had_error = True
            return
        except:
            err_diag = p2p_diag.P2PMessageDialog( p2p_gui.widgets["main"], None, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK, "%s" % str( sys.exc_info()[1] ) )
            err_diag.run()
            err_diag.destroy()
            self.had_error = True
            return

    def populate_listview(self):
        """ Populate the winex_verlist treeview with all available versions of the
            Cedega engine. """

        # Attach rendering widgets to the game tree
        cell = gtk.CellRendererText()
        column = gtk.TreeViewColumn( _("Versions"), cell, text=0 )
        self.widgets["winex_verlist"].append_column( column )


        # update the list widget with the versions available
        version_list = list()
        if self.ver_list:
           for ver in self.ver_list:
              version_list.append (ver["version"])
           version_list.sort(Point2Play.version_cmp)
           version_list.reverse()
        
        self.model = gtk.ListStore(gobject.TYPE_STRING)
        for verinfo in version_list:
           iter = self.model.append()
           # Get the full version object - its guaranteed to exist as we just sorted from the
           # list above
           for ver in self.ver_list:
              if ver["version"] == verinfo:
                 break

           versionparse = cedega_version_parse.ParseVersion(ver["version"])

           if versionparse.any_credentials_parsed():
              # Any version that has credentials is a Cedega version
              productname = Point2Play.WINEX_DISPLAY_TITLE
              self.model.set_value(iter, 0, _("%s Engine Version %s") % (productname, versionparse.get_parsed_version_string() ) )
           else:
              # Legacy (pre 4.0) products were always called WineX
              productname = "WineX"
              self.model.set_value(iter, 0, _("%s Engine Version %s") % (productname,  versionparse.get_parsed_version_string() ) )

           self.WineXVersions[_("%s Engine Version %s") % (productname, versionparse.get_parsed_version_string() ) ] = ver

        
        self.widgets["winex_verlist"].set_model(self.model)

    def continue_cb(self, button):
        """ Prepare the values the caller will need."""
        log.logger.info("In ok_cb.")
        ver_sel = self.widgets["winex_verlist"].get_selection().get_selected()
        if ver_sel:
            ver_model, ver_iter = ver_sel
            if ver_iter:
                self.version_string = ver_model.get_value( ver_iter, 0 )
                self.ver = self.WineXVersions[self.version_string]
                self.sel_versionparse = cedega_version_parse.ParseVersion(self.ver["version"])
                self.ver["version"] = self.sel_versionparse.get_parsed_version_string()
                self.version = self.ver["version"]
                log.logger.info("Version is " + str(self.version) + " and version string is " + str(self.version_string)) 

